@@ -20,6 +20,7 @@ package org.apache.atlas.hive.bridge;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.sun.jersey.api.client.ClientResponse;
+import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.type.AtlasTypeUtil;
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasClientV2;
@@ -27,16 +28,21 @@ import org.apache.atlas.AtlasServiceException;
 import org.apache.atlas.hive.hook.events.BaseHiveEvent;
 import org.apache.atlas.hive.model.HiveDataTypes;
 import org.apache.atlas.hook.AtlasHookException;
+import org.apache.atlas.model.discovery.AtlasSearchResult;
+import org.apache.atlas.model.discovery.SearchParameters;
 import org.apache.atlas.model.instance.AtlasEntityHeader;
 import org.apache.atlas.model.instance.EntityMutationResponse;
 import org.apache.atlas.model.instance.EntityMutations;
+import org.apache.atlas.utils.AtlasPathExtractorUtil;
 import org.apache.atlas.utils.AuthenticationUtil;
 import org.apache.atlas.utils.HdfsNameServiceResolver;
+import org.apache.atlas.utils.AtlasConfigurationUtil;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntitiesWithExtInfo;
 import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
+import org.apache.atlas.utils.PathExtractorContext;
 import org.apache.commons.cli.ParseException;
 import org.apache.commons.collections.CollectionUtils;
 
@@ -58,6 +64,7 @@ import org.apache.hadoop.hive.metastore.api.SerDeInfo;
 import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 import org.apache.hadoop.hive.ql.metadata.Hive;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
+import org.apache.hadoop.hive.ql.metadata.InvalidTableException;
 import org.apache.hadoop.hive.ql.metadata.Table;
 import org.apache.hadoop.hive.ql.session.SessionState;
 import org.apache.hadoop.security.UserGroupInformation;
@@ -85,24 +92,32 @@ public class HiveMetaStoreBridge {
 
     public static final String CONF_PREFIX                     = "atlas.hook.hive.";
     public static final String CLUSTER_NAME_KEY                = "atlas.cluster.name";
+    public static final String HIVE_USERNAME                   = "atlas.hook.hive.default.username";
     public static final String HIVE_METADATA_NAMESPACE         = "atlas.metadata.namespace";
     public static final String HDFS_PATH_CONVERT_TO_LOWER_CASE = CONF_PREFIX + "hdfs_path.convert_to_lowercase";
+    public static final String HOOK_AWS_S3_ATLAS_MODEL_VERSION = CONF_PREFIX + "aws_s3.atlas.model.version";
     public static final String DEFAULT_CLUSTER_NAME            = "primary";
     public static final String TEMP_TABLE_PREFIX               = "_temp-";
     public static final String ATLAS_ENDPOINT                  = "atlas.rest.address";
     public static final String SEP                             = ":".intern();
     public static final String HDFS_PATH                       = "hdfs_path";
     public static final String DEFAULT_METASTORE_CATALOG       = "hive";
+    public static final String HIVE_TABLE_DB_EDGE_LABEL        = "__hive_table.db";
+    public static final String HOOK_HIVE_PAGE_LIMIT            = CONF_PREFIX + "page.limit";
+
+    public static final String HOOK_AWS_S3_ATLAS_MODEL_VERSION_V2  = "v2";
 
     private static final int    EXIT_CODE_SUCCESS = 0;
     private static final int    EXIT_CODE_FAILED  = 1;
     private static final String DEFAULT_ATLAS_URL = "http://localhost:21000/";
+    private static       int    pageLimit         = 10000;
 
     private final String        metadataNamespace;
     private final Hive          hiveClient;
     private final AtlasClientV2 atlasClientV2;
     private final boolean       convertHdfsPathToLowerCase;
 
+    private String awsS3AtlasModelVersion = null;
 
     public static void main(String[] args) {
         int exitCode = EXIT_CODE_FAILED;
@@ -114,9 +129,13 @@ public class HiveMetaStoreBridge {
             options.addOption("t", "table", true, "Table name");
             options.addOption("f", "filename", true, "Filename");
             options.addOption("failOnError", false, "failOnError");
+            options.addOption("deleteNonExisting", false, "Delete database and table entities in Atlas if not present in Hive");
+
+            CommandLine   cmd               = new BasicParser().parse(options, args);
+            boolean       failOnError       = cmd.hasOption("failOnError");
+            boolean       deleteNonExisting = cmd.hasOption("deleteNonExisting");
+            LOG.info("delete non existing flag : {} ", deleteNonExisting);
 
-            CommandLine   cmd              = new BasicParser().parse(options, args);
-            boolean       failOnError      = cmd.hasOption("failOnError");
             String        databaseToImport = cmd.getOptionValue("d");
             String        tableToImport    = cmd.getOptionValue("t");
             String        fileToImport     = cmd.getOptionValue("f");
@@ -140,7 +159,9 @@ public class HiveMetaStoreBridge {
 
             HiveMetaStoreBridge hiveMetaStoreBridge = new HiveMetaStoreBridge(atlasConf, new HiveConf(), atlasClientV2);
 
-            if (StringUtils.isNotEmpty(fileToImport)) {
+            if (deleteNonExisting) {
+                hiveMetaStoreBridge.deleteEntitiesForNonExistingHiveMetadata(failOnError);
+            } else if (StringUtils.isNotEmpty(fileToImport)) {
                 File f = new File(fileToImport);
 
                 if (f.exists() && f.canRead()) {
@@ -204,6 +225,8 @@ public class HiveMetaStoreBridge {
         System.out.println("    database1:tbl1");
         System.out.println("    database1:tbl2");
         System.out.println("    database2:tbl2");
+        System.out.println("Usage 5: import-hive.sh [-deleteNonExisting] "  );
+        System.out.println("    Deletes databases and tables which are not in Hive ...");
         System.out.println();
     }
 
@@ -216,6 +239,10 @@ public class HiveMetaStoreBridge {
         this.hiveClient                 = Hive.get(hiveConf);
         this.atlasClientV2              = atlasClientV2;
         this.convertHdfsPathToLowerCase = atlasProperties.getBoolean(HDFS_PATH_CONVERT_TO_LOWER_CASE, false);
+        this.awsS3AtlasModelVersion     = atlasProperties.getString(HOOK_AWS_S3_ATLAS_MODEL_VERSION, HOOK_AWS_S3_ATLAS_MODEL_VERSION_V2);
+        if (atlasProperties != null) {
+            pageLimit = atlasProperties.getInteger(HOOK_HIVE_PAGE_LIMIT, 10000);
+        }
     }
 
     /**
@@ -238,7 +265,7 @@ public class HiveMetaStoreBridge {
     }
 
     public String getMetadataNamespace(Configuration config) {
-        return config.getString(HIVE_METADATA_NAMESPACE, getClusterName(config));
+        return AtlasConfigurationUtil.getRecentString(config, HIVE_METADATA_NAMESPACE, getClusterName(config));
     }
 
     private String getClusterName(Configuration config) {
@@ -355,19 +382,28 @@ public class HiveMetaStoreBridge {
                 AtlasEntityWithExtInfo processEntity        = findProcessEntity(processQualifiedName);
 
                 if (processEntity == null) {
-                    String      tableLocation = isConvertHdfsPathToLowerCase() ? lower(table.getDataLocation().toString()) : table.getDataLocation().toString();
-                    String      query         = getCreateTableString(table, tableLocation);
-                    AtlasEntity pathInst      = toHdfsPathEntity(tableLocation);
-                    AtlasEntity tableInst     = tableEntity.getEntity();
-                    AtlasEntity processInst   = new AtlasEntity(HiveDataTypes.HIVE_PROCESS.getName());
-                    long        now           = System.currentTimeMillis();
+                    String tableLocationString = isConvertHdfsPathToLowerCase() ? lower(table.getDataLocation().toString()) : table.getDataLocation().toString();
+                    Path   location            = table.getDataLocation();
+                    String query               = getCreateTableString(table, tableLocationString);
+
+                    PathExtractorContext   pathExtractorCtx  = new PathExtractorContext(getMetadataNamespace(), isConvertHdfsPathToLowerCase(), awsS3AtlasModelVersion);
+                    AtlasEntityWithExtInfo entityWithExtInfo = AtlasPathExtractorUtil.getPathEntity(location, pathExtractorCtx);
+                    AtlasEntity            pathInst          = entityWithExtInfo.getEntity();
+                    AtlasEntity            tableInst         = tableEntity.getEntity();
+                    AtlasEntity            processInst       = new AtlasEntity(HiveDataTypes.HIVE_PROCESS.getName());
+
+                    long now = System.currentTimeMillis();
 
                     processInst.setAttribute(ATTRIBUTE_QUALIFIED_NAME, processQualifiedName);
                     processInst.setAttribute(ATTRIBUTE_NAME, query);
                     processInst.setAttribute(ATTRIBUTE_CLUSTER_NAME, metadataNamespace);
                     processInst.setRelationshipAttribute(ATTRIBUTE_INPUTS, Collections.singletonList(AtlasTypeUtil.getAtlasRelatedObjectId(pathInst, RELATIONSHIP_DATASET_PROCESS_INPUTS)));
                     processInst.setRelationshipAttribute(ATTRIBUTE_OUTPUTS, Collections.singletonList(AtlasTypeUtil.getAtlasRelatedObjectId(tableInst, RELATIONSHIP_PROCESS_DATASET_OUTPUTS)));
-                    processInst.setAttribute(ATTRIBUTE_USER_NAME, table.getOwner());
+                    String userName = table.getOwner();
+                    if (StringUtils.isEmpty(userName)) {
+                        userName = ApplicationProperties.get().getString(HIVE_USERNAME, "hive");
+                    }
+                    processInst.setAttribute(ATTRIBUTE_USER_NAME, userName);
                     processInst.setAttribute(ATTRIBUTE_START_TIME, now);
                     processInst.setAttribute(ATTRIBUTE_END_TIME, now);
                     processInst.setAttribute(ATTRIBUTE_OPERATION_TYPE, "CREATETABLE");
@@ -379,7 +415,12 @@ public class HiveMetaStoreBridge {
                     AtlasEntitiesWithExtInfo createTableProcess = new AtlasEntitiesWithExtInfo();
 
                     createTableProcess.addEntity(processInst);
-                    createTableProcess.addEntity(pathInst);
+
+                    if (pathExtractorCtx.getKnownEntities() != null) {
+                        pathExtractorCtx.getKnownEntities().values().forEach(entity -> createTableProcess.addEntity(entity));
+                    } else {
+                        createTableProcess.addEntity(pathInst);
+                    }
 
                     registerInstances(createTableProcess);
                 } else {
@@ -725,35 +766,6 @@ public class HiveMetaStoreBridge {
         return ret;
     }
 
-    private AtlasEntity toHdfsPathEntity(String pathUri) {
-        AtlasEntity ret           = new AtlasEntity(HDFS_PATH);
-        String      nameServiceID = HdfsNameServiceResolver.getNameServiceIDForPath(pathUri);
-        Path        path          = new Path(pathUri);
-
-        ret.setAttribute(ATTRIBUTE_NAME, Path.getPathWithoutSchemeAndAuthority(path).toString());
-        ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, metadataNamespace);
-
-        if (StringUtils.isNotEmpty(nameServiceID)) {
-            // Name service resolution is successful, now get updated HDFS path where the host port info is replaced by resolved name service
-            String updatedHdfsPath = HdfsNameServiceResolver.getPathWithNameServiceID(pathUri);
-
-            ret.setAttribute(ATTRIBUTE_PATH, updatedHdfsPath);
-            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHdfsPathQualifiedName(updatedHdfsPath));
-            ret.setAttribute(ATTRIBUTE_NAMESERVICE_ID, nameServiceID);
-        } else {
-            ret.setAttribute(ATTRIBUTE_PATH, pathUri);
-
-            // Only append metadataNamespace for the HDFS path
-            if (pathUri.startsWith(HdfsNameServiceResolver.HDFS_SCHEME)) {
-                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHdfsPathQualifiedName(pathUri));
-            } else {
-                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathUri);
-            }
-        }
-
-        return ret;
-    }
-
     /**
      * Gets the atlas entity for the database
      * @param databaseName  database Name
@@ -768,7 +780,7 @@ public class HiveMetaStoreBridge {
 
         String typeName = HiveDataTypes.HIVE_DB.getName();
 
-        return findEntity(typeName, getDBQualifiedName(metadataNamespace, databaseName));
+        return findEntity(typeName, getDBQualifiedName(metadataNamespace, databaseName), true, true);
     }
 
     /**
@@ -786,7 +798,7 @@ public class HiveMetaStoreBridge {
         String typeName         = HiveDataTypes.HIVE_TABLE.getName();
         String tblQualifiedName = getTableQualifiedName(getMetadataNamespace(), hiveTable.getDbName(), hiveTable.getTableName());
 
-        return findEntity(typeName, tblQualifiedName);
+        return findEntity(typeName, tblQualifiedName, true, true);
     }
 
     private AtlasEntityWithExtInfo findProcessEntity(String qualifiedName) throws Exception{
@@ -796,14 +808,14 @@ public class HiveMetaStoreBridge {
 
         String typeName = HiveDataTypes.HIVE_PROCESS.getName();
 
-        return findEntity(typeName, qualifiedName);
+        return findEntity(typeName, qualifiedName , true , true);
     }
 
-    private AtlasEntityWithExtInfo findEntity(final String typeName, final String qualifiedName) throws AtlasServiceException {
+    private AtlasEntityWithExtInfo findEntity(final String typeName, final String qualifiedName ,  boolean minExtInfo, boolean ignoreRelationship) throws AtlasServiceException {
         AtlasEntityWithExtInfo ret = null;
 
         try {
-            ret = atlasClientV2.getEntityByAttribute(typeName, Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName));
+            ret = atlasClientV2.getEntityByAttribute(typeName, Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName), minExtInfo, ignoreRelationship);
         } catch (AtlasServiceException e) {
             if(e.getStatus() == ClientResponse.Status.NOT_FOUND) {
                 return null;
@@ -812,8 +824,6 @@ public class HiveMetaStoreBridge {
             throw e;
         }
 
-        clearRelationshipAttributes(ret);
-
         return ret;
     }
 
@@ -971,4 +981,214 @@ public class HiveMetaStoreBridge {
         }
         return ret;
     }
-}
\ No newline at end of file
+
+    private List<AtlasEntityHeader> getAllDatabaseInCluster() throws AtlasServiceException {
+
+        List<AtlasEntityHeader> entities   = new ArrayList<>();
+        final int               pageSize   = pageLimit;
+
+        SearchParameters.FilterCriteria fc = new SearchParameters.FilterCriteria();
+        fc.setAttributeName(ATTRIBUTE_CLUSTER_NAME);
+        fc.setAttributeValue(metadataNamespace);
+        fc.setOperator(SearchParameters.Operator.EQ);
+
+        for (int i = 0; ; i++) {
+            int offset = pageSize * i;
+            LOG.info("Retrieving databases: offset={}, pageSize={}", offset, pageSize);
+
+            AtlasSearchResult searchResult = atlasClientV2.basicSearch(HIVE_TYPE_DB, fc,null, null, true, pageSize, offset);
+
+            List<AtlasEntityHeader> entityHeaders = searchResult == null ? null : searchResult.getEntities();
+            int                     dbCount       = entityHeaders == null ? 0 : entityHeaders.size();
+
+            LOG.info("Retrieved {} databases of {} cluster", dbCount, metadataNamespace);
+
+            if (dbCount > 0) {
+                entities.addAll(entityHeaders);
+            }
+
+            if (dbCount < pageSize) { // last page
+                break;
+            }
+        }
+
+        return entities;
+    }
+
+    private List<AtlasEntityHeader> getAllTablesInDb(String databaseGuid) throws AtlasServiceException {
+
+        List<AtlasEntityHeader> entities = new ArrayList<>();
+        final int               pageSize = pageLimit;
+
+        for (int i = 0; ; i++) {
+            int offset = pageSize * i;
+            LOG.info("Retrieving tables: offset={}, pageSize={}", offset, pageSize);
+
+            AtlasSearchResult searchResult = atlasClientV2.relationshipSearch(databaseGuid, HIVE_TABLE_DB_EDGE_LABEL, null, null, true, pageSize, offset);
+
+            List<AtlasEntityHeader> entityHeaders = searchResult == null ? null : searchResult.getEntities();
+            int                     tableCount    = entityHeaders == null ? 0 : entityHeaders.size();
+
+            LOG.info("Retrieved {} tables of {} database", tableCount, databaseGuid);
+
+            if (tableCount > 0) {
+                entities.addAll(entityHeaders);
+            }
+
+            if (tableCount < pageSize) { // last page
+                break;
+            }
+        }
+
+        return entities;
+    }
+
+    public String getHiveDatabaseName(String qualifiedName) {
+
+        if (StringUtils.isNotEmpty(qualifiedName)) {
+            String[] split = qualifiedName.split("@");
+            if (split.length > 0) {
+                return split[0];
+            }
+        }
+        return null;
+    }
+
+
+    public String getHiveTableName(String qualifiedName, boolean isTemporary) {
+
+        if (StringUtils.isNotEmpty(qualifiedName)) {
+            String tableName = StringUtils.substringBetween(qualifiedName, ".", "@");
+            if (!isTemporary) {
+                return tableName;
+            } else {
+                if (StringUtils.isNotEmpty(tableName)) {
+                    String[] splitTemp = tableName.split(TEMP_TABLE_PREFIX);
+                    if (splitTemp.length > 0) {
+                        return splitTemp[0];
+                    }
+                }
+            }
+        }
+        return null;
+    }
+
+    private void deleteByGuid(List<String> guidTodelete) throws AtlasServiceException {
+
+        if (CollectionUtils.isNotEmpty(guidTodelete)) {
+
+            for (String guid : guidTodelete) {
+                EntityMutationResponse response = atlasClientV2.deleteEntityByGuid(guid);
+
+                if (response.getDeletedEntities().size() < 1) {
+                    LOG.info("Entity with guid : {} is not deleted", guid);
+                } else {
+                    LOG.info("Entity with guid : {} is deleted", guid);
+                }
+            }
+        } else {
+            LOG.info("No Entity to delete from Atlas");
+        }
+    }
+
+    public void deleteEntitiesForNonExistingHiveMetadata(boolean failOnError) throws Exception {
+
+        //fetch databases from Atlas
+        List<AtlasEntityHeader> dbs = null;
+        try {
+            dbs = getAllDatabaseInCluster();
+            LOG.info("Total Databases in cluster {} : {} ", metadataNamespace, dbs.size());
+        } catch (AtlasServiceException e) {
+            LOG.error("Failed to retrieve database entities for cluster {} from Atlas", metadataNamespace, e);
+            if (failOnError) {
+                throw e;
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(dbs)) {
+            //iterate all dbs to check if exists in hive
+            for (AtlasEntityHeader db : dbs) {
+
+                String dbGuid     = db.getGuid();
+                String hiveDbName = getHiveDatabaseName((String) db.getAttribute(ATTRIBUTE_QUALIFIED_NAME));
+
+                if (StringUtils.isEmpty(hiveDbName)) {
+                    LOG.error("Failed to get database from qualifiedName: {}, guid: {} ", db.getAttribute(ATTRIBUTE_QUALIFIED_NAME), dbGuid);
+                    continue;
+                }
+
+                List<AtlasEntityHeader> tables;
+                try {
+                    tables = getAllTablesInDb(dbGuid);
+                    LOG.info("Total Tables in database {} : {} ", hiveDbName, tables.size());
+                } catch (AtlasServiceException e) {
+                    LOG.error("Failed to retrieve table entities for database {} from Atlas", hiveDbName, e);
+                    if (failOnError) {
+                        throw e;
+                    }
+                    continue;
+                }
+
+                List<String> guidsToDelete = new ArrayList<>();
+                if (!hiveClient.databaseExists(hiveDbName)) {
+
+                    //table guids
+                    if (CollectionUtils.isNotEmpty(tables)) {
+                        for (AtlasEntityHeader table : tables) {
+                            guidsToDelete.add(table.getGuid());
+                        }
+                    }
+
+                    //db guid
+                    guidsToDelete.add(db.getGuid());
+                    LOG.info("Added database {}.{} and its {} tables to delete", metadataNamespace, hiveDbName, tables.size());
+
+                } else {
+                    //iterate all table of db to check if it exists
+                    if (CollectionUtils.isNotEmpty(tables)) {
+                        for (AtlasEntityHeader table : tables) {
+                            String hiveTableName = getHiveTableName((String) table.getAttribute(ATTRIBUTE_QUALIFIED_NAME), true);
+
+                            if (StringUtils.isEmpty(hiveTableName)) {
+                                LOG.error("Failed to get table from qualifiedName: {}, guid: {} ", table.getAttribute(ATTRIBUTE_QUALIFIED_NAME), table.getGuid());
+                                continue;
+                            }
+
+                            try {
+                                hiveClient.getTable(hiveDbName, hiveTableName, true);
+                            } catch (InvalidTableException e) { //table doesn't exists
+                                LOG.info("Added table {}.{} to delete", hiveDbName, hiveTableName);
+
+                                guidsToDelete.add(table.getGuid());
+                            } catch (HiveException e) {
+                                LOG.error("Failed to get table {}.{} from Hive", hiveDbName, hiveTableName, e);
+
+                                if (failOnError) {
+                                    throw e;
+                                }
+                            }
+                        }
+                    }
+                }
+
+                //delete entities
+                if (CollectionUtils.isNotEmpty(guidsToDelete)) {
+                    try {
+                        deleteByGuid(guidsToDelete);
+                    } catch (AtlasServiceException e) {
+                        LOG.error("Failed to delete Atlas entities for database {}", hiveDbName, e);
+
+                        if (failOnError) {
+                            throw e;
+                        }
+                    }
+
+                }
+            }
+
+        } else {
+            LOG.info("No database found in service.");
+        }
+
+    }
+}
