@@ -19,6 +19,7 @@
 package org.apache.atlas.hive.hook;
 
 import org.apache.atlas.hive.hook.events.*;
+import org.apache.atlas.hive.hook.utils.ActiveEntityFilter;
 import org.apache.atlas.hook.AtlasHook;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.utils.LruCache;
@@ -68,6 +69,9 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
     public static final String HOOK_HIVE_TABLE_IGNORE_PATTERN                            = CONF_PREFIX + "hive_table.ignore.pattern";
     public static final String HOOK_HIVE_TABLE_PRUNE_PATTERN                             = CONF_PREFIX + "hive_table.prune.pattern";
     public static final String HOOK_HIVE_TABLE_CACHE_SIZE                                = CONF_PREFIX + "hive_table.cache.size";
+    public static final String HOOK_HIVE_IGNORE_DDL_OPERATIONS                           = CONF_PREFIX + "hs2.ignore.ddl.operations";
+    public static final String HOOK_HIVE_FILTER_ENTITY_ADDITIONAL_TYPES_TO_RETAIN        = CONF_PREFIX + "hs2.filter.entity.additional.types.to.retain";
+    public static final String HOOK_HIVE_SKIP_TEMP_TABLES                                = CONF_PREFIX + "skip.temp.tables";
     public static final String DEFAULT_HOST_NAME = "localhost";
 
     private static final Map<String, HiveOperation> OPERATION_MAP = new HashMap<>();
@@ -88,8 +92,9 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
     private static final List                          ignoreDummyTableName;
     private static final String                        ignoreValuesTmpTableNamePrefix;
     private static final boolean                       hiveProcessPopulateDeprecatedAttributes;
-    private static HiveHookObjectNamesCache knownObjects = null;
+    private static HiveHookObjectNamesCache            knownObjects = null;
     private static String hostName;
+    private static boolean                             skipTempTables = true;
 
     static {
         for (HiveOperation hiveOperation : HiveOperation.values()) {
@@ -151,6 +156,7 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
         ignoreDummyDatabaseName        = atlasProperties.getList("atlas.hook.hive.ignore.dummy.database.name", defaultDummyDatabase);
         ignoreDummyTableName           = atlasProperties.getList("atlas.hook.hive.ignore.dummy.table.name", defaultDummyTable);
         ignoreValuesTmpTableNamePrefix = atlasProperties.getString("atlas.hook.hive.ignore.values.tmp.table.name.prefix", "Values__Tmp__Table__");
+        skipTempTables                 = atlasProperties.getBoolean(HOOK_HIVE_SKIP_TEMP_TABLES, true);
 
         try {
             hostName = InetAddress.getLocalHost().getHostName();
@@ -158,12 +164,19 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
             LOG.warn("No hostname found. Setting the hostname to default value {}", DEFAULT_HOST_NAME, e);
             hostName = DEFAULT_HOST_NAME;
         }
+
+        ActiveEntityFilter.init(atlasProperties);
     }
 
 
     public HiveHook() {
     }
 
+    public HiveHook(String name) {
+        super(name);
+    }
+
+
     @Override
     public void run(HookContext hookContext) throws Exception {
         if (LOG.isDebugEnabled()) {
@@ -172,7 +185,7 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
 
         try {
             HiveOperation        oper    = OPERATION_MAP.get(hookContext.getOperationName());
-            AtlasHiveHookContext context = new AtlasHiveHookContext(this, oper, hookContext, getKnownObjects());
+            AtlasHiveHookContext context = new AtlasHiveHookContext(this, oper, hookContext, getKnownObjects(), isSkipTempTables());
             BaseHiveEvent        event   = null;
 
             switch (oper) {
@@ -191,22 +204,24 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
                 break;
 
                 case CREATETABLE:
-                    event = new CreateTable(context, true);
+                    event = new CreateTable(context);
                 break;
 
                 case DROPTABLE:
                 case DROPVIEW:
+                case DROP_MATERIALIZED_VIEW:
                     event = new DropTable(context);
                 break;
 
                 case CREATETABLE_AS_SELECT:
+                case CREATE_MATERIALIZED_VIEW:
                 case CREATEVIEW:
                 case ALTERVIEW_AS:
                 case LOAD:
                 case EXPORT:
                 case IMPORT:
                 case QUERY:
-                    event = new CreateHiveProcess(context, true);
+                    event = new CreateHiveProcess(context);
                 break;
 
                 case ALTERTABLE_FILEFORMAT:
@@ -242,7 +257,7 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
             if (event != null) {
                 final UserGroupInformation ugi = hookContext.getUgi() == null ? Utils.getUGI() : hookContext.getUgi();
 
-                super.notifyEntities(event.getNotificationMessages(), ugi);
+                super.notifyEntities(ActiveEntityFilter.apply(event.getNotificationMessages()), ugi);
             }
         } catch (Throwable t) {
             LOG.error("HiveHook.run(): failed to process operation {}", hookContext.getOperationName(), t);
@@ -285,6 +300,10 @@ public class HiveHook extends AtlasHook implements ExecuteWithHookContext {
         return hiveProcessPopulateDeprecatedAttributes;
     }
 
+    public static boolean isSkipTempTables() {
+        return skipTempTables;
+    }
+
     public PreprocessAction getPreprocessActionForHiveTable(String qualifiedName) {
         PreprocessAction ret = PreprocessAction.NONE;
 
