@@ -18,7 +18,6 @@
 
 package org.apache.atlas.hive.hook.events;
 
-import com.google.common.collect.ImmutableMap;
 import org.apache.atlas.hive.hook.AtlasHiveHookContext;
 import org.apache.atlas.hive.hook.HiveHook.PreprocessAction;
 import org.apache.atlas.utils.PathExtractorContext;
@@ -30,7 +29,6 @@ import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasRelatedObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
 import org.apache.atlas.model.notification.HookNotification;
-import org.apache.atlas.repository.Constants;
 import org.apache.atlas.type.AtlasTypeUtil;
 import org.apache.atlas.utils.AtlasPathExtractorUtil;
 import org.apache.atlas.utils.HdfsNameServiceResolver;
@@ -55,7 +53,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
-import java.net.InetAddress;
 import java.net.URI;
 import java.util.*;
 
@@ -86,6 +83,7 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_OWNER                     = "owner";
     public static final String ATTRIBUTE_CLUSTER_NAME              = "clusterName";
     public static final String ATTRIBUTE_LOCATION                  = "location";
+    public static final String ATTRIBUTE_LOCATION_PATH             = "locationPath";
     public static final String ATTRIBUTE_PARAMETERS                = "parameters";
     public static final String ATTRIBUTE_OWNER_TYPE                = "ownerType";
     public static final String ATTRIBUTE_COMMENT                   = "comment";
@@ -97,6 +95,7 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_TEMPORARY                 = "temporary";
     public static final String ATTRIBUTE_RETENTION                 = "retention";
     public static final String ATTRIBUTE_DB                        = "db";
+    public static final String ATTRIBUTE_HIVE_DB                   = "hiveDb";
     public static final String ATTRIBUTE_STORAGEDESC               = "sd";
     public static final String ATTRIBUTE_PARTITION_KEYS            = "partitionKeys";
     public static final String ATTRIBUTE_COLUMNS                   = "columns";
@@ -137,6 +136,8 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_EXEC_TIME                 = "execTime";
     public static final String ATTRIBUTE_DDL_QUERIES               = "ddlQueries";
     public static final String ATTRIBUTE_SERVICE_TYPE              = "serviceType";
+    public static final String ATTRIBUTE_GUID                      = "guid";
+    public static final String ATTRIBUTE_UNIQUE_ATTRIBUTES         = "uniqueAttributes";
     public static final String HBASE_STORAGE_HANDLER_CLASS         = "org.apache.hadoop.hive.hbase.HBaseStorageHandler";
     public static final String HBASE_DEFAULT_NAMESPACE             = "default";
     public static final String HBASE_NAMESPACE_TABLE_DELIMITER     = ":";
@@ -154,12 +155,14 @@ public abstract class BaseHiveEvent {
     public static final String RELATIONSHIP_HIVE_TABLE_STORAGE_DESC       = "hive_table_storagedesc";
     public static final String RELATIONSHIP_HIVE_PROCESS_PROCESS_EXE      = "hive_process_process_executions";
     public static final String RELATIONSHIP_HIVE_DB_DDL_QUERIES           = "hive_db_ddl_queries";
+    public static final String RELATIONSHIP_HIVE_DB_LOCATION              = "hive_db_location";
     public static final String RELATIONSHIP_HIVE_TABLE_DDL_QUERIES        = "hive_table_ddl_queries";
     public static final String RELATIONSHIP_HBASE_TABLE_NAMESPACE         = "hbase_table_namespace";
 
 
     public static final Map<Integer, String> OWNER_TYPE_TO_ENUM_VALUE = new HashMap<>();
 
+    protected final boolean skipTempTables;
 
     static {
         OWNER_TYPE_TO_ENUM_VALUE.put(1, "USER");
@@ -171,7 +174,8 @@ public abstract class BaseHiveEvent {
 
 
     protected BaseHiveEvent(AtlasHiveHookContext context) {
-        this.context = context;
+        this.context        = context;
+        this.skipTempTables = context.isSkipTempTables();
     }
 
     public AtlasHiveHookContext getContext() {
@@ -228,7 +232,8 @@ public abstract class BaseHiveEvent {
         switch(entity.getType()) {
             case TABLE:
             case PARTITION:
-            case DFS_DIR: {
+            case DFS_DIR:
+            case LOCAL_DIR: {
                 ret = toAtlasEntity(entity, entityExtInfo, skipTempTables);
             }
             break;
@@ -276,7 +281,8 @@ public abstract class BaseHiveEvent {
             }
             break;
 
-            case DFS_DIR: {
+            case DFS_DIR:
+            case LOCAL_DIR: {
                 URI location = entity.getLocation();
 
                 if (location != null) {
@@ -692,6 +698,33 @@ public abstract class BaseHiveEvent {
         return hiveDDL;
     }
 
+    protected AtlasEntity createHiveLocationEntity(AtlasEntity dbEntity, AtlasEntitiesWithExtInfo extInfoEntity) {
+        AtlasEntity ret          = null;
+        String      locationUri  = (String)dbEntity.getAttribute(ATTRIBUTE_LOCATION);
+
+        if (StringUtils.isNotEmpty(locationUri)) {
+            Path path = null;
+
+            try {
+                path = new Path(locationUri);
+            } catch (IllegalArgumentException excp) {
+                LOG.warn("failed to create Path from locationUri {}", locationUri, excp);
+            }
+
+            if (path != null) {
+                ret = getPathEntity(path, extInfoEntity);
+
+                if (ret != null) {
+                    AtlasRelatedObjectId dbRelatedObjectId = AtlasTypeUtil.getAtlasRelatedObjectId(dbEntity, RELATIONSHIP_HIVE_DB_LOCATION);
+
+                    ret.setRelationshipAttribute(ATTRIBUTE_HIVE_DB, dbRelatedObjectId);
+                }
+            }
+        }
+
+        return ret;
+    }
+
     protected String getMetadataNamespace() {
         return context.getMetadataNamespace();
     }
@@ -790,6 +823,7 @@ public abstract class BaseHiveEvent {
                 return getQualifiedName(entity.getTable());
 
             case DFS_DIR:
+            case LOCAL_DIR:
                 return getQualifiedName(entity.getLocation());
         }
 
@@ -1008,6 +1042,7 @@ public abstract class BaseHiveEvent {
         Collections.sort(sortedEntities, entityComparator);
 
         Set<String> dataSetsProcessed = new HashSet<>();
+        Map<String, Table> tableMap   = new HashMap<>();
 
         for (Entity entity : sortedEntities) {
             if (ignoreHDFSPaths && (Entity.Type.DFS_DIR.equals(entity.getType()) || Entity.Type.LOCAL_DIR.equals(entity.getType()))) {
@@ -1019,8 +1054,14 @@ public abstract class BaseHiveEvent {
 
             try {
                 if (entity.getType() == Entity.Type.PARTITION || entity.getType() == Entity.Type.TABLE) {
-                    Table table = getHive().getTable(entity.getTable().getDbName(), entity.getTable().getTableName());
+                    String tableKey = entity.getTable().getDbName() + "." + entity.getTable().getTableName();
+                    Table  table    = tableMap.get(tableKey);
 
+                    if (table == null) {
+                        table = getHive().getTable(entity.getTable().getDbName(), entity.getTable().getTableName());
+
+                        tableMap.put(tableKey, table); //since there could be several partitions in a table, store it to avoid hive calls.
+                    }
                     if (table != null) {
                         createTime    = getTableCreateTime(table);
                         qualifiedName = getQualifiedName(table);
