@@ -175,7 +175,7 @@ public class HiveITBase {
     }
 
     protected String createTestDFSPath(String path) throws Exception {
-        return "pfile://" + mkdir(path);
+        return "file://" + mkdir(path);
     }
 
     protected String file(String tag) throws Exception {
@@ -224,7 +224,7 @@ public class HiveITBase {
 
     protected String assertEntityIsRegistered(final String typeName, final String property, final String value,
                                               final HiveHookIT.AssertPredicate assertPredicate) throws Exception {
-        waitFor(80000, new HiveHookIT.Predicate() {
+        waitFor(100000, new HiveHookIT.Predicate() {
             @Override
             public void evaluate() throws Exception {
                 AtlasEntity.AtlasEntityWithExtInfo atlasEntityWithExtInfo = atlasClientV2.getEntityByAttribute(typeName, Collections.singletonMap(property,value));
@@ -240,6 +240,25 @@ public class HiveITBase {
         return (String) entity.getGuid();
     }
 
+    protected String assertEntityIsRegisteredViaGuid(String guid,
+                                              final HiveHookIT.AssertPredicate assertPredicate) throws Exception {
+        waitFor(100000, new HiveHookIT.Predicate() {
+            @Override
+            public void evaluate() throws Exception {
+                    AtlasEntity.AtlasEntityWithExtInfo atlasEntityWithExtInfo = atlasClientV2.getEntityByGuid(guid);
+                    AtlasEntity entity = atlasEntityWithExtInfo.getEntity();
+                    assertNotNull(entity);
+                    if (assertPredicate != null) {
+                        assertPredicate.assertOnEntity(entity);
+                    }
+
+            }
+        });
+        AtlasEntity.AtlasEntityWithExtInfo atlasEntityWithExtInfo = atlasClientV2.getEntityByGuid(guid);
+        AtlasEntity entity = atlasEntityWithExtInfo.getEntity();
+        return (String) entity.getGuid();
+    }
+
     protected AtlasEntity assertEntityIsRegistedViaEntity(final String typeName, final String property, final String value,
                                               final HiveHookIT.AssertPredicate assertPredicate) throws Exception {
         waitFor(80000, new HiveHookIT.Predicate() {
@@ -354,7 +373,7 @@ public class HiveITBase {
 
     protected void assertEntityIsNotRegistered(final String typeName, final String property, final String value) throws Exception {
         // wait for sufficient time before checking if entity is not available.
-        long waitTime = 2000;
+        long waitTime = 10000;
         LOG.debug("Waiting for {} msecs, before asserting entity is not registered.", waitTime);
         Thread.sleep(waitTime);
 
@@ -500,7 +519,7 @@ public class HiveITBase {
                 Table outTable = entity.getTable();
                 //refresh table
                 outTable = dgiBridge.getHiveClient().getTable(outTable.getDbName(), outTable.getTableName());
-                return HiveMetaStoreBridge.getTableProcessQualifiedName(dgiBridge.getClusterName(), outTable);
+                return HiveMetaStoreBridge.getTableProcessQualifiedName(dgiBridge.getMetadataNamespace(), outTable);
             }
         }
 
@@ -518,7 +537,6 @@ public class HiveITBase {
         return buffer.toString();
     }
 
-
     protected static Entity getEntityByType(Set<? extends Entity> entities, Entity.Type entityType) {
         for (Entity entity : entities) {
             if (entity.getType() == entityType) {
