@@ -1,4 +1,3 @@
-
 /**
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -529,6 +528,9 @@ public abstract class BaseImpalaEvent {
             queryStr = queryStr.toLowerCase().trim();
         }
 
+        Long startTime = getQueryStartTime();
+        Long endTime   = getQueryEndTime();
+
         ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getQualifiedName(inputs, outputs));
         ret.setAttribute(ATTRIBUTE_INPUTS, getObjectIds(inputs));
         ret.setAttribute(ATTRIBUTE_OUTPUTS,  getObjectIds(outputs));
@@ -538,8 +540,8 @@ public abstract class BaseImpalaEvent {
         // We are setting an empty value to these attributes, since now we have a new entity type called impala process
         // execution which captures these values. We have to set empty values here because these attributes are
         // mandatory attributes for impala process entity type.
-        ret.setAttribute(ATTRIBUTE_START_TIME, EMPTY_ATTRIBUTE_VALUE);
-        ret.setAttribute(ATTRIBUTE_END_TIME, EMPTY_ATTRIBUTE_VALUE);
+        ret.setAttribute(ATTRIBUTE_START_TIME, startTime);
+        ret.setAttribute(ATTRIBUTE_END_TIME, endTime);
         ret.setAttribute(ATTRIBUTE_USER_NAME, EMPTY_ATTRIBUTE_VALUE);
         ret.setAttribute(ATTRIBUTE_QUERY_TEXT, EMPTY_ATTRIBUTE_VALUE);
         ret.setAttribute(ATTRIBUTE_QUERY_ID, EMPTY_ATTRIBUTE_VALUE);
@@ -577,13 +579,7 @@ public abstract class BaseImpalaEvent {
     }
 
     protected Long getQueryStartTime() {
-        Long startTime = null;
-        if (context.getLineageQuery() == null || context.getLineageQuery().getTimestamp() == null) {
-            startTime = System.currentTimeMillis();
-        } else {
-            startTime = context.getLineageQuery().getTimestamp() * BaseImpalaEvent.MILLIS_CONVERT_FACTOR;
-        }
-        return startTime;
+        return context.getLineageQuery().getTimestamp() * BaseImpalaEvent.MILLIS_CONVERT_FACTOR;
     }
 
     protected Long getQueryEndTime() {
@@ -649,16 +645,13 @@ public abstract class BaseImpalaEvent {
             hiveDDL = new AtlasEntity(ImpalaDataType.HIVE_TABLE_DDL.getName(), ATTRIBUTE_TABLE, objId);
         }
 
-        Long startTime = getQueryStartTime();
-
         if (hiveDDL != null) {
             hiveDDL.setAttribute(ATTRIBUTE_SERVICE_TYPE, "impala");
-            hiveDDL.setAttribute(ATTRIBUTE_EXEC_TIME, startTime);
+            hiveDDL.setAttribute(ATTRIBUTE_EXEC_TIME, getQueryStartTime());
             hiveDDL.setAttribute(ATTRIBUTE_QUERY_TEXT, context.getQueryStr());
             hiveDDL.setAttribute(ATTRIBUTE_USER_NAME, getUserName());
-            hiveDDL.setAttribute(ATTRIBUTE_NAME, context.getQueryStr());
-            hiveDDL.setAttribute(ATTRIBUTE_QUALIFIED_NAME, dbOrTable.getAttribute(ATTRIBUTE_QUALIFIED_NAME).toString()
-                + QNAME_SEP_PROCESS + startTime.toString());
+            hiveDDL.setAttribute(ATTRIBUTE_NAME, context.getQueryStr() + QNAME_SEP_PROCESS + getQueryStartTime().toString());
+            hiveDDL.setAttribute(ATTRIBUTE_QUALIFIED_NAME, hiveDDL.getAttribute(ATTRIBUTE_NAME));
         }
 
         return hiveDDL;
