@@ -78,6 +78,12 @@ public class AtlasPathExtractorUtilTest {
     private static final String S3_PATH                          = S3_SCHEME + "aws_my_bucket1/1234567890/renders/Irradiance_A.csv";
     private static final String S3A_PATH                         = S3A_SCHEME + "aws_my_bucket1/1234567890/renders/Irradiance_A.csv";
 
+    // Google Cloud Storage
+    private static final String GCS_VIRTUAL_DIR            = "gcp_storage_virtual_directory";
+    private static final String GCS_BUCKET                 = "gcp_storage_bucket";
+    private static final String GCS_SCHEME                 = "gs" + SCHEME_SEPARATOR;
+    private static final String GCS_PATH                   = GCS_SCHEME + "gcs_test_bucket1/1234567890/data";
+
     @DataProvider(name = "ozonePathProvider")
     private Object[][] ozonePathProvider(){
         return new Object[][]{
@@ -264,6 +270,22 @@ public class AtlasPathExtractorUtilTest {
         verifyS3KnownEntities(S3A_SCHEME, S3A_PATH, extractorContext.getKnownEntities());
     }
 
+    @Test
+    public void testGetPathEntityGCSPath() {
+        PathExtractorContext extractorContext = new PathExtractorContext(METADATA_NAMESPACE);
+
+        Path path = new Path(GCS_PATH);
+        AtlasEntityWithExtInfo entityWithExtInfo = AtlasPathExtractorUtil.getPathEntity(path, extractorContext);
+        AtlasEntity entity = entityWithExtInfo.getEntity();
+
+        assertNotNull(entity);
+        assertEquals(entity.getTypeName(), GCS_VIRTUAL_DIR);
+        assertEquals(entityWithExtInfo.getReferredEntities().size(), 1);
+
+        verifyGCSVirtualDir(GCS_SCHEME, GCS_PATH, entity);
+        verifyGCSKnownEntities(GCS_SCHEME, GCS_PATH, extractorContext.getKnownEntities());
+    }
+
     private void verifyOzoneEntities(Map<String, AtlasEntity> knownEntities, OzoneKeyValidator validator) {
         for (AtlasEntity knownEntity : knownEntities.values()) {
             switch (knownEntity.getTypeName()){
@@ -304,17 +326,17 @@ public class AtlasPathExtractorUtilTest {
     }
 
     private void verifyABFSAdlsGen2Dir(String abfsScheme, String path, AtlasEntity entity){
-        String pathQName = abfsScheme + "data@razrangersan/tmp/cdp-demo/sample.csv/" + QNAME_METADATA_NAMESPACE;
+        String pathQName = abfsScheme + "data@razrangersan/tmp/cdp-demo/sample.csv" + QNAME_METADATA_NAMESPACE;
         String entityQName = (String) entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME);
 
         if (pathQName.equalsIgnoreCase(entityQName)){
             assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "sample.csv");
         } else {
-            pathQName = abfsScheme + "data@razrangersan/tmp/cdp-demo/" + QNAME_METADATA_NAMESPACE;
+            pathQName = abfsScheme + "data@razrangersan/tmp/cdp-demo" + QNAME_METADATA_NAMESPACE;
             if (pathQName.equalsIgnoreCase(entityQName)){
                 assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "cdp-demo");
             } else {
-                assertEquals(entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME), abfsScheme + "data@razrangersan/tmp/" + QNAME_METADATA_NAMESPACE);
+                assertEquals(entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME), abfsScheme + "data@razrangersan/tmp" + QNAME_METADATA_NAMESPACE);
                 assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "tmp");
             }
         }
@@ -350,20 +372,35 @@ public class AtlasPathExtractorUtilTest {
 
         if (pathQName.equalsIgnoreCase(entityQName)){
             assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "Irradiance_A.csv");
-            assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/renders/Irradiance_A.csv/");
+            assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/renders/");
         } else {
             pathQName = s3Scheme + "aws_my_bucket1/1234567890/" + QNAME_METADATA_NAMESPACE;
             if (pathQName.equalsIgnoreCase(entityQName)){
                 assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "1234567890");
-                assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/");
+                assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/");
             } else {
                 assertEquals(entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME), s3Scheme + "aws_my_bucket1/1234567890/renders/" + QNAME_METADATA_NAMESPACE);
                 assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "renders");
-                assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/renders/");
+                assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/");
             }
         }
     }
 
+    private void verifyGCSVirtualDir(String s3Scheme, String path, AtlasEntity entity) {
+        String pathQName = path + "/" + QNAME_METADATA_NAMESPACE;
+        String entityQName = (String) entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME);
+
+        if (pathQName.equalsIgnoreCase(entityQName)){
+            assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "data");
+            assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/1234567890/");
+        } else {
+            pathQName = s3Scheme + "gcs_test_bucket1/1234567890/" + QNAME_METADATA_NAMESPACE;
+            assertEquals(entityQName, pathQName);
+            assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "1234567890");
+            assertEquals(entity.getAttribute(ATTRIBUTE_OBJECT_PREFIX), "/");
+        }
+    }
+
     private void verifyS3V2KnownEntities(String scheme, String path, Map<String, AtlasEntity> knownEntities) {
         assertEquals(knownEntities.size(), 4);
         int dirCount = 0;
@@ -411,6 +448,29 @@ public class AtlasPathExtractorUtilTest {
         assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "aws_my_bucket1");
     }
 
+    private void verifyGCSKnownEntities(String scheme, String path, Map<String, AtlasEntity> knownEntities) {
+        assertEquals(knownEntities.size(), 3);
+        int dirCount = 0;
+        for (AtlasEntity knownEntity : knownEntities.values()) {
+            switch (knownEntity.getTypeName()){
+                case GCS_VIRTUAL_DIR:
+                    verifyGCSVirtualDir(scheme, path, knownEntity);
+                    dirCount++;
+                    break;
+
+                case GCS_BUCKET:
+                    verifyGCSBucketEntity(scheme, knownEntity);
+                    break;
+            }
+        }
+        assertEquals(dirCount, 2);
+    }
+
+    private void verifyGCSBucketEntity(String scheme, AtlasEntity entity) {
+        assertEquals(entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME), scheme + "gcs_test_bucket1" + QNAME_METADATA_NAMESPACE);
+        assertEquals(entity.getAttribute(ATTRIBUTE_NAME), "gcs_test_bucket1");
+    }
+
     private class OzoneKeyValidator {
         private final String              scheme;
         private final String              location;
