@@ -99,10 +99,17 @@ require.config({
             'deps': ['d3'],
             'exports': ['d3-tip']
         },
+        'LineageHelper': {
+            'deps': ['d3'],
+        },
         'dagreD3': {
             'deps': ['d3'],
             'exports': ['dagreD3']
         },
+        'sparkline': {
+            'deps': ['jquery'],
+            'exports': ['sparkline']
+        },
         'pnotify': {
             'exports': ['pnotify']
         },
@@ -121,13 +128,6 @@ require.config({
         'moment': {
             'exports': ['moment']
         },
-        'nvd3': {
-            'deps': ['d3']
-        },
-        'sparkline': {
-            'deps': ['jquery'],
-            'exports': ['sparkline']
-        },
         'jstree': {
             'deps': ['jquery']
         },
@@ -154,12 +154,14 @@ require.config({
         'asBreadcrumbs': 'libs/jquery-asBreadcrumbs/js/jquery-asBreadcrumbs.min',
         'd3': 'libs/d3/d3.min',
         'd3-tip': 'libs/d3/index',
+        'LineageHelper': 'external_lib/atlas-lineage/dist/index',
+        'dagreD3': 'libs/dagre-d3/dagre-d3.min',
+        'sparkline': 'libs/sparkline/jquery.sparkline.min',
         'tmpl': 'templates',
         'requirejs.text': 'libs/requirejs-text/text',
         'handlebars': 'external_lib/require-handlebars-plugin/js/handlebars',
         'hbs': 'external_lib/require-handlebars-plugin/js/hbs',
         'i18nprecompile': 'external_lib/require-handlebars-plugin/js/i18nprecompile',
-        'dagreD3': 'libs/dagre-d3/dagre-d3.min',
         'select2': 'libs/select2/select2.full.min',
         'backgrid-select-all': 'libs/backgrid-select-all/backgrid-select-all.min',
         'moment': 'libs/moment/js/moment.min',
@@ -172,12 +174,11 @@ require.config({
         'platform': 'libs/platform/platform',
         'query-builder': 'libs/jQueryQueryBuilder/js/query-builder.standalone.min',
         'daterangepicker': 'libs/bootstrap-daterangepicker/js/daterangepicker',
-        'nvd3': 'libs/nvd3/nv.d3.min',
-        'sparkline': 'libs/sparkline/jquery.sparkline.min',
         'table-dragger': 'libs/table-dragger/table-dragger',
         'jstree': 'libs/jstree/jstree.min',
         'jquery-steps': 'libs/jquery-steps/jquery.steps.min',
-        'dropzone': 'libs/dropzone/js/dropzone-amd-module'
+        'dropzone': 'libs/dropzone/js/dropzone-amd-module',
+        'lossless-json': 'libs/lossless-json/lossless-json'
     },
 
     /**
@@ -204,7 +205,7 @@ require(['App',
     'select2'
 ], function(App, Router, Helper, CommonViewFunction, Globals, UrlLinks, VEntityList, VTagList, Enums) {
     var that = this;
-    this.asyncFetchCounter = 7 + (Enums.addOnEntities.length + 1);
+    this.asyncFetchCounter = 5 + (Enums.addOnEntities.length + 1);
     // entity
     this.entityDefCollection = new VEntityList();
     this.entityDefCollection.url = UrlLinks.entitiesDefApiUrl();
@@ -221,6 +222,7 @@ require(['App',
     this.metricCollection = new VTagList();
     this.metricCollection.url = UrlLinks.metricsApiUrl();
     this.metricCollection.modelAttrName = "data";
+    this.classificationAndMetricEvent = new Backbone.Wreqr.EventAggregator();
     // businessMetadata
     this.businessMetadataDefCollection = new VEntityList();
     this.businessMetadataDefCollection.url = UrlLinks.businessMetadataDefApiUrl();
@@ -232,6 +234,7 @@ require(['App',
         enumDefCollection: this.enumDefCollection,
         classificationDefCollection: this.classificationDefCollection,
         metricCollection: this.metricCollection,
+        classificationAndMetricEvent: this.classificationAndMetricEvent,
         businessMetadataDefCollection: this.businessMetadataDefCollection
     });
 
@@ -267,6 +270,22 @@ require(['App',
                 if (response['atlas.ui.default.version'] !== undefined) {
                     Globals.DEFAULT_UI = response['atlas.ui.default.version'];
                 }
+                if (response['atlas.ui.date.format'] !== undefined) {
+                    Globals.dateTimeFormat = response['atlas.ui.date.format'];
+                    var dateFormatSeperated = Globals.dateTimeFormat.split(' ');
+                    if (dateFormatSeperated[0]) {
+                        Globals.dateFormat = dateFormatSeperated[0]; //date
+                    }
+                }
+                if (response['atlas.ui.date.timezone.format.enabled'] !== undefined) {
+                    Globals.isTimezoneFormatEnabled = response['atlas.ui.date.timezone.format.enabled'];
+                }
+                if (response['atlas.debug.metrics.enabled'] !== undefined) {
+                    Globals.isDebugMetricsEnabled = response["atlas.debug.metrics.enabled"];
+                }
+                if (response['atlas.tasks.enabled'] !== undefined) {
+                    Globals.isTasksEnabled = response['atlas.tasks.enabled'];
+                }
             }
             --that.asyncFetchCounter;
             startApp();
@@ -283,6 +302,7 @@ require(['App',
         }
     });
     this.typeHeaders.fetch({
+        async: true,
         complete: function() {
             that.typeHeaders.fullCollection.comparator = function(model) {
                 return model.get('name').toLowerCase();
@@ -303,23 +323,22 @@ require(['App',
         }
     });
     this.classificationDefCollection.fetch({
+        async: true,
         complete: function() {
             that.classificationDefCollection.fullCollection.comparator = function(model) {
                 return model.get('name').toLowerCase();
             };
             that.classificationDefCollection.fullCollection.sort({ silent: true });
-            --that.asyncFetchCounter;
-            startApp();
+            that.classificationAndMetricEvent.trigger("classification:Update:ClassificationTab");
+            that.classificationAndMetricEvent.trigger("classification:Update:Search");
         }
     });
-
     this.metricCollection.fetch({
-        complete: function() {
-            --that.asyncFetchCounter;
-            startApp();
+        async: true,
+        success: function() {
+            that.classificationAndMetricEvent.trigger("metricCollection:Update");
         }
     });
-
     this.businessMetadataDefCollection.fetch({
         complete: function() {
             that.businessMetadataDefCollection.fullCollection.comparator = function(model) {
@@ -330,7 +349,6 @@ require(['App',
             startApp();
         }
     });
-
     CommonViewFunction.fetchRootEntityAttributes({
         url: UrlLinks.rootEntityDefUrl(Enums.addOnEntities[0]),
         entity: Enums.addOnEntities,
@@ -339,7 +357,6 @@ require(['App',
             startApp();
         }
     });
-
     CommonViewFunction.fetchRootClassificationAttributes({
         url: UrlLinks.rootClassificationDefUrl(Enums.addOnClassification[0]),
         classification: Enums.addOnClassification,
