@@ -89,6 +89,12 @@ define(['require',
             case '>=':
                 return (v1 >= v2) ? options.fn(this) : options.inverse(this);
                 break;
+            case 'isEmpty':
+                return (_.isEmpty(v1)) ? options.fn(this) : options.inverse(this);
+                break;
+            case 'has':
+                return (_.has(v1, v2)) ? options.fn(this) : options.inverse(this);
+                break;
             default:
                 return options.inverse(this);
                 break;
@@ -96,6 +102,37 @@ define(['require',
         //return options.inverse(this);
     });
 
+    Handlebars.registerHelper('arithmetic', function(val1, operator, val2, commaFormat, options) {
+        var v1 = (val1 && parseInt(val1.toString().replace(/\,/g, ''))) || 0,
+            v2 = (val2 && parseInt(val2.toString().replace(/\,/g, ''))) || 0,
+            val = null;
+        switch (operator) {
+            case '+':
+                val = v1 + v2;
+                break;
+            case '-':
+                val = v1 - v2;
+                break;
+            case '/':
+                val = v1 / v2;
+                break;
+            case '*':
+                val = v1 * v2;
+                break;
+            case '%':
+                val = v1 % v2;
+                break;
+            default:
+                val = 0;
+                break;
+        }
+        if (commaFormat === false) {
+            return val;
+        }
+        return _.numberFormatWithComma(val);;
+
+    });
+
     Handlebars.registerHelper('lookup', function(obj, field, defaulValue) {
         return (obj[field] ? obj[field] : (defaulValue ? defaulValue : ""));
     });
