@@ -46,11 +46,13 @@ define([
             //Administrator page
             '!/administrator': 'administrator',
             '!/administrator/businessMetadata/:id': 'businessMetadataDetailPage',
+            //Debug Metrics
+            '!/debugMetrics': 'debugMetrics',
             // Default
             '*actions': 'defaultAction'
         },
         initialize: function(options) {
-            _.extend(this, _.pick(options, 'entityDefCollection', 'typeHeaders', 'enumDefCollection', 'classificationDefCollection', 'metricCollection', 'businessMetadataDefCollection'));
+            _.extend(this, _.pick(options, 'entityDefCollection', 'typeHeaders', 'enumDefCollection', 'classificationDefCollection', 'metricCollection', 'classificationAndMetricEvent', 'businessMetadataDefCollection'));
             this.showRegions();
             this.bindCommonEvents();
             this.listenTo(this, 'route', this.postRouteExecute, this);
@@ -68,6 +70,7 @@ define([
                 'classificationDefCollection': this.classificationDefCollection,
                 'glossaryCollection': this.glossaryCollection,
                 'metricCollection': this.metricCollection,
+                'classificationAndMetricEvent': this.classificationAndMetricEvent,
                 'businessMetadataDefCollection': this.businessMetadataDefCollection
             }
             this.ventObj = {
@@ -106,11 +109,18 @@ define([
         renderViewIfNotExists: function(options) {
             var view = options.view,
                 render = options.render,
+                viewName = options.viewName,
                 manualRender = options.manualRender;
             if (!view.currentView) {
-                if (render) view.show(options.render());
+                if (render) view.show(options.render(options));
+            } else if (manualRender && viewName) {
+                if (viewName === view.currentView._viewName) {
+                    options.manualRender(options);
+                } else {
+                    if (render) view.show(options.render(options));
+                }
             } else {
-                if (manualRender) options.manualRender();
+                if (manualRender) options.manualRender(options);
             }
         },
 
@@ -149,13 +159,7 @@ define([
         detailPage: function(id) {
             var that = this;
             if (id) {
-                require([
-                    'views/site/Header',
-                    'views/detail_page/DetailPageLayoutView',
-                    'views/site/SideNavLayoutView',
-                    'collection/VEntityList'
-                ], function(Header, DetailPageLayoutView, SideNavLayoutView, VEntityList) {
-                    this.entityCollection = new VEntityList([], {});
+                require(["views/site/Header", "views/detail_page/DetailPageLayoutView", "views/site/SideNavLayoutView"], function(Header, DetailPageLayoutView, SideNavLayoutView) {
                     var paramObj = Utils.getUrlState.getQueryParams(),
                         options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj);
                     that.renderViewIfNotExists(that.getHeaderOptions(Header));
@@ -168,9 +172,18 @@ define([
                             return new SideNavLayoutView(options);
                         }
                     });
-                    App.rNContent.show(new DetailPageLayoutView(_.extend({ 'collection': this.entityCollection, 'id': id, 'value': paramObj }, options)));
-                    this.entityCollection.url = UrlLinks.entitiesApiUrl({ guid: id, minExtInfo: true });
-                    this.entityCollection.fetch({ reset: true });
+
+                    var dOptions = _.extend({ id: id, value: paramObj }, options);
+                    that.renderViewIfNotExists({
+                        view: App.rNContent,
+                        viewName: "DetailPageLayoutView",
+                        manualRender: function() {
+                            this.view.currentView.manualRender(dOptions);
+                        },
+                        render: function() {
+                            return new DetailPageLayoutView(dOptions);
+                        }
+                    });
                 });
             }
         },
@@ -249,58 +262,76 @@ define([
             require([
                 'views/site/Header',
                 'views/site/SideNavLayoutView',
-                'views/search/SearchDetailLayoutView'
-            ], function(Header, SideNavLayoutView, SearchDetailLayoutView) {
+                'views/search/SearchDetailLayoutView',
+                'collection/VTagList'
+            ], function(Header, SideNavLayoutView, SearchDetailLayoutView, VTagList) {
                 var paramObj = Utils.getUrlState.getQueryParams(),
-                    options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj);
+                    options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj),
+                    tag = new VTagList();
                 if (paramObj.tag) {
                     var tagValidate = paramObj.tag,
                         isTagPresent = false;
                     if ((tagValidate.indexOf('*') == -1)) {
-                        classificationDefCollection.fullCollection.each(function(model) {
+                        isTagPresent = classificationDefCollection.fullCollection.some(function(model) {
                             var name = Utils.getName(model.toJSON(), 'name');
                             if (model.get('category') == 'CLASSIFICATION') {
-                                if (tagValidate) {
-                                    if (name === tagValidate) {
-                                        isTagPresent = true;
-                                    }
-                                }
+                                return name === tagValidate;
                             }
-                        });
-                        _.each(Enums.addOnClassification, function(classificationName) {
-                            if (classificationName === tagValidate) {
-                                isTagPresent = true;
-                            }
-                        });
+                            return false;
+                        })
                         if (!isTagPresent) {
-                            paramObj.tag = null;
+                            isTagPresent = Enums.addOnClassification.some(function(classificationName) {
+                                return classificationName === tagValidate;
+                            })
+                        }
+                        if (!isTagPresent) {
+                            tag.url = UrlLinks.classicationApiUrl(tagValidate);
+                            tag.fetch({
+                                success: function(tagCollection) {
+                                    isTagPresent = true;
+                                },
+                                cust_error: function(model, response) {
+                                    paramObj.tag = null;
+                                },
+                                complete: function() {
+                                    renderSearchView.call();
+                                }
+                            });
                         }
                     }
-                }
-                var isinitialView = true,
-                    isTypeTagNotExists = false,
-                    tempParam = $.extend(true, {}, paramObj);
-                that.renderViewIfNotExists(that.getHeaderOptions(Header));
-                that.renderViewIfNotExists({
-                    view: App.rSideNav,
-                    manualRender: function() {
-                        this.view.currentView.RSearchLayoutView.currentView.manualRender(tempParam);
-                    },
-                    render: function() {
-                        return new SideNavLayoutView(_.extend({ 'value': tempParam }, options));
+                    if (tagValidate.indexOf('*') >= 0 || isTagPresent) {
+                        renderSearchView();
                     }
-                });
-                App.rSideNav.currentView.selectTab();
-                if (paramObj) {
-                    isinitialView = (paramObj.type || (paramObj.dslChecked == "true" ? "" : (paramObj.tag || paramObj.term)) || (paramObj.query ? paramObj.query.trim() : "")).length === 0;
+                } else {
+                    renderSearchView();
+                }
+
+                function renderSearchView() {
+                    var isinitialView = true,
+                        isTypeTagNotExists = false,
+                        tempParam = $.extend(true, {}, paramObj);
+                    that.renderViewIfNotExists(that.getHeaderOptions(Header));
+                    that.renderViewIfNotExists({
+                        view: App.rSideNav,
+                        manualRender: function() {
+                            this.view.currentView.RSearchLayoutView.currentView.manualRender(tempParam);
+                        },
+                        render: function() {
+                            return new SideNavLayoutView(_.extend({ 'value': tempParam }, options));
+                        }
+                    });
+                    App.rSideNav.currentView.selectTab();
+                    if (paramObj) {
+                        isinitialView = (paramObj.type || (paramObj.dslChecked == "true" ? "" : (paramObj.tag || paramObj.term)) || (paramObj.query ? paramObj.query.trim() : "")).length === 0;
+                    }
+                    App.rNContent.show(new SearchDetailLayoutView(
+                        _.extend({
+                            'value': paramObj,
+                            'initialView': isinitialView,
+                            'isTypeTagNotExists': ((paramObj.type != tempParam.type) || (tempParam.tag != paramObj.tag))
+                        }, options)
+                    ));
                 }
-                App.rNContent.show(new SearchDetailLayoutView(
-                    _.extend({
-                        'value': paramObj,
-                        'initialView': isinitialView,
-                        'isTypeTagNotExists': ((paramObj.type != tempParam.type) || (tempParam.tag != paramObj.tag))
-                    }, options)
-                ));
             });
         },
         administrator: function() {
@@ -326,6 +357,29 @@ define([
                 App.rNContent.show(new AdministratorLayoutView(_.extend({ value: paramObj, guid: null }, options)));
             });
         },
+        debugMetrics: function() {
+            var that = this;
+            require(["views/site/Header", "views/site/SideNavLayoutView", 'views/dev_debug/DebugMetricsLayoutView'], function(Header, SideNavLayoutView, DebugMetricsLayoutView) {
+                var paramObj = Utils.getUrlState.getQueryParams(),
+                    options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj);
+                that.renderViewIfNotExists(that.getHeaderOptions(Header));
+                that.renderViewIfNotExists({
+                    view: App.rSideNav,
+                    manualRender: function() {
+                        this.view.currentView.selectTab();
+                        if (Utils.getUrlState.isTagTab()) {
+                            this.view.currentView.RTagLayoutView.currentView.manualRender();
+                        } else if (Utils.getUrlState.isGlossaryTab()) {
+                            this.view.currentView.RGlossaryLayoutView.currentView.manualRender(_.extend({ "isTrigger": true, "value": paramObj }));
+                        }
+                    },
+                    render: function() {
+                        return new SideNavLayoutView(options);
+                    }
+                });
+                App.rNContent.show(new DebugMetricsLayoutView(options));
+            });
+        },
         businessMetadataDetailPage: function(guid) {
             var that = this;
             require(["views/site/Header", "views/site/SideNavLayoutView", "views/business_metadata/BusinessMetadataContainerLayoutView", ], function(Header, SideNavLayoutView, BusinessMetadataContainerLayoutView) {
@@ -374,7 +428,7 @@ define([
                     }
                 });
 
-                if (Globals.entityCreate && Utils.getUrlState.isSearchTab()) {
+                if (Utils.getUrlState.isSearchTab()) {
                     App.rNContent.show(new SearchDetailLayoutView(_.extend({ 'value': paramObj, 'initialView': true }, options)));
                 } else {
                     if (App.rNContent.currentView) {
