@@ -16,7 +16,7 @@
  * limitations under the License.
  */
 
-define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enums', 'moment', 'utils/Globals'], function(require, Utils, Modal, Messages, Enums, moment, Globals) {
+define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enums', 'moment', 'utils/Globals', 'moment-timezone'], function(require, Utils, Modal, Messages, Enums, moment, Globals) {
     'use strict';
 
     var CommonViewFunction = {};
@@ -79,13 +79,15 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             sortBy = options.sortBy,
             valueObject = options.valueObject,
             extractJSON = options.extractJSON,
+            getArrayOfStringElement = options.getArrayOfStringElement,
+            getArrayOfStringFormat = options.getArrayOfStringFormat,
             isTable = _.isUndefined(options.isTable) ? true : options.isTable,
             attributeDefs = options.attributeDefs,
             formatIntVal = options.formatIntVal,
             showListCount = options.showListCount || true,
             highlightString = options.highlightString,
             formatStringVal = options.formatStringVal,
-            numberFormat = options.numberFormat || _.numberFormatWithComa;
+            numberFormat = options.numberFormat || _.numberFormatWithComma;
 
         var table = "",
             getHighlightedString = function(resultStr) {
@@ -101,8 +103,17 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     return resultStr;
                 }
             },
-            getValue = function(val) {
-                if (val) {
+            getEmptyString = function(key) {
+                if (options.getEmptyString) {
+                    return options.getEmptyString(key);
+                }
+                return "N/A";
+            },
+            getValue = function(val, key) {
+                if (options && options.getValue) {
+                    val = options.getValue(val, key);
+                }
+                if (!_.isUndefined(val) && !_.isNull(val)) {
                     if ((_.isNumber(val) || !_.isNaN(parseInt(val))) && formatIntVal) {
                         return numberFormat(val);
                     } else {
@@ -118,7 +129,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         return getHighlightedString(_.escape(newVal));
                     }
                 } else {
-                    return "N/A";
+                    return getEmptyString(key);
                 }
             },
             fetchInputOutputValue = function(id, defEntity) {
@@ -179,7 +190,12 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     if (_.isString(inputOutputField) || _.isBoolean(inputOutputField) || _.isNumber(inputOutputField)) {
                         var tempVarfor$check = inputOutputField.toString();
                         if (tempVarfor$check.indexOf("$") == -1) {
-                            valueOfArray.push('<span class="json-string">' + getValue(inputOutputField) + '</span>');
+                            var tmpVal = getValue(inputOutputField, key)
+                            if (getArrayOfStringElement) {
+                                valueOfArray.push(getArrayOfStringElement(tmpVal, key));
+                            } else {
+                                valueOfArray.push('<span class="json-string">' + tmpVal + '</span>');
+                            }
                         }
                     } else if (_.isObject(inputOutputField) && !id) {
                         var attributesList = inputOutputField;
@@ -245,9 +261,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     }
                 }
                 if (valueOfArray.length) {
-                    subLink = valueOfArray.join(', ');
+                    if (getArrayOfStringFormat) {
+                        subLink = getArrayOfStringFormat(valueOfArray, key);
+                    } else {
+                        subLink = valueOfArray.join(', ');
+                    }
                 }
-                return subLink === "" ? "N/A" : subLink;
+                return subLink === "" ? getEmptyString(key) : subLink;
             }
         var valueObjectKeysList = _.keys(valueObject);
         if (_.isUndefined(sortBy) || sortBy == true) {
@@ -263,7 +283,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             if (defEntity && defEntity.typeName) {
                 var defEntityType = defEntity.typeName.toLocaleLowerCase();
                 if (defEntityType === 'date') {
-                    keyValue = keyValue > 0 ? new Date(keyValue) : "";
+                    keyValue = keyValue > 0 ? Utils.formatDate({ date: keyValue }) : null;
                 } else if (_.isObject(keyValue)) {
                     keyValue = extractObject({ "keyValue": keyValue, "key": key, 'defEntity': defEntity });
                 }
@@ -276,13 +296,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             if (_.isObject(valueObject[key])) {
                 val = keyValue
             } else if (key === 'guid' || key === "__guid") {
-                if (options.fromAdminAudit) {
-                    val = getValue(keyValue);
+                if (options.guidHyperLink === false) {
+                    val = getValue(keyValue, key);
                 } else {
-                    val = '<a title="' + key + '" href="#!/detailPage/' + _.escape(keyValue) + '">' + getValue(keyValue) + '</a>';
+                    val = '<a title="' + key + '" href="#!/detailPage/' + _.escape(keyValue) + '">' + getValue(keyValue, key) + '</a>';
                 }
             } else {
-                val = getValue(keyValue);
+                val = getValue(keyValue, key);
             }
             if (isTable) {
                 var value = val,
@@ -291,12 +311,17 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 if (_.isObject(valueObject[key]) && !_.isEmpty(valueObject[key])) {
                     var matchedLinkString = val.match(/href|value-loader\w*/g),
                         matchedJson = val.match(/json-value|json-string\w*/g),
+                        matchedKey = val.match(/json-key\w*/g),
                         isMatchLinkStringIsSingle = matchedLinkString && matchedLinkString.length <= 5,
                         isMatchJSONStringIsSingle = matchedJson && matchedJson.length == 1,
                         expandCollapseButton = "";
-                    if ((matchedJson && !isMatchJSONStringIsSingle) || (matchedLinkString && !isMatchLinkStringIsSingle)) {
-                        expandCollapseButton = '<button class="expand-collapse-button"><i class="fa"></i></button>';
-                        htmlTag = '<pre class="shrink code-block ' + (isMatchJSONStringIsSingle ? 'fixed-height' : '') + '">' + expandCollapseButton + '<code>' + val + '</code></pre>';
+                    if ((matchedJson) || (matchedLinkString)) {
+                        var className = "code-block fixed-height";
+                        if (!isMatchJSONStringIsSingle) {
+                            className += " shrink";
+                            expandCollapseButton = '<button class="expand-collapse-button"><i class="fa"></i></button>';
+                        }
+                        htmlTag = '<pre class="' + className + '">' + expandCollapseButton + '<code>' + val + '</code></pre>';
                     }
                 }
                 table += '<tr class="' + appendClass + '"><td>' + (_.escape(key) + listCount) + '</td><td>' + htmlTag + '</td></tr>';
@@ -387,9 +412,17 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
 
         function objToString(filterObj) {
             var generatedQuery = _.map(filterObj.rules, function(obj, key) {
+                var obj = $.extend(true, {}, obj); // not to update the timezone abbr on original obj , copy of obj is used 
                 if (_.has(obj, 'condition')) {
                     return '&nbsp<span class="operator">' + obj.condition + '</span>&nbsp' + '(' + objToString(obj) + ')';
                 } else {
+                    if (obj.type === "date") {
+                        if (Enums.queryBuilderDateRangeUIValueToAPI[obj.value]) {
+                            obj.value = Enums.queryBuilderDateRangeUIValueToAPI[obj.value];
+                        } else {
+                            obj.value = obj.value + " (" + moment.tz(moment.tz.guess()).zoneAbbr() + ")";
+                        }
+                    }
                     return '<span class="key">' + (Enums.systemAttributes[obj.id] ? Enums.systemAttributes[obj.id] : _.escape(obj.id)) + '</span>&nbsp<span class="operator">' + _.escape(obj.operator) + '</span>&nbsp<span class="value">' + (Enums[obj.id] ? Enums[obj.id][obj.value] : _.escape(obj.value)) + "</span>";
                 }
             });
@@ -487,6 +520,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                     if (Globals[value[skey].typeName]) {
                                         attributeDefs = Globals[value[skey].typeName].attributeDefs;
                                     }
+                                    if (Globals._ALL_CLASSIFICATION_TYPES && Globals._ALL_CLASSIFICATION_TYPES.attributeDefs) {
+                                        attributeDefs = attributeDefs.concat(Globals._ALL_CLASSIFICATION_TYPES.attributeDefs);
+                                    }
                                 }
                                 val = CommonViewFunction.attributeFilter.generateUrl({ "value": val, "attributeDefs": attributeDefs });
                             } else if (k == "entityFilters") {
@@ -503,6 +539,9 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                     if (Globals[value[skey].typeName]) {
                                         attributeDefs = Globals[value[skey].typeName].attributeDefs;
                                     }
+                                    if (Globals._ALL_ENTITY_TYPES && Globals._ALL_ENTITY_TYPES.attributeDefs) {
+                                        attributeDefs = attributeDefs.concat(Globals._ALL_ENTITY_TYPES.attributeDefs);
+                                    }
                                 }
                                 val = CommonViewFunction.attributeFilter.generateUrl({ "value": val, "attributeDefs": attributeDefs });
                             } else if (_.contains(["includeDE", "excludeST", "excludeSC"], k)) {
@@ -544,7 +583,18 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         var type = (obj.type || obj.attributeType),
                             //obj.value will come as an object when selected type is Date and operator is isNull or not_null;
                             value = ((_.isString(obj.value) && _.contains(["is_null", "not_null"], obj.operator) && type === 'date') || _.isObject(obj.value) ? "" : _.trim(obj.value || obj.attributeValue)),
-                            url = [(obj.id || obj.attributeName), mapApiOperatorToUI(obj.operator), (type === 'date' && formatedDateToLong && value.length ? Date.parse(value) : value)];
+                            url = [(obj.id || obj.attributeName), mapApiOperatorToUI(obj.operator), value];
+                        if (obj.operator === "TIME_RANGE") {
+                            if (value.indexOf("-") > -1) {
+                                url[2] = value.split('-').map(function(udKey) {
+                                    return Date.parse(udKey.trim()).toString()
+                                }).join(",")
+                            } else {
+                                url[2] = Enums.queryBuilderDateRangeUIValueToAPI[_.trim(value)] || value;
+                            }
+                        } else if (value.length && type === 'date' && formatedDateToLong) {
+                            url[2] = Date.parse(value);
+                        }
                         if (type) {
                             url.push(type);
                         }
@@ -561,30 +611,8 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             }
 
             function mapApiOperatorToUI(oper) {
-                if (oper == "eq") {
-                    return "=";
-                } else if (oper == "neq") {
-                    return "!=";
-                } else if (oper == "lt") {
-                    return "<";
-                } else if (oper == "lte") {
-                    return "<=";
-                } else if (oper == "gt") {
-                    return ">";
-                } else if (oper == "gte") {
-                    return ">=";
-                } else if (oper == "startsWith") {
-                    return "begins_with";
-                } else if (oper == "endsWith") {
-                    return "ends_with";
-                } else if (oper == "contains") {
-                    return "contains";
-                } else if (oper == "notNull") {
-                    return "not_null";
-                } else if (oper == "isNull") {
-                    return "is_null";
-                }
-                return oper;
+                // Enum will be in effect once we click on save search.
+                return Enums.queryBuilderApiOperatorToUI[oper] || oper;
             }
         },
         extractUrl: function(options) {
@@ -594,30 +622,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 spliter = 1,
                 apiObj = options.apiObj,
                 mapUiOperatorToAPI = function(oper) {
-                    if (oper == "=") {
-                        return "eq";
-                    } else if (oper == "!=") {
-                        return "neq";
-                    } else if (oper == "<") {
-                        return "lt";
-                    } else if (oper == "<=") {
-                        return "lte";
-                    } else if (oper == ">") {
-                        return "gt";
-                    } else if (oper == ">=") {
-                        return "gte";
-                    } else if (oper == "begins_with") {
-                        return "startsWith";
-                    } else if (oper == "ends_with") {
-                        return "endsWith";
-                    } else if (oper == "contains") {
-                        return "contains";
-                    } else if (oper == "not_null") {
-                        return "notNull";
-                    } else if (oper == "is_null") {
-                        return "isNull";
-                    }
-                    return oper;
+                    return Enums.queryBuilderUIOperatorToAPI[oper] || oper;
                 },
                 createObject = function(urlObj) {
                     var finalObj = {};
@@ -636,13 +641,23 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                                 rule = {};
                             if (apiObj) {
                                 rule = { attributeName: temp[0], operator: mapUiOperatorToAPI(temp[1]), attributeValue: _.trim(temp[2]) }
-                                rule.attributeValue = rule.type === 'date' && formatDate && rule.attributeValue.length ? moment(parseInt(rule.attributeValue)).format(Globals.dateTimeFormat) : rule.attributeValue;
+                                rule.attributeValue = rule.type === 'date' && formatDate && rule.attributeValue.length ? Utils.formatDate({ date: parseInt(rule.attributeValue), zone: false }) : rule.attributeValue;
                             } else {
                                 rule = { id: temp[0], operator: temp[1], value: _.trim(temp[2]) }
                                 if (temp[3]) {
                                     rule['type'] = temp[3];
                                 }
-                                rule.value = rule.type === 'date' && formatDate && rule.value.length ? moment(parseInt(rule.value)).format(Globals.dateTimeFormat) : rule.value;
+                                if (rule.operator === "TIME_RANGE") {
+                                    if (temp[2].indexOf(",") > -1) {
+                                        rule.value = temp[2].split(",").map(function(udKey) {
+                                            return Utils.formatDate({ date: parseInt(udKey.trim()), zone: false })
+                                        }).join(" - ")
+                                    } else {
+                                        rule.value = Enums.queryBuilderDateRangeAPIValueToUI[_.trim(rule.value)] || rule.value;
+                                    }
+                                } else if (rule.type === 'date' && formatDate && rule.value.length) {
+                                    rule.value = Utils.formatDate({ date: parseInt(rule.value), zone: false })
+                                }
                             }
                             return rule;
                         }
@@ -695,21 +710,10 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 "okText": model ? "Update" : "Create",
                 "allowCancel": true
             }).open();
-            modal.$el.find('button.ok').attr("disabled", "true");
-            if (model) {
-                view.$('input,textarea').on('keyup', function(e) {
-                    modal.$el.find('button.ok').attr("disabled", false);
-                });
-            } else {
-                view.ui.name.on('keyup', function(e) {
-                    modal.$el.find('button.ok').attr("disabled", false);
-                });
-            }
-            view.ui.name.on('keyup', function(e) {
-                if ((e.keyCode == 8 || e.keyCode == 32 || e.keyCode == 46) && e.currentTarget.value.trim() == "") {
-                    modal.$el.find('button.ok').attr("disabled", true);
-                }
+            modal.$el.find('input[data-id=shortDescription]').on('input keydown', function(e) {
+                $(this).val($(this).val().replace(/\s+/g, ' '));
             });
+            modal.$el.find('button.ok').attr("disabled", "true");
             modal.on('ok', function() {
                 modal.$el.find('button.ok').showButtonLoader();
                 CommonViewFunction.createEditGlossaryCategoryTermSubmit(_.extend({ "ref": view, "modal": modal }, options));
@@ -792,7 +796,6 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 }));
             }
         }
-
     }
     CommonViewFunction.removeCategoryTermAssociation = function(options) {
         if (options) {
@@ -863,13 +866,10 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
         }
     }
     CommonViewFunction.addRestCsrfCustomHeader = function(xhr, settings) {
-        if (settings.url == null) {
-            return;
-        }
-        var method = settings.type;
-        if (CommonViewFunction.restCsrfCustomHeader != null && !CommonViewFunction.restCsrfMethodsToIgnore[method]) {
-            // The value of the header is unimportant.  Only its presence matters.
-            xhr.setRequestHeader(CommonViewFunction.restCsrfCustomHeader, '""');
+        if (null != settings.url) {
+            var method = settings.type;
+            var csrfToken = CommonViewFunction.restCsrfValue;
+            null == CommonViewFunction.restCsrfCustomHeader || CommonViewFunction.restCsrfMethodsToIgnore[method] || xhr.setRequestHeader(CommonViewFunction.restCsrfCustomHeader, csrfToken);
         }
     }
     CommonViewFunction.restCsrfCustomHeader = null;
@@ -899,16 +899,14 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                             var str = "" + response['atlas.rest-csrf.enabled'];
                             csrfEnabled = (str.toLowerCase() == 'true');
                         }
-                        if (response['atlas.rest-csrf.custom-header']) {
-                            header = response['atlas.rest-csrf.custom-header'].trim();
-                        }
-                        if (response['atlas.rest-csrf.methods-to-ignore']) {
-                            methods = getTrimmedStringArrayValue(response['atlas.rest-csrf.methods-to-ignore']);
-                        }
-                        if (csrfEnabled) {
-                            CommonViewFunction.restCsrfCustomHeader = header;
-                            CommonViewFunction.restCsrfMethodsToIgnore = {};
-                            methods.map(function(method) { CommonViewFunction.restCsrfMethodsToIgnore[method] = true; });
+                        if (response["atlas.rest-csrf.custom-header"] && (header = response["atlas.rest-csrf.custom-header"].trim()),
+                            response["atlas.rest-csrf.methods-to-ignore"] && (methods = getTrimmedStringArrayValue(response["atlas.rest-csrf.methods-to-ignore"])),
+                            csrfEnabled) {
+                            CommonViewFunction.restCsrfCustomHeader = header, CommonViewFunction.restCsrfMethodsToIgnore = {},
+                                CommonViewFunction.restCsrfValue = response["_csrfToken"] || '""',
+                                methods.map(function(method) {
+                                    CommonViewFunction.restCsrfMethodsToIgnore[method] = !0;
+                                });
                             var statusCodeErrorFn = function(error) {
                                 Utils.defaultErrorHandler(null, error)
                             }
