@@ -16,8 +16,8 @@
  * limitations under the License.
  */
 
-define(['require'], function(require) {
-    'use strict';
+define(["require"], function(require) {
+    "use strict";
 
     var Globals = {};
     Globals.settings = {};
@@ -27,8 +27,10 @@ define(['require'], function(require) {
         tabState: {
             stateChanged: false,
             tagUrl: "#!/tag",
-            taxonomyUrl: "#!/taxonomy",
-            searchUrl: "#!/search"
+            searchUrl: "#!/search",
+            glossaryUrl: "#!/glossary",
+            administratorUrl: "#!/administrator",
+            debugMetricsUrl: "#!/debugMetrics"
         },
         detailPageState: {}
     };
@@ -36,6 +38,17 @@ define(['require'], function(require) {
         status: false,
         response: {}
     };
+    Globals.serviceTypeMap = {};
     Globals.entityImgPath = "/img/entity-icon/";
+    Globals.DEFAULT_UI = "v2";
+
+    // Date Format
+    Globals.dateTimeFormat = "MM/DD/YYYY hh:mm:ss A";
+    Globals.dateFormat = "MM/DD/YYYY";
+    Globals.isTimezoneFormatEnabled = true;
+
+    Globals.isDebugMetricsEnabled = false;
+    Globals.isTasksEnabled = false;
+
     return Globals;
 });
\ No newline at end of file
