@@ -165,7 +165,7 @@ define(['require',
                     }
                 };
                 events["click " + this.ui.addTerm] = 'onClickAddTermBtn';
-                events["click " + this.ui.addCategory] = 'onClickAddCategoryBtn';
+                events["click " + this.ui.addCategory] = 'onClickAddTermBtn';
                 events["click " + this.ui.addTag] = 'onClickAddTagBtn';
                 return events;
             },
@@ -232,7 +232,36 @@ define(['require',
                                 if (that.isDestroyed) {
                                     return;
                                 }
+                                that.data = data;
                                 if (that.isTermView) {
+                                    var tags = {
+                                        'self': [],
+                                        'propagated': [],
+                                        'propagatedMap': {},
+                                        'combineMap': {}
+                                    };
+                                    if (that.data) {
+                                        var tagObject = that.data.classifications;
+                                        _.each(tagObject, function(val) {
+                                            var typeName = val.typeName;
+                                            if (val.entityGuid === that.guid) {
+                                                tags['self'].push(val)
+                                            } else {
+                                                tags['propagated'].push(val);
+                                                if (tags.propagatedMap[typeName]) {
+                                                    tags.propagatedMap[typeName]["count"] += tags.propagatedMap[typeName]["count"];
+                                                } else {
+                                                    tags.propagatedMap[typeName] = val;
+                                                    tags.propagatedMap[typeName]["count"] = 1;
+                                                }
+                                            }
+                                            if (tags.combineMap[typeName] === undefined) {
+                                                tags.combineMap[typeName] = val;
+                                            }
+                                        });
+                                        tags.self = _.sortBy(tags.self, "typeName");
+                                        tags.propagated = _.sortBy(tags.propagated, "typeName");
+                                    }
                                     var obj = {
                                         "guid": that.guid,
                                         "entityDefCollection": that.entityDefCollection,
@@ -242,6 +271,7 @@ define(['require',
                                         "classificationDefCollection": that.classificationDefCollection,
                                         "glossaryCollection": that.glossaryCollection,
                                         "searchVent": that.searchVent,
+                                        "tags": tags,
                                         "getSelectedTermAttribute": function() {
                                             return that.selectedTermAttribute;
                                         },
@@ -253,7 +283,6 @@ define(['require',
                                     that.renderTagTableLayoutView(obj);
                                     that.renderRelationLayoutView(obj);
                                 }
-                                that.data = data;
                                 that.glossaryCollection.trigger("data:updated", $.extend(true, {}, data));
                                 that.glossary.selectedItem.model = data;
                                 that.glossary.selectedItem.guid = data.guid;
@@ -268,8 +297,8 @@ define(['require',
                 Utils.hideTitleLoader(this.$('.fontLoader'), this.ui.details);
                 if (data) {
                     this.ui.title.text(data.name || data.displayText || data.qualifiedName);
-                    this.ui.shortDescription.text(data.shortDescription);
-                    this.ui.longDescription.text(data.longDescription);
+                    this.ui.shortDescription.text(data.shortDescription ? data.shortDescription : "");
+                    this.ui.longDescription.text(data.longDescription ? data.longDescription : "");
                     this.generateCategories(data.categories);
                     this.generateTerm(data.terms);
                     this.generateTag(data.classifications);
@@ -307,60 +336,64 @@ define(['require',
                 this.ui.tagList.find("span.btn").remove();
                 this.ui.tagList.prepend(tagData);
             },
-            onClickAddTermBtn: function(e) {
-                var that = this;
-                require(['views/glossary/AssignTermLayoutView'], function(AssignTermLayoutView) {
-                    var glossary = that.glossaryCollection;
-                    if (that.value && that.value.gId) {
-                        var foundModel = that.glossaryCollection.find({ guid: that.value.gId });
-                        if (foundModel) {
-                            glossary = new VGlossaryList([foundModel.toJSON()], {
-                                comparator: function(item) {
-                                    return item.get("name");
-                                }
-                            });
-                        }
+            getCategoryTermCount: function(collection, matchString) {
+                var terms = 0;
+                _.each(collection, function(model) {
+                    if (model.get(matchString)) {
+                        terms += model.get(matchString).length;
                     }
-
-                    var view = new AssignTermLayoutView({
-                        categoryData: that.data,
-                        associatedTerms: that.data && that.data.terms && that.data.terms.length > 0 ? that.data.terms : [],
-                        isCategoryView: that.isCategoryView,
-                        callback: function() {
-                            that.getData();
-                        },
-                        glossaryCollection: glossary
-                    });
-                    view.modal.on('ok', function() {
-                        that.hideLoader();
-                    });
                 });
+                return terms;
             },
-            onClickAddCategoryBtn: function(e) {
-                var that = this;
-                require(['views/glossary/AssignTermLayoutView'], function(AssignTermLayoutView) {
-                    var glossary = that.glossaryCollection;
-                    if (that.value && that.value.gId) {
-                        var foundModel = that.glossaryCollection.find({ guid: that.value.gId });
-                        if (foundModel) {
-                            glossary = new VGlossaryList([foundModel.toJSON()], {
-                                comparator: function(item) {
-                                    return item.get("name");
-                                }
-                            });
-                        }
+            onClickAddTermBtn: function(e) {
+                var that = this,
+                    glossary = this.glossaryCollection;
+                if (this.value && this.value.gId) {
+                    var foundModel = this.glossaryCollection.find({ guid: this.value.gId });
+                    if (foundModel) {
+                        glossary = new VGlossaryList([foundModel.toJSON()], {
+                            comparator: function(item) {
+                                return item.get("name");
+                            }
+                        });
                     }
-                    var view = new AssignTermLayoutView({
-                        termData: that.data,
-                        isTermView: that.isTermView,
+                }
+                var obj = {
                         callback: function() {
                             that.getData();
                         },
-                        glossaryCollection: glossary
+                        glossaryCollection: glossary,
+                    },
+                    emptyListMessage = this.isCategoryView ? "There are no available terms that can be associated with this category" : "There are no available categories that can be associated with this term";
+
+                if (this.isCategoryView) {
+                    obj = _.extend(obj, {
+                        categoryData: this.data,
+                        associatedTerms: (this.data && this.data.terms && this.data.terms.length > 0) ? this.data.terms : [],
+                        isCategoryView: this.isCategoryView,
                     });
+                } else {
+                    obj = _.extend(obj, {
+                        termData: this.data,
+                        isTermView: this.isTermView,
+                    });
+                }
+                if (this.getCategoryTermCount(glossary.fullCollection.models, this.isCategoryView ? "terms" : "categories")) {
+                    this.AssignTermLayoutViewModal(obj);
+                } else {
+                    Utils.notifyInfo({
+                        content: emptyListMessage
+                    });
+                }
+            },
+            AssignTermLayoutViewModal: function(termCategoryObj) {
+                var that = this;
+                require(['views/glossary/AssignTermLayoutView'], function(AssignTermLayoutView) {
+                    var view = new AssignTermLayoutView(termCategoryObj);
                     view.modal.on('ok', function() {
                         that.hideLoader();
                     });
+
                 });
             },
             onClickAddTagBtn: function(e) {
