@@ -90,9 +90,9 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'value', 'typeHeaders', 'searchVent', 'entityDefCollection', 'enumDefCollection', 'classificationDefCollection', 'businessMetadataDefCollection', 'searchTableColumns', 'searchTableFilters', 'metricCollection'));
+                _.extend(this, _.pick(options, 'value', 'typeHeaders', 'searchVent', 'entityDefCollection', 'enumDefCollection', 'classificationDefCollection', 'businessMetadataDefCollection', 'searchTableColumns', 'searchTableFilters', 'metricCollection', 'classificationAndMetricEvent'));
                 this.type = "basic";
-                this.entityCountObj = _.first(this.metricCollection.toJSON());
+                this.entityCountObj = _.first(this.metricCollection.toJSON()) || { entity: { entityActive: {}, entityDeleted: {} }, tag: { tagEntities: {} } };
                 this.selectedFilter = {
                     'basic': [],
                     'dsl': []
@@ -207,12 +207,19 @@ define(['require',
                 });
             },
             bindEvents: function(param) {
+                var that = this;
                 this.listenTo(this.typeHeaders, "reset", function(value) {
                     this.initializeValues();
                 }, this);
                 this.listenTo(this.searchVent, "entityList:refresh", function(model, response) {
                     this.onRefreshButton();
                 }, this);
+                this.classificationAndMetricEvent.on("classification:Update:Search", function(options) {
+                    that.entityCountObj = _.first(that.metricCollection.toJSON());
+                    that.value = Utils.getUrlState.getQueryParams() || {};
+                    if (!that.value.type) that.setInitialEntityVal = true;
+                    that.initializeValues();
+                });
             },
             initializeValues: function() {
                 this.renderTypeTagList();
@@ -545,7 +552,7 @@ define(['require',
                     var name = Utils.getName(model.toJSON(), 'name');
                     if (model.get('category') == 'ENTITY' && (serviceTypeToBefiltered && serviceTypeToBefiltered.length ? _.contains(serviceTypeToBefiltered, model.get('serviceType')) : true)) {
                         var entityCount = (that.entityCountObj.entity.entityActive[name] || 0) + (that.entityCountObj.entity.entityDeleted[name] || 0);
-                        typeStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (entityCount ? "(" + _.numberFormatWithComa(entityCount) + ")" : '') + '</option>';
+                        typeStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (entityCount ? "(" + _.numberFormatWithComma(entityCount) + ")" : '') + '</option>';
                     }
                     if (isTypeOnly == undefined && model.get('category') == 'CLASSIFICATION') {
                         var tagEntityCount = that.entityCountObj.tag.tagEntities[name];
@@ -554,7 +561,7 @@ define(['require',
                                 foundNewClassification = true;
                             }
                         }
-                        tagStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (tagEntityCount ? "(" + _.numberFormatWithComa(tagEntityCount) + ")" : '') + '</option>';
+                        tagStr += '<option value="' + (name) + '" data-name="' + (name) + '">' + (name) + ' ' + (tagEntityCount ? "(" + _.numberFormatWithComma(tagEntityCount) + ")" : '') + '</option>';
                     }
                 });
 
@@ -618,6 +625,8 @@ define(['require',
                 }
             },
             renderTermList: function() {
+                this.glossaryTermArray = null; //This Value is created to store the result of search Term while basic search through term.
+                var that = this;
                 var getTypeAheadData = function(data, params) {
                     var dataList = data.entities,
                         foundOptions = [];
@@ -630,7 +639,7 @@ define(['require',
                         }
                     });
                     return foundOptions;
-                }
+                };
                 this.ui.termLov.select2({
                     placeholder: "Search Term",
                     allowClear: true,
@@ -647,6 +656,7 @@ define(['require',
                             };
                         },
                         processResults: function(data, params) {
+                            that.glossaryTermArray = getTypeAheadData(data, params); //storing the search Results obj
                             return {
                                 results: getTypeAheadData(data, params)
                             };
@@ -727,6 +737,18 @@ define(['require',
             findSearchResult: function() {
                 this.triggerSearch(this.ui.searchInput.val());
             },
+            //This below function returns the searched Term Guid.
+            getSearchedTermGuid: function() {
+                var searchedTerm = this.ui.termLov.select2('val'),
+                    searchedTermGuid = null;
+                if (searchedTerm) {
+                    this.glossaryTermArray.find(function(obj) {
+                        if (searchedTerm === obj.id)
+                            searchedTermGuid = obj.guid;
+                    });
+                }
+                return searchedTermGuid;
+            },
             triggerSearch: function(value) {
                 var params = {
                         searchType: this.type,
@@ -736,31 +758,37 @@ define(['require',
                     },
                     typeLovValue = this.ui.typeLov.find(':selected').data('name'), // to get count of selected element used data
                     tagLovValue = this.ui.tagLov.find(':selected').data('name') || this.ui.tagLov.val(),
-                    termLovValue = this.ui.termLov.select2('val')
+                    termLovValue = this.ui.termLov.select2('val');
                 params['type'] = typeLovValue || null;
                 if (!this.dsl) {
                     params['tag'] = tagLovValue || null;
                     params['term'] = termLovValue || null;
+                    params['guid'] = this.getSearchedTermGuid(); //Adding Guid in the URL for selection while switching. 
                     var entityFilterObj = this.searchTableFilters['entityFilters'],
                         tagFilterObj = this.searchTableFilters['tagFilters'];
-                    if (this.value.tag) {
-                        params['tagFilters'] = tagFilterObj[this.value.tag]
+                    params['includeDE'] = false;
+                    params['excludeST'] = false;
+                    params['excludeSC'] = false;
+                    if (this.value) {
+                        if (this.value.tag) {
+                            params['tagFilters'] = tagFilterObj[this.value.tag]
+                        }
+                        if (this.value.type) {
+                            params['entityFilters'] = entityFilterObj[this.value.type]
+                        }
+                        var columnList = this.value.type && this.searchTableColumns ? this.searchTableColumns[this.value.type] : null;
+                        if (columnList) {
+                            params['attributes'] = columnList.join(',');
+                        }
+                        params['includeDE'] = _.isUndefinedNull(this.value.includeDE) ? false : this.value.includeDE;
+                        params['excludeST'] = _.isUndefinedNull(this.value.excludeST) ? false : this.value.excludeST;
+                        params['excludeSC'] = _.isUndefinedNull(this.value.excludeSC) ? false : this.value.excludeSC;
                     }
-                    if (this.value.type) {
-                        params['entityFilters'] = entityFilterObj[this.value.type]
-                    }
-                    var columnList = this.value && this.value.type && this.searchTableColumns ? this.searchTableColumns[this.value.type] : null;
-                    if (columnList) {
-                        params['attributes'] = columnList.join(',');
-                    }
-                    params['includeDE'] = _.isUndefinedNull(this.value.includeDE) ? false : this.value.includeDE;
-                    params['excludeST'] = _.isUndefinedNull(this.value.excludeST) ? false : this.value.excludeST;
-                    params['excludeSC'] = _.isUndefinedNull(this.value.excludeSC) ? false : this.value.excludeSC;
                 }
-                if (!_.isUndefinedNull(this.value.pageLimit)) {
+                if (this.value && !_.isUndefinedNull(this.value.pageLimit)) {
                     params['pageLimit'] = this.value.pageLimit;
                 }
-                if (!_.isUndefinedNull(this.value.pageOffset)) {
+                if (this.value && !_.isUndefinedNull(this.value.pageOffset)) {
                     if (!_.isUndefinedNull(this.query[this.type]) && this.query[this.type].query != value) {
                         params['pageOffset'] = 0;
                     } else {
