@@ -147,7 +147,7 @@ define(['require',
             if (options && options.model) {
                 var searchParameters = options.model.toJSON().searchParameters,
                     params = CommonViewFunction.generateUrlFromSaveSearchObject({
-                        value: {"searchParameters" : searchParameters, "uiParameters": options.model.get('uiParameters')},
+                        value: { "searchParameters": searchParameters, "uiParameters": options.model.get('uiParameters') },
                         classificationDefCollection: this.classificationDefCollection,
                         entityDefCollection: this.entityDefCollection
                     });
@@ -177,9 +177,12 @@ define(['require',
                     modal: true,
                     html: true,
                     text: Messages.conformation.deleteMessage + "<b>" + _.escape(options.model.get('name')) + "</b>" + " ?",
-                    ok: function(argument) {
+                    ok: function(obj) {
+                        that.notificationModal = obj;
+                        obj.showButtonLoader();
                         that.onDeleteNotifyOk(options);
                     },
+                    okCloses: false,
                     cancel: function(argument) {}
                 }
                 Utils.notifyConfirm(notifyObj);
@@ -194,8 +197,10 @@ define(['require',
                     if (that.collection) {
                         that.collection.remove(model);
                     }
+                    that.notificationModal.hideButtonLoader();
+                    that.notificationModal.remove();
                     Utils.notifySuccess({
-                        content: options.model.get('name') + Messages.deleteSuccessMessage
+                        content: options.model.get('name') + Messages.getAbbreviationMsg(false, 'deleteSuccessMessage')
                     });
                 }
             });
