@@ -28,9 +28,10 @@ define(['require',
     'utils/Enums',
     'utils/Messages',
     'moment',
+    'utils/Globals',
     'moment-timezone',
     'daterangepicker'
-], function(require, AddTagModalViewTmpl, AddTimezoneItemView, VTagList, VCommonList, Modal, VEntity, Utils, UrlLinks, Enums, Messages, moment) {
+], function(require, AddTagModalViewTmpl, AddTimezoneItemView, VTagList, VCommonList, Modal, VEntity, Utils, UrlLinks, Enums, Messages, moment, Globals) {
     'use strict';
 
     var AddTagModel = Backbone.Marionette.CompositeView.extend({
@@ -125,6 +126,7 @@ define(['require',
                     cancelText: "Cancel",
                     mainClass: 'modal-lg',
                     allowCancel: true,
+                    okCloses: false
                 };
             if (this.tagModel) {
                 modalObj.title = 'Edit Classification';
@@ -134,6 +136,15 @@ define(['require',
             this.modal.open();
             this.modal.$el.find('button.ok').attr("disabled", true);
             this.on('ok', function() {
+                if (this.ui.checkTimeZone.is(':checked')) {
+                    if (this.validateValues()) {
+                        if (this.hideLoader) {
+                            this.hideLoader();
+                        };
+                        return;
+                    };
+                }
+                that.modal.$el.find('button.ok').showButtonLoader();
                 var tagName = this.tagModel ? this.tagModel.typeName : this.ui.addTagOptions.val(),
                     tagAttributes = {},
                     tagAttributeNames = this.$(".attrName"),
@@ -244,6 +255,33 @@ define(['require',
             });
             this.bindEvents();
         },
+        validateValues: function(attributeDefs) {
+            var isValidate = true,
+                applyErrorClass = function(scope) {
+                    if (this.value == '' || this.value == null || this.value.indexOf('Select Timezone') > -1) {
+                        $(this).addClass('errorValidate');
+                        if (isValidate) { isValidate = false; }
+                    } else {
+                        $(this).removeClass('errorValidate');
+                    }
+                };
+
+            this.$el.find('.start-time').each(function(element) {
+                applyErrorClass.call(this);
+            });
+            this.$el.find('.end-time').each(function(element) {
+                applyErrorClass.call(this);
+            });
+            this.$el.find('.time-zone').each(function(element) {
+                applyErrorClass.call(this);
+            });
+            if (!isValidate) {
+                Utils.notifyInfo({
+                    content: "Please fill the details"
+                });
+                return true;
+            }
+        },
 
         onRender: function() {
             var that = this;
@@ -352,7 +390,7 @@ define(['require',
                         var str = '<option value=""' + (!that.tagModel ? 'selected' : '') + '>-- Select ' + typeName + " --</option>";
                         var enumValue = typeNameValue.get('elementDefs');
                         _.each(enumValue, function(key, value) {
-                            str += '<option ' + ((that.tagModel && key.value === that.tagModel.attributes[name]) ? 'selected' : '') + '>' + key.value + '</option>';
+                            str += '<option ' + ((that.tagModel && key.value === that.tagModel.attributes[name]) ? 'selected' : '') + '>' + _.escape(key.value) + '</option>';
                         })
                         that.ui.tagAttribute.append('<div class="form-group"><label>' + name + '</label>' + ' (' + typeName + ')' +
                             '<select class="form-control attributeInputVal attrName" data-key="' + name + '">' + str + '</select></div>');
@@ -369,13 +407,16 @@ define(['require',
                             "singleDatePicker": true,
                             "showDropdowns": true,
                             "timePicker": true,
+                            startDate: new Date(),
                             locale: {
-                                format: 'MM/DD/YYYY h:mm A'
+                                format: Globals.dateTimeFormat
                             }
                         };
                         if (that.tagModel) {
-                            var formatDate = Number(this.value);
-                            dateObj["startDate"] = new Date(formatDate);
+                            if (this.value.length) {
+                                var formatDate = Number(this.value);
+                                dateObj["startDate"] = new Date(formatDate);
+                            }
                         }
                         $(this).daterangepicker(dateObj);
                     }
@@ -390,7 +431,7 @@ define(['require',
             }
         },
         getElement: function(labelName, typeName) {
-            var value = this.tagModel && this.tagModel.attributes ? (this.tagModel.attributes[labelName] || "") : "",
+            var value = this.tagModel && this.tagModel.attributes ? (this.tagModel.attributes[_.unescape(labelName)] || "") : "",
                 isTypeNumber = typeName === "int" || typeName === "byte" || typeName === "short" || typeName === "double" || typeName === "float",
                 inputClassName = "form-control attributeInputVal attrName";
             if (isTypeNumber) {
@@ -402,7 +443,7 @@ define(['require',
                     '<option value="true">true</option>' +
                     '<option value="false">false</option></select>';
             } else {
-                return '<input type="text" value="' + value + '" class="' + inputClassName + '" data-key="' + labelName + '" data-type="' + typeName + '"/>';
+                return '<input type="text" value="' + _.escape(value) + '" class="' + inputClassName + '" data-key="' + labelName + '" data-type="' + typeName + '"/>';
             }
 
         },
@@ -439,9 +480,9 @@ define(['require',
                 this.showLoader();
             }
             this.entityModel.saveTraitsEntity(this.tagModel ? options.guid : null, {
-                skipDefaultError: true,
                 data: JSON.stringify(json),
                 type: this.tagModel ? 'PUT' : 'POST',
+                defaultErrorMessage: "Tag " + tagName + " could not be added",
                 success: function(data) {
                     var addupdatetext = that.tagModel ? 'updated successfully to ' : 'added to ';
                     Utils.notifySuccess({
@@ -453,15 +494,10 @@ define(['require',
                     if (that.callback) {
                         that.callback();
                     }
+                    that.modal.close();
                 },
                 cust_error: function(model, response) {
-                    var message = "Tag " + tagName + " could not be added";
-                    if (response && response.responseJSON) {
-                        message = response.responseJSON.errorMessage;
-                    }
-                    Utils.notifyError({
-                        content: message
-                    });
+                    that.modal.$el.find('button.ok').hideButtonLoader();
                     if (that.hideLoader) {
                         that.hideLoader();
                     }
