@@ -147,7 +147,7 @@ define(['require',
                 var validate = true;
 
                 this.modal.$el.find(".attributeInput").each(function() {
-                    if ($(this).val() === "") {
+                    if ($(this).val().trim() === "") {
                         $(this).css('borderColor', "red")
                         validate = false;
                     }
@@ -276,14 +276,14 @@ define(['require',
                     });
             },
             textAreaChangeEvent: function(view) {
-                if (this.model.get('description') === view.ui.description.val()) {
+                if (this.model.get('description') === view.ui.description.val() || view.ui.description.val().length == 0 || view.ui.description.val().trim().length === 0) {
                     this.modal.$el.find('button.ok').prop('disabled', true);
                 } else {
                     this.modal.$el.find('button.ok').prop('disabled', false);
                 }
             },
             onPublishClick: function(view) {
-                var saveObj = _.extend(this.model.toJSON(), { 'description': view.ui.description.val() });
+                var saveObj = _.extend(this.model.toJSON(), { 'description': view.ui.description.val().trim() });
                 this.onSaveButton(saveObj, Messages.tag.updateTagDescriptionMessage);
                 this.ui.description.show();
             },
@@ -302,8 +302,10 @@ define(['require',
                         okText: 'Save',
                         allowCancel: true,
                     }).open();
-                    view.ui.description.on('keyup', function(e) {
+                    view.ui.description.on('keyup input', function(e) {
+                        $(this).val($(this).val().replace(/\s+/g, ' '));
                         that.textAreaChangeEvent(view);
+                        e.stopPropagation();
                     });
                     that.modal.$el.find('button.ok').prop('disabled', true);
                     that.modal.on('ok', function() {
