@@ -23,8 +23,8 @@ define(['require',
     'utils/Messages',
     'utils/Globals',
     'utils/UrlLinks',
-    'models/VTag'
-], function(require, Backbone, TagLayoutViewTmpl, Utils, Messages, Globals, UrlLinks, VTag) {
+    'collection/VTagList'
+], function(require, Backbone, TagLayoutViewTmpl, Utils, Messages, Globals, UrlLinks, VTagList) {
     'use strict';
 
     var TagLayoutView = Backbone.Marionette.LayoutView.extend(
@@ -49,6 +49,7 @@ define(['require',
                 tagView: 'input[name="tagView"]',
                 expandArrow: '[data-id="expandArrow"]',
                 tagTreeLoader: '[data-id="tagTreeLoader"]',
+                tagTreeTextLoader: '[data-id="tagTreeTextLoader"]',
                 tagTreeView: '[data-id="tagTreeView"]'
             },
             /** ui events hash */
@@ -68,7 +69,7 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'tag', 'collection', 'typeHeaders', 'value', 'enumDefCollection'));
+                _.extend(this, _.pick(options, 'tag', 'collection', 'typeHeaders', 'value', 'enumDefCollection', 'classificationAndMetricEvent'));
                 this.viewType = "flat";
                 this.query = {
                     flat: {
@@ -87,6 +88,7 @@ define(['require',
                 var that = this;
                 this.listenTo(this.collection.fullCollection, "reset add remove", function() {
                     this.tagsGenerator();
+                    this.changeLoaderTextState(false);
                 }, this);
                 this.ui.tagsList.on('click', 'li.parent-node a', function() {
                     that.setUrl(this.getAttribute("href"));
@@ -95,12 +97,36 @@ define(['require',
                     that.$('.tagPopover').popover('hide');
                     that[$(this).find('a').data('fn')](e)
                 });
+                this.classificationAndMetricEvent.on("classification:Update:ClassificationTab", function(options) {
+                    that.changeLoaderTextState(false);
+                });
             },
             onRender: function() {
                 var that = this;
+                this.changeLoaderTextState(false);
                 this.changeLoaderState(true);
                 this.bindEvents();
-                this.tagsGenerator();
+                this.checkTagOnRefresh();
+            },
+            checkTagOnRefresh: function() {
+                var that = this,
+                    tagName = this.options.tag,
+                    presentTag = this.collection.fullCollection.findWhere({ name: tagName }),
+                    tag = new VTagList();
+                if (!presentTag && tagName) {
+                    tag.url = UrlLinks.classicationApiUrl(tagName);
+                    tag.fetch({
+                        success: function(dataOrCollection, tagDetails) {
+                            that.collection.fullCollection.add(tagDetails);
+                            that.changeLoaderTextState(true);
+                        },
+                        cust_error: function(model, response) {
+                            that.tagsGenerator();
+                        }
+                    });
+                } else {
+                    this.tagsGenerator();
+                }
             },
             changeLoaderState: function(showLoader) {
                 if (showLoader) {
@@ -111,6 +137,13 @@ define(['require',
                     this.ui.tagTreeView.show();
                 }
             },
+            changeLoaderTextState: function(showLoader) {
+                if (showLoader) {
+                    this.ui.tagTreeTextLoader.show();
+                } else {
+                    this.ui.tagTreeTextLoader.hide();
+                }
+            },
             fetchCollections: function() {
                 this.changeLoaderState(true);
                 this.ui.refreshTag.attr("disabled", true);
@@ -123,6 +156,7 @@ define(['require',
                 this.query[this.viewType].tagName = this.tag;
                 if (options && options.viewType) {
                     this.viewType = options.viewType;
+                    this.changeLoaderTextState(false);
                 }
                 if (!this.createTag) {
                     this.setValues(true);
@@ -206,16 +240,10 @@ define(['require',
                             if (target.children('div').find('a').text() === tag) {
                                 target.addClass('active');
                                 target.parents('ul').addClass('show').removeClass('hide'); // Don't use toggle
-                                if (this.createTag || !manual) {
-                                    if (target.offset()) {
-                                        $('#sidebar-wrapper').animate({
-                                            scrollTop: target.offset().top - 100
-                                        }, 500);
-                                    }
-                                }
                                 return false;
                             }
                         });
+
                     }
                 }
             },
@@ -293,7 +321,7 @@ define(['require',
                         return '<li class="parent-node" data-id="tags">' +
                             '<div><div class="tools"><i class="fa fa-ellipsis-h tagPopover"></i></div>' +
                             (hasChild ? '<i class="fa toggleArrow fa-angle-right" data-id="expandArrow" data-name="' + name + '"></i>' : '') +
-                            '<a href="#!/tag/tagAttribute/' + name + '?viewType=' + (isTree ? 'tree' : 'flat') + '&searchType=basic"  data-name="' + name + '">' + name + '</a></div>' +
+                            '<a href="#!/tag/tagAttribute/' + name + '?viewType=' + (isTree ? 'tree' : 'flat') + '&searchType=basic&tag=' + name + '" data-name="' + name + '">' + name + '</a></div>' +
                             (isTree && hasChild ? '<ul class="child hide">' + that.generateTree({ 'data': options.children, 'isTree': isTree }) + '</ul>' : '') + '</li>';
                     };
                 if (isTree) {
@@ -374,13 +402,11 @@ define(['require',
                             allowCancel: true,
                         }).open();
                     modal.$el.find('button.ok').attr("disabled", "true");
-                    view.ui.tagName.on('keyup', function(e) {
-                        modal.$el.find('button.ok').removeAttr("disabled");
+                    view.ui.tagName.on('keyup input', function(e) {
+                        view.ui.description.val($(this).val().replace(/\s+/g, ' '));
                     });
-                    view.ui.tagName.on('keyup', function(e) {
-                        if ((e.keyCode == 8 || e.keyCode == 32 || e.keyCode == 46) && e.currentTarget.value.trim() == "") {
-                            modal.$el.find('button.ok').attr("disabled", "true");
-                        }
+                    view.ui.description.on('input keydown', function(e) {
+                        $(this).val($(this).val().replace(/\s+/g, ' '));
                     });
                     modal.on('shownModal', function() {
                         view.ui.parentTag.select2({
@@ -390,7 +416,6 @@ define(['require',
                         });
                     });
                     modal.on('ok', function() {
-                        // modal.$el.find('button.ok').attr("disabled", "true");
                         modal.$el.find('button.ok').showButtonLoader();
                         that.onCreateButton(view, modal);
                     });
@@ -511,7 +536,7 @@ define(['require',
                             });
                         }
                         that.collection.fullCollection.add(classificationDefs);
-                        that.setUrl('#!/tag/tagAttribute/' + ref.ui.tagName.val(), true);
+                        that.setUrl('#!/tag/tagAttribute/' + ref.ui.tagName.val() + '?tag=' + ref.ui.tagName.val(), true);
                         Utils.notifySuccess({
                             content: "Classification " + that.name + Messages.getAbbreviationMsg(false, 'addSuccessMessage')
                         });
@@ -597,13 +622,23 @@ define(['require',
             },
             onNotifyOk: function(data) {
                 var that = this,
-                    deleteTagData = this.collection.fullCollection.findWhere({ name: this.tag });
+                    deleteTagData = this.collection.fullCollection.findWhere({ name: this.tag }),
+                    superTypeOfDeleteTag = deleteTagData.get('superTypes'),
+                    superTypeObj = superTypeOfDeleteTag ? this.collection.fullCollection.findWhere({ name: superTypeOfDeleteTag[0] }) : null;
+
                 deleteTagData.deleteTag({
                     typeName: that.tag,
                     success: function() {
                         Utils.notifySuccess({
                             content: "Classification " + that.tag + Messages.getAbbreviationMsg(false, 'deleteSuccessMessage')
                         });
+                        //delete current classification from subTypes list of parent classification if any
+                        if (superTypeObj) {
+                            var parentSubTypeUpdate = _.reject(superTypeObj.get('subTypes'), function(subtype) {
+                                return subtype === that.tag;
+                            });
+                            superTypeObj.set('subTypes', parentSubTypeUpdate);
+                        }
                         // if deleted tag is prviously searched then remove that tag url from save state of tab.
                         var searchUrl = Globals.saveApplicationState.tabState.searchUrl;
                         var urlObj = Utils.getUrlState.getQueryParams(searchUrl);
