@@ -47,13 +47,15 @@ define([
             //Audit table
             '!/administrator': 'administrator',
             '!/administrator/businessMetadata/:id': 'businessMetadataDetailPage',
+            //Debug Metrics
+            '!/debugMetrics': 'debugMetrics',
             // Default
             "*actions": "defaultAction"
         },
         initialize: function(options) {
             _.extend(
                 this,
-                _.pick(options, "entityDefCollection", "typeHeaders", "enumDefCollection", "classificationDefCollection", "metricCollection", "businessMetadataDefCollection")
+                _.pick(options, "entityDefCollection", "typeHeaders", "enumDefCollection", "classificationDefCollection", "metricCollection", "classificationAndMetricEvent", "businessMetadataDefCollection")
             );
             this.showRegions();
             this.bindCommonEvents();
@@ -72,6 +74,7 @@ define([
                 classificationDefCollection: this.classificationDefCollection,
                 glossaryCollection: this.glossaryCollection,
                 metricCollection: this.metricCollection,
+                classificationAndMetricEvent: this.classificationAndMetricEvent,
                 businessMetadataDefCollection: this.businessMetadataDefCollection
             };
             this.ventObj = {
@@ -177,8 +180,7 @@ define([
         detailPage: function(id) {
             var that = this;
             if (id) {
-                require(["views/site/Header", "views/detail_page/DetailPageLayoutView", "collection/VEntityList", "views/site/SideNavLayoutView"], function(Header, DetailPageLayoutView, VEntityList, SideNavLayoutView) {
-                    this.entityCollection = new VEntityList([], {});
+                require(["views/site/Header", "views/detail_page/DetailPageLayoutView", "views/site/SideNavLayoutView"], function(Header, DetailPageLayoutView, SideNavLayoutView) {
                     var paramObj = Utils.getUrlState.getQueryParams(),
                         options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj);
                     that.renderViewIfNotExists(that.getHeaderOptions(Header));
@@ -191,9 +193,18 @@ define([
                             return new SideNavLayoutView(options);
                         }
                     });
-                    App.rContent.show(new DetailPageLayoutView(_.extend({ collection: this.entityCollection, id: id, value: paramObj }, options)));
-                    this.entityCollection.url = UrlLinks.entitiesApiUrl({ guid: id, minExtInfo: true });
-                    this.entityCollection.fetch({ reset: true });
+
+                    var dOptions = _.extend({ id: id, value: paramObj }, options);
+                    that.renderViewIfNotExists({
+                        view: App.rContent,
+                        viewName: "DetailPageLayoutView",
+                        manualRender: function() {
+                            this.view.currentView.manualRender(dOptions);
+                        },
+                        render: function() {
+                            return new DetailPageLayoutView(dOptions);
+                        }
+                    });
                 });
             }
         },
@@ -224,8 +235,13 @@ define([
         },
         renderGlossaryLayoutView: function(id) {
             var that = this;
-            require(["views/site/Header", "views/glossary/GlossaryContainerLayoutView", "views/site/SideNavLayoutView"], function(Header, GlossaryContainerLayoutView, SideNavLayoutView) {
+            require(["views/site/Header", "views/glossary/GlossaryContainerLayoutView", "views/search/SearchDefaultLayoutView", "views/site/SideNavLayoutView"], function(Header, GlossaryContainerLayoutView, SearchDefaultLayoutView, SideNavLayoutView) {
                 var paramObj = Utils.getUrlState.getQueryParams();
+                //Below if condition is added  to handle "when Glossary tab does not have any glossary and selected in Old UI and switched to New UI is show continous loading
+                if (paramObj === undefined) {
+                    that.defaultAction();
+                    return;
+                }
                 that.renderViewIfNotExists(that.getHeaderOptions(Header));
                 var options = _.extend({
                         guid: id,
@@ -245,22 +261,36 @@ define([
                         return new SideNavLayoutView(options);
                     }
                 });
-                that.renderViewIfNotExists({
-                    view: App.rContent,
-                    viewName: "GlossaryContainerLayoutView",
-                    manualRender: function() {
-                        this.view.currentView.manualRender(options);
-                    },
-                    render: function() {
-                        return new GlossaryContainerLayoutView(options)
-                    }
-                });
+                //Below condition is added for showing Detail Page for only Term and Category,
+                //because we are showing default search Page on Glossary Selection.
+                if (paramObj.gType !== "glossary") {
+                    that.renderViewIfNotExists({
+                        view: App.rContent,
+                        viewName: "GlossaryContainerLayoutView",
+                        manualRender: function() {
+                            this.view.currentView.manualRender(options);
+                        },
+                        render: function() {
+                            return new GlossaryContainerLayoutView(options)
+                        }
+                    });
+                } else {
+                    options["value"] = "";
+                    options["initialView"] = true;
+                    that.renderViewIfNotExists(
+                        that.getHeaderOptions(Header, {
+                            fromDefaultSearch: true
+                        })
+                    );
+                    App.rContent.show(new SearchDefaultLayoutView(options));
+                }
             });
         },
         renderDefaultSearchLayoutView: function(opt) {
             var that = this;
-            require(["views/site/Header", "views/search/SearchDefaultLayoutView", "views/site/SideNavLayoutView"], function(Header, SearchDefaultLayoutView, SideNavLayoutView) {
+            require(["views/site/Header", "views/search/SearchDefaultLayoutView", "views/site/SideNavLayoutView", "collection/VTagList"], function(Header, SearchDefaultLayoutView, SideNavLayoutView, VTagList) {
                 var paramObj = Utils.getUrlState.getQueryParams();
+                tag = new VTagList();
                 if (paramObj && (paramObj.type || paramObj.tag || paramObj.term || paramObj.query || paramObj.udKeys || paramObj.udLabels) === undefined) {
                     Utils.setUrl({
                         url: "#!/search",
@@ -289,6 +319,7 @@ define([
                         if (paramObj.tagFilters) {
                             paramObj.tagFilters = null;
                         }
+                        renderSearchView();
                     } else {
                         var tagValidate = paramObj.tag,
                             isTagPresent = false;
@@ -309,59 +340,78 @@ define([
                                 }
                             });
                             if (!isTagPresent) {
-                                paramObj.tag = null;
+                                tag.url = UrlLinks.classificationDefApiUrl(tagValidate);
+                                tag.fetch({
+                                    success: function(tagCollection) {
+                                        isTagPresent = true;
+                                    },
+                                    cust_error: function(model, response) {
+                                        paramObj.tag = null;
+                                    },
+                                    complete: function() {
+                                        renderSearchView.call();
+                                    }
+                                });
+                            } else {
+                                renderSearchView();
                             }
+                        } else {
+                            renderSearchView();
                         }
                     }
+                } else {
+                    renderSearchView();
+                }
 
-                }
-                var isinitialView = true,
-                    isTypeTagNotExists = false,
-                    tempParam = _.extend({}, paramObj);
-                if (paramObj) {
-                    isinitialView =
-                        (
-                            paramObj.type ||
-                            (paramObj.dslChecked == "true" ? "" : paramObj.tag || paramObj.term) ||
-                            (paramObj.query ? paramObj.query.trim() : "")
-                        ).length === 0;
-                }
-                var options = _.extend({
-                        value: paramObj,
-                        initialView: isinitialView,
-                        fromDefaultSearch: opt ? (opt && !opt.fromSearchResultView) : true,
-                        fromSearchResultView: (opt && opt.fromSearchResultView) || false,
-                        fromCustomFilterView: (opt && opt.fromCustomFilterView) || false,
-                        isTypeTagNotExists: paramObj && (paramObj.type != tempParam.type || tempParam.tag != paramObj.tag)
-                    },
-                    that.preFetchedCollectionLists,
-                    that.sharedObj,
-                    that.ventObj
-                );
-                that.renderViewIfNotExists(
-                    that.getHeaderOptions(Header, {
-                        fromDefaultSearch: options.fromDefaultSearch
-                    })
-                );
-                that.renderViewIfNotExists({
-                    view: App.rSideNav,
-                    manualRender: function() {
-                        this.view.currentView.manualRender(options);
-                    },
-                    render: function() {
-                        return new SideNavLayoutView(options);
+                function renderSearchView() {
+                    var isinitialView = true,
+                        isTypeTagNotExists = false,
+                        tempParam = _.extend({}, paramObj);
+                    if (paramObj) {
+                        isinitialView =
+                            (
+                                paramObj.type ||
+                                (paramObj.dslChecked == "true" ? "" : paramObj.tag || paramObj.term) ||
+                                (paramObj.query ? paramObj.query.trim() : "")
+                            ).length === 0;
                     }
-                });
-                that.renderViewIfNotExists({
-                    view: App.rContent,
-                    viewName: "SearchDefaultlLayoutView",
-                    manualRender: function() {
-                        this.view.currentView.manualRender(options);
-                    },
-                    render: function() {
-                        return new SearchDefaultLayoutView(options);
-                    }
-                });
+                    var options = _.extend({
+                            value: paramObj,
+                            initialView: isinitialView,
+                            fromDefaultSearch: opt ? (opt && !opt.fromSearchResultView) : true,
+                            fromSearchResultView: (opt && opt.fromSearchResultView) || false,
+                            fromCustomFilterView: (opt && opt.fromCustomFilterView) || false,
+                            isTypeTagNotExists: paramObj && (paramObj.type != tempParam.type || tempParam.tag != paramObj.tag)
+                        },
+                        that.preFetchedCollectionLists,
+                        that.sharedObj,
+                        that.ventObj
+                    );
+                    that.renderViewIfNotExists(
+                        that.getHeaderOptions(Header, {
+                            fromDefaultSearch: options.fromDefaultSearch
+                        })
+                    );
+                    that.renderViewIfNotExists({
+                        view: App.rSideNav,
+                        manualRender: function() {
+                            this.view.currentView.manualRender(options);
+                        },
+                        render: function() {
+                            return new SideNavLayoutView(options);
+                        }
+                    });
+                    that.renderViewIfNotExists({
+                        view: App.rContent,
+                        viewName: "SearchDefaultlLayoutView",
+                        manualRender: function() {
+                            this.view.currentView.manualRender(options);
+                        },
+                        render: function() {
+                            return new SearchDefaultLayoutView(options);
+                        }
+                    });
+                }
             });
         },
         administrator: function() {
@@ -382,6 +432,24 @@ define([
                 App.rContent.show(new AdministratorLayoutView(_.extend({ value: paramObj, guid: null }, options)));
             });
         },
+        debugMetrics: function() {
+            var that = this;
+            require(["views/site/Header", "views/site/SideNavLayoutView", 'views/dev_debug/DebugMetricsLayoutView'], function(Header, SideNavLayoutView, DebugMetricsLayoutView) {
+                var paramObj = Utils.getUrlState.getQueryParams(),
+                    options = _.extend({}, that.preFetchedCollectionLists, that.sharedObj, that.ventObj);
+                that.renderViewIfNotExists(that.getHeaderOptions(Header));
+                that.renderViewIfNotExists({
+                    view: App.rSideNav,
+                    manualRender: function() {
+                        this.view.currentView.manualRender(options);
+                    },
+                    render: function() {
+                        return new SideNavLayoutView(options);
+                    }
+                });
+                App.rContent.show(new DebugMetricsLayoutView(options));
+            });
+        },
         businessMetadataDetailPage: function(guid) {
             var that = this;
             require(["views/site/Header", "views/site/SideNavLayoutView", "views/business_metadata/BusinessMetadataContainerLayoutView", ], function(Header, SideNavLayoutView, BusinessMetadataContainerLayoutView) {
