@@ -26,8 +26,9 @@ define([
     "collection/VSearchList",
     "collection/VGlossaryList",
     "utils/Enums",
+    "collection/VTagList",
     "jstree"
-], function(require, ClassificationTreeLayoutViewTmpl, Utils, Messages, Globals, UrlLinks, CommonViewFunction, VSearchList, VGlossaryList, Enums) {
+], function(require, ClassificationTreeLayoutViewTmpl, Utils, Messages, Globals, UrlLinks, CommonViewFunction, VSearchList, VGlossaryList, Enums, VTagList) {
     "use strict";
 
     var ClassificationTreeLayoutView = Marionette.LayoutView.extend({
@@ -162,11 +163,30 @@ define([
         },
         onRender: function() {
             this.changeLoaderState(true);
-            this.renderClassificationTree();
+            this.checkTagOnRefresh();
             this.createClassificationAction();
             this.ui.clearWildCard.addClass('hide-icon');
             this.changeLoaderState(false);
         },
+        checkTagOnRefresh: function() {
+            var that = this,
+                tagName = (this.options && this.options.value) ? this.options.value.tag : null,
+                presentTag = this.classificationDefCollection.fullCollection.findWhere({ name: tagName }),
+                tag = new VTagList();
+            if (!presentTag && tagName) {
+                tag.url = UrlLinks.classificationDefApiUrl(tagName);
+                tag.fetch({
+                    success: function(dataOrCollection, tagDetails) {
+                        that.classificationDefCollection.fullCollection.add(tagDetails);
+                    },
+                    cust_error: function(model, response) {
+                        that.renderClassificationTree();
+                    }
+                });
+            } else {
+                this.renderClassificationTree();
+            }
+        },
         changeLoaderState: function(showLoader) {
             if (showLoader) {
                 this.ui.classificationSearchTree.hide();
@@ -181,13 +201,19 @@ define([
             var that = this;
             this.listenTo(
                 this.classificationDefCollection.fullCollection,
-                "reset add remove",
-                function() {
-                    if (this.ui.classificationSearchTree.jstree(true)) {
-                        that.ui.classificationSearchTree.jstree(true).refresh();
-                    } else {
-                        this.renderClassificationTree();
-                    }
+                "reset add",
+                function(model) {
+                    that.classificationTreeUpdate = true;
+                    that.classificationTreeRefresh();
+                },
+                this
+            );
+            this.listenTo(
+                this.classificationDefCollection.fullCollection,
+                "remove",
+                function(model) {
+                    that.classificationTreeUpdate = false;
+                    that.classificationTreeRefresh();
                 },
                 this
             );
@@ -215,6 +241,13 @@ define([
 
             });
         },
+        classificationTreeRefresh: function() {
+            if (this.ui.classificationSearchTree.jstree(true)) {
+                this.ui.classificationSearchTree.jstree(true).refresh();
+            } else {
+                this.renderClassificationTree();
+            }
+        },
         findSearchResult: function(tagValue) {
             if (tagValue) {
                 var params = {
@@ -442,8 +475,8 @@ define([
                             var child = collection.find({
                                 name: name
                             });
-                            var tagEntityCount = that.entityCountObj.tag.tagEntities[name];
-                            var tagname = tagEntityCount ? name + " (" + _.numberFormatWithComa(tagEntityCount) + ")" : name;
+                            var tagEntityCount = that.entityCountObj ? that.entityCountObj.tag.tagEntities[name] : null;
+                            var tagname = tagEntityCount ? name + " (" + _.numberFormatWithComma(tagEntityCount) + ")" : name;
 
                             if (that.options.value) {
                                 isSelectedChild = that.options.value.tag ? that.options.value.tag == name : false;
@@ -483,10 +516,11 @@ define([
                     return tagData;
                 }
             collection.each(function(model) {
-                var modelJSON = model.toJSON();
-                var name = modelJSON.name;
-                var tagEntityCount = that.entityCountObj.tag.tagEntities[name];
-                var tagname = tagEntityCount ? name + " (" + _.numberFormatWithComa(tagEntityCount) + ")" : name,
+                var modelJSON = model.toJSON(),
+                    name = modelJSON.name,
+                    tagEntityCount = that.entityCountObj ? that.entityCountObj.tag.tagEntities[name] : null,
+                    tagname = tagEntityCount ? name + " (" + _.numberFormatWithComma(tagEntityCount) + ")" : name,
+                    isSelectedChildted = false,
                     isSelected = false;
 
                 if (that.options.value) {
@@ -540,22 +574,22 @@ define([
         pushRootClassificationToJstree: function(data) {
             var that = this;
             _.each(Enums.addOnClassification, function(addOnClassification) {
-                var rootClassification = Globals[addOnClassification];
-                var isSelected = that.options.value && that.options.value.tag ? that.options.value.tag == rootClassification.name : false;
-                var rootClassificationNode = {
-                    text: _.escape(rootClassification.name),
-                    name: rootClassification.name,
-                    type: rootClassification.category,
-                    gType: "Classification",
-                    guid: rootClassification.guid,
-                    id: rootClassification.guid,
-                    model: rootClassification,
-                    children: [],
-                    icon: "fa fa-tag",
-                    state: {
-                        selected: isSelected
+                var rootClassification = Globals[addOnClassification],
+                    isSelected = (that.options.value && that.options.value.tag) ? that.options.value.tag == rootClassification.name : false,
+                    rootClassificationNode = {
+                        text: _.escape(rootClassification.name),
+                        name: rootClassification.name,
+                        type: rootClassification.category,
+                        gType: "Classification",
+                        guid: rootClassification.guid,
+                        id: rootClassification.guid,
+                        model: rootClassification,
+                        children: [],
+                        icon: "fa fa-tag",
+                        state: {
+                            selected: isSelected
+                        }
                     }
-                }
                 data.push(rootClassificationNode);
             });
             return data;
@@ -652,13 +686,11 @@ define([
                         allowCancel: true
                     }).open();
                 modal.$el.find("button.ok").attr("disabled", "true");
-                view.ui.tagName.on("keyup", function(e) {
-                    modal.$el.find("button.ok").removeAttr("disabled");
+                view.ui.tagName.on('keyup input', function(e) {
+                    view.ui.description.val($(this).val().replace(/\s+/g, ' '));
                 });
-                view.ui.tagName.on("keyup", function(e) {
-                    if ((e.keyCode == 8 || e.keyCode == 32 || e.keyCode == 46) && e.currentTarget.value.trim() == "") {
-                        modal.$el.find("button.ok").attr("disabled", "true");
-                    }
+                view.ui.description.on('input keydown', function(e) {
+                    $(this).val($(this).val().replace(/\s+/g, ' '));
                 });
                 modal.on("shownModal", function() {
                     view.ui.parentTag.select2({
@@ -806,7 +838,7 @@ define([
         onViewEditClassification: function() {
             var selectedNode = this.ui.classificationSearchTree.jstree("get_selected", true);
             if (selectedNode && selectedNode[0]) {
-                var url = "#!/tag/tagAttribute/" + selectedNode[0].original.name;
+                var url = "#!/tag/tagAttribute/" + selectedNode[0].original.name + "?tag=" + selectedNode[0].original.name;
                 this.onClassificationUpdate(url);
             }
         },
@@ -839,16 +871,25 @@ define([
             if (this.tagId) {
                 var deleteTagData = this.classificationDefCollection.fullCollection.findWhere({ guid: this.tagId });
                 if (deleteTagData) {
-                    var tagName = deleteTagData.get("name");
+                    var tagName = deleteTagData.get("name"),
+                        superTypeOfDeleteTag = deleteTagData.get('superTypes'),
+                        superTypeObj = superTypeOfDeleteTag ? this.classificationDefCollection.fullCollection.findWhere({ name: superTypeOfDeleteTag[0] }) : null;
                     deleteTagData.deleteTag({
                         typeName: tagName,
                         success: function() {
                             Utils.notifySuccess({
                                 content: "Classification " + tagName + Messages.getAbbreviationMsg(false, 'deleteSuccessMessage')
                             });
+                            //delete current classification from subTypes list of parent classification if any
+                            if (superTypeObj) {
+                                var parentSubTypeUpdate = _.reject(superTypeObj.get('subTypes'), function(subtype) {
+                                    return subtype === tagName;
+                                });
+                                superTypeObj.set('subTypes', parentSubTypeUpdate);
+                            }
                             // if deleted tag is prviously searched then remove that tag url from save state of tab.
-                            var searchUrl = Globals.saveApplicationState.tabState.searchUrl;
-                            var urlObj = Utils.getUrlState.getQueryParams(searchUrl);
+                            var searchUrl = Globals.saveApplicationState.tabState.searchUrl,
+                                urlObj = Utils.getUrlState.getQueryParams(searchUrl);
                             that.classificationDefCollection.fullCollection.remove(deleteTagData);
                             // to update tag list of search tab fetch typeHeaders.
                             //that.typeHeaders.fetch({ reset: true });
