@@ -80,7 +80,10 @@ define([
             };
             events["click " + this.ui.importGlossary] = function(e) {
                 e.stopPropagation();
-                that.onClickImportGlossary();
+                var $target = $(e.target);
+                if ($target.parents(".disable-list-option").length == 0 && $target.hasClass("disable-list-option") == false) {
+                    that.onClickImportGlossary();
+                }
             };
             events['click ' + this.ui.downloadTemplate] = function(e) {
                 e.stopPropagation();
@@ -93,6 +96,7 @@ define([
                 this.glossaryCollection.fullCollection, "reset add change",
                 function(skip) {
                     if (this.ui.termSearchTree.jstree(true)) {
+                        that.isGlossaryTree = true; //To Keep the selection of Term after any new Glossary is Created.
                         this.ui.termSearchTree.jstree(true).refresh();
                     } else {
                         this.renderGlossaryTree();
@@ -114,9 +118,17 @@ define([
             });
         },
         glossarySwitchBtnUpdate: function() {
-            this.ui.showGlossaryType.attr("data-original-title", (this.isTermView ? "Show Category" : "Show Term"));
+            var tooltipTitle = (this.isTermView ? "Show Category" : "Show Term");
+            this.ui.showGlossaryType.attr({ "data-original-title": tooltipTitle, "title": tooltipTitle });
             this.ui.showGlossaryType.tooltip('hide');
             this.ui.showGlossaryType.find("i").toggleClass("switch-button");
+            if (this.isTermView) {
+                this.ui.importGlossary.removeClass("disable-list-option").find('a').attr("href", "javascript:void(0)");
+                this.ui.downloadTemplate.removeClass("disable-list-option").find('a').attr("href", UrlLinks.glossaryImportTempUrl());
+            } else {
+                this.ui.importGlossary.addClass("disable-list-option").find('a').removeAttr("href");
+                this.ui.downloadTemplate.addClass("disable-list-option").find('a').removeAttr("href");
+            }
             this.ui.termSearchTree.jstree(true).refresh();
         },
         initialize: function(options) {
@@ -145,6 +157,14 @@ define([
             });
             this.getViewType();
             this.bindEvents();
+            //To stop the trigger Search event, if the node is selected in Old UI and swicthed to New UI.
+            this.isGlossaryTree = this.isGlossryTreeview();
+        },
+        isGlossryTreeview: function() {
+            var queryParams = Utils.getUrlState.getQueryParams();
+            if (queryParams && (queryParams.gType === "term" || queryParams.gType === "category")) {
+                return true;
+            }
         },
         onRender: function() {
             this.changeLoaderState(true);
@@ -163,7 +183,7 @@ define([
             this.options.categoryEvent.off("Success:TermRename")
         },
         getViewType: function() {
-            if (this.options.value) {
+            if (Utils.getUrlState.isGlossaryTab()) {
                 this.isTermView = this.options.value.viewType ? this.options.value.viewType == "term" ? true : false : true;
             } else {
                 this.isTermView = true;
@@ -216,6 +236,12 @@ define([
             this.createTermAction();
         },
         onNodeSelect: function(options, showCategory) {
+            var nodeType = options.node.original.type;
+            if (this.isGlossaryTree && (nodeType === "GlossaryTerm" || nodeType === "GlossaryCategory")) {
+                //To stop the trigger Search event,if the node is selected in Old UI and swicthed to New UI.
+                this.isGlossaryTree = false;
+                return;
+            }
             var name, type, selectedNodeId, that = this,
                 glossaryType = options.node.original.gType;
             if (glossaryType == "category") {
@@ -265,6 +291,27 @@ define([
                 if (that.searchVent) {
                     that.searchVent.trigger("Success:Category");
                 }
+            } else if (glossaryType = "glossary") { //This condition is added to setUrl after click on Glossary for highlighting issue from New UI to Old UI.                
+                that.glossaryTermId = null;
+                if (that.glossaryId != options.node.id) {
+                    that.glossaryId = options.node.id;
+                    var params = {
+                        "gId": that.glossaryId,
+                        "gType": options.node.original.gType,
+                        "viewType": (this.isTermView) ? "term" : "category"
+                    };
+                    Utils.setUrl({
+                        url: '#!/glossary/' + that.glossaryId,
+                        urlParams: params,
+                        mergeBrowserUrl: false,
+                        trigger: false,
+                        updateTabState: true
+                    });
+                } else {
+                    that.glossaryId = null;
+                    that.ui.termSearchTree.jstree(true).deselect_all(true);
+                    this.showDefaultPage();
+                }
             } else {
                 that.glossaryTermId = null;
                 if (that.glossaryId != options.node.id) {
@@ -334,8 +381,10 @@ define([
             var that = this,
                 collection = (options && options.collection) || this.glossaryCollection.fullCollection,
                 listOfParents = [],
-                type = "term";
-
+                type = "term",
+                queryParams = Utils.getUrlState.getQueryParams(),
+                glossaryGuid = queryParams ? queryParams.gId : null,
+                gType = queryParams ? queryParams.gType : "term";
             return this.glossaryCollection.fullCollection.map(function(model, i) {
                 var obj = model.toJSON(),
                     parent = {
@@ -348,7 +397,10 @@ define([
                         type: obj.typeName ? obj.typeName : "GLOSSARY",
                         gType: "glossary",
                         children: [],
-                        state: { opened: true }
+                        state: {
+                            opened: true,
+                            selected: (model.id === glossaryGuid && gType === "glossary") ? true : false
+                        }
                     },
                     openGlossaryNodesState = function(treeDate) {
                         if (treeDate.length == 1) {
@@ -379,36 +431,46 @@ define([
                     var isSelected = false,
                         parentGuid = obj.guid,
                         parentCategoryGuid = null,
-                        getParentCategory = function() {
-                            var parentCategory = _.find(parent.model.categories, function(subCategory) {
-                                return subCategory.categoryGuid === parentCategoryGuid;
-                            });
-                            return parentCategory;
-                        };
+                        categoryList = [],
+                        catrgoryRelation = [];
                     _.each(obj.categories, function(category) {
                         if (that.options.value) {
                             isSelected = that.options.value.guid ? that.options.value.guid == category.categoryGuid : false;
                         }
-                        if (category.parentCategoryGuid) {
-                            return;
-                        }
+
                         var typeName = category.typeName || "GlossaryCategory",
                             guid = category.categoryGuid,
                             categoryObj = {
-                                "text": _.escape(category.displayText),
-                                "type": typeName,
-                                "gType": "category",
-                                "guid": guid,
-                                "id": guid,
-                                "parent": obj,
-                                "glossaryId": obj.guid,
-                                "glossaryName": obj.name,
-                                "model": category,
-                                "children": true,
-                                "icon": "fa fa-files-o"
+                                id: guid,
+                                guid: guid,
+                                text: _.escape(category.displayText),
+                                type: typeName,
+                                gType: "category",
+                                glossaryId: obj.guid,
+                                glossaryName: obj.name,
+                                children: [],
+                                model: category,
+                                icon: "fa fa-files-o"
                             };
-                        parent.children.push(categoryObj)
+                        if (category.parentCategoryGuid) {
+                            catrgoryRelation.push({ parent: category.parentCategoryGuid, child: guid })
+                        }
+                        categoryList.push(categoryObj);
                     });
+                    _.each(categoryList, function(category) {
+                        var getRelation = _.find(catrgoryRelation, function(catrgoryObj) {
+                            if (catrgoryObj.child == category.guid) return catrgoryObj;
+                        })
+                        if (getRelation) {
+                            _.map(categoryList, function(catrgoryObj) {
+                                if (catrgoryObj.guid == getRelation.parent) {
+                                    catrgoryObj["children"].push(category);
+                                };
+                            })
+                        } else {
+                            parent.children.push(category)
+                        }
+                    })
                 }
                 if (that.isTermView && obj.terms) {
                     var isSelected = false;
@@ -481,7 +543,26 @@ define([
                 CommonViewFunction.createEditGlossaryCategoryTerm({
                     "isCategoryView": true,
                     "collection": that.glossaryCollection,
-                    "callback": function() {
+                    "callback": function(updateCollection) {
+                        var updatedObj = {
+                                categoryGuid: updateCollection.guid,
+                                displayText: updateCollection.name,
+                                relationGuid: updateCollection.anchor ? updateCollection.anchor.relationGuid : null
+                            },
+                            glossary = that.glossaryCollection.fullCollection.findWhere({ guid: updateCollection.anchor.glossaryGuid });
+                        if (updateCollection.parentCategory) {
+                            updatedObj["parentCategoryGuid"] = updateCollection.parentCategory.categoryGuid;
+                        }
+                        if (glossary) {
+                            var glossaryAttributes = glossary.attributes || null;
+                            if (glossaryAttributes) {
+                                if (glossaryAttributes.categories) {
+                                    glossaryAttributes['categories'].push(updatedObj);
+                                } else {
+                                    glossaryAttributes['categories'] = [updatedObj];
+                                }
+                            }
+                        }
                         that.ui.termSearchTree.jstree(true).refresh();
                     },
                     "node": selectednode[0].original
@@ -581,7 +662,7 @@ define([
                 gId = selectednode[0].original.parent && selectednode[0].original.parent.guid,
                 isGlossaryView = (type == 'GlossaryTerm' || type == 'GlossaryCategory') ? false : true,
                 model = this.glossaryCollection.fullCollection.get(guid),
-                termModel = this.glossaryCollection.fullCollection.get(gId);;
+                termModel = this.glossaryCollection.fullCollection.get(gId);
             if (isGlossaryView) {
                 CommonViewFunction.createEditGlossaryCategoryTerm({
                     "model": model,
@@ -610,6 +691,10 @@ define([
                     viewType: that.isTermView ? 'term' : 'category',
                     searchType: "basic"
                 }
+                if (type === "GlossaryTerm") {
+                    //Below condition is used to keep the selection or Highlight after clicking on the viewEdit option on term.
+                    params['term'] = selectednode[0].original.name + '@' + selectednode[0].original.parent.name;
+                }
                 var serachUrl = '#!/glossary/' + guid;
                 this.triggerSearch(params, serachUrl);
                 if (!this.isTermView && this.options.categoryEvent) {
@@ -662,6 +747,10 @@ define([
                 ).on("load_node.jstree", function(e, data) {}).on("open_node.jstree", function(e, data) {}).on("select_node.jstree", function(e, data) {
                     if (that.fromManualRender !== true) {
                         that.onNodeSelect(data);
+                        if (that.glossaryId === data.node.original.id) {
+                            //This condition is to reset the Glossary ID to null after clicking on the Logo.
+                            that.glossaryId = null;
+                        }
                     } else {
                         that.fromManualRender = false;
                     }
@@ -687,6 +776,10 @@ define([
                             that.options.categoryEvent.trigger("Success:Category", true);
                         }
                     }
+                    //Below condition is for switching the the Show Term/Show Category toggle button on switching from Old to New UI.
+                    if (that.isTermView === false) {
+                        that.glossarySwitchBtnUpdate();
+                    }
 
                 }).on("hover_node.jstree", function(nodes, str, res) {
                     var aTerm = that.$("#" + str.node.a_attr.id),
@@ -710,6 +803,7 @@ define([
         refresh: function(options) {
             this.glossaryTermId = null;
             this.fetchGlossary();
+            this.isGlossaryTree = true;
         },
         onClickImportGlossary: function() {
             var that = this;
