@@ -39,6 +39,7 @@ import org.apache.atlas.repository.graphdb.janus.AtlasJanusGraphDatabase;
 import org.apache.atlas.repository.graphdb.janus.AtlasJanusVertex;
 import org.apache.tinkerpop.gremlin.process.traversal.Compare;
 import org.apache.tinkerpop.gremlin.structure.Vertex;
+import org.janusgraph.graphdb.query.JanusGraphPredicateUtils;
 import org.janusgraph.graphdb.query.graph.GraphCentricQueryBuilder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -55,7 +56,7 @@ public class NativeJanusGraphQuery implements NativeTinkerpopGraphQuery<AtlasJan
     private JanusGraphQuery<?> query;
 
     public NativeJanusGraphQuery(AtlasJanusGraph graph) {
-        this.query = AtlasJanusGraphDatabase.getGraphInstance().query();
+        this.query = graph.getGraph().query();
         this.graph = graph;
     }
 
@@ -226,7 +227,7 @@ public class NativeJanusGraphQuery implements NativeTinkerpopGraphQuery<AtlasJan
         JanusGraphPredicate pred;
         if (op instanceof ComparisionOperator) {
             Compare c = getGremlinPredicate((ComparisionOperator) op);
-            pred = JanusGraphPredicate.Converter.convert(c);
+            pred = JanusGraphPredicateUtils.convert(c);
         } else {
             pred = getGremlinPredicate((MatchingOperator)op);
         }
