@@ -17,16 +17,18 @@
  */
 package org.apache.atlas.model.instance;
 
+
+import com.fasterxml.jackson.annotation.JsonAutoDetect;
+import com.fasterxml.jackson.annotation.JsonIgnore;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.databind.annotation.JsonSerialize;
 import org.apache.atlas.model.PList;
 import org.apache.atlas.model.SearchFilter.SortType;
+import org.apache.atlas.model.glossary.relations.AtlasTermAssignmentHeader;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
-import org.codehaus.jackson.annotate.JsonAutoDetect;
-import org.codehaus.jackson.annotate.JsonIgnore;
-import org.codehaus.jackson.annotate.JsonIgnoreProperties;
-import org.codehaus.jackson.map.annotate.JsonSerialize;
 
 import javax.xml.bind.annotation.XmlAccessType;
 import javax.xml.bind.annotation.XmlAccessorType;
@@ -40,10 +42,11 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicLong;
 
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE;
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.PUBLIC_ONLY;
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;
 
 
 /**
@@ -57,20 +60,42 @@ import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.PUBLIC_ONL
 public class AtlasEntity extends AtlasStruct implements Serializable {
     private static final long serialVersionUID = 1L;
 
+    public static final String KEY_GUID            = "guid";
+    public static final String KEY_HOME_ID         = "homeId";
+    public static final String KEY_IS_PROXY        = "isProxy";
+    public static final String KEY_IS_INCOMPLETE   = "isIncomplete";
+    public static final String KEY_PROVENANCE_TYPE = "provenanceType";
+    public static final String KEY_STATUS          = "status";
+    public static final String KEY_CREATED_BY      = "createdBy";
+    public static final String KEY_UPDATED_BY      = "updatedBy";
+    public static final String KEY_CREATE_TIME     = "createTime";
+    public static final String KEY_UPDATE_TIME     = "updateTime";
+    public static final String KEY_VERSION         = "version";
+
     /**
      * Status of the entity - can be active or deleted. Deleted entities are not removed from Atlas store.
      */
-    public enum Status { ACTIVE, DELETED }
+    public enum Status { ACTIVE, DELETED, PURGED }
 
-    private String guid       = null;
-    private Status status     = Status.ACTIVE;
-    private String createdBy  = null;
-    private String updatedBy  = null;
-    private Date   createTime = null;
-    private Date   updateTime = null;
-    private Long   version    = 0L;
+    private String  guid           = null;
+    private String  homeId         = null;
+    private Boolean isProxy        = Boolean.FALSE;
+    private Boolean isIncomplete   = Boolean.FALSE;
+    private Integer provenanceType = 0;
+    private Status  status         = Status.ACTIVE;
+    private String  createdBy      = null;
+    private String  updatedBy      = null;
+    private Date    createTime     = null;
+    private Date    updateTime     = null;
+    private Long    version        = 0L;
 
-    private List<AtlasClassification> classifications;
+    private Map<String, Object>              relationshipAttributes;
+    private List<AtlasClassification>        classifications;
+    private List<AtlasTermAssignmentHeader>  meanings;
+    private Map<String, String>              customAttributes;
+    private Map<String, Map<String, Object>> businessAttributes;
+    private Set<String>                      labels;
+    private Set<String>                      pendingTasks; // read-only field i.e. value provided is ignored during entity create/update
 
     @JsonIgnore
     private static AtomicLong s_nextId = new AtomicLong(System.nanoTime());
@@ -105,6 +130,74 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
         setGuid(header.getGuid());
         setStatus(header.getStatus());
         setClassifications(header.getClassifications());
+        setMeanings(header.getMeanings());
+    }
+
+    public AtlasEntity(Map map) {
+        super(map);
+
+        if (map != null) {
+            Object oGuid             = map.get(KEY_GUID);
+            Object homeId            = map.get(KEY_HOME_ID);
+            Object isProxy           = map.get(KEY_IS_PROXY);
+            Object isIncomplete      = map.get(KEY_IS_INCOMPLETE);
+            Object provenanceType    = map.get(KEY_PROVENANCE_TYPE);
+            Object status            = map.get(KEY_STATUS);
+            Object createdBy         = map.get(KEY_CREATED_BY);
+            Object updatedBy         = map.get(KEY_UPDATED_BY);
+            Object createTime        = map.get(KEY_CREATE_TIME);
+            Object updateTime        = map.get(KEY_UPDATE_TIME);
+            Object version           = map.get(KEY_VERSION);
+
+            if (oGuid != null) {
+                setGuid(oGuid.toString());
+            }
+
+            if (homeId != null) {
+                setHomeId(homeId.toString());
+            }
+
+            if (isProxy != null) {
+                setIsProxy((Boolean)isProxy);
+            }
+            else {
+                setIsProxy(Boolean.FALSE);
+            }
+
+            if (isIncomplete != null) {
+                setIsIncomplete((Boolean) isIncomplete);
+            } else {
+                setIsIncomplete(Boolean.FALSE);
+            }
+
+            if (provenanceType instanceof Number) {
+                setProvenanceType(((Number) version).intValue());
+            }
+
+            if (status != null) {
+                setStatus(Status.valueOf(status.toString()));
+            }
+
+            if (createdBy != null) {
+                setCreatedBy(createdBy.toString());
+            }
+
+            if (createTime instanceof Number) {
+                setCreateTime(new Date(((Number) createTime).longValue()));
+            }
+
+            if (updatedBy != null) {
+                setUpdatedBy(updatedBy.toString());
+            }
+
+            if (updateTime instanceof Number) {
+                setUpdateTime(new Date(((Number) updateTime).longValue()));
+            }
+
+            if (version instanceof Number) {
+                setVersion(((Number) version).longValue());
+            }
+        }
     }
 
     public AtlasEntity(AtlasEntity other) {
@@ -112,6 +205,10 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
 
         if (other != null) {
             setGuid(other.getGuid());
+            setHomeId(other.getHomeId());
+            setIsProxy(other.isProxy());
+            setIsIncomplete(other.getIsIncomplete());
+            setProvenanceType(other.getProvenanceType());
             setStatus(other.getStatus());
             setCreatedBy(other.getCreatedBy());
             setUpdatedBy(other.getUpdatedBy());
@@ -119,6 +216,12 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
             setUpdateTime(other.getUpdateTime());
             setVersion(other.getVersion());
             setClassifications(other.getClassifications());
+            setRelationshipAttributes(other.getRelationshipAttributes());
+            setMeanings(other.getMeanings());
+            setCustomAttributes(other.getCustomAttributes());
+            setBusinessAttributes(other.getBusinessAttributes());
+            setLabels(other.getLabels());
+            setPendingTasks(other.getPendingTasks());
         }
     }
 
@@ -130,6 +233,38 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
         this.guid = guid;
     }
 
+    public String getHomeId() {
+        return homeId;
+    }
+
+    public void setHomeId(String homeId) {
+        this.homeId = homeId;
+    }
+
+    public Boolean isProxy() {
+        return isProxy;
+    }
+
+    public void setIsProxy(Boolean isProxy) {
+        this.isProxy = isProxy;
+    }
+
+    public Boolean getIsIncomplete() {
+        return isIncomplete;
+    }
+
+    public void setIsIncomplete(Boolean isIncomplete) {
+        this.isIncomplete = isIncomplete;
+    }
+
+    public Integer getProvenanceType() {
+        return provenanceType;
+    }
+
+    public void setProvenanceType(Integer provenanceType) {
+        this.provenanceType = provenanceType;
+    }
+
     public Status getStatus() {
         return status;
     }
@@ -178,19 +313,147 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
         this.version = version;
     }
 
+    public Map<String, Object> getRelationshipAttributes() { return relationshipAttributes; }
+
+    public void setRelationshipAttributes(Map<String, Object> relationshipAttributes) {
+        this.relationshipAttributes = relationshipAttributes;
+    }
+
+    public void setRelationshipAttribute(String name, Object value) {
+        Map<String, Object> r = this.relationshipAttributes;
+
+        if (r != null) {
+            r.put(name, value);
+        } else {
+            r = new HashMap<>();
+            r.put(name, value);
+
+            this.relationshipAttributes = r;
+        }
+    }
+
+    public Object getRelationshipAttribute(String name) {
+        Map<String, Object> a = this.relationshipAttributes;
+
+        return a != null ? a.get(name) : null;
+    }
+
+    public boolean hasRelationshipAttribute(String name) {
+        Map<String, Object> r = this.relationshipAttributes;
+
+        return r != null ? r.containsKey(name) : false;
+    }
+
+    public Map<String, String> getCustomAttributes() {
+        return customAttributes;
+    }
+
+    public void setCustomAttributes(Map<String, String> customAttributes) {
+        this.customAttributes = customAttributes;
+    }
+
+    public Map<String, Map<String, Object>> getBusinessAttributes() {
+        return businessAttributes;
+    }
+
+    public void setBusinessAttributes(Map<String, Map<String, Object>> businessAttributes) {
+        this.businessAttributes = businessAttributes;
+    }
+
+    public void setBusinessAttribute(String nsName, String nsAttrName, Object nsValue) {
+        Map<String, Map<String, Object>> businessAttributes = this.businessAttributes;
+
+        if (businessAttributes == null) {
+            businessAttributes = new HashMap<>();
+
+            this.businessAttributes = businessAttributes;
+        }
+
+        Map<String, Object> businessAttributeMap = businessAttributes.get(nsName);
+
+        if (businessAttributeMap == null) {
+            businessAttributeMap = new HashMap<>();
+
+            businessAttributes.put(nsName, businessAttributeMap);
+        }
+
+        businessAttributeMap.put(nsAttrName, nsValue);
+    }
+
+    public Object getBusinessAttribute(String bmName, String bmAttrName) {
+        Map<String, Map<String, Object>> businessAttributes   = this.businessAttributes;
+        Map<String, Object>              businessAttributeMap = businessAttributes == null ? null : businessAttributes.get(bmName);
+
+        return businessAttributeMap == null ? null : businessAttributeMap.get(bmAttrName);
+    }
+
+    public Set<String> getLabels() {
+        return labels;
+    }
+
+    public void setLabels(Set<String> labels) {
+        this.labels = labels;
+    }
+
+    public Set<String> getPendingTasks() {
+        return pendingTasks;
+    }
+
+    public void setPendingTasks(Set<String> pendingTasks) {
+        this.pendingTasks = pendingTasks;
+    }
+
     public List<AtlasClassification> getClassifications() { return classifications; }
 
     public void setClassifications(List<AtlasClassification> classifications) { this.classifications = classifications; }
 
+    public void addClassifications(List<AtlasClassification> classifications) {
+        List<AtlasClassification> c = this.classifications;
+
+        if (c == null) {
+            c = new ArrayList<>();
+        }
+
+        c.addAll(classifications);
+
+        this.classifications = c;
+    }
+
+    public List<AtlasTermAssignmentHeader> getMeanings() {
+        return meanings;
+    }
+
+    public void setMeanings(final List<AtlasTermAssignmentHeader> meanings) {
+        this.meanings = meanings;
+    }
+
+    public void addMeaning(AtlasTermAssignmentHeader meaning) {
+        List<AtlasTermAssignmentHeader> meanings = this.meanings;
+
+        if (meanings == null) {
+            meanings = new ArrayList<>();
+        }
+        meanings.add(meaning);
+        setMeanings(meanings);
+    }
 
     private void init() {
         setGuid(nextInternalId());
+        setHomeId(null);
+        setIsProxy(Boolean.FALSE);
+        setIsIncomplete(Boolean.FALSE);
+        setProvenanceType(0);
         setStatus(null);
         setCreatedBy(null);
         setUpdatedBy(null);
         setCreateTime(null);
         setUpdateTime(null);
         setClassifications(null);
+        setMeanings(null);
+        setCustomAttributes(null);
+        setBusinessAttributes(null);
+        setLabels(null);
+        setPendingTasks(null);
     }
 
     private static String nextInternalId() {
@@ -206,16 +469,37 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
         sb.append("AtlasEntity{");
         super.toString(sb);
         sb.append("guid='").append(guid).append('\'');
+        sb.append(", homeId='").append(homeId).append('\'');
+        sb.append(", isProxy='").append(isProxy).append('\'');
+        sb.append(", isIncomplete=").append(isIncomplete);
+        sb.append(", provenanceType=").append(provenanceType);
         sb.append(", status=").append(status);
         sb.append(", createdBy='").append(createdBy).append('\'');
         sb.append(", updatedBy='").append(updatedBy).append('\'');
         dumpDateField(", createTime=", createTime, sb);
         dumpDateField(", updateTime=", updateTime, sb);
         sb.append(", version=").append(version);
+        sb.append(", relationshipAttributes=[");
+        dumpObjects(relationshipAttributes, sb);
+        sb.append("]");
         sb.append(", classifications=[");
         AtlasBaseTypeDef.dumpObjects(classifications, sb);
         sb.append(']');
-        sb.append(", ");
+        sb.append(", meanings=[");
+        AtlasBaseTypeDef.dumpObjects(meanings, sb);
+        sb.append(']');
+        sb.append(", customAttributes=[");
+        dumpObjects(customAttributes, sb);
+        sb.append("]");
+        sb.append(", businessAttributes=[");
+        dumpObjects(businessAttributes, sb);
+        sb.append("]");
+        sb.append(", labels=[");
+        dumpObjects(labels, sb);
+        sb.append("]");
+        sb.append(", pendingTasks=[");
+        dumpObjects(pendingTasks, sb);
+        sb.append("]");
         sb.append('}');
 
         return sb;
@@ -229,19 +513,27 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
 
         AtlasEntity that = (AtlasEntity) o;
         return Objects.equals(guid, that.guid) &&
+                Objects.equals(homeId, that.homeId) &&
+                Objects.equals(isProxy, that.isProxy) &&
+                Objects.equals(isIncomplete, that.isIncomplete) &&
+                Objects.equals(provenanceType, that.provenanceType) &&
                 status == that.status &&
                 Objects.equals(createdBy, that.createdBy) &&
                 Objects.equals(updatedBy, that.updatedBy) &&
                 Objects.equals(createTime, that.createTime) &&
                 Objects.equals(updateTime, that.updateTime) &&
                 Objects.equals(version, that.version) &&
+                Objects.equals(relationshipAttributes, that.relationshipAttributes) &&
+                Objects.equals(customAttributes, that.customAttributes) &&
+                Objects.equals(businessAttributes, that.businessAttributes) &&
+                Objects.equals(labels, that.labels) &&
                 Objects.equals(classifications, that.classifications);
     }
 
     @Override
     public int hashCode() {
-        return Objects.hash(super.hashCode(), guid, status, createdBy, updatedBy, createTime, updateTime, version,
-                            classifications);
+        return Objects.hash(super.hashCode(), guid, homeId, isProxy, isIncomplete, provenanceType, status, createdBy, updatedBy,
+                createTime, updateTime, version, relationshipAttributes, classifications, customAttributes, businessAttributes, labels);
     }
 
     @Override
@@ -623,4 +915,4 @@ public class AtlasEntity extends AtlasStruct implements Serializable {
             super(list, startIndex, pageSize, totalCount, sortType, sortBy);
         }
     }
-}
+}
\ No newline at end of file
