@@ -17,6 +17,11 @@
  */
 package org.apache.atlas.model.typedef;
 
+import com.fasterxml.jackson.annotation.JsonAutoDetect;
+import com.fasterxml.jackson.annotation.JsonIgnore;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.databind.annotation.JsonSerialize;
+
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -39,13 +44,9 @@ import org.apache.atlas.model.SearchFilter.SortType;
 import org.apache.atlas.model.TypeCategory;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.hadoop.util.StringUtils;
-import org.codehaus.jackson.annotate.JsonAutoDetect;
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.PUBLIC_ONLY;
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE;
 
-import org.codehaus.jackson.annotate.JsonIgnore;
-import org.codehaus.jackson.annotate.JsonIgnoreProperties;
-import org.codehaus.jackson.map.annotate.JsonSerialize;
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;
 
 
 /**
@@ -88,7 +89,11 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
     }
 
     protected AtlasStructDef(TypeCategory category, String name, String description, String typeVersion, List<AtlasAttributeDef> attributeDefs, Map<String, String> options) {
-        super(category, name, description, typeVersion, options);
+        this(category, name, description, typeVersion, attributeDefs, null, options);
+    }
+
+    protected AtlasStructDef(TypeCategory category, String name, String description, String typeVersion, List<AtlasAttributeDef> attributeDefs, String serviceType, Map<String, String> options) {
+        super(category, name, description, typeVersion, serviceType, options);
 
         setAttributeDefs(attributeDefs);
     }
@@ -254,16 +259,23 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
     @XmlRootElement
     @XmlAccessorType(XmlAccessType.PROPERTY)
     public static class AtlasAttributeDef implements Serializable {
-        private static final long serialVersionUID = 1L;
+        private static final long     serialVersionUID              = 1L;
+        public static final int       DEFAULT_SEARCHWEIGHT          = -1;
 
-        public static final String ATTRDEF_OPTION_SOFT_REFERENCE = "isSoftReference";
-        private final String STRING_TRUE = "true";
+
+        public static final String    SEARCH_WEIGHT_ATTR_NAME                 = "searchWeight";
+        public static final String    INDEX_TYPE_ATTR_NAME                    = "indexType";
+        public static final String    ATTRDEF_OPTION_SOFT_REFERENCE           = "isSoftReference";
+        public static final String    ATTRDEF_OPTION_APPEND_ON_PARTIAL_UPDATE = "isAppendOnPartialUpdate";
+        private final String          STRING_TRUE                             = "true";
 
         /**
          * single-valued attribute or multi-valued attribute.
          */
         public enum Cardinality { SINGLE, LIST, SET }
 
+        public enum IndexType { DEFAULT, STRING}
+
         public static final int COUNT_NOT_SET = -1;
 
         private String                   name;
@@ -274,23 +286,60 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
         private int                      valuesMaxCount;
         private boolean                  isUnique;
         private boolean                  isIndexable;
+        private boolean                  includeInNotification;
+        private String                   defaultValue;
+        private String                   description;
+        private int                      searchWeight = DEFAULT_SEARCHWEIGHT;
+        private IndexType                indexType    = null;
         private List<AtlasConstraintDef> constraints;
         private Map<String, String>      options;
+        private String                   displayName;
 
         public AtlasAttributeDef() { this(null, null); }
 
         public AtlasAttributeDef(String name, String typeName) {
-            this(name, typeName, false, Cardinality.SINGLE, COUNT_NOT_SET, COUNT_NOT_SET, false, false, null);
+            this(name, typeName, DEFAULT_SEARCHWEIGHT);
+        }
+
+        public AtlasAttributeDef(String name, String typeName, boolean isUnique, boolean isIndexable) {
+            this(name, typeName, false, Cardinality.SINGLE, COUNT_NOT_SET, COUNT_NOT_SET, isUnique, isIndexable,
+                false, null,null, null, null, DEFAULT_SEARCHWEIGHT, null);
+        }
+
+        public AtlasAttributeDef(String name, String typeName, Cardinality cardinality, boolean isUnique, boolean isIndexable) {
+            this(name, typeName, false, cardinality, COUNT_NOT_SET, COUNT_NOT_SET, isUnique, isIndexable,
+                false, null,null, null, null, DEFAULT_SEARCHWEIGHT, null);
+        }
+
+        public AtlasAttributeDef(String name, String typeName, int searchWeight) {
+            this(name, typeName, false, Cardinality.SINGLE, searchWeight, null);
+        }
+
+        public AtlasAttributeDef(String name, String typeName, int searchWeight, IndexType indexType) {
+            this(name, typeName, false, Cardinality.SINGLE, searchWeight, indexType);
+        }
+
+        public AtlasAttributeDef(String name, String typeName, boolean isOptional, Cardinality cardinality) {
+            this(name, typeName, isOptional, cardinality, DEFAULT_SEARCHWEIGHT, null);
+        }
+
+        private AtlasAttributeDef(String name, String typeName, boolean isOptional, Cardinality cardinality, int searchWeight, IndexType indexType) {
+            this(name, typeName, isOptional, cardinality, COUNT_NOT_SET, COUNT_NOT_SET, false, false, false, null, searchWeight, indexType);
         }
 
         public AtlasAttributeDef(String name, String typeName, boolean isOptional, Cardinality cardinality,
-                                 int valuesMinCount, int valuesMaxCount, boolean isUnique, boolean isIndexable, List<AtlasConstraintDef> constraints) {
-            this(name, typeName, isOptional, cardinality, valuesMinCount, valuesMaxCount, isUnique, isIndexable, null, constraints, null, null);
+                                 int valuesMinCount, int valuesMaxCount, boolean isUnique, boolean isIndexable, boolean includeInNotification, List<AtlasConstraintDef> constraints) {
+            this(name, typeName, isOptional, cardinality, valuesMinCount, valuesMaxCount, isUnique, isIndexable, includeInNotification, constraints, DEFAULT_SEARCHWEIGHT, null);
+        }
+
+        private AtlasAttributeDef(String name, String typeName, boolean isOptional, Cardinality cardinality,
+                                  int valuesMinCount, int valuesMaxCount, boolean isUnique, boolean isIndexable, boolean includeInNotification, List<AtlasConstraintDef> constraints, int searchWeight, IndexType indexType) {
+            this(name, typeName, isOptional, cardinality, valuesMinCount, valuesMaxCount, isUnique, isIndexable, includeInNotification, null, constraints, null, null, searchWeight, indexType);
         }
 
         public AtlasAttributeDef(String name, String typeName, boolean isOptional, Cardinality cardinality,
-                                 int valuesMinCount, int valuesMaxCount, boolean isUnique, boolean isIndexable, String defaultValue,
-                                 List<AtlasConstraintDef> constraints, Map<String,String> options, String description) {
+                                 int valuesMinCount, int valuesMaxCount, boolean isUnique, boolean isIndexable, boolean includeInNotification, String defaultValue,
+                                 List<AtlasConstraintDef> constraints, Map<String,String> options, String description, int searchWeight, IndexType indexType) {
             setName(name);
             setTypeName(typeName);
             setIsOptional(isOptional);
@@ -299,8 +348,13 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
             setValuesMaxCount(valuesMaxCount);
             setIsUnique(isUnique);
             setIsIndexable(isIndexable);
+            setIncludeInNotification(includeInNotification);
+            setDefaultValue(defaultValue);
             setConstraints(constraints);
             setOptions(options);
+            setDescription(description);
+            setSearchWeight(searchWeight);
+            setIndexType(indexType);
         }
 
         public AtlasAttributeDef(AtlasAttributeDef other) {
@@ -313,11 +367,41 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
                 setValuesMaxCount(other.getValuesMaxCount());
                 setIsUnique(other.getIsUnique());
                 setIsIndexable(other.getIsIndexable());
+                setIncludeInNotification(other.getIncludeInNotification());
+                setDefaultValue(other.getDefaultValue());
                 setConstraints(other.getConstraints());
                 setOptions(other.getOptions());
+                setDescription((other.getDescription()));
+                setSearchWeight(other.getSearchWeight());
+                setIndexType(other.getIndexType());
+                setDisplayName(other.getDisplayName());
             }
         }
 
+        public void setDisplayName(String displayName) {
+            this.displayName = displayName;
+        }
+
+        public String getDisplayName() {
+            return displayName;
+        }
+
+        public int getSearchWeight() {
+            return searchWeight;
+        }
+
+        public void setSearchWeight(int searchWeight) {
+            this.searchWeight = searchWeight;
+        }
+
+        public void setIndexType(IndexType indexType) {
+            this.indexType = indexType;
+        }
+
+        public IndexType getIndexType() {
+            return indexType;
+        }
+
         public String getName() {
             return name;
         }
@@ -376,6 +460,18 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
             return isIndexable;
         }
 
+        public boolean getIncludeInNotification() { return includeInNotification; }
+
+        public void setIncludeInNotification(Boolean isInNotification) { this.includeInNotification = isInNotification == null ? Boolean.FALSE : isInNotification; }
+
+        public String getDefaultValue(){
+            return defaultValue;
+        }
+
+        public void setDefaultValue(String defaultValue){
+            this.defaultValue = defaultValue;
+        }
+
         public void setIsIndexable(boolean idexable) {
             isIndexable = idexable;
         }
@@ -424,6 +520,37 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
                     getOptions().get(AtlasAttributeDef.ATTRDEF_OPTION_SOFT_REFERENCE).equals(STRING_TRUE);
         }
 
+        @JsonIgnore
+        public boolean isAppendOnPartialUpdate() {
+            String val = getOption(AtlasAttributeDef.ATTRDEF_OPTION_APPEND_ON_PARTIAL_UPDATE);
+
+            return val != null && Boolean.valueOf(val);
+        }
+
+        @JsonIgnore
+        public void setOption(String name, String value) {
+            if (this.options == null) {
+                this.options = new HashMap<>();
+            }
+
+            this.options.put(name, value);
+        }
+
+        @JsonIgnore
+        public String getOption(String name) {
+            Map<String, String> option = this.options;
+
+            return option != null ? option.get(name) : null;
+        }
+
+        public String getDescription() {
+            return description;
+        }
+
+        public void setDescription(String description) {
+            this.description = description;
+        }
+
         public StringBuilder toString(StringBuilder sb) {
             if (sb == null) {
                 sb = new StringBuilder();
@@ -432,13 +559,19 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
             sb.append("AtlasAttributeDef{");
             sb.append("name='").append(name).append('\'');
             sb.append(", typeName='").append(typeName).append('\'');
+            sb.append(", description='").append(description).append('\'');
             sb.append(", getIsOptional=").append(isOptional);
             sb.append(", cardinality=").append(cardinality);
             sb.append(", valuesMinCount=").append(valuesMinCount);
             sb.append(", valuesMaxCount=").append(valuesMaxCount);
             sb.append(", isUnique=").append(isUnique);
             sb.append(", isIndexable=").append(isIndexable);
+            sb.append(", includeInNotification=").append(includeInNotification);
+            sb.append(", defaultValue=").append(defaultValue);
             sb.append(", options='").append(options).append('\'');
+            sb.append(", searchWeight='").append(searchWeight).append('\'');
+            sb.append(", indexType='").append(indexType).append('\'');
+            sb.append(", displayName='").append(displayName).append('\'');
             sb.append(", constraints=[");
             if (CollectionUtils.isNotEmpty(constraints)) {
                 int i = 0;
@@ -466,16 +599,22 @@ public class AtlasStructDef extends AtlasBaseTypeDef implements Serializable {
                     valuesMaxCount == that.valuesMaxCount &&
                     isUnique == that.isUnique &&
                     isIndexable == that.isIndexable &&
+                    includeInNotification == that.includeInNotification &&
                     Objects.equals(name, that.name) &&
                     Objects.equals(typeName, that.typeName) &&
                     cardinality == that.cardinality &&
+                    Objects.equals(defaultValue, that.defaultValue) &&
+                    Objects.equals(description, that.description) &&
                     Objects.equals(constraints, that.constraints) &&
-                    Objects.equals(options, that.options);
+                    Objects.equals(options, that.options) &&
+                    Objects.equals(searchWeight, that.searchWeight) &&
+                    Objects.equals(indexType, that.indexType) &&
+                    Objects.equals(displayName, that.displayName);
         }
 
         @Override
         public int hashCode() {
-            return Objects.hash(name, typeName, isOptional, cardinality, valuesMinCount, valuesMaxCount, isUnique, isIndexable, constraints, options);
+            return Objects.hash(name, typeName, isOptional, cardinality, valuesMinCount, valuesMaxCount, isUnique, isIndexable, includeInNotification, defaultValue, constraints, options, description, searchWeight, indexType, displayName);
         }
 
         @Override
