@@ -42,18 +42,20 @@ import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_
 @XmlRootElement
 @XmlAccessorType(XmlAccessType.PROPERTY)
 public class AtlasTypesDef {
-    private List<AtlasEnumDef>           enumDefs;
-    private List<AtlasStructDef>         structDefs;
-    private List<AtlasClassificationDef> classificationDefs;
-    private List<AtlasEntityDef>         entityDefs;
-    private List<AtlasRelationshipDef>   relationshipDefs;
+    private List<AtlasEnumDef>             enumDefs;
+    private List<AtlasStructDef>           structDefs;
+    private List<AtlasClassificationDef>   classificationDefs;
+    private List<AtlasEntityDef>           entityDefs;
+    private List<AtlasRelationshipDef>     relationshipDefs;
+    private List<AtlasBusinessMetadataDef> businessMetadataDefs;
 
     public AtlasTypesDef() {
-        enumDefs           = new ArrayList<>();
-        structDefs         = new ArrayList<>();
-        classificationDefs = new ArrayList<>();
-        entityDefs         = new ArrayList<>();
-        relationshipDefs   = new ArrayList<>();
+        enumDefs             = new ArrayList<>();
+        structDefs           = new ArrayList<>();
+        classificationDefs   = new ArrayList<>();
+        entityDefs           = new ArrayList<>();
+        relationshipDefs     = new ArrayList<>();
+        businessMetadataDefs = new ArrayList<>();
     }
 
     /**
@@ -66,7 +68,7 @@ public class AtlasTypesDef {
      */
     public AtlasTypesDef(List<AtlasEnumDef> enumDefs, List<AtlasStructDef> structDefs,
                          List<AtlasClassificationDef> classificationDefs, List<AtlasEntityDef> entityDefs) {
-       this(enumDefs, structDefs, classificationDefs, entityDefs,new ArrayList<AtlasRelationshipDef>());
+       this(enumDefs, structDefs, classificationDefs, entityDefs, new ArrayList<>(), new ArrayList<>());
     }
     /**
      * Create the TypesDef. This created definitions for each of the types.
@@ -81,12 +83,23 @@ public class AtlasTypesDef {
                          List<AtlasClassificationDef> classificationDefs,
                          List<AtlasEntityDef>         entityDefs,
                          List<AtlasRelationshipDef>   relationshipDefs) {
-        this.enumDefs           = enumDefs;
-        this.structDefs         = structDefs;
-        this.classificationDefs = classificationDefs;
-        this.entityDefs         = entityDefs;
-        this.relationshipDefs   = relationshipDefs;
+        this(enumDefs, structDefs, classificationDefs, entityDefs, relationshipDefs, new ArrayList<>());
     }
+
+    public AtlasTypesDef(List<AtlasEnumDef>             enumDefs,
+                         List<AtlasStructDef>           structDefs,
+                         List<AtlasClassificationDef>   classificationDefs,
+                         List<AtlasEntityDef>           entityDefs,
+                         List<AtlasRelationshipDef>     relationshipDefs,
+                         List<AtlasBusinessMetadataDef> businessMetadataDefs) {
+        this.enumDefs             = enumDefs;
+        this.structDefs           = structDefs;
+        this.classificationDefs   = classificationDefs;
+        this.entityDefs           = entityDefs;
+        this.relationshipDefs     = relationshipDefs;
+        this.businessMetadataDefs = businessMetadataDefs;
+    }
+
     public List<AtlasEnumDef> getEnumDefs() {
         return enumDefs;
     }
@@ -125,6 +138,14 @@ public class AtlasTypesDef {
         this.relationshipDefs = relationshipDefs;
     }
 
+    public void setBusinessMetadataDefs(List<AtlasBusinessMetadataDef> businessMetadataDefs) {
+        this.businessMetadataDefs = businessMetadataDefs;
+    }
+
+    public List<AtlasBusinessMetadataDef> getBusinessMetadataDefs() {
+        return businessMetadataDefs;
+    }
+
     public boolean hasClassificationDef(String name) {
         return hasTypeDef(classificationDefs, name);
     }
@@ -144,6 +165,9 @@ public class AtlasTypesDef {
         return hasTypeDef(relationshipDefs, name);
     }
 
+    public boolean hasBusinessMetadataDef(String name) {
+        return hasTypeDef(businessMetadataDefs, name);
+    }
 
     private <T extends AtlasBaseTypeDef> boolean hasTypeDef(Collection<T> typeDefs, String name) {
         if (CollectionUtils.isNotEmpty(typeDefs)) {
@@ -163,7 +187,8 @@ public class AtlasTypesDef {
                 CollectionUtils.isEmpty(structDefs) &&
                 CollectionUtils.isEmpty(classificationDefs) &&
                 CollectionUtils.isEmpty(entityDefs) &&
-                CollectionUtils.isEmpty(relationshipDefs);
+                CollectionUtils.isEmpty(relationshipDefs) &&
+                CollectionUtils.isEmpty(businessMetadataDefs);
     }
 
     public void clear() {
@@ -185,6 +210,10 @@ public class AtlasTypesDef {
         if (relationshipDefs != null) {
             relationshipDefs.clear();
         }
+
+        if (businessMetadataDefs != null) {
+            businessMetadataDefs.clear();
+        }
     }
     public StringBuilder toString(StringBuilder sb) {
         if (sb == null) {
@@ -206,6 +235,8 @@ public class AtlasTypesDef {
         sb.append("}");
         sb.append("relationshipDefs={");
         AtlasBaseTypeDef.dumpObjects(relationshipDefs, sb);
+        sb.append("businessMetadataDefs={");
+        AtlasBaseTypeDef.dumpObjects(businessMetadataDefs, sb);
         sb.append("}");
 
         return sb;
