@@ -20,6 +20,7 @@ package org.apache.atlas.type;
 
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasRelationship;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.model.typedef.AtlasRelationshipDef;
@@ -91,19 +92,36 @@ public class AtlasRelationshipType extends AtlasStructType {
         String end1TypeName = relationshipDef.getEndDef1() != null ? relationshipDef.getEndDef1().getType() : null;
         String end2TypeName = relationshipDef.getEndDef2() != null ? relationshipDef.getEndDef2().getType() : null;
 
-        AtlasType type1 = typeRegistry.getType(end1TypeName);
-        AtlasType type2 = typeRegistry.getType(end2TypeName);
-
-        if (type1 instanceof AtlasEntityType) {
-            end1Type = (AtlasEntityType) type1;
+        if (StringUtils.isEmpty(end1TypeName)) {
+            throw new AtlasBaseException(AtlasErrorCode.MISSING_MANDATORY_ATTRIBUTE, "endDef1", "type");
         } else {
-            throw new AtlasBaseException(AtlasErrorCode.RELATIONSHIPDEF_INVALID_END_TYPE, getTypeName(), end1TypeName);
+            AtlasType type1 = typeRegistry.getType(end1TypeName);
+
+            if (type1 instanceof AtlasEntityType) {
+                end1Type = (AtlasEntityType) type1;
+            } else {
+                throw new AtlasBaseException(AtlasErrorCode.RELATIONSHIPDEF_INVALID_END_TYPE, getTypeName(), end1TypeName);
+            }
         }
 
-        if (type2 instanceof AtlasEntityType) {
-            end2Type = (AtlasEntityType) type2;
+        if (StringUtils.isEmpty(end2TypeName)) {
+            throw new AtlasBaseException(AtlasErrorCode.MISSING_MANDATORY_ATTRIBUTE, "endDef2", "type");
         } else {
-            throw new AtlasBaseException(AtlasErrorCode.RELATIONSHIPDEF_INVALID_END_TYPE, getTypeName(), end2TypeName);
+            AtlasType type2 = typeRegistry.getType(end2TypeName);
+
+            if (type2 instanceof AtlasEntityType) {
+                end2Type = (AtlasEntityType) type2;
+            } else {
+                throw new AtlasBaseException(AtlasErrorCode.RELATIONSHIPDEF_INVALID_END_TYPE, getTypeName(), end2TypeName);
+            }
+        }
+
+        if (StringUtils.isEmpty(relationshipDef.getEndDef1().getName())) {
+            throw new AtlasBaseException(AtlasErrorCode.MISSING_MANDATORY_ATTRIBUTE, "endDef1", "name");
+        }
+
+        if (StringUtils.isEmpty(relationshipDef.getEndDef2().getName())) {
+            throw new AtlasBaseException(AtlasErrorCode.MISSING_MANDATORY_ATTRIBUTE, "endDef2", "name");
         }
 
         validateAtlasRelationshipDef(relationshipDef);
@@ -242,14 +260,29 @@ public class AtlasRelationshipType extends AtlasStructType {
      * @throws AtlasBaseException
      */
     private boolean validateRelationship(AtlasRelationship relationship) {
-        String end1TypeName = relationship.getEnd1() != null ? relationship.getEnd1().getTypeName() : null;
-        String end2TypeName = relationship.getEnd2() != null ? relationship.getEnd2().getTypeName() : null;
 
-        if (StringUtils.isNotEmpty(end1TypeName) && StringUtils.isNotEmpty(end2TypeName)) {
-            return end1Type.isTypeOrSuperTypeOf(end1TypeName) && end2Type.isTypeOrSuperTypeOf(end2TypeName) && super.isValidValue(relationship);
+        AtlasObjectId end1 = relationship.getEnd1();
+        AtlasObjectId end2 = relationship.getEnd2();
+
+        if (end1 != null && end2 != null) {
+
+            String end1TypeName = end1.getTypeName();
+            String end2TypeName = end2.getTypeName();
+
+            if (StringUtils.isNotEmpty(end1TypeName) && StringUtils.isNotEmpty(end2TypeName)) {
+
+                return end1Type.isTypeOrSuperTypeOf(end1TypeName) && end2Type.isTypeOrSuperTypeOf(end2TypeName) && super.isValidValue(relationship);
+
+            } else {
+
+                return StringUtils.isNotEmpty(end1.getGuid()) && StringUtils.isNotEmpty(end2.getGuid());
+
+            }
+
         }
 
         return false;
+
     }
 
     /**
