@@ -20,12 +20,14 @@ package org.apache.atlas.type;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
+import org.apache.atlas.model.typedef.AtlasBusinessMetadataDef;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
 import org.apache.atlas.model.typedef.AtlasRelationshipDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
+import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
@@ -33,13 +35,7 @@ import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Component;
 
 import javax.inject.Singleton;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.ReentrantLock;
@@ -57,19 +53,27 @@ public class AtlasTypeRegistry {
     protected       RegistryData                   registryData;
     private   final TypeRegistryUpdateSynchronizer updateSynchronizer;
     private   final Set<String>                    missingRelationshipDefs;
-
+    private   final Map<String, String>            commonIndexFieldNameCache;
 
     public AtlasTypeRegistry() {
-        registryData            = new RegistryData();
-        updateSynchronizer      = new TypeRegistryUpdateSynchronizer(this);
-        missingRelationshipDefs = new HashSet<>();
+        registryData              = new RegistryData();
+        updateSynchronizer        = new TypeRegistryUpdateSynchronizer(this);
+        missingRelationshipDefs   = new HashSet<>();
+        commonIndexFieldNameCache = new LinkedHashMap<>();
+
+        resolveReferencesForRootTypes();
+        resolveIndexFieldNamesForRootTypes();
     }
 
     // used only by AtlasTransientTypeRegistry
     protected AtlasTypeRegistry(AtlasTypeRegistry other) {
-        registryData            = new RegistryData();
-        updateSynchronizer      = other.updateSynchronizer;
-        missingRelationshipDefs = other.missingRelationshipDefs;
+        registryData              = new RegistryData();
+        updateSynchronizer        = other.updateSynchronizer;
+        missingRelationshipDefs   = other.missingRelationshipDefs;
+        commonIndexFieldNameCache = other.commonIndexFieldNameCache;
+
+        resolveReferencesForRootTypes();
+        resolveIndexFieldNamesForRootTypes();
     }
 
     public Collection<String> getAllTypeNames() { return registryData.allTypes.getAllTypeNames(); }
@@ -116,12 +120,14 @@ public class AtlasTypeRegistry {
         return ret;
     }
 
-    public AtlasType getTypeByGuid(String guid) {
+    public AtlasType getTypeByGuid(String guid) throws AtlasBaseException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> AtlasTypeRegistry.getTypeByGuid({})", guid);
         }
 
         AtlasType ret = registryData.allTypes.getTypeByGuid(guid);
+        if(ret == null)
+            throw new AtlasBaseException(AtlasErrorCode.TYPE_GUID_NOT_FOUND, guid);
 
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== AtlasTypeRegistry.getTypeByGuid({}): {}", guid, ret);
@@ -189,6 +195,16 @@ public class AtlasTypeRegistry {
         return registryData.classificationDefs.getTypeByName(name);
     }
 
+    public Collection<AtlasBusinessMetadataType> getAllBusinessMetadataTypes() {
+        return registryData.businessMetadataDefs.getAllTypes();
+    }
+
+    public Collection<AtlasBusinessMetadataDef> getAllBusinessMetadataDefs() {
+        return registryData.businessMetadataDefs.getAll();
+    }
+
+    public AtlasBusinessMetadataType getBusinessMetadataTypeByName(String name) { return registryData.businessMetadataDefs.getTypeByName(name); }
+
     public Collection<AtlasRelationshipDef> getAllRelationshipDefs() { return registryData.relationshipDefs.getAll(); }
 
     public Collection<AtlasEntityDef> getAllEntityDefs() { return registryData.entityDefs.getAll(); }
@@ -214,6 +230,15 @@ public class AtlasTypeRegistry {
     public AtlasRelationshipDef getRelationshipDefByName(String name) {
         return registryData.relationshipDefs.getTypeDefByName(name);
     }
+
+    public AtlasBusinessMetadataDef getBusinessMetadataDefByGuid(String guid) {
+        return registryData.businessMetadataDefs.getTypeDefByGuid(guid);
+    }
+
+    public AtlasBusinessMetadataDef getBusinessMetadataDefByName(String name) {
+        return registryData.businessMetadataDefs.getTypeDefByName(name);
+    }
+
     public AtlasRelationshipType getRelationshipTypeByName(String name) { return registryData.relationshipDefs.getTypeByName(name); }
 
     public AtlasTransientTypeRegistry lockTypeRegistryForUpdate() throws AtlasBaseException {
@@ -238,23 +263,64 @@ public class AtlasTypeRegistry {
         }
     }
 
+    public void addIndexFieldName(String propertyName, String indexFieldName) {
+        commonIndexFieldNameCache.put(propertyName, indexFieldName);
+    }
+
+    private void resolveReferencesForRootTypes() {
+        try {
+            AtlasEntityType.ENTITY_ROOT.resolveReferences(this);
+            AtlasClassificationType.CLASSIFICATION_ROOT.resolveReferences(this);
+        } catch (AtlasBaseException e) {
+            LOG.error("Failed to initialize root types", e);
+            throw new RuntimeException(e);
+        }
+    }
+
+    private void resolveIndexFieldNamesForRootTypes() {
+        for (AtlasStructType structType : Arrays.asList(AtlasEntityType.ENTITY_ROOT, AtlasClassificationType.CLASSIFICATION_ROOT)) {
+            for (AtlasAttribute attribute : structType.getAllAttributes().values()) {
+                String indexFieldName = getIndexFieldName(attribute.getVertexPropertyName());
+
+                if (StringUtils.isNotEmpty(indexFieldName)) {
+                    attribute.setIndexFieldName(indexFieldName);
+                }
+
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Attribute {} with index name {} is added", attribute.getVertexPropertyName(), attribute.getIndexFieldName());
+                }
+            }
+        }
+    }
+
+    /**
+     * retrieves the index field name for the common field passed in.
+     * @param propertyName the name of the common field.
+     * @return the index name for the common field passed in.
+     */
+    public String getIndexFieldName(String propertyName) {
+        return commonIndexFieldNameCache.get(propertyName);
+    }
+
     static class RegistryData {
-        final TypeCache                                                       allTypes;
-        final TypeDefCache<AtlasEnumDef, AtlasEnumType>                       enumDefs;
-        final TypeDefCache<AtlasStructDef, AtlasStructType>                   structDefs;
-        final TypeDefCache<AtlasClassificationDef, AtlasClassificationType>   classificationDefs;
-        final TypeDefCache<AtlasEntityDef, AtlasEntityType>                   entityDefs;
-        final TypeDefCache<AtlasRelationshipDef, AtlasRelationshipType>       relationshipDefs;
-        final TypeDefCache<? extends AtlasBaseTypeDef, ? extends AtlasType>[] allDefCaches;
+        final TypeCache                                                         allTypes;
+        final TypeDefCache<AtlasEnumDef, AtlasEnumType>                         enumDefs;
+        final TypeDefCache<AtlasStructDef, AtlasStructType>                     structDefs;
+        final TypeDefCache<AtlasClassificationDef, AtlasClassificationType>     classificationDefs;
+        final TypeDefCache<AtlasEntityDef, AtlasEntityType>                     entityDefs;
+        final TypeDefCache<AtlasRelationshipDef, AtlasRelationshipType>         relationshipDefs;
+        final TypeDefCache<AtlasBusinessMetadataDef, AtlasBusinessMetadataType> businessMetadataDefs;
+        final TypeDefCache<? extends AtlasBaseTypeDef, ? extends AtlasType>[]   allDefCaches;
 
         RegistryData() {
-            allTypes           = new TypeCache();
-            enumDefs           = new TypeDefCache<>(allTypes);
-            structDefs         = new TypeDefCache<>(allTypes);
-            classificationDefs = new TypeDefCache<>(allTypes);
-            entityDefs         = new TypeDefCache<>(allTypes);
-            relationshipDefs   = new TypeDefCache<>(allTypes);
-            allDefCaches       = new TypeDefCache[] { enumDefs, structDefs, classificationDefs, entityDefs, relationshipDefs };
+            allTypes             = new TypeCache();
+            enumDefs             = new TypeDefCache<>(allTypes);
+            structDefs           = new TypeDefCache<>(allTypes);
+            classificationDefs   = new TypeDefCache<>(allTypes);
+            entityDefs           = new TypeDefCache<>(allTypes);
+            relationshipDefs     = new TypeDefCache<>(allTypes);
+            businessMetadataDefs = new TypeDefCache<>(allTypes);
+            allDefCaches         = new TypeDefCache[] { enumDefs, structDefs, classificationDefs, entityDefs, relationshipDefs, businessMetadataDefs};
 
             init();
         }
@@ -313,6 +379,7 @@ public class AtlasTypeRegistry {
                 classificationDefs.updateGuid(typeName, guid);
                 entityDefs.updateGuid(typeName, guid);
                 relationshipDefs.updateGuid(typeName, guid);
+                businessMetadataDefs.updateGuid(typeName, guid);
             }
         }
 
@@ -323,6 +390,7 @@ public class AtlasTypeRegistry {
                 classificationDefs.removeTypeDefByGuid(guid);
                 entityDefs.removeTypeDefByGuid(guid);
                 relationshipDefs.removeTypeDefByGuid(guid);
+                businessMetadataDefs.removeTypeDefByGuid(guid);
             }
         }
 
@@ -333,6 +401,7 @@ public class AtlasTypeRegistry {
                 classificationDefs.removeTypeDefByName(typeName);
                 entityDefs.removeTypeDefByName(typeName);
                 relationshipDefs.removeTypeDefByName(typeName);
+                businessMetadataDefs.removeTypeDefByName(typeName);
             }
         }
 
@@ -343,7 +412,7 @@ public class AtlasTypeRegistry {
             classificationDefs.clear();
             entityDefs.clear();
             relationshipDefs.clear();
-
+            businessMetadataDefs.clear();
             init();
         }
     }
@@ -362,6 +431,7 @@ public class AtlasTypeRegistry {
             addTypesWithNoRefResolve(parent.getAllClassificationDefs());
             addTypesWithNoRefResolve(parent.getAllEntityDefs());
             addTypesWithNoRefResolve(parent.getAllRelationshipDefs());
+            addTypesWithNoRefResolve(parent.getAllBusinessMetadataDefs());
 
             addedTypes.clear();
             updatedTypes.clear();
@@ -393,10 +463,10 @@ public class AtlasTypeRegistry {
 
             if (typeDef != null) {
                 addTypeWithNoRefResolve(typeDef);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.addType({})", typeDef);
             }
@@ -421,10 +491,10 @@ public class AtlasTypeRegistry {
 
             if (CollectionUtils.isNotEmpty(typeDefs)) {
                 addTypesWithNoRefResolve(typeDefs);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.addTypes(length={})", (typeDefs == null ? 0 : typeDefs.size()));
             }
@@ -441,10 +511,11 @@ public class AtlasTypeRegistry {
                 addTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 addTypesWithNoRefResolve(typesDef.getEntityDefs());
                 addTypesWithNoRefResolve(typesDef.getRelationshipDefs());
-
-                resolveReferences();
+                addTypesWithNoRefResolve(typesDef.getBusinessMetadataDefs());
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.addTypes({})", typesDef);
             }
@@ -457,10 +528,10 @@ public class AtlasTypeRegistry {
 
             if (typeDef != null) {
                 updateTypeWithNoRefResolve(typeDef);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateType({})", typeDef);
             }
@@ -473,10 +544,10 @@ public class AtlasTypeRegistry {
 
             if (guid != null && typeDef != null) {
                 updateTypeByGuidWithNoRefResolve(guid, typeDef);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateTypeByGuid({})", guid);
             }
@@ -489,10 +560,10 @@ public class AtlasTypeRegistry {
 
             if (name != null && typeDef != null) {
                 updateTypeByNameWithNoRefResolve(name, typeDef);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateEnumDefByName({})", name);
             }
@@ -505,10 +576,10 @@ public class AtlasTypeRegistry {
 
             if (CollectionUtils.isNotEmpty(typeDefs)) {
                 updateTypesWithNoRefResolve(typeDefs);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateTypes(length={})", (typeDefs == null ? 0 : typeDefs.size()));
             }
@@ -521,10 +592,10 @@ public class AtlasTypeRegistry {
 
             if (typesDef != null) {
                 updateTypesWithNoRefResolve(typesDef);
-
-                resolveReferences();
             }
 
+            resolveReferences();
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateTypes({})", typesDef);
             }
@@ -541,6 +612,7 @@ public class AtlasTypeRegistry {
                 updateTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 updateTypesWithNoRefResolve(typesDef.getEntityDefs());
                 updateTypesWithNoRefResolve(typesDef.getRelationshipDefs());
+                updateTypesWithNoRefResolve(typesDef.getBusinessMetadataDefs());
             }
 
             if (LOG.isDebugEnabled()) {
@@ -555,9 +627,10 @@ public class AtlasTypeRegistry {
                 removeTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 removeTypesWithNoRefResolve(typesDef.getEntityDefs());
                 removeTypesWithNoRefResolve(typesDef.getRelationshipDefs());
-
-                resolveReferences();
+                removeTypesWithNoRefResolve(typesDef.getBusinessMetadataDefs());
             }
+
+            resolveReferences();
         }
 
         private void removeTypesWithNoRefResolve(Collection<? extends AtlasBaseTypeDef> typeDefs) {
@@ -589,6 +662,9 @@ public class AtlasTypeRegistry {
                 case RELATIONSHIP:
                     registryData.relationshipDefs.removeTypeDefByName(typeDef.getName());
                     break;
+                case BUSINESS_METADATA:
+                    registryData.businessMetadataDefs.removeTypeDefByName(typeDef.getName());
+                    break;
             }
             deletedTypes.add(typeDef);
         }
@@ -610,6 +686,9 @@ public class AtlasTypeRegistry {
                 case RELATIONSHIP:
                     registryData.relationshipDefs.removeTypeDefByGuid(typeDef.getGuid());
                     break;
+                case BUSINESS_METADATA:
+                    registryData.businessMetadataDefs.removeTypeDefByGuid(typeDef.getGuid());
+                    break;
             }
             deletedTypes.add(typeDef);
         }
@@ -619,16 +698,16 @@ public class AtlasTypeRegistry {
                 LOG.debug("==> AtlasTypeRegistry.removeTypeByGuid({})", guid);
             }
 
+            AtlasBaseTypeDef typeDef = getTypeDefByGuid(guid);
+
             if (guid != null) {
-                AtlasBaseTypeDef typeDef = getTypeDefByGuid(guid);
-
                 registryData.removeByGuid(guid);
+            }
 
-                resolveReferences();
+            resolveReferences();
 
-                if (typeDef != null) {
-                    deletedTypes.add(typeDef);
-                }
+            if (typeDef != null) {
+                deletedTypes.add(typeDef);
             }
 
             if (LOG.isDebugEnabled()) {
@@ -641,16 +720,16 @@ public class AtlasTypeRegistry {
                 LOG.debug("==> AtlasTypeRegistry.removeTypeByName({})", name);
             }
 
+            AtlasBaseTypeDef typeDef = getTypeDefByName(name);
+
             if (name != null) {
-                AtlasBaseTypeDef typeDef = getTypeDefByName(name);
-
                 registryData.removeByName(name);
+            }
 
-                resolveReferences();
+            resolveReferences();
 
-                if (typeDef != null) {
-                    deletedTypes.add(typeDef);
-                }
+            if (typeDef != null) {
+                deletedTypes.add(typeDef);
             }
 
             if (LOG.isDebugEnabled()) {
@@ -671,10 +750,6 @@ public class AtlasTypeRegistry {
             }
 
             if (typeDef != null) {
-                if (this.isRegisteredType(typeDef.getName())) {
-                    throw new AtlasBaseException(AtlasErrorCode.TYPE_ALREADY_EXISTS, typeDef.getName());
-                }
-
                 if (typeDef.getClass().equals(AtlasEnumDef.class)) {
                     AtlasEnumDef enumDef = (AtlasEnumDef) typeDef;
 
@@ -696,6 +771,9 @@ public class AtlasTypeRegistry {
                     AtlasRelationshipDef relationshipDef = (AtlasRelationshipDef) typeDef;
 
                     registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
+                } else if (typeDef.getClass().equals(AtlasBusinessMetadataDef.class)) {
+                    AtlasBusinessMetadataDef businessMetadataDef = (AtlasBusinessMetadataDef) typeDef;
+                    registryData.businessMetadataDefs.addType(businessMetadataDef, new AtlasBusinessMetadataType(businessMetadataDef));
                 }
 
                 addedTypes.add(typeDef);
@@ -775,6 +853,11 @@ public class AtlasTypeRegistry {
 
                     registryData.relationshipDefs.removeTypeDefByGuid(guid);
                     registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
+                } else if (typeDef.getClass().equals(AtlasBusinessMetadataDef.class)) {
+                    AtlasBusinessMetadataDef businessMetadataDef = (AtlasBusinessMetadataDef) typeDef;
+
+                    registryData.businessMetadataDefs.removeTypeDefByGuid(guid);
+                    registryData.businessMetadataDefs.addType(businessMetadataDef, new AtlasBusinessMetadataType(businessMetadataDef));
                 }
 
                 updatedTypes.add(typeDef);
@@ -817,6 +900,11 @@ public class AtlasTypeRegistry {
 
                     registryData.relationshipDefs.removeTypeDefByName(name);
                     registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
+                } else if (typeDef.getClass().equals(AtlasBusinessMetadataDef.class)) {
+                    AtlasBusinessMetadataDef businessMetadataDef = (AtlasBusinessMetadataDef) typeDef;
+
+                    registryData.businessMetadataDefs.removeTypeDefByName(name);
+                    registryData.businessMetadataDefs.addType(businessMetadataDef, new AtlasBusinessMetadataType(businessMetadataDef));
                 }
 
                 updatedTypes.add(typeDef);
@@ -906,6 +994,10 @@ public class AtlasTypeRegistry {
                                 new Exception().fillInStackTrace());
                     } else if (typeRegistryUpdateLock.getHoldCount() == 1) {
                         if (ttr != null && commitUpdates) {
+                            // copy indexName for attributes from current typeRegistry to new one
+                            copyIndexNameFromCurrent(ttr.getAllEntityTypes());
+                            copyIndexNameFromCurrent(ttr.getAllBusinessMetadataTypes());
+
                             typeRegistry.registryData = ttr.registryData;
                         }
                     }
@@ -927,6 +1019,34 @@ public class AtlasTypeRegistry {
             LOG.debug("<== releaseTypeRegistryForUpdate()");
         }
 
+        private void copyIndexNameFromCurrent(Collection<? extends AtlasStructType> ttrTypes) {
+            for (AtlasStructType ttrType : ttrTypes) {
+                final AtlasStructType currType;
+
+                if (ttrType instanceof AtlasEntityType) {
+                    currType = typeRegistry.getEntityTypeByName(ttrType.getTypeName());
+                } else if (ttrType instanceof AtlasBusinessMetadataType) {
+                    currType = typeRegistry.getBusinessMetadataTypeByName(ttrType.getTypeName());
+                } else {
+                    currType = null;
+                }
+
+                if (currType == null) { // ttrType could be a new type introduced
+                    continue;
+                }
+
+                for (AtlasAttribute ttrAttribute : ttrType.getAllAttributes().values()) {
+                    if (StringUtils.isEmpty(ttrAttribute.getIndexFieldName())) {
+                        AtlasAttribute currAttribute = currType.getAttribute(ttrAttribute.getName());
+
+                        if (currAttribute != null) {
+                            ttrAttribute.setIndexFieldName(currAttribute.getIndexFieldName());
+                        }
+                    }
+                }
+            }
+
+        }
     }
 }
 
@@ -1161,4 +1281,5 @@ class TypeDefCache<T1 extends AtlasBaseTypeDef, T2 extends AtlasType> {
         typeDefNameMap.clear();
         typeNameMap.clear();
     }
+
 }
