@@ -18,15 +18,25 @@
 
 package org.apache.atlas.kafka;
 
-public class AtlasKafkaMessage<T> {
-    private final T    message;
-    private final long offset;
-    private final int  partition;
+import org.apache.kafka.common.TopicPartition;
 
-    public AtlasKafkaMessage(T message, long offset, int partition) {
-        this.message   = message;
-        this.offset    = offset;
-        this.partition = partition;
+public class AtlasKafkaMessage<T> {
+    private final T              message;
+    private final long           offset;
+    private final TopicPartition topicPartition;
+    private final boolean        spooled;
+    private final long           msgCreated;
+
+    public AtlasKafkaMessage(T message, long offset, String topic, int partition, long msgCreated, boolean spooled) {
+        this.message        = message;
+        this.offset         = offset;
+        this.topicPartition = new TopicPartition(topic, partition);
+        this.msgCreated     = msgCreated;
+        this.spooled        = spooled;
+    }
+
+    public AtlasKafkaMessage(T message, long offset, String topic, int partition) {
+        this(message, offset, topic, partition, 0, false);
     }
 
     public T getMessage() {
@@ -37,8 +47,23 @@ public class AtlasKafkaMessage<T> {
         return offset;
     }
 
-    public int getPartition() {
-        return partition;
+    public TopicPartition getTopicPartition() {
+        return topicPartition;
     }
 
+    public String getTopic() {
+        return topicPartition.topic();
+    }
+
+    public int getPartition() {
+        return topicPartition.partition();
+    }
+
+    public boolean getSpooled() {
+        return this.spooled;
+    }
+
+    public long getMsgCreated() {
+        return this.msgCreated;
+    }
 }
