@@ -59,6 +59,14 @@ public interface NotificationInterface {
         }
     }
 
+    /**
+     *
+     * @param source: Name of the source
+     * @param failedMessagesLogger: Logger for failed messages
+     * @return
+     */
+    void init(String source, Object failedMessagesLogger);
+
     /**
      *
      * @param user Name of the user under which the processes is running
@@ -102,4 +110,13 @@ public interface NotificationInterface {
      * Shutdown any notification producers and consumers associated with this interface instance.
      */
     void close();
+
+    /**
+     *  Check if underlying notification mechanism is ready for use.
+     *
+     * @param type tye message type
+     * @return true if available, false otherwise
+     *
+     */
+    boolean isReady(NotificationType type);
 }
