@@ -112,7 +112,7 @@ public class AbstractNotificationTest {
         }
 
         @Override
-        protected void sendInternal(NotificationType notificationType, List<String> notificationMessages)
+        public void sendInternal(NotificationType notificationType, List<String> notificationMessages)
             throws NotificationException {
 
             type     = notificationType;
@@ -127,5 +127,10 @@ public class AbstractNotificationTest {
         @Override
         public void close() {
         }
+
+        @Override
+        public boolean isReady(NotificationType type) {
+            return true;
+        }
     }
 }
