@@ -23,9 +23,12 @@ import org.aopalliance.intercept.MethodInvocation;
 import org.apache.atlas.annotation.GraphTransaction;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.exception.NotFoundException;
+import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.repository.graphdb.AtlasGraph;
 import org.apache.atlas.repository.graphdb.AtlasVertex;
+import org.apache.atlas.tasks.TaskManagement;
 import org.apache.atlas.utils.AtlasPerfMetrics.MetricRecorder;
+import org.apache.commons.collections.CollectionUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Component;
@@ -52,11 +55,37 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
     private static final ThreadLocal<Boolean>                   innerFailure               = ThreadLocal.withInitial(() -> Boolean.FALSE);
     private static final ThreadLocal<Map<String, AtlasVertex>>  guidVertexCache            = ThreadLocal.withInitial(() -> new HashMap<>());
 
-    private final AtlasGraph graph;
+    private final AtlasGraph     graph;
+    private final TaskManagement taskManagement;
+
+    private static final ThreadLocal<Map<Object, String>> vertexGuidCache =
+            new ThreadLocal<Map<Object, String>>() {
+                @Override
+                public Map<Object, String> initialValue() {
+                    return new HashMap<Object, String>();
+                }
+            };
+
+    private static final ThreadLocal<Map<Object, AtlasEntity.Status>> vertexStateCache =
+            new ThreadLocal<Map<Object, AtlasEntity.Status>>() {
+                @Override
+                public Map<Object, AtlasEntity.Status> initialValue() {
+                    return new HashMap<Object, AtlasEntity.Status>();
+                }
+            };
+
+    private static final ThreadLocal<Map<Object, AtlasEntity.Status>> edgeStateCache =
+            new ThreadLocal<Map<Object, AtlasEntity.Status>>() {
+                @Override
+                public Map<Object, AtlasEntity.Status> initialValue() {
+                    return new HashMap<Object, AtlasEntity.Status>();
+                }
+            };
 
     @Inject
-    public GraphTransactionInterceptor(AtlasGraph graph) {
-        this.graph = graph;
+    public GraphTransactionInterceptor(AtlasGraph graph, TaskManagement taskManagement) {
+        this.graph          = graph;
+        this.taskManagement = taskManagement;
     }
 
     @Override
@@ -64,7 +93,7 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
         Method        method            = invocation.getMethod();
         String        invokingClass     = method.getDeclaringClass().getSimpleName();
         String        invokedMethodName = method.getName();
-        boolean       logRollback       = method.getAnnotation(GraphTransaction.class).logRollback();
+        boolean       logRollback       = method.getAnnotation(GraphTransaction.class) == null || method.getAnnotation(GraphTransaction.class).logRollback();
 
         boolean isInnerTxn = isTxnOpen.get();
         // Outermost txn marks any subsequent transaction as inner
@@ -117,7 +146,7 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
                 // Reset the boolean flags
                 isTxnOpen.set(Boolean.FALSE);
                 innerFailure.set(Boolean.FALSE);
-                guidVertexCache.get().clear();
+                clearCache();
 
                 List<PostTransactionHook> trxHooks = postTransactionHooks.get();
 
@@ -139,6 +168,10 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
             }
 
             OBJECT_UPDATE_SYNCHRONIZER.releaseLockedObjects();
+
+            if (isSuccess) {
+                submitTasks();
+            }
         }
     }
 
@@ -201,6 +234,17 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
 
     public static void clearCache() {
         guidVertexCache.get().clear();
+        vertexGuidCache.get().clear();
+        vertexStateCache.get().clear();
+        edgeStateCache.get().clear();
+    }
+
+    private void submitTasks() {
+        if (CollectionUtils.isEmpty(RequestContext.get().getQueuedTasks()) || taskManagement == null) {
+            return;
+        }
+
+        taskManagement.addAll(RequestContext.get().getQueuedTasks());
     }
 
     boolean logException(Throwable t) {
@@ -214,6 +258,72 @@ public class GraphTransactionInterceptor implements MethodInterceptor {
         }
     }
 
+    public static void addToVertexGuidCache(Object vertexId, String guid) {
+
+        if (guid == null) {
+            removeFromVertexGuidCache(vertexId);
+        } else {
+            Map<Object, String> cache = vertexGuidCache.get();
+            cache.put(vertexId, guid);
+        }
+    }
+
+    public static void removeFromVertexGuidCache(Object vertexId) {
+        Map<Object, String> cache = vertexGuidCache.get();
+
+        cache.remove(vertexId);
+    }
+
+    public static String getVertexGuidFromCache(Object vertexId) {
+        Map<Object, String> cache = vertexGuidCache.get();
+
+        return cache.get(vertexId);
+    }
+
+    public static void addToVertexStateCache(Object vertexId, AtlasEntity.Status status) {
+
+        if (status == null) {
+            removeFromVertexStateCache(vertexId);
+        } else {
+            Map<Object, AtlasEntity.Status> cache = vertexStateCache.get();
+            cache.put(vertexId, status);
+        }
+    }
+
+    public static void removeFromVertexStateCache(Object vertexId) {
+        Map<Object, AtlasEntity.Status> cache = vertexStateCache.get();
+
+        cache.remove(vertexId);
+    }
+
+    public static AtlasEntity.Status getVertexStateFromCache(Object vertexId) {
+        Map<Object, AtlasEntity.Status> cache = vertexStateCache.get();
+
+        return cache.get(vertexId);
+    }
+
+    public static void addToEdgeStateCache(Object edgeId, AtlasEntity.Status status) {
+
+        if (status == null) {
+            removeFromEdgeStateCache(edgeId);
+        } else {
+            Map<Object, AtlasEntity.Status> cache = edgeStateCache.get();
+            cache.put(edgeId, status);
+        }
+    }
+
+    public static void removeFromEdgeStateCache(Object edgeId) {
+        Map<Object, AtlasEntity.Status> cache = edgeStateCache.get();
+
+        cache.remove(edgeId);
+    }
+
+    public static AtlasEntity.Status getEdgeStateFromCache(Object edgeId) {
+        Map<Object, AtlasEntity.Status> cache = edgeStateCache.get();
+
+        return cache.get(edgeId);
+    }
+
     public static abstract class PostTransactionHook {
         protected PostTransactionHook() {
             List<PostTransactionHook> trxHooks = postTransactionHooks.get();
