@@ -18,16 +18,16 @@
 package org.apache.atlas.discovery;
 
 import org.apache.atlas.SortOrder;
-import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.discovery.SearchParameters.FilterCriteria;
+import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.repository.Constants;
 import org.apache.atlas.repository.graphdb.*;
 import org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2;
 import org.apache.atlas.type.AtlasClassificationType;
-import org.apache.atlas.util.AtlasGremlinQueryProvider;
 import org.apache.atlas.util.SearchPredicateUtil;
 import org.apache.atlas.utils.AtlasPerfTracer;
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.collections.MapUtils;
 import org.apache.commons.collections.Predicate;
 import org.apache.commons.collections.PredicateUtils;
 import org.apache.commons.lang3.StringUtils;
@@ -35,8 +35,6 @@ import org.apache.tinkerpop.gremlin.process.traversal.Order;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import javax.script.ScriptEngine;
-import javax.script.ScriptException;
 import java.util.*;
 
 /**
@@ -52,10 +50,9 @@ public class ClassificationSearchProcessor extends SearchProcessor {
     private final AtlasIndexQuery        indexQuery;
     private final AtlasIndexQuery        classificationIndexQuery;
     private final AtlasGraphQuery        tagGraphQueryWithAttributes;
-    private final Map<String, Object>    gremlinQueryBindings;
-    private final String                 gremlinTagFilterQuery;
     private final Predicate              traitPredicate;
     private final Predicate              isEntityPredicate;
+    private       Predicate              activePredicate;
 
     // Some index engines may take space as a delimiter, when basic search is
     // executed, unsatisfying results may be returned.
@@ -67,17 +64,16 @@ public class ClassificationSearchProcessor extends SearchProcessor {
     public ClassificationSearchProcessor(SearchContext context) {
         super(context);
 
-        final AtlasClassificationType classificationType    = context.getClassificationType();
         final FilterCriteria          filterCriteria        = context.getSearchParameters().getTagFilters();
         final Set<String>             indexAttributes       = new HashSet<>();
         final Set<String>             graphAttributes       = new HashSet<>();
         final Set<String>             allAttributes         = new HashSet<>();
-        final Set<String>             typeAndSubTypes       = context.getClassificationTypes();
+        final Set<String>             typeAndSubTypes       = context.getClassificationTypeNames();
         final String                  typeAndSubTypesQryStr = context.getClassificationTypesQryStr();
-        final boolean isBuiltInType                         = context.isBuiltInClassificationType();
         final boolean isWildcardSearch                      = context.isWildCardSearch();
+        final Set<AtlasClassificationType> classificationTypes = context.getClassificationTypes();
 
-        processSearchAttributes(classificationType, filterCriteria, indexAttributes, graphAttributes, allAttributes);
+        processSearchAttributes(classificationTypes, filterCriteria, indexAttributes, graphAttributes, allAttributes);
 
         /* for classification search, if any attribute can't be handled by index query - switch to all filter by Graph query
            There are four cases in the classification type :
@@ -87,21 +83,35 @@ public class ClassificationSearchProcessor extends SearchProcessor {
            4. classification is not present in the search parameter
            each of above cases with either has empty/or not tagFilters
          */
-        final boolean useIndexSearchForEntity = (classificationType != null || isWildcardSearch) &&
+        final boolean useIndexSearchForEntity = (CollectionUtils.isNotEmpty(classificationTypes) || isWildcardSearch) &&
                                                 !context.hasAttributeFilter(filterCriteria)  &&
                                                 (typeAndSubTypesQryStr.length() <= MAX_QUERY_STR_LENGTH_TAGS);
 
         /* If classification's attributes can be applied index filter, we can use direct index
          * to query classification index as well.
          */
-        final boolean useIndexSearchForClassification = (!isBuiltInType && !isWildcardSearch) &&
+        final boolean useIndexSearchForClassification = (CollectionUtils.isNotEmpty(classificationTypes) &&
+                                                         classificationTypes.iterator().next() != SearchContext.MATCH_ALL_NOT_CLASSIFIED &&
+                                                          !isWildcardSearch) &&
                                                         (typeAndSubTypesQryStr.length() <= MAX_QUERY_STR_LENGTH_TAGS) &&
                                                         CollectionUtils.isNotEmpty(indexAttributes) &&
-                                                        canApplyIndexFilter(classificationType, filterCriteria, false);
+                                                        canApplyIndexFilter(classificationTypes, filterCriteria, false);
 
-        traitPredicate    = buildTraitPredict(classificationType);
+        final boolean useGraphSearchForClassification = (CollectionUtils.isNotEmpty(classificationTypes) &&
+                                                        classificationTypes.iterator().next() != SearchContext.MATCH_ALL_NOT_CLASSIFIED &&
+                                                        !isWildcardSearch && CollectionUtils.isNotEmpty(graphAttributes));
+
+        traitPredicate    = buildTraitPredict(classificationTypes);
         isEntityPredicate = SearchPredicateUtil.generateIsEntityVertexPredicate(context.getTypeRegistry());
 
+        if (context.getSearchParameters().getExcludeDeletedEntities()) {
+            activePredicate = SearchPredicateUtil.getEQPredicateGenerator()
+                    .generatePredicate(Constants.STATE_PROPERTY_KEY, AtlasEntity.Status.ACTIVE.name(), String.class);
+        }
+
+        Predicate attributePredicate = null;
+        Predicate typeNamePredicate  = null;
+
         AtlasGraph graph = context.getGraph();
 
         // index query directly on entity
@@ -115,8 +125,7 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                 // tagFilters is not allowed in wildcard search
                 graphIndexQueryBuilder.addClassificationTypeFilter(queryString);
             } else {
-                if (isBuiltInType) {
-
+                if (classificationTypes.iterator().next() == SearchContext.MATCH_ALL_NOT_CLASSIFIED) {
                     // tagFilters is not allowed in unique classificationType search
                     graphIndexQueryBuilder.addClassificationFilterForBuiltInTypes(queryString);
 
@@ -146,65 +155,46 @@ public class ClassificationSearchProcessor extends SearchProcessor {
             graphIndexQueryBuilder.addActiveStateQueryFilter(queryString);
             graphIndexQueryBuilder.addTypeAndSubTypesQueryFilter(queryString, typeAndSubTypesQryStr);
 
-            constructFilterQuery(queryString, classificationType, filterCriteria, indexAttributes);
+            constructFilterQuery(queryString, classificationTypes, filterCriteria, indexAttributes);
 
             String indexQueryString = STRAY_AND_PATTERN.matcher(queryString).replaceAll(")");
             indexQueryString = STRAY_OR_PATTERN.matcher(indexQueryString).replaceAll(")");
             indexQueryString = STRAY_ELIPSIS_PATTERN.matcher(indexQueryString).replaceAll("");
 
-            Predicate typeNamePredicate  = isClassificationRootType() ? null : SearchPredicateUtil.getINPredicateGenerator().generatePredicate(Constants.TYPE_NAME_PROPERTY_KEY, typeAndSubTypes, String.class);
-
-            if (typeNamePredicate != null) {
-                inMemoryPredicate = inMemoryPredicate == null ? typeNamePredicate : PredicateUtils.andPredicate(inMemoryPredicate, typeNamePredicate);
-            }
-
-            Predicate attributePredicate = constructInMemoryPredicate(classificationType, filterCriteria, indexAttributes);
-
-            if (attributePredicate != null) {
-                inMemoryPredicate = inMemoryPredicate == null ? attributePredicate : PredicateUtils.andPredicate(inMemoryPredicate, attributePredicate);
-            }
-
             this.classificationIndexQuery = graph.indexQuery(Constants.VERTEX_INDEX, indexQueryString);
+
+            typeNamePredicate  = isClassificationRootType() ? null :
+                                 SearchPredicateUtil.getINPredicateGenerator().generatePredicate(Constants.TYPE_NAME_PROPERTY_KEY, typeAndSubTypes, String.class);
+            attributePredicate = constructInMemoryPredicate(classificationTypes, filterCriteria, indexAttributes);
+
         } else {
             classificationIndexQuery = null;
         }
 
         // only registered classification will search with tag filters
-        if (!isWildcardSearch && !isBuiltInType && !graphAttributes.isEmpty()) {
+        if (useGraphSearchForClassification) {
 
-            AtlasGremlinQueryProvider queryProvider = AtlasGremlinQueryProvider.INSTANCE;
             AtlasGraphQuery query = graph.query();
 
             if (!isClassificationRootType()) {
                 query.in(Constants.TYPE_NAME_PROPERTY_KEY, typeAndSubTypes);
             }
 
-            tagGraphQueryWithAttributes = toGraphFilterQuery(classificationType, filterCriteria, allAttributes, query);
-            gremlinQueryBindings       = new HashMap<>();
-            StringBuilder gremlinQuery = new StringBuilder();
+            tagGraphQueryWithAttributes = toGraphFilterQuery(classificationTypes, filterCriteria, allAttributes, query);
 
-            gremlinQuery.append("g.V().has('__guid', within(guids))");
-            gremlinQuery.append(queryProvider.getQuery(AtlasGremlinQueryProvider.AtlasGremlinQuery.BASIC_SEARCH_CLASSIFICATION_FILTER));
-            gremlinQuery.append(".as('e').filter(out()");
-            gremlinQuery.append(queryProvider.getQuery(AtlasGremlinQueryProvider.AtlasGremlinQuery.BASIC_SEARCH_TYPE_FILTER));
+            typeNamePredicate = isClassificationRootType() ? null :
+                                SearchPredicateUtil.getINPredicateGenerator().generatePredicate(Constants.TYPE_NAME_PROPERTY_KEY, typeAndSubTypes, String.class);
+            attributePredicate = constructInMemoryPredicate(classificationTypes, filterCriteria, allAttributes);
 
-            constructGremlinFilterQuery(gremlinQuery, gremlinQueryBindings, context.getClassificationType(), context.getSearchParameters().getTagFilters());
-
-            // After filtering on tags go back to e and output the list of entity vertices
-            gremlinQuery.append(").toList()");
-
-            gremlinQueryBindings.put("traitNames", typeAndSubTypes);
-            gremlinQueryBindings.put("typeNames", typeAndSubTypes); // classification typeName
-
-            gremlinTagFilterQuery = gremlinQuery.toString();
-
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("gremlinTagFilterQuery={}", gremlinTagFilterQuery);
-            }
         } else {
             tagGraphQueryWithAttributes = null;
-            gremlinTagFilterQuery = null;
-            gremlinQueryBindings = null;
+        }
+
+        if (typeNamePredicate != null) {
+            inMemoryPredicate = inMemoryPredicate == null ? typeNamePredicate : PredicateUtils.andPredicate(inMemoryPredicate, typeNamePredicate);
+        }
+        if (attributePredicate != null) {
+            inMemoryPredicate = inMemoryPredicate == null ? attributePredicate : PredicateUtils.andPredicate(inMemoryPredicate, attributePredicate);
         }
     }
 
@@ -223,26 +213,32 @@ public class ClassificationSearchProcessor extends SearchProcessor {
         }
 
         try {
-            final int     startIdx   = context.getSearchParameters().getOffset();
             final int     limit      = context.getSearchParameters().getLimit();
+                  Integer marker     = context.getMarker();
 
             // query to start at 0, even though startIdx can be higher - because few results in earlier retrieval could
             // have been dropped: like non-active-entities or duplicate-entities (same entity pointed to by multiple
             // classifications in the result)
             //
             // first 'startIdx' number of entries will be ignored
-            int qryOffset = 0;
+            //marker functionality will not work when there is need to fetch classificationVertices and get entities from it
+            if (indexQuery == null) {
+                marker = null;
+            }
+            // if marker is provided, start query with marker offset
+            int startIdx  = marker != null ? marker : context.getSearchParameters().getOffset();
+            int qryOffset = marker != null ? marker : 0;
             int resultIdx = qryOffset;
 
-            final Set<String>       processedGuids         = new HashSet<>();
-            final List<AtlasVertex> entityVertices         = new ArrayList<>();
-            final List<AtlasVertex> classificationVertices = new ArrayList<>();
+            final Set<String>                   processedGuids         = new HashSet<>();
+            LinkedHashMap<Integer, AtlasVertex> offsetEntityVertexMap  = new LinkedHashMap<>();
+            final List<AtlasVertex>             classificationVertices = new ArrayList<>();
 
             final String          sortBy                = context.getSearchParameters().getSortBy();
             final SortOrder       sortOrder             = context.getSearchParameters().getSortOrder();
 
             for (; ret.size() < limit; qryOffset += limit) {
-                entityVertices.clear();
+                offsetEntityVertexMap.clear();
                 classificationVertices.clear();
 
                 if (context.terminateSearch()) {
@@ -262,23 +258,15 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                         queryResult = indexQuery.vertices(qryOffset, limit);
                     }
 
-                    getVerticesFromIndexQueryResult(queryResult, entityVertices);
-                    isLastResultPage = entityVertices.size() < limit;
+                    offsetEntityVertexMap   = getVerticesFromIndexQueryResult(queryResult, offsetEntityVertexMap, qryOffset);
+                    isLastResultPage        = offsetEntityVertexMap.size() < limit;
 
                     // Do in-memory filtering
-                    CollectionUtils.filter(entityVertices, traitPredicate);
-                    CollectionUtils.filter(entityVertices, isEntityPredicate);
+                    offsetEntityVertexMap = super.filter(offsetEntityVertexMap, traitPredicate);
+                    offsetEntityVertexMap = super.filter(offsetEntityVertexMap, isEntityPredicate);
 
                 } else {
-                    if (tagGraphQueryWithAttributes != null) {
-
-                        Iterator<AtlasVertex> queryResult = tagGraphQueryWithAttributes.vertices(qryOffset, limit).iterator();
-
-                        getVertices(queryResult, classificationVertices);
-
-                        isLastResultPage = classificationVertices.size() < limit;
-
-                    } else if (classificationIndexQuery != null){
+                    if (classificationIndexQuery != null) {
 
                         Iterator<AtlasIndexQuery.Result> queryResult = classificationIndexQuery.vertices(qryOffset, limit);
 
@@ -286,7 +274,15 @@ public class ClassificationSearchProcessor extends SearchProcessor {
 
                         isLastResultPage = classificationVertices.size() < limit;
 
-                        // Do in-memory filtering before the graph query
+                        CollectionUtils.filter(classificationVertices, inMemoryPredicate);
+                    } else if (tagGraphQueryWithAttributes != null) {
+
+                        Iterator<AtlasVertex> queryResult = tagGraphQueryWithAttributes.vertices(qryOffset, limit).iterator();
+
+                        getVertices(queryResult, classificationVertices);
+
+                        isLastResultPage = classificationVertices.size() < limit;
+
                         CollectionUtils.filter(classificationVertices, inMemoryPredicate);
                     }
                 }
@@ -294,11 +290,14 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                 // Since tag filters are present, we need to collect the entity vertices after filtering the classification
                 // vertex results (as these might be lower in number)
                 if (CollectionUtils.isNotEmpty(classificationVertices)) {
+                    int resultCount = 0;
+
                     for (AtlasVertex classificationVertex : classificationVertices) {
                         Iterable<AtlasEdge> edges = classificationVertex.getEdges(AtlasEdgeDirection.IN, Constants.CLASSIFICATION_LABEL);
 
                         for (AtlasEdge edge : edges) {
                             AtlasVertex entityVertex = edge.getOutVertex();
+                            resultCount++;
 
                             String guid = AtlasGraphUtilsV2.getIdFromVertex(entityVertex);
 
@@ -306,7 +305,7 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                                 continue;
                             }
 
-                            entityVertices.add(entityVertex);
+                            offsetEntityVertexMap.put((qryOffset + resultCount) - 1, entityVertex);
 
                             processedGuids.add(guid);
                         }
@@ -314,19 +313,28 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                 }
 
                 if (whiteSpaceFilter) {
-                    filterWhiteSpaceClassification(entityVertices);
+                    offsetEntityVertexMap = filterWhiteSpaceClassification(offsetEntityVertexMap);
+                }
+                // Do in-memory filtering
+                offsetEntityVertexMap = super.filter(offsetEntityVertexMap, isEntityPredicate);
+                if (activePredicate != null) {
+                    offsetEntityVertexMap = super.filter(offsetEntityVertexMap, activePredicate);
                 }
-                    // Do in-memory filtering
-                CollectionUtils.filter(entityVertices, isEntityPredicate);
 
-                super.filter(entityVertices);
+                offsetEntityVertexMap = super.filter(offsetEntityVertexMap);
 
-                resultIdx = collectResultVertices(ret, startIdx, limit, resultIdx, entityVertices);
+                resultIdx = collectResultVertices(ret, startIdx, limit, resultIdx, offsetEntityVertexMap, marker);
 
                 if (isLastResultPage) {
+                    resultIdx = SearchContext.MarkerUtil.MARKER_END - 1;
                     break;
                 }
             }
+
+            if (marker != null) {
+                nextOffset = resultIdx + 1;
+            }
+
         } finally {
             AtlasPerfTracer.log(perf);
         }
@@ -339,42 +347,23 @@ public class ClassificationSearchProcessor extends SearchProcessor {
     }
 
     @Override
-    public void filter(List<AtlasVertex> entityVertices) {
+    public LinkedHashMap<Integer, AtlasVertex> filter(LinkedHashMap<Integer, AtlasVertex> offsetEntityVertexMap) {
         if (LOG.isDebugEnabled()) {
-            LOG.debug("==> ClassificationSearchProcessor.filter({})", entityVertices.size());
+            LOG.debug("==> ClassificationSearchProcessor.filter({})", offsetEntityVertexMap.size());
         }
-        //in case of classification type + graph attributes
-        if (gremlinTagFilterQuery != null && gremlinQueryBindings != null) {
-            // Now filter on the tag attributes
-            Set<String> guids = getGuids(entityVertices);
 
-            // Clear prior results
-            entityVertices.clear();
-
-            if (CollectionUtils.isNotEmpty(guids)) {
-                gremlinQueryBindings.put("guids", guids);
-
-                try {
-                    AtlasGraph        graph               = context.getGraph();
-                    ScriptEngine      gremlinScriptEngine = graph.getGremlinScriptEngine();
-                    List<AtlasVertex> atlasVertices       = (List<AtlasVertex>) graph.executeGremlinScript(gremlinScriptEngine, gremlinQueryBindings, gremlinTagFilterQuery, false);
-
-                    if (CollectionUtils.isNotEmpty(atlasVertices)) {
-                        entityVertices.addAll(atlasVertices);
-                    }
-                } catch (AtlasBaseException | ScriptException e) {
-                    LOG.warn(e.getMessage(), e);
-                }
-            }
-        } else if (inMemoryPredicate != null) {
+        if (inMemoryPredicate != null) {
             //in case of classification type + index attributes
-            CollectionUtils.filter(entityVertices, traitPredicate);
+            offsetEntityVertexMap = super.filter(offsetEntityVertexMap, traitPredicate);
 
             //filter attributes (filterCriteria). Find classification vertex(typeName = classification) from entity vertex (traitName = classification)
             final Set<String> processedGuids = new HashSet<>();
-            List<AtlasVertex> matchEntityVertices = new ArrayList<>();
-            if (CollectionUtils.isNotEmpty(entityVertices)) {
-                for (AtlasVertex entityVertex : entityVertices) {
+            LinkedHashMap<Integer, AtlasVertex> matchEntityVertices = new LinkedHashMap<>();
+
+            if (MapUtils.isNotEmpty(offsetEntityVertexMap)) {
+                for (Map.Entry<Integer, AtlasVertex> offsetToEntity : offsetEntityVertexMap.entrySet()) {
+
+                    AtlasVertex entityVertex  = offsetToEntity.getValue();
                     Iterable<AtlasEdge> edges = entityVertex.getEdges(AtlasEdgeDirection.OUT, Constants.CLASSIFICATION_LABEL);
 
                     for (AtlasEdge edge : edges) {
@@ -388,7 +377,7 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                                 continue;
                             }
 
-                            matchEntityVertices.add(entityVertex);
+                            matchEntityVertices.put(offsetToEntity.getKey(), entityVertex);
                             processedGuids.add(guid);
                             break;
 
@@ -396,20 +385,22 @@ public class ClassificationSearchProcessor extends SearchProcessor {
                     }
                 }
             }
-            entityVertices.clear();
-            entityVertices.addAll(matchEntityVertices);
+            offsetEntityVertexMap.clear();
+            offsetEntityVertexMap.putAll(matchEntityVertices);
 
         } else {
             //in case of only classsification type
-            CollectionUtils.filter(entityVertices, traitPredicate);
-            CollectionUtils.filter(entityVertices, isEntityPredicate);
+            offsetEntityVertexMap = super.filter(offsetEntityVertexMap, traitPredicate);
+            offsetEntityVertexMap = super.filter(offsetEntityVertexMap, isEntityPredicate);
         }
 
-        super.filter(entityVertices);
+        offsetEntityVertexMap = super.filter(offsetEntityVertexMap);
 
         if (LOG.isDebugEnabled()) {
-            LOG.debug("<== ClassificationSearchProcessor.filter(): ret.size()={}", entityVertices.size());
+            LOG.debug("<== ClassificationSearchProcessor.filter(): ret.size()={}", offsetEntityVertexMap.size());
         }
+
+        return offsetEntityVertexMap;
     }
 
     @Override
