@@ -20,6 +20,8 @@ package org.apache.atlas.glossary;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.SortOrder;
 import org.apache.atlas.annotation.GraphTransaction;
+import org.apache.atlas.bulkimport.BulkImportResponse;
+import org.apache.atlas.bulkimport.BulkImportResponse.ImportInfo;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.glossary.AtlasGlossary;
 import org.apache.atlas.model.glossary.AtlasGlossaryCategory;
@@ -35,6 +37,7 @@ import org.apache.atlas.repository.store.graph.v2.AtlasEntityChangeNotifier;
 import org.apache.atlas.repository.store.graph.v2.AtlasGraphUtilsV2;
 import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.util.FileUtils;
+import org.apache.atlas.utils.AtlasJson;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
@@ -42,7 +45,6 @@ import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Service;
 
 import javax.inject.Inject;
-import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -54,7 +56,11 @@ import java.util.Objects;
 import java.util.Set;
 import java.util.stream.Collectors;
 
-import static org.apache.atlas.glossary.GlossaryUtils.*;
+import static org.apache.atlas.bulkimport.BulkImportResponse.ImportStatus.FAILED;
+import static org.apache.atlas.bulkimport.BulkImportResponse.ImportStatus.SUCCESS;
+import static org.apache.atlas.glossary.GlossaryUtils.getAtlasGlossaryCategorySkeleton;
+import static org.apache.atlas.glossary.GlossaryUtils.getAtlasGlossaryTermSkeleton;
+import static org.apache.atlas.glossary.GlossaryUtils.getGlossarySkeleton;
 
 @Service
 public class GlossaryService {
@@ -149,7 +155,7 @@ public class GlossaryService {
         }
 
         if (glossaryExists(atlasGlossary)) {
-            throw new AtlasBaseException(AtlasErrorCode.GLOSSARY_ALREADY_EXISTS, atlasGlossary.getQualifiedName());
+            throw new AtlasBaseException(AtlasErrorCode.GLOSSARY_ALREADY_EXISTS, atlasGlossary.getName());
         }
 
         AtlasGlossary storeObject = dataAccess.save(atlasGlossary);
@@ -389,8 +395,12 @@ public class GlossaryService {
         return ret;
     }
 
-    @GraphTransaction
     public AtlasGlossaryTerm updateTerm(AtlasGlossaryTerm atlasGlossaryTerm) throws AtlasBaseException {
+        return updateTerm(atlasGlossaryTerm, true);
+    }
+
+    @GraphTransaction
+    public AtlasGlossaryTerm updateTerm(AtlasGlossaryTerm atlasGlossaryTerm, boolean ignoreUpdateIfTermExists) throws AtlasBaseException {
         if (DEBUG_ENABLED) {
             LOG.debug("==> GlossaryService.updateTerm({})", atlasGlossaryTerm);
         }
@@ -407,16 +417,20 @@ public class GlossaryService {
             throw new AtlasBaseException(AtlasErrorCode.INVALID_DISPLAY_NAME);
         }
 
+        if (ignoreUpdateIfTermExists) {
+            String qualifiedName = getDuplicateGlossaryRelatedTerm(atlasGlossaryTerm);
+
+            if (StringUtils.isNotEmpty(qualifiedName)) {
+                throw new AtlasBaseException(AtlasErrorCode.GLOSSARY_TERM_ALREADY_EXISTS, qualifiedName);
+            }
+        }
+
         AtlasGlossaryTerm storeObject = dataAccess.load(atlasGlossaryTerm);
         if (!storeObject.equals(atlasGlossaryTerm)) {
-            try {
-                atlasGlossaryTerm.setGuid(storeObject.getGuid());
-                atlasGlossaryTerm.setQualifiedName(storeObject.getQualifiedName());
+            atlasGlossaryTerm.setGuid(storeObject.getGuid());
+            atlasGlossaryTerm.setQualifiedName(storeObject.getQualifiedName());
 
-                storeObject = dataAccess.save(atlasGlossaryTerm);
-            } catch (AtlasBaseException e) {
-                LOG.debug("Glossary term had no immediate attr updates. Exception: {}", e.getMessage());
-            }
+            storeObject = dataAccess.save(atlasGlossaryTerm);
 
             glossaryTermUtils.processTermRelations(storeObject, atlasGlossaryTerm, GlossaryUtils.RelationshipOperation.UPDATE);
 
@@ -1034,6 +1048,29 @@ public class GlossaryService {
         return StringUtils.containsAny(name, invalidNameChars);
     }
 
+    private String getDuplicateGlossaryRelatedTerm(AtlasGlossaryTerm atlasGlossaryTerm) throws AtlasBaseException {
+        Map<AtlasGlossaryTerm.Relation, Set<AtlasRelatedTermHeader>> relatedTermsMap = atlasGlossaryTerm.getRelatedTerms();
+        for (Map.Entry<AtlasGlossaryTerm.Relation, Set<AtlasRelatedTermHeader>> relatedTermsMapEntry : relatedTermsMap.entrySet()) {
+            Set<AtlasRelatedTermHeader> termHeaders = relatedTermsMapEntry.getValue();
+
+            if (CollectionUtils.isNotEmpty(termHeaders)) {
+                List<AtlasRelatedTermHeader> duplicateTermHeaders = termHeaders.stream()
+                        .collect(Collectors.groupingBy(AtlasRelatedTermHeader::getTermGuid))
+                        .values().stream()
+                        .filter(duplicates -> duplicates.size() > 1)
+                        .flatMap(Collection::stream)
+                        .collect(Collectors.toList());
+
+                if (CollectionUtils.isNotEmpty(duplicateTermHeaders) && duplicateTermHeaders.size() > 0) {
+                    String dupTermGuid = duplicateTermHeaders.get(0).getTermGuid();
+                    AtlasGlossaryTerm glossaryTerm = getTerm(dupTermGuid);
+                    return glossaryTerm.getQualifiedName();
+                }
+            }
+        }
+        return StringUtils.EMPTY;
+    }
+
     private String getDisplayText(AtlasGlossaryTerm term) {
         return term != null ? term.getName() : null;
     }
@@ -1081,37 +1118,83 @@ public class GlossaryService {
         }
     }
 
-    public List<AtlasGlossaryTerm> importGlossaryData(InputStream inputStream, String fileName) throws AtlasBaseException {
-        List<AtlasGlossaryTerm> ret;
+    @GraphTransaction
+    public BulkImportResponse importGlossaryData(InputStream inputStream, String fileName) throws AtlasBaseException {
+        BulkImportResponse ret = new BulkImportResponse();
+
+        if (StringUtils.isBlank(fileName)) {
+            throw new AtlasBaseException(AtlasErrorCode.INVALID_FILE_TYPE, fileName);
+        }
 
         try {
-            if (StringUtils.isBlank(fileName)) {
-                throw new AtlasBaseException(AtlasErrorCode.INVALID_FILE_TYPE, fileName);
-            }
+            List<String[]> fileData = FileUtils.readFileData(fileName, inputStream);
 
-            List<String[]> fileData       = FileUtils.readFileData(fileName, inputStream);
-            List<String>   failedTermMsgs = new ArrayList<>();
+            List<AtlasGlossaryTerm> glossaryTermsWithoutRelations = glossaryTermUtils.getGlossaryTermDataWithoutRelations(fileData, ret);
+            createGlossaryTerms(glossaryTermsWithoutRelations, ret);
 
-            ret = glossaryTermUtils.getGlossaryTermDataList(fileData, failedTermMsgs);
-            ret = createGlossaryTerms(ret);
-        } catch (IOException e) {
-            throw new AtlasBaseException(AtlasErrorCode.FAILED_TO_UPLOAD, e);
+            List<AtlasGlossaryTerm> glossaryTermsWithRelations = glossaryTermUtils.getGlossaryTermDataWithRelations(fileData, ret);
+            updateGlossaryTermsRelation(glossaryTermsWithRelations, ret);
+        } finally {
+            glossaryTermUtils.clearImportCache();
         }
 
         return ret;
     }
 
-    private List<AtlasGlossaryTerm> createGlossaryTerms(List<AtlasGlossaryTerm> glossaryTerms) throws AtlasBaseException {
-        List<AtlasGlossaryTerm> ret = new ArrayList<>();
-
+    private void createGlossaryTerms(List<AtlasGlossaryTerm> glossaryTerms, BulkImportResponse bulkImportResponse) throws AtlasBaseException {
         for (AtlasGlossaryTerm glossaryTerm : glossaryTerms) {
+            String glossaryTermName = glossaryTerm.getName();
+            String glossaryName     = getGlossaryName(glossaryTerm);
+
             try {
-                ret.add(createTerm(glossaryTerm));
+                AtlasGlossaryTerm createdTerm = createTerm(glossaryTerm);
+
+                bulkImportResponse.addToSuccessImportInfoList(new ImportInfo(glossaryName, glossaryTermName, SUCCESS, AtlasJson.toJson(createdTerm.getGlossaryTermHeader())));
             } catch (AtlasBaseException e) {
-                throw new AtlasBaseException(AtlasErrorCode.FAILED_TO_CREATE_GLOSSARY_TERM, e);
+                LOG.error(AtlasErrorCode.FAILED_TO_CREATE_GLOSSARY_TERM.toString(), glossaryTermName, e);
+
+                bulkImportResponse.addToFailedImportInfoList(new ImportInfo(glossaryName, glossaryTermName, FAILED, e.getMessage()));
             }
         }
 
+        checkForSuccessImports(bulkImportResponse);
+    }
+
+    private void updateGlossaryTermsRelation(List<AtlasGlossaryTerm> glossaryTerms, BulkImportResponse bulkImportResponse) {
+        for (AtlasGlossaryTerm glossaryTerm : glossaryTerms) {
+            glossaryTermUtils.updateGlossaryTermRelations(glossaryTerm);
+
+            if (glossaryTerm.hasTerms()) {
+                String glossaryTermName = glossaryTerm.getName();
+                String glossaryName     = getGlossaryName(glossaryTerm);
+
+                try {
+                    updateTerm(glossaryTerm, false);
+                } catch (AtlasBaseException e) {
+                    LOG.error(AtlasErrorCode.FAILED_TO_UPDATE_GLOSSARY_TERM.toString(), glossaryTermName, e);
+
+                    bulkImportResponse.addToFailedImportInfoList(new ImportInfo(glossaryName, glossaryTermName, FAILED, e.getMessage()));
+                }
+            }
+        }
+    }
+
+    private String getGlossaryName(AtlasGlossaryTerm glossaryTerm) {
+        String ret               = "";
+        String glossaryTermQName = glossaryTerm.getQualifiedName();
+
+        if (StringUtils.isNotBlank(glossaryTermQName)){
+            String[] glossaryQnameSplit = glossaryTermQName.split("@");
+
+            ret = (glossaryQnameSplit.length == 2) ? glossaryQnameSplit[1] : "";
+        }
+
         return ret;
     }
+
+    private void checkForSuccessImports(BulkImportResponse bulkImportResponse) throws AtlasBaseException {
+        if (CollectionUtils.isEmpty(bulkImportResponse.getSuccessImportInfoList())) {
+            throw new AtlasBaseException(AtlasErrorCode.GLOSSARY_IMPORT_FAILED);
+        }
+    }
 }
