@@ -18,6 +18,8 @@
 package org.apache.atlas.glossary;
 
 import org.apache.atlas.AtlasErrorCode;
+import org.apache.atlas.bulkimport.BulkImportResponse;
+import org.apache.atlas.bulkimport.BulkImportResponse.ImportInfo;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.glossary.AtlasGlossary;
 import org.apache.atlas.model.glossary.AtlasGlossaryTerm;
@@ -37,6 +39,7 @@ import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.util.FileUtils;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
+import org.apache.commons.lang.ArrayUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -46,6 +49,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
@@ -53,10 +57,19 @@ import java.util.Objects;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import static org.apache.atlas.bulkimport.BulkImportResponse.ImportStatus.FAILED;
+
 public class GlossaryTermUtils extends GlossaryUtils {
     private static final Logger  LOG           = LoggerFactory.getLogger(GlossaryTermUtils.class);
     private static final boolean DEBUG_ENABLED = LOG.isDebugEnabled();
 
+    private static final int INDEX_FOR_GLOSSARY_AT_RECORD = 0;
+    private static final int INDEX_FOR_TERM_AT_RECORD     = 1;
+
+    private static final ThreadLocal<Map<String, String>>  glossaryNameGuidCache      = ThreadLocal.withInitial(() -> new LinkedHashMap<>());
+    private static final ThreadLocal<Map<String, Integer>> glossaryTermOrderCache     = ThreadLocal.withInitial(() -> new HashMap<>());
+    private static final ThreadLocal<Map<String, String>>  glossaryTermQNameGuidCache = ThreadLocal.withInitial(() -> new HashMap<>());
+
     protected GlossaryTermUtils(AtlasRelationshipStore relationshipStore, AtlasTypeRegistry typeRegistry, DataAccess dataAccess) {
         super(relationshipStore, typeRegistry, dataAccess);
     }
@@ -138,6 +151,12 @@ public class GlossaryTermUtils extends GlossaryUtils {
         }
     }
 
+    public void clearImportCache() {
+        glossaryNameGuidCache.get().clear();
+        glossaryTermOrderCache.get().clear();
+        glossaryTermQNameGuidCache.get().clear();
+    }
+
     private boolean isRelationshipGuidSame(AtlasRelatedObjectId storeObject, AtlasRelatedObjectId relatedObjectId) {
         return StringUtils.equals(relatedObjectId.getRelationshipGuid(), storeObject.getRelationshipGuid());
     }
@@ -529,56 +548,90 @@ public class GlossaryTermUtils extends GlossaryUtils {
         }
     }
 
-    protected List<AtlasGlossaryTerm> getGlossaryTermDataList(List<String[]> fileData, List<String> failedTermMsgs) throws AtlasBaseException {
-        List<AtlasGlossaryTerm> glossaryTerms     = new ArrayList<>();
-        Map<String, String>     glossaryNameCache = new HashMap<>();
+    protected List<AtlasGlossaryTerm> getGlossaryTermDataWithoutRelations(List<String[]> fileData, BulkImportResponse bulkImportResponse) throws AtlasBaseException {
+        List<AtlasGlossaryTerm> glossaryTerms = new ArrayList<>();
+        int                     rowCount      = 1;
 
         for (String[] record : fileData) {
-            AtlasGlossaryTerm glossaryTerm = new AtlasGlossaryTerm();
+            List<String>      failedTermMsgs = new ArrayList<>();
+            AtlasGlossaryTerm glossaryTerm   = new AtlasGlossaryTerm();
+            String            glossaryName   = StringUtils.EMPTY;
 
-            if ((record.length < 1) || StringUtils.isBlank(record[0])) {
+            if (ArrayUtils.isEmpty(record) || StringUtils.isBlank(record[INDEX_FOR_GLOSSARY_AT_RECORD])) {
                 LOG.error("The GlossaryName is blank for the record : ", Arrays.toString(record));
+
                 failedTermMsgs.add("The GlossaryName is blank for the record : " + Arrays.toString(record));
-            }
-
-            String glossaryName = record[0];
-            String glossaryGuid;
-
-            if (glossaryNameCache.get(glossaryName) != null) {
-                glossaryGuid = glossaryNameCache.get(glossaryName);
-
             } else {
-                AtlasVertex vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TYPENAME, GlossaryUtils.ATLAS_GLOSSARY_TYPENAME + "." + QUALIFIED_NAME_ATTR, glossaryName);
+                glossaryName = record[INDEX_FOR_GLOSSARY_AT_RECORD];
 
-                glossaryGuid = (vertex != null) ? AtlasGraphUtilsV2.getIdFromVertex(vertex) : null;
-            }
+                String glossaryGuid = glossaryNameGuidCache.get().get(glossaryName);
 
-            if (glossaryGuid == null) {
-                if (GlossaryService.isNameInvalid(glossaryName)) {
-                    LOG.error("The provided Glossary Name is invalid : " + glossaryName);
-                    failedTermMsgs.add("The provided Glossary Name is invalid : " + glossaryName);
-                } else {
-                    AtlasGlossary glossary = new AtlasGlossary();
-                    glossary.setQualifiedName(glossaryName);
-                    glossary.setName(glossaryName);
+                if (StringUtils.isEmpty(glossaryGuid)) {
+                    glossaryGuid = getGlossaryGUIDFromGraphDB(glossaryName);
 
-                    glossary      = dataAccess.save(glossary);
-                    glossaryGuid  = glossary.getGuid();
+                    if (StringUtils.isEmpty(glossaryGuid)) {
+                        glossaryGuid = createGlossary(glossaryName, failedTermMsgs);
+                    }
+
+                    glossaryNameGuidCache.get().put(glossaryName, glossaryGuid);
+                }
+
+                if (StringUtils.isNotEmpty(glossaryGuid)) {
+                    glossaryTerm = populateGlossaryTermObject(failedTermMsgs, record, glossaryGuid, false);
+
+                    glossaryTerm.setQualifiedName(getGlossaryTermQualifiedName(glossaryTerm.getName(), glossaryName));
+
+                    glossaryTermOrderCache.get().put(glossaryTerm.getQualifiedName(), rowCount);
+
+                    glossaryTerms.add(glossaryTerm);
                 }
             }
 
-            if (glossaryGuid != null) {
-                glossaryNameCache.put(glossaryName, glossaryGuid);
-                glossaryTerm = populateGlossaryTermObject(failedTermMsgs, record, glossaryGuid);
-                glossaryTerms.add(glossaryTerm);
+            if (failedTermMsgs.size() > 0) {
+                String failedTermMsg    = StringUtils.join(failedTermMsgs, System.lineSeparator());
+                String glossaryTermName = glossaryTerm.getName();
+
+                bulkImportResponse.addToFailedImportInfoList(new ImportInfo(glossaryName, glossaryTermName, FAILED, failedTermMsg, rowCount));
             }
+
+            rowCount++;
         }
 
-        if (failedTermMsgs.size() == 0) {
-            return glossaryTerms;
-        } else {
-            throw new AtlasBaseException("The uploaded file has not been processed due to the following errors : " + "\n" + failedTermMsgs.toString());
+        return glossaryTerms;
+    }
+
+    protected List<AtlasGlossaryTerm> getGlossaryTermDataWithRelations(List<String[]> fileData, BulkImportResponse bulkImportResponse) throws AtlasBaseException {
+        List<AtlasGlossaryTerm> glossaryTerms = new ArrayList<>();
+        int                     rowCount      = 1;
+
+        for (String[] record : fileData) {
+            List<String>      failedTermMsgs = new ArrayList<>();
+
+            if (ArrayUtils.isNotEmpty(record) && StringUtils.isNotBlank(record[INDEX_FOR_GLOSSARY_AT_RECORD])) {
+                AtlasGlossaryTerm glossaryTerm = new AtlasGlossaryTerm();
+                String            glossaryName = record[INDEX_FOR_GLOSSARY_AT_RECORD];
+                String            glossaryGuid = glossaryNameGuidCache.get().get(glossaryName);
+
+                if (StringUtils.isNotEmpty(glossaryGuid)) {
+                    glossaryTerm = populateGlossaryTermObject(failedTermMsgs, record, glossaryGuid, true);
+
+                    glossaryTerm.setQualifiedName(getGlossaryTermQualifiedName(glossaryTerm.getName(), glossaryName));
+
+                    glossaryTerms.add(glossaryTerm);
+                }
+
+                if (failedTermMsgs.size() > 0) {
+                    String failedTermMsg    = StringUtils.join(failedTermMsgs, System.lineSeparator());
+                    String glossaryTermName = glossaryTerm.getName();
+
+                    bulkImportResponse.addToFailedImportInfoList(new ImportInfo(glossaryName, glossaryTermName, FAILED, failedTermMsg, rowCount));
+                }
+            }
+
+            rowCount++;
         }
+
+        return glossaryTerms;
     }
 
     public static String getGlossaryTermHeaders() {
@@ -609,7 +662,20 @@ public class GlossaryTermUtils extends GlossaryUtils {
         return String.join(", ", ret);
     }
 
-    protected Map getMapValue(String csvRecord, List<String> failedTermMsgs) {
+    public void updateGlossaryTermRelations(AtlasGlossaryTerm updatedGlossaryTerm) {
+        if (glossaryTermQNameGuidCache.get().containsKey(updatedGlossaryTerm.getQualifiedName())) {
+            try {
+                AtlasGlossaryTerm glossaryTermFromDB = getGlossaryTem(glossaryTermQNameGuidCache.get().get(updatedGlossaryTerm.getQualifiedName()));
+                copyRelations(updatedGlossaryTerm, glossaryTermFromDB);
+            } catch (AtlasBaseException e) {
+                if (DEBUG_ENABLED) {
+                    LOG.debug("Error occurred while loading glossary Term", e);
+                }
+            }
+        }
+    }
+
+    protected Map getMapValue(String csvRecord, List<String> failedTermMsgs, boolean populateRelations) {
         Map ret = null;
 
         if (StringUtils.isNotBlank(csvRecord)) {
@@ -622,8 +688,9 @@ public class GlossaryTermUtils extends GlossaryUtils {
 
                 if ((recordArray.length % 2) == 0) {
                     ret.put(recordArray[0], recordArray[1]);
-                } else {
-                    failedTermMsgs.add("\n" + "The Data in the uploaded file is incorrectly specified  : " + csvRecord);
+                } else if (!populateRelations) {
+                    failedTermMsgs.add("The Data in the uploaded file is incorrectly specified  : " + csvRecord
+                            + System.lineSeparator() + "AdditionalAttributes needs to be a key:value pair");
                 }
             }
         }
@@ -645,48 +712,64 @@ public class GlossaryTermUtils extends GlossaryUtils {
         Set ret = null;
 
         if (StringUtils.isNotBlank(csvRecord)) {
-            ret                                     = new HashSet();
-            String                 csvRecordArray[] = csvRecord.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER);
+            ret                     = new HashSet();
+            String csvRecordArray[] = csvRecord.split(FileUtils.ESCAPE_CHARACTER + FileUtils.PIPE_CHARACTER);
+
             AtlasRelatedTermHeader relatedTermHeader;
 
             for (String data : csvRecordArray) {
                 AtlasVertex vertex      = null;
                 String      dataArray[] = data.split(FileUtils.ESCAPE_CHARACTER + FileUtils.COLON_CHARACTER);
 
-                if ((dataArray.length % 2) == 0) {
-                    vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME,
-                            GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME + invalidNameChars[1] + QUALIFIED_NAME_ATTR, dataArray[1] + invalidNameChars[0] + dataArray[0]);
-                } else {
-                    failedTermMsgs.add("\n" + "Either incorrect data specified for Term or Term does not exist : " +termName);
-                }
+                if (dataArray.length == 2) {
+                    String relatedTermQualifiedName = dataArray[1] + invalidNameChars[0] + dataArray[0];
+                    String currTermQualifiedName    = termName + invalidNameChars[0] + glossaryName;
 
-                if (vertex != null) {
-                    String glossaryTermGuid = AtlasGraphUtilsV2.getIdFromVertex(vertex);
-                    relatedTermHeader       = new AtlasRelatedTermHeader();
-                    relatedTermHeader.setTermGuid(glossaryTermGuid);
-                    ret.add(relatedTermHeader);
+                    if (relatedTermQualifiedName.equalsIgnoreCase(currTermQualifiedName)) {
+                        failedTermMsgs.add("Invalid relationship specified for Term. Term cannot have a relationship with self");
+                    } else {
+                        vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME,
+                                GlossaryUtils.ATLAS_GLOSSARY_TERM_TYPENAME + invalidNameChars[1] + QUALIFIED_NAME_ATTR, relatedTermQualifiedName);
+
+                        if (vertex != null) {
+                            String glossaryTermGuid = AtlasGraphUtilsV2.getIdFromVertex(vertex);
+
+                            relatedTermHeader = new AtlasRelatedTermHeader();
+                            relatedTermHeader.setTermGuid(glossaryTermGuid);
+
+                            cacheRelatedTermQNameGuid(currTermQualifiedName, relatedTermQualifiedName, glossaryTermGuid);
+
+                            ret.add(relatedTermHeader);
+                        } else {
+                            failedTermMsgs.add("The provided Reference " + dataArray[1] + "@" + dataArray[0] +
+                                    " does not exist at Atlas referred at record with TermName  : " + termName + " and GlossaryName : " + glossaryName);
+                        }
+                    }
                 } else {
-                    failedTermMsgs.add("\n" + "The provided Reference Glossary and TermName does not exist in the system " +
-                            dataArray[1] + FileUtils.COLON_CHARACTER + dataArray[0] + " for record with TermName  : " + termName + " and GlossaryName : " + glossaryName);
+                    failedTermMsgs.add("Incorrect relation data specified for the term : " + termName + "@" + glossaryName);
                 }
             }
-
-            return ret;
         }
 
         return ret;
     }
 
-    protected AtlasGlossaryTerm populateGlossaryTermObject(List<String> failedTermMsgList, String[] record, String glossaryGuid) {
-        AtlasGlossaryTerm ret    = new AtlasGlossaryTerm();
-        int               i      = 0;
+    protected AtlasGlossaryTerm populateGlossaryTermObject(List<String> failedTermMsgList, String[] record, String glossaryGuid, boolean populateRelations) {
         int               length = record.length;
+        int               i      = INDEX_FOR_TERM_AT_RECORD;
+        AtlasGlossaryTerm ret    = new AtlasGlossaryTerm();
 
-        ret.setName((length > ++i) ? record[i] : null);
+        if (length > i) {
+            ret.setName(record[i]);
+        }
 
-        if (!StringUtils.isNotBlank(ret.getName())) {
-            failedTermMsgList.add("\n" + "The TermName is blank for provided record: " + Arrays.toString(record));
+        if (StringUtils.isBlank(ret.getName())) {
+            if (!populateRelations) {
+                failedTermMsgList.add("The TermName is blank for provided record: " + Arrays.toString(record));
+            }
         } else {
+            ret.setAnchor(new AtlasGlossaryHeader(glossaryGuid));
+
             ret.setShortDescription((length > ++i) ? record[i] : null);
 
             ret.setLongDescription((length > ++i) ? record[i] : null);
@@ -697,37 +780,199 @@ public class GlossaryTermUtils extends GlossaryUtils {
 
             ret.setUsage((length > ++i) ? record[i] : null);
 
-            ret.setAdditionalAttributes(((length > ++i) ? (Map<String, Object>) getMapValue(record[i], failedTermMsgList) : null));
+            ret.setAdditionalAttributes(((length > ++i) ? (Map<String, Object>) getMapValue(record[i], failedTermMsgList, populateRelations) : null));
 
-            ret.setTranslationTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+            if (populateRelations) {
+                ret.setTranslationTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setValidValuesFor((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setValidValuesFor((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setSynonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setSynonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setReplacedBy((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setReplacedBy((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setValidValues((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setValidValues((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setReplacementTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setReplacementTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setSeeAlso((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setSeeAlso((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setTranslatedTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setTranslatedTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setIsA((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setIsA((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setAnchor(new AtlasGlossaryHeader(glossaryGuid));
+                ret.setAntonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setAntonyms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setClassifies((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setClassifies((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setPreferredToTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
 
-            ret.setPreferredToTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
-
-            ret.setPreferredTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[0], failedTermMsgList) : null);
+                ret.setPreferredTerms((length > ++i) ? (Set<AtlasRelatedTermHeader>) getAtlasRelatedTermHeaderSet(record[i], ret.getName(), record[INDEX_FOR_GLOSSARY_AT_RECORD], failedTermMsgList) : null);
+            }
         }
 
         return ret;
     }
+
+    private String getGlossaryTermQualifiedName(String glossaryTermName, String glossaryName) throws AtlasBaseException {
+        return glossaryTermName + "@" + glossaryName;
+    }
+
+    private String getGlossaryGUIDFromGraphDB(String glossaryName) {
+        AtlasVertex vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(GlossaryUtils.ATLAS_GLOSSARY_TYPENAME, GlossaryUtils.ATLAS_GLOSSARY_TYPENAME + "." + QUALIFIED_NAME_ATTR, glossaryName);
+
+        return (vertex != null) ? AtlasGraphUtilsV2.getIdFromVertex(vertex) : null;
+    }
+
+    private String createGlossary(String glossaryName, List<String> failedTermMsgs) throws AtlasBaseException {
+        String ret = null;
+
+        if (GlossaryService.isNameInvalid(glossaryName)) {
+            LOG.error("The provided Glossary Name is invalid : " + glossaryName);
+
+            failedTermMsgs.add("The provided Glossary Name {" + glossaryName +  "} is invalid : " + AtlasErrorCode.INVALID_DISPLAY_NAME.getFormattedErrorMessage());
+        } else {
+            AtlasGlossary glossary = new AtlasGlossary();
+            glossary.setQualifiedName(glossaryName);
+            glossary.setName(glossaryName);
+
+            glossary = dataAccess.save(glossary);
+            ret      = glossary.getGuid();
+        }
+
+        return ret;
+    }
+
+    private void cacheRelatedTermQNameGuid(String currTermQualifiedName, String relatedTermQualifiedName, String termGuid) {
+        if (!glossaryTermQNameGuidCache.get().containsKey(relatedTermQualifiedName) &&
+                glossaryTermOrderCache.get().containsKey(currTermQualifiedName) &&
+                glossaryTermOrderCache.get().containsKey(relatedTermQualifiedName) &&
+                glossaryTermOrderCache.get().get(currTermQualifiedName) < glossaryTermOrderCache.get().get(relatedTermQualifiedName)) {
+            glossaryTermQNameGuidCache.get().put(relatedTermQualifiedName, termGuid);
+        }
+    }
+
+    private AtlasGlossaryTerm getGlossaryTem(String termGuid) throws AtlasBaseException {
+        AtlasGlossaryTerm ret = null;
+
+        if (DEBUG_ENABLED) {
+            LOG.debug("==> GlossaryTemUtils.getGlossaryTem({})", termGuid);
+        }
+
+        if (!Objects.isNull(termGuid)) {
+            AtlasGlossaryTerm atlasGlossary = getAtlasGlossaryTermSkeleton(termGuid);
+            ret = dataAccess.load(atlasGlossary);
+
+            if (DEBUG_ENABLED) {
+                LOG.debug("<== GlossaryTemUtils.getGlossaryTem() : {}", ret);
+            }
+        }
+        return ret;
+    }
+
+    private void copyRelations(AtlasGlossaryTerm toGlossaryTerm, AtlasGlossaryTerm fromGlossaryTerm) {
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getSeeAlso())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getSeeAlso())) {
+                toGlossaryTerm.getSeeAlso().addAll(fromGlossaryTerm.getSeeAlso());
+            } else {
+                toGlossaryTerm.setSeeAlso(fromGlossaryTerm.getSeeAlso());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getSynonyms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getSynonyms())) {
+                toGlossaryTerm.getSynonyms().addAll(fromGlossaryTerm.getSynonyms());
+            } else {
+                toGlossaryTerm.setSynonyms(fromGlossaryTerm.getSynonyms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getAntonyms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getAntonyms())) {
+                toGlossaryTerm.getAntonyms().addAll(fromGlossaryTerm.getAntonyms());
+            } else {
+                toGlossaryTerm.setAntonyms(fromGlossaryTerm.getAntonyms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getPreferredTerms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getPreferredTerms())) {
+                toGlossaryTerm.getPreferredTerms().addAll(fromGlossaryTerm.getPreferredTerms());
+            } else {
+                toGlossaryTerm.setPreferredTerms(fromGlossaryTerm.getPreferredTerms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getPreferredToTerms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getPreferredToTerms())) {
+                toGlossaryTerm.getPreferredToTerms().addAll(fromGlossaryTerm.getPreferredToTerms());
+            } else {
+                toGlossaryTerm.setPreferredToTerms(fromGlossaryTerm.getPreferredToTerms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getReplacementTerms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getReplacementTerms())) {
+                toGlossaryTerm.getReplacementTerms().addAll(fromGlossaryTerm.getReplacementTerms());
+            } else {
+                toGlossaryTerm.setReplacementTerms(fromGlossaryTerm.getReplacementTerms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getReplacedBy())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getReplacedBy())) {
+                toGlossaryTerm.getReplacedBy().addAll(fromGlossaryTerm.getReplacedBy());
+            } else {
+                toGlossaryTerm.setReplacedBy(fromGlossaryTerm.getReplacedBy());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getTranslationTerms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getTranslationTerms())) {
+                toGlossaryTerm.getTranslationTerms().addAll(fromGlossaryTerm.getTranslationTerms());
+            } else {
+                toGlossaryTerm.setTranslationTerms(fromGlossaryTerm.getTranslationTerms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getTranslatedTerms())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getTranslatedTerms())) {
+                toGlossaryTerm.getTranslatedTerms().addAll(fromGlossaryTerm.getTranslatedTerms());
+            } else {
+                toGlossaryTerm.setTranslatedTerms(fromGlossaryTerm.getTranslatedTerms());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getIsA())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getIsA())) {
+                toGlossaryTerm.getIsA().addAll(fromGlossaryTerm.getIsA());
+            } else {
+                toGlossaryTerm.setIsA(fromGlossaryTerm.getIsA());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getClassifies())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getClassifies())) {
+                toGlossaryTerm.getClassifies().addAll(fromGlossaryTerm.getClassifies());
+            } else {
+                toGlossaryTerm.setClassifies(fromGlossaryTerm.getClassifies());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getValidValues())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getValidValues())) {
+                toGlossaryTerm.getValidValues().addAll(fromGlossaryTerm.getValidValues());
+            } else {
+                toGlossaryTerm.setValidValues(fromGlossaryTerm.getValidValues());
+            }
+        }
+
+        if (CollectionUtils.isNotEmpty(fromGlossaryTerm.getValidValuesFor())) {
+            if (CollectionUtils.isNotEmpty(toGlossaryTerm.getValidValuesFor())) {
+                toGlossaryTerm.getValidValuesFor().addAll(fromGlossaryTerm.getValidValuesFor());
+            } else {
+                toGlossaryTerm.setValidValuesFor(fromGlossaryTerm.getValidValuesFor());
+            }
+        }
+    }
 }
