@@ -24,6 +24,7 @@ import com.google.common.collect.HashBiMap;
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.AtlasException;
+import org.apache.atlas.GraphTransactionInterceptor;
 import org.apache.atlas.RequestContext;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.instance.AtlasEntity;
@@ -815,8 +816,17 @@ public final class GraphHelper {
         return element.getProperty(Constants.RELATIONSHIP_GUID_PROPERTY_KEY, String.class);
     }
 
-    public static String getGuid(AtlasElement element) {
-        return element.<String>getProperty(Constants.GUID_PROPERTY_KEY, String.class);
+    public static String getGuid(AtlasVertex vertex) {
+        Object vertexId = vertex.getId();
+        String ret = GraphTransactionInterceptor.getVertexGuidFromCache(vertexId);
+
+        if (ret == null) {
+            ret = vertex.<String>getProperty(Constants.GUID_PROPERTY_KEY, String.class);
+
+            GraphTransactionInterceptor.addToVertexGuidCache(vertexId, ret);
+        }
+
+        return ret;
     }
 
     public static String getHomeId(AtlasElement element) {
@@ -870,10 +880,33 @@ public final class GraphHelper {
         return element.getProperty(STATE_PROPERTY_KEY, String.class);
     }
 
-    public static Status getStatus(AtlasElement element) {
-        return (getState(element) == Id.EntityState.DELETED) ? Status.DELETED : Status.ACTIVE;
+    public static Status getStatus(AtlasVertex vertex) {
+        Object vertexId = vertex.getId();
+        Status ret = GraphTransactionInterceptor.getVertexStateFromCache(vertexId);
+
+        if (ret == null) {
+            ret = (getState(vertex) == Id.EntityState.DELETED) ? Status.DELETED : Status.ACTIVE;
+
+            GraphTransactionInterceptor.addToVertexStateCache(vertexId, ret);
+        }
+
+        return ret;
     }
 
+    public static Status getStatus(AtlasEdge edge) {
+        Object edgeId = edge.getId();
+        Status ret = GraphTransactionInterceptor.getEdgeStateFromCache(edgeId);
+
+        if (ret == null) {
+            ret = (getState(edge) == Id.EntityState.DELETED) ? Status.DELETED : Status.ACTIVE;
+
+            GraphTransactionInterceptor.addToEdgeStateCache(edgeId, ret);
+        }
+
+        return ret;
+    }
+
+
     public static AtlasRelationship.Status getEdgeStatus(AtlasElement element) {
         return (getState(element) == Id.EntityState.DELETED) ? AtlasRelationship.Status.DELETED : AtlasRelationship.Status.ACTIVE;
     }
@@ -1572,6 +1605,27 @@ public final class GraphHelper {
         return ret;
     }
 
+    //get entity type of relationship (End vertex entity type) from relationship label
+    public static String getReferencedEntityTypeName(AtlasVertex entityVertex, String relation) {
+        String ret = null;
+        Iterator<AtlasEdge> edges    = GraphHelper.getAdjacentEdgesByLabel(entityVertex, AtlasEdgeDirection.BOTH, relation);
+
+        if (edges != null && edges.hasNext()) {
+            AtlasEdge   relationEdge = edges.next();
+            AtlasVertex outVertex    = relationEdge.getOutVertex();
+            AtlasVertex inVertex     = relationEdge.getInVertex();
+
+            if (outVertex != null && inVertex != null) {
+                String outVertexId    = outVertex.getIdForDisplay();
+                String entityVertexId = entityVertex.getIdForDisplay();
+                AtlasVertex endVertex = StringUtils.equals(outVertexId, entityVertexId) ? inVertex : outVertex;
+                ret                   = GraphHelper.getTypeName(endVertex);
+            }
+        }
+
+       return ret;
+    }
+
     public static boolean isRelationshipEdge(AtlasEdge edge) {
         if (edge == null) {
             return false;
