@@ -18,7 +18,9 @@
 package org.apache.atlas.repository.store.graph.v2;
 
 
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.atlas.AtlasErrorCode;
+import org.apache.atlas.DeleteType;
 import org.apache.atlas.GraphTransactionInterceptor;
 import org.apache.atlas.RequestContext;
 import org.apache.atlas.annotation.GraphTransaction;
@@ -48,7 +50,7 @@ import org.apache.atlas.repository.store.graph.AtlasEntityStore;
 import org.apache.atlas.repository.store.graph.EntityGraphDiscovery;
 import org.apache.atlas.repository.store.graph.EntityGraphDiscoveryContext;
 import org.apache.atlas.repository.store.graph.v1.DeleteHandlerDelegate;
-import org.apache.atlas.store.DeleteType;
+import org.apache.atlas.repository.store.graph.v2.AtlasEntityComparator.AtlasEntityDiffResult;
 import org.apache.atlas.type.AtlasArrayType;
 import org.apache.atlas.type.AtlasBusinessMetadataType.AtlasBusinessAttribute;
 import org.apache.atlas.type.AtlasClassificationType;
@@ -59,6 +61,7 @@ import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.type.AtlasTypeUtil;
 import org.apache.atlas.bulkimport.BulkImportResponse;
+import org.apache.atlas.bulkimport.BulkImportResponse.ImportInfo;
 import org.apache.atlas.util.FileUtils;
 import org.apache.atlas.utils.AtlasEntityUtil;
 import org.apache.atlas.utils.AtlasPerfMetrics.MetricRecorder;
@@ -85,11 +88,12 @@ import java.util.Objects;
 import java.util.Set;
 
 import static java.lang.Boolean.FALSE;
+import static org.apache.atlas.AtlasConfiguration.STORE_DIFFERENTIAL_AUDITS;
+import static org.apache.atlas.bulkimport.BulkImportResponse.ImportStatus.FAILED;
 import static org.apache.atlas.model.instance.EntityMutations.EntityOperation.DELETE;
 import static org.apache.atlas.model.instance.EntityMutations.EntityOperation.PURGE;
 import static org.apache.atlas.model.instance.EntityMutations.EntityOperation.UPDATE;
 import static org.apache.atlas.repository.Constants.IS_INCOMPLETE_PROPERTY_KEY;
-import static org.apache.atlas.repository.graph.GraphHelper.getCustomAttributes;
 import static org.apache.atlas.repository.graph.GraphHelper.getTypeName;
 import static org.apache.atlas.repository.graph.GraphHelper.isEntityIncomplete;
 import static org.apache.atlas.repository.store.graph.v2.EntityGraphMapper.validateLabels;
@@ -100,12 +104,13 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
     private static final Logger LOG = LoggerFactory.getLogger(AtlasEntityStoreV2.class);
     private static final Logger PERF_LOG = AtlasPerfTracer.getPerfLogger("store.EntityStore");
 
-    private final AtlasGraph                graph;
-    private final DeleteHandlerDelegate     deleteDelegate;
-    private final AtlasTypeRegistry         typeRegistry;
+    private final AtlasGraph                 graph;
+    private final DeleteHandlerDelegate      deleteDelegate;
+    private final AtlasTypeRegistry          typeRegistry;
     private final IAtlasEntityChangeNotifier entityChangeNotifier;
-    private final EntityGraphMapper         entityGraphMapper;
-    private final EntityGraphRetriever      entityRetriever;
+    private final EntityGraphMapper          entityGraphMapper;
+    private final EntityGraphRetriever       entityRetriever;
+    private       boolean                    storeDifferentialAudits;
 
     @Inject
     public AtlasEntityStoreV2(AtlasGraph graph, DeleteHandlerDelegate deleteDelegate, AtlasTypeRegistry typeRegistry,
@@ -116,6 +121,12 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         this.entityChangeNotifier = entityChangeNotifier;
         this.entityGraphMapper    = entityGraphMapper;
         this.entityRetriever      = new EntityGraphRetriever(graph, typeRegistry);
+        this.storeDifferentialAudits = STORE_DIFFERENTIAL_AUDITS.getBoolean();
+    }
+
+    @VisibleForTesting
+    public void setStoreDifferentialAudits(boolean val) {
+        this.storeDifferentialAudits = val;
     }
 
     @Override
@@ -210,10 +221,26 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         AtlasEntitiesWithExtInfo ret = entityRetriever.toAtlasEntitiesWithExtInfo(guids, isMinExtInfo);
 
         if(ret != null){
-            for(String guid : guids){
+            for(String guid : guids) {
                 AtlasEntity entity = ret.getEntity(guid);
+                try {
+                    AtlasAuthorizationUtils.verifyAccess(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_READ, new AtlasEntityHeader(entity)), "read entity: guid=", guid);
+                } catch (AtlasBaseException e) {
+                    if (RequestContext.get().isSkipFailedEntities()) {
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("getByIds(): ignoring failure for entity {}: error code={}, message={}", guid, e.getAtlasErrorCode(), e.getMessage());
+                        }
 
-                AtlasAuthorizationUtils.verifyAccess(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_READ, new AtlasEntityHeader(entity)), "read entity: guid=", guid);
+                        //Remove from referred entities
+                        ret.removeEntity(guid);
+                        //Remove from entities
+                        ret.removeEntity(entity);
+
+                        continue;
+                    }
+
+                    throw e;
+                }
             }
         }
 
@@ -706,37 +733,50 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         if (CollectionUtils.isEmpty(guids)) {
             throw new AtlasBaseException(AtlasErrorCode.INVALID_PARAMETERS, "Guid(s) not specified");
         }
+
         if (classification == null) {
             throw new AtlasBaseException(AtlasErrorCode.INVALID_PARAMETERS, "classification not specified");
         }
 
-        EntityMutationContext context = new EntityMutationContext();
+        validateAndNormalize(classification);
+
+        EntityMutationContext     context         = new EntityMutationContext();
+        List<AtlasClassification> classifications = Collections.singletonList(classification);
+        List<String>              validGuids      =  new ArrayList<>();
 
         GraphTransactionInterceptor.lockObjectAndReleasePostCommit(guids);
 
         for (String guid : guids) {
-            AtlasVertex entityVertex = AtlasGraphUtilsV2.findByGuid(graph, guid);
+            try {
+                AtlasVertex entityVertex = AtlasGraphUtilsV2.findByGuid(graph, guid);
 
-            if (entityVertex == null) {
-                throw new AtlasBaseException(AtlasErrorCode.INSTANCE_GUID_NOT_FOUND, guid);
+                if (entityVertex == null) {
+                    throw new AtlasBaseException(AtlasErrorCode.INSTANCE_GUID_NOT_FOUND, guid);
+                }
+
+                AtlasEntityHeader entityHeader = entityRetriever.toAtlasEntityHeaderWithClassifications(entityVertex);
+
+                AtlasAuthorizationUtils.verifyAccess(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_ADD_CLASSIFICATION, entityHeader, classification),
+                        "add classification: guid=", guid, ", classification=", classification.getTypeName());
+
+                validateEntityAssociations(guid, classifications);
+
+                validGuids.add(guid);
+                context.cacheEntity(guid, entityVertex, typeRegistry.getEntityTypeByName(entityHeader.getTypeName()));
+            } catch (AtlasBaseException abe) {
+                if (RequestContext.get().isSkipFailedEntities()) {
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("addClassification(): ignoring failure for entity {}: error code={}, message={}", guid, abe.getAtlasErrorCode(), abe.getMessage());
+                    }
+
+                    continue;
+                }
+
+                throw abe;
             }
-
-            AtlasEntityHeader entityHeader = entityRetriever.toAtlasEntityHeaderWithClassifications(entityVertex);
-
-            AtlasAuthorizationUtils.verifyAccess(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_ADD_CLASSIFICATION, entityHeader, classification),
-                                                 "add classification: guid=", guid, ", classification=", classification.getTypeName());
-
-            context.cacheEntity(guid, entityVertex, typeRegistry.getEntityTypeByName(entityHeader.getTypeName()));
         }
 
-
-        validateAndNormalize(classification);
-
-        List<AtlasClassification> classifications = Collections.singletonList(classification);
-
-        for (String guid : guids) {
-            validateEntityAssociations(guid, classifications);
-
+        for (String guid : validGuids) {
             entityGraphMapper.addClassifications(context, guid, classifications);
         }
     }
@@ -1084,7 +1124,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         }
     }
 
-        private EntityMutationResponse createOrUpdate(EntityStream entityStream, boolean isPartialUpdate, boolean replaceClassifications, boolean replaceBusinessAttributes) throws AtlasBaseException {
+    private EntityMutationResponse createOrUpdate(EntityStream entityStream, boolean isPartialUpdate, boolean replaceClassifications, boolean replaceBusinessAttributes) throws AtlasBaseException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> createOrUpdate()");
         }
@@ -1116,113 +1156,55 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
             if (CollectionUtils.isNotEmpty(context.getUpdatedEntities())) {
                 MetricRecorder checkForUnchangedEntities = RequestContext.get().startMetricRecord("checkForUnchangedEntities");
 
-                List<AtlasEntity> entitiesToSkipUpdate = null;
+                List<AtlasEntity>     entitiesToSkipUpdate = new ArrayList<>();
+                AtlasEntityComparator entityComparator     = new AtlasEntityComparator(typeRegistry, entityRetriever, context.getGuidAssignments(), !replaceClassifications, !replaceBusinessAttributes);
+                RequestContext        reqContext           = RequestContext.get();
 
                 for (AtlasEntity entity : context.getUpdatedEntities()) {
-                    String          guid       = entity.getGuid();
-                    AtlasVertex     vertex     = context.getVertex(guid);
-                    AtlasEntityType entityType = typeRegistry.getEntityTypeByName(entity.getTypeName());
-                    boolean         hasUpdates = false;
-
-                    if (!hasUpdates) {
-                        hasUpdates = entity.getStatus() == AtlasEntity.Status.DELETED; // entity status could be updated during import
+                    if (entity.getStatus() == AtlasEntity.Status.DELETED) {// entity status could be updated during import
+                        continue;
                     }
 
-                    if (!hasUpdates && MapUtils.isNotEmpty(entity.getAttributes())) { // check for attribute value change
-                        for (AtlasAttribute attribute : entityType.getAllAttributes().values()) {
-                            if (!entity.getAttributes().containsKey(attribute.getName())) {  // if value is not provided, current value will not be updated
-                                continue;
-                            }
+                    AtlasVertex           storedVertex = context.getVertex(entity.getGuid());
+                    AtlasEntityDiffResult diffResult   = entityComparator.getDiffResult(entity, storedVertex, !storeDifferentialAudits);
 
-                            Object newVal  = entity.getAttribute(attribute.getName());
-                            Object currVal = entityRetriever.getEntityAttribute(vertex, attribute);
-
-                            if (!attribute.getAttributeType().areEqualValues(currVal, newVal, context.getGuidAssignments())) {
-                                hasUpdates = true;
-
-                                if (LOG.isDebugEnabled()) {
-                                    LOG.debug("found attribute update: entity(guid={}, typeName={}), attrName={}, currValue={}, newValue={}", guid, entity.getTypeName(), attribute.getName(), currVal, newVal);
-                                }
-
-                                break;
-                            }
-                        }
-                    }
-
-                    if (!hasUpdates && MapUtils.isNotEmpty(entity.getRelationshipAttributes())) { // check of relationsship-attribute value change
-                        for (String attributeName : entityType.getRelationshipAttributes().keySet()) {
-                            if (!entity.getRelationshipAttributes().containsKey(attributeName)) {  // if value is not provided, current value will not be updated
-                                continue;
-                            }
-
-                            Object         newVal           = entity.getRelationshipAttribute(attributeName);
-                            String         relationshipType = AtlasEntityUtil.getRelationshipType(newVal);
-                            AtlasAttribute attribute        = entityType.getRelationshipAttribute(attributeName, relationshipType);
-                            Object         currVal          = entityRetriever.getEntityAttribute(vertex, attribute);
-
-                            if (!attribute.getAttributeType().areEqualValues(currVal, newVal, context.getGuidAssignments())) {
-                                hasUpdates = true;
-
-                                if (LOG.isDebugEnabled()) {
-                                    LOG.debug("found relationship attribute update: entity(guid={}, typeName={}), attrName={}, currValue={}, newValue={}", guid, entity.getTypeName(), attribute.getName(), currVal, newVal);
-                                }
-
-                                break;
-                            }
-                        }
-                    }
-
-                    if (!hasUpdates && entity.getCustomAttributes() != null) {
-                        Map<String, String> currCustomAttributes = getCustomAttributes(vertex);
-                        Map<String, String> newCustomAttributes  = entity.getCustomAttributes();
-
-                        if (!Objects.equals(currCustomAttributes, newCustomAttributes)) {
-                            hasUpdates = true;
-                        }
-                    }
-
-                    // if classifications are to be replaced, then skip updates only when no change in classifications
-                    if (!hasUpdates && replaceClassifications) {
-                        List<AtlasClassification> newVal  = entity.getClassifications();
-                        List<AtlasClassification> currVal = entityRetriever.getAllClassifications(vertex);
-
-                        if (!Objects.equals(currVal, newVal)) {
-                            hasUpdates = true;
-
-                            if (LOG.isDebugEnabled()) {
-                                LOG.debug("found classifications update: entity(guid={}, typeName={}), currValue={}, newValue={}", guid, entity.getTypeName(), currVal, newVal);
-                            }
-                        }
-                    }
-
-                    if (!hasUpdates) {
-                        if (entitiesToSkipUpdate == null) {
-                            entitiesToSkipUpdate = new ArrayList<>();
+                    if (diffResult.hasDifference()) {
+                        if (storeDifferentialAudits) {
+                            diffResult.getDiffEntity().setGuid(entity.getGuid());
+                            reqContext.cacheDifferentialEntity(diffResult.getDiffEntity());
                         }
 
+                        if (diffResult.hasDifferenceOnlyInCustomAttributes()) {
+                            reqContext.recordEntityWithCustomAttributeUpdate(entity.getGuid());
+                        }
+
+                        if (diffResult.hasDifferenceOnlyInBusinessAttributes()) {
+                            reqContext.recordEntityWithBusinessAttributeUpdate(entity.getGuid());
+                        }
+                    } else {
                         if (LOG.isDebugEnabled()) {
                             LOG.debug("skipping unchanged entity: {}", entity);
                         }
 
                         entitiesToSkipUpdate.add(entity);
-                        RequestContext.get().recordEntityToSkip(entity.getGuid());
+                        reqContext.recordEntityToSkip(entity.getGuid());
                     }
                 }
 
-                if (entitiesToSkipUpdate != null) {
+                if (entitiesToSkipUpdate.size() > 0) {
                     // remove entitiesToSkipUpdate from EntityMutationContext
                     context.getUpdatedEntities().removeAll(entitiesToSkipUpdate);
                 }
 
                 // Check if authorized to update entities
-                if (!RequestContext.get().isImportInProgress()) {
+                if (!reqContext.isImportInProgress()) {
                     for (AtlasEntity entity : context.getUpdatedEntities()) {
                         AtlasAuthorizationUtils.verifyAccess(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_UPDATE, new AtlasEntityHeader(entity)),
                                                              "update entity: type=", entity.getTypeName());
                     }
                 }
 
-                RequestContext.get().endMetricRecord(checkForUnchangedEntities);
+                reqContext.endMetricRecord(checkForUnchangedEntities);
             }
 
             EntityMutationResponse ret = entityGraphMapper.mapAttributesAndClassifications(context, isPartialUpdate, replaceClassifications, replaceBusinessAttributes);
@@ -1551,33 +1533,26 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
     public BulkImportResponse bulkCreateOrUpdateBusinessAttributes(InputStream inputStream, String fileName) throws AtlasBaseException {
         BulkImportResponse ret = new BulkImportResponse();
 
-        try {
-            if (StringUtils.isBlank(fileName)) {
-                throw new AtlasBaseException(AtlasErrorCode.FILE_NAME_NOT_FOUND, fileName);
+        if (StringUtils.isBlank(fileName)) {
+            throw new AtlasBaseException(AtlasErrorCode.FILE_NAME_NOT_FOUND, fileName);
+        }
+
+        List<String[]>           fileData              = FileUtils.readFileData(fileName, inputStream);
+        Map<String, AtlasEntity> attributesToAssociate = getBusinessMetadataDefList(fileData, ret);
+
+        for (AtlasEntity entity : attributesToAssociate.values()) {
+            Map<String, Map<String, Object>> businessAttributes = entity.getBusinessAttributes();
+            String                           guid               = entity.getGuid();
+
+            try {
+                addOrUpdateBusinessAttributes(guid, businessAttributes, true);
+
+                ret.addToSuccessImportInfoList(new ImportInfo(guid, businessAttributes.toString()));
+            } catch (Exception e) {
+                LOG.error("Error occurred while updating BusinessMetadata Attributes for Entity " + guid);
+
+                ret.addToFailedImportInfoList(new ImportInfo(guid, businessAttributes.toString(), FAILED, e.getMessage()));
             }
-
-            List<String[]>           fileData              = FileUtils.readFileData(fileName, inputStream);
-            Map<String, AtlasEntity> attributesToAssociate = getBusinessMetadataDefList(fileData, ret);
-
-            for (AtlasEntity entity : attributesToAssociate.values()) {
-                try {
-                    addOrUpdateBusinessAttributes(entity.getGuid(), entity.getBusinessAttributes(), true);
-
-                    BulkImportResponse.ImportInfo successImportInfo = new BulkImportResponse.ImportInfo(entity.getGuid(), entity.getBusinessAttributes().toString());
-
-                    ret.setSuccessImportInfoList(successImportInfo);
-                }catch (Exception e) {
-                    LOG.error("Error occurred while updating BusinessMetadata Attributes for Entity " + entity.getGuid());
-
-                    BulkImportResponse.ImportInfo failedImportInfo = new BulkImportResponse.ImportInfo(entity.getGuid(), entity.getBusinessAttributes().toString(), BulkImportResponse.ImportStatus.FAILED, e.getMessage());
-
-                    ret.getFailedImportInfoList().add(failedImportInfo);
-                }
-            }
-        } catch (IOException e) {
-            LOG.error("An Exception occurred while uploading the file {}", fileName, e);
-
-            throw new AtlasBaseException(AtlasErrorCode.FAILED_TO_UPLOAD, e);
         }
 
         return ret;
@@ -1589,7 +1564,8 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         List<String>             failedMsgList = new ArrayList<>();
 
         for (int lineIndex = 0; lineIndex < fileData.size(); lineIndex++) {
-            String[] record = fileData.get(lineIndex);
+            String[] record         = fileData.get(lineIndex);
+            int      lineIndexToLog = lineIndex + 2;
 
             boolean missingFields = record.length < FileUtils.UNIQUE_ATTR_NAME_COLUMN_INDEX ||
                                     StringUtils.isBlank(record[FileUtils.TYPENAME_COLUMN_INDEX]) ||
@@ -1598,7 +1574,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
                                     StringUtils.isBlank(record[FileUtils.BM_ATTR_VALUE_COLUMN_INDEX]);
 
             if (missingFields){
-                failedMsgList.add("Line #" + (lineIndex + 1) + ": missing fields. " + Arrays.toString(record));
+                failedMsgList.add("Line #" + lineIndexToLog + ": missing fields. " + Arrays.toString(record));
 
                 continue;
             }
@@ -1607,7 +1583,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
             AtlasEntityType entityType = typeRegistry.getEntityTypeByName(typeName);
 
             if (entityType == null) {
-                failedMsgList.add("Line #" + (lineIndex + 1) + ": invalid entity-type '" + typeName + "'");
+                failedMsgList.add("Line #" + lineIndexToLog + ": invalid entity-type '" + typeName + "'");
 
                 continue;
             }
@@ -1617,20 +1593,20 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
             String bmAttributeValue = record[FileUtils.BM_ATTR_VALUE_COLUMN_INDEX];
             String uniqueAttrName   = AtlasTypeUtil.ATTRIBUTE_QUALIFIED_NAME;
 
-            if (record.length > FileUtils.UNIQUE_ATTR_NAME_COLUMN_INDEX) {
+            if (record.length > FileUtils.UNIQUE_ATTR_NAME_COLUMN_INDEX && StringUtils.isNotBlank(record[FileUtils.UNIQUE_ATTR_NAME_COLUMN_INDEX])) {
                 uniqueAttrName = record[FileUtils.UNIQUE_ATTR_NAME_COLUMN_INDEX];
             }
 
             AtlasAttribute uniqueAttribute = entityType.getAttribute(uniqueAttrName);
 
             if (uniqueAttribute == null) {
-                failedMsgList.add("Line #" + (lineIndex + 1) + ": attribute '" + uniqueAttrName + "' not found in entity-type '" + typeName + "'");
+                failedMsgList.add("Line #" + lineIndexToLog + ": attribute '" + uniqueAttrName + "' not found in entity-type '" + typeName + "'");
 
                 continue;
             }
 
             if (!uniqueAttribute.getAttributeDef().getIsUnique()) {
-                failedMsgList.add("Line #" + (lineIndex + 1) + ": attribute '" + uniqueAttrName + "' is not an unique attribute in entity-type '" + typeName + "'");
+                failedMsgList.add("Line #" + lineIndexToLog + ": attribute '" + uniqueAttrName + "' is not an unique attribute in entity-type '" + typeName + "'");
 
                 continue;
             }
@@ -1642,7 +1618,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
                 vertex = AtlasGraphUtilsV2.findByTypeAndUniquePropertyName(graph, typeName, uniqueAttribute.getVertexUniquePropertyName(), uniqueAttrValue);
 
                 if (vertex == null) {
-                    failedMsgList.add("Line #" + (lineIndex + 1) + ": no " + typeName + " entity found with " + uniqueAttrName + "=" + uniqueAttrValue);
+                    failedMsgList.add("Line #" + lineIndexToLog + ": no " + typeName + " entity found with " + uniqueAttrName + "=" + uniqueAttrValue);
 
                     continue;
                 }
@@ -1653,7 +1629,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
             AtlasBusinessAttribute businessAttribute = entityType.getBusinesAAttribute(bmAttribute);
 
             if (businessAttribute == null) {
-                failedMsgList.add("Line #" + (lineIndex + 1) + ": invalid business attribute name '" + bmAttribute + "'");
+                failedMsgList.add("Line #" + lineIndexToLog + ": invalid business-metadata '"+ bmAttribute + "' for entity type '" + entityType.getTypeName() + "'");
 
                 continue;
             }
@@ -1702,9 +1678,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
         for (String failedMsg : failedMsgList) {
             LOG.error(failedMsg);
 
-            BulkImportResponse.ImportInfo importInfo = new BulkImportResponse.ImportInfo(BulkImportResponse.ImportStatus.FAILED, failedMsg);
-
-            bulkImportResponse.getFailedImportInfoList().add(importInfo);
+            bulkImportResponse.addToFailedImportInfoList(new ImportInfo(FAILED, failedMsg));
         }
 
         return ret;
@@ -1733,7 +1707,7 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
                     return AtlasGraphUtilsV2.doubleParser(arr, failedTermMsgList, lineIndex);
 
                 case AtlasBaseTypeDef.ATLAS_TYPE_DATE:
-                    return AtlasGraphUtilsV2.dateParser(arr, failedTermMsgList, lineIndex);
+                    return AtlasGraphUtilsV2.longParser(arr, failedTermMsgList, lineIndex);
 
                 case AtlasBaseTypeDef.ATLAS_TYPE_BOOLEAN:
                     return AtlasGraphUtilsV2.booleanParser(arr, failedTermMsgList, lineIndex);
@@ -1757,9 +1731,10 @@ public class AtlasEntityStoreV2 implements AtlasEntityStore {
 
         if(missingFieldsCheck){
             LOG.error("Missing fields: " + Arrays.toString(record) + " at line #" + lineIndex);
+
             String failedTermMsgs = "Missing fields: " + Arrays.toString(record) + " at line #" + lineIndex;
-            BulkImportResponse.ImportInfo importInfo = new BulkImportResponse.ImportInfo(BulkImportResponse.ImportStatus.FAILED, failedTermMsgs, lineIndex);
-            bulkImportResponse.getFailedImportInfoList().add(importInfo);
+
+            bulkImportResponse.addToFailedImportInfoList(new ImportInfo(FAILED, failedTermMsgs, lineIndex));
         }
         return missingFieldsCheck;
     }
