@@ -68,6 +68,7 @@ import org.springframework.stereotype.Component;
 import javax.inject.Inject;
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -79,6 +80,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
+import java.util.Queue;
 import java.util.Set;
 import java.util.stream.Collectors;
 
@@ -116,6 +118,7 @@ import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelation
 import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.BOTH;
 import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.IN;
 import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.OUT;
+import static org.apache.atlas.type.Constants.PENDING_TASKS_PROPERTY_KEY;
 
 @Component
 public class EntityGraphRetriever {
@@ -201,6 +204,10 @@ public class EntityGraphRetriever {
         return toAtlasEntityHeader(getEntityVertex(guid));
     }
 
+    public AtlasEntityHeader toAtlasEntityHeader(String guid, Set<String> attributes) throws AtlasBaseException {
+        return toAtlasEntityHeader(getEntityVertex(guid), attributes);
+    }
+
     public AtlasEntityHeader toAtlasEntityHeader(AtlasVertex entityVertex) throws AtlasBaseException {
         return toAtlasEntityHeader(entityVertex, Collections.<String>emptySet());
     }
@@ -264,7 +271,7 @@ public class EntityGraphRetriever {
         Object ret = null;
 
         try {
-            ret = getVertexAttribute(entityVertex, attribute);
+            ret = getVertexAttributeIgnoreInactive(entityVertex, attribute);
         } catch (AtlasBaseException excp) {
             // ignore
         }
@@ -375,6 +382,10 @@ public class EntityGraphRetriever {
             throw new AtlasBaseException(AtlasErrorCode.INSTANCE_GUID_NOT_FOUND, guid);
         }
 
+        if (StringUtils.isEmpty(GraphHelper.getTypeName(ret))) {
+            throw new AtlasBaseException(AtlasErrorCode.NO_TYPE_NAME_ON_VERTEX, guid);
+        }
+
         return ret;
     }
 
@@ -493,15 +504,19 @@ public class EntityGraphRetriever {
     public List<AtlasVertex> getImpactedVerticesV2(AtlasVertex entityVertex, String relationshipGuidToExclude) {
         List<AtlasVertex> ret = new ArrayList<>();
 
-        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, null, new HashSet<>(), ret);
+        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, null, ret);
 
         return ret;
     }
 
     public List<AtlasVertex> getIncludedImpactedVerticesV2(AtlasVertex entityVertex, String relationshipGuidToExclude) {
+        return getIncludedImpactedVerticesV2(entityVertex, relationshipGuidToExclude, null);
+    }
+
+    public List<AtlasVertex> getIncludedImpactedVerticesV2(AtlasVertex entityVertex, String relationshipGuidToExclude, String classificationId) {
         List<AtlasVertex> ret = new ArrayList<>(Arrays.asList(entityVertex));
 
-        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, null, new HashSet<>(), ret);
+        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, classificationId, ret);
 
         return ret;
     }
@@ -509,18 +524,36 @@ public class EntityGraphRetriever {
     public List<AtlasVertex> getImpactedVerticesV2(AtlasVertex entityVertex, String relationshipGuidToExclude, String classificationId) {
         List<AtlasVertex> ret = new ArrayList<>();
 
-        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, classificationId, new HashSet<>(), ret);
+        traverseImpactedVertices(entityVertex, relationshipGuidToExclude, classificationId, ret);
 
         return ret;
     }
 
-    private void traverseImpactedVertices(AtlasVertex entityVertex, String relationshipGuidToExclude, String classificationId, Set<String> visitedVertices, List<AtlasVertex> result) {
-        visitedVertices.add(entityVertex.getIdForDisplay());
+    private void traverseImpactedVertices(final AtlasVertex entityVertexStart, final String relationshipGuidToExclude,
+                                          final String classificationId, final List<AtlasVertex> result) {
+        Set<String>              visitedVertices = new HashSet<>();
+        Queue<AtlasVertex>       queue           = new ArrayDeque<AtlasVertex>() {{ add(entityVertexStart); }};
+        Map<String, AtlasVertex> resultsMap      = new HashMap<>();
 
-        AtlasEntityType entityType          = typeRegistry.getEntityTypeByName(getTypeName(entityVertex));
-        String[]        tagPropagationEdges = entityType != null ? entityType.getTagPropagationEdgesArray() : null;
+        while (!queue.isEmpty()) {
+            AtlasVertex entityVertex   = queue.poll();
+            String      entityVertexId = entityVertex.getIdForDisplay();
+
+            if (visitedVertices.contains(entityVertexId)) {
+                LOG.info("Already visited: {}", entityVertexId);
+
+                continue;
+            }
+
+            visitedVertices.add(entityVertexId);
+
+            AtlasEntityType entityType          = typeRegistry.getEntityTypeByName(getTypeName(entityVertex));
+            String[]        tagPropagationEdges = entityType != null ? entityType.getTagPropagationEdgesArray() : null;
+
+            if (tagPropagationEdges == null) {
+                continue;
+            }
 
-        if (tagPropagationEdges != null) {
             Iterable<AtlasEdge> propagationEdges = entityVertex.getEdges(AtlasEdgeDirection.BOTH, tagPropagationEdges);
 
             for (AtlasEdge propagationEdge : propagationEdges) {
@@ -532,7 +565,7 @@ public class EntityGraphRetriever {
 
                 if (tagPropagation == null || tagPropagation == NONE) {
                     continue;
-                } else if (tagPropagation == TWO_TO_ONE)  {
+                } else if (tagPropagation == TWO_TO_ONE) {
                     if (isOutVertex(entityVertex, propagationEdge)) {
                         continue;
                     }
@@ -556,15 +589,18 @@ public class EntityGraphRetriever {
                     }
                 }
 
-                AtlasVertex adjacentVertex = getOtherVertex(propagationEdge, entityVertex);
+                AtlasVertex adjacentVertex             = getOtherVertex(propagationEdge, entityVertex);
+                String      adjacentVertexIdForDisplay = adjacentVertex.getIdForDisplay();
 
-                if (!visitedVertices.contains(adjacentVertex.getIdForDisplay())) {
-                    result.add(adjacentVertex);
+                if (!visitedVertices.contains(adjacentVertexIdForDisplay) && !resultsMap.containsKey(adjacentVertexIdForDisplay)) {
+                    resultsMap.put(adjacentVertexIdForDisplay, adjacentVertex);
 
-                    traverseImpactedVertices(adjacentVertex, relationshipGuidToExclude, classificationId, visitedVertices, result);
+                    queue.add(adjacentVertex);
                 }
             }
         }
+
+        result.addAll(resultsMap.values());
     }
 
     private boolean isOutVertex(AtlasVertex vertex, AtlasEdge edge) {
@@ -760,25 +796,32 @@ public class EntityGraphRetriever {
             LOG.debug("Mapping system attributes for type {}", entity.getTypeName());
         }
 
-        entity.setGuid(getGuid(entityVertex));
-        entity.setTypeName(getTypeName(entityVertex));
-        entity.setStatus(GraphHelper.getStatus(entityVertex));
-        entity.setVersion(GraphHelper.getVersion(entityVertex));
+        try {
+            if (entityVertex != null) {
+                entity.setGuid(getGuid(entityVertex));
+                entity.setTypeName(getTypeName(entityVertex));
+                entity.setStatus(GraphHelper.getStatus(entityVertex));
+                entity.setVersion(GraphHelper.getVersion(entityVertex));
 
-        entity.setCreatedBy(GraphHelper.getCreatedByAsString(entityVertex));
-        entity.setUpdatedBy(GraphHelper.getModifiedByAsString(entityVertex));
+                entity.setCreatedBy(GraphHelper.getCreatedByAsString(entityVertex));
+                entity.setUpdatedBy(GraphHelper.getModifiedByAsString(entityVertex));
 
-        entity.setCreateTime(new Date(GraphHelper.getCreatedTime(entityVertex)));
-        entity.setUpdateTime(new Date(GraphHelper.getModifiedTime(entityVertex)));
+                entity.setCreateTime(new Date(GraphHelper.getCreatedTime(entityVertex)));
+                entity.setUpdateTime(new Date(GraphHelper.getModifiedTime(entityVertex)));
 
-        entity.setHomeId(GraphHelper.getHomeId(entityVertex));
+                entity.setHomeId(GraphHelper.getHomeId(entityVertex));
 
-        entity.setIsProxy(GraphHelper.isProxy(entityVertex));
-        entity.setIsIncomplete(isEntityIncomplete(entityVertex));
+                entity.setIsProxy(GraphHelper.isProxy(entityVertex));
+                entity.setIsIncomplete(isEntityIncomplete(entityVertex));
 
-        entity.setProvenanceType(GraphHelper.getProvenanceType(entityVertex));
-        entity.setCustomAttributes(getCustomAttributes(entityVertex));
-        entity.setLabels(getLabels(entityVertex));
+                entity.setProvenanceType(GraphHelper.getProvenanceType(entityVertex));
+                entity.setCustomAttributes(getCustomAttributes(entityVertex));
+                entity.setLabels(getLabels(entityVertex));
+                entity.setPendingTasks(getPendingTasks(entityVertex));
+            }
+        } catch (Throwable t) {
+            LOG.warn("Got exception while mapping system attributes for type {} : ", entity.getTypeName(), t);
+        }
 
         return entity;
     }
@@ -931,6 +974,10 @@ public class EntityGraphRetriever {
     }
 
     private Object mapVertexToAttribute(AtlasVertex entityVertex, AtlasAttribute attribute, AtlasEntityExtInfo entityExtInfo, final boolean isMinExtInfo, boolean includeReferences) throws AtlasBaseException {
+        return mapVertexToAttribute(entityVertex, attribute, entityExtInfo, isMinExtInfo, includeReferences, false);
+    }
+
+    private Object mapVertexToAttribute(AtlasVertex entityVertex, AtlasAttribute attribute, AtlasEntityExtInfo entityExtInfo, final boolean isMinExtInfo, boolean includeReferences, boolean ignoreInactive) throws AtlasBaseException {
         Object    ret                = null;
         AtlasType attrType           = attribute.getAttributeType();
         String    edgeLabel          = attribute.getRelationshipEdgeLabel();
@@ -976,7 +1023,7 @@ public class EntityGraphRetriever {
                     if (attribute.getAttributeDef().isSoftReferenced()) {
                         ret = mapVertexToArrayForSoftRef(entityVertex, attribute, entityExtInfo, isMinExtInfo);
                     } else {
-                        ret = mapVertexToArray(entityVertex, entityExtInfo, isOwnedAttribute, attribute, isMinExtInfo, includeReferences);
+                        ret = mapVertexToArray(entityVertex, entityExtInfo, isOwnedAttribute, attribute, isMinExtInfo, includeReferences, ignoreInactive);
                     }
                 }
             }
@@ -1123,7 +1170,8 @@ public class EntityGraphRetriever {
     }
 
     private List<Object> mapVertexToArray(AtlasVertex entityVertex, AtlasEntityExtInfo entityExtInfo,
-                                          boolean isOwnedAttribute, AtlasAttribute attribute, final boolean isMinExtInfo, boolean includeReferences) throws AtlasBaseException {
+                                          boolean isOwnedAttribute, AtlasAttribute attribute, final boolean isMinExtInfo,
+                                          boolean includeReferences, boolean ignoreInactive) throws AtlasBaseException {
 
         AtlasArrayType arrayType        = (AtlasArrayType) attribute.getAttributeType();
         AtlasType      arrayElementType = arrayType.getElementType();
@@ -1149,6 +1197,10 @@ public class EntityGraphRetriever {
                 continue;
             }
 
+            if (isInactiveEdge(element, ignoreInactive)) {
+                continue;
+            }
+
             Object arrValue = mapVertexToCollectionEntry(entityVertex, arrayElementType, element, edgeLabel,
                                                          entityExtInfo, isOwnedAttribute, edgeDirection, isMinExtInfo, includeReferences);
 
@@ -1298,6 +1350,10 @@ public class EntityGraphRetriever {
         return vertex != null && attribute != null ? mapVertexToAttribute(vertex, attribute, null, false) : null;
     }
 
+    private Object getVertexAttributeIgnoreInactive(AtlasVertex vertex, AtlasAttribute attribute) throws AtlasBaseException {
+        return vertex != null && attribute != null ? mapVertexToAttribute(vertex, attribute, null, false, true, true) : null;
+    }
+
     private void mapRelationshipAttributes(AtlasVertex entityVertex, AtlasEntity entity, AtlasEntityExtInfo entityExtInfo, boolean isMinExtInfo) throws AtlasBaseException {
         AtlasEntityType entityType = typeRegistry.getEntityTypeByName(entity.getTypeName());
 
@@ -1348,12 +1404,11 @@ public class EntityGraphRetriever {
                 break;
         }
 
-        if (ret != null) {
-            entity.setRelationshipAttribute(attributeName, ret);
+        // Set Relationship attributes, even if the value is null
+        entity.setRelationshipAttribute(attributeName, ret);
 
-            if (attributeEndDef.getIsLegacyAttribute() && !entity.hasAttribute(attributeName)) {
-                entity.setAttribute(attributeName, toLegacyAttribute(ret));
-            }
+        if (attributeEndDef.getIsLegacyAttribute() && !entity.hasAttribute(attributeName)) {
+            entity.setAttribute(attributeName, toLegacyAttribute(ret));
         }
 
         return ret;
@@ -1653,4 +1708,18 @@ public class EntityGraphRetriever {
             relationship.setAttribute(attribute.getName(), attrValue);
         }
     }
+
+    private Set<String> getPendingTasks(AtlasVertex entityVertex) {
+        Collection<String> ret = entityVertex.getPropertyValues(PENDING_TASKS_PROPERTY_KEY, String.class);
+
+        if (CollectionUtils.isEmpty(ret)) {
+            return null;
+        }
+
+        return new HashSet<>(ret);
+    }
+
+    private boolean isInactiveEdge(Object element, boolean ignoreInactive) {
+        return ignoreInactive && element instanceof AtlasEdge && getStatus((AtlasEdge) element) != AtlasEntity.Status.ACTIVE;
+    }
 }
