@@ -19,15 +19,19 @@ package org.apache.atlas;
 
 import com.google.common.collect.ImmutableList;
 import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.glossary.GlossaryService;
 import org.apache.atlas.model.discovery.SearchParameters;
-import org.apache.atlas.model.instance.AtlasClassification;
-import org.apache.atlas.model.instance.AtlasEntity;
-import org.apache.atlas.model.instance.AtlasObjectId;
+import org.apache.atlas.model.glossary.AtlasGlossary;
+import org.apache.atlas.model.glossary.AtlasGlossaryTerm;
+import org.apache.atlas.model.glossary.relations.AtlasGlossaryHeader;
+import org.apache.atlas.model.instance.*;
 import org.apache.atlas.model.typedef.*;
+import org.apache.atlas.repository.AtlasTestBase;
 import org.apache.atlas.repository.store.graph.AtlasEntityStore;
 import org.apache.atlas.repository.store.graph.v2.AtlasEntityStream;
 import org.apache.atlas.store.AtlasTypeDefStore;
 import org.apache.atlas.type.AtlasTypeRegistry;
+import org.apache.commons.lang.StringUtils;
 
 import javax.inject.Inject;
 import java.io.IOException;
@@ -40,8 +44,9 @@ import java.util.stream.Stream;
 import static org.apache.atlas.utils.TestLoadModelUtils.loadModelFromJson;
 import static org.testng.Assert.fail;
 
-public abstract class BasicTestSetup {
+public abstract class BasicTestSetup extends AtlasTestBase {
 
+    // Entity type //
     protected static final String DATABASE_TYPE     = "hive_db";
     protected static final String HIVE_TABLE_TYPE   = "hive_table";
     private static final   String COLUMN_TYPE       = "hive_column";
@@ -50,6 +55,7 @@ public abstract class BasicTestSetup {
     private static final   String VIEW_TYPE         = "hive_process";
     protected static final String DATASET_SUBTYPE   = "Asset";
 
+    //Classification type //
     public static final String DIMENSION_CLASSIFICATION    = "Dimension";
     public static final String FACT_CLASSIFICATION         = "Fact";
     public static final String PII_CLASSIFICATION          = "PII";
@@ -59,24 +65,39 @@ public abstract class BasicTestSetup {
     public static final String LOGDATA_CLASSIFICATION      = "Log Data";
     public static final String DIMENSIONAL_CLASSIFICATION  = "Dimensional";
 
+    // Glossary type //
+    public static final String SALES_GLOSSARY = "salesGlossary";
+    public static final String SALES_TERM     = "salesTerm";
+    public static final String MODERNTRADE_TERM = "modernTrade";
+    public static final String ECOMMERCE_TERM   = "ecommerce";
+
     @Inject
     protected AtlasTypeRegistry typeRegistry;
     @Inject
     protected AtlasTypeDefStore typeDefStore;
     @Inject
     protected AtlasEntityStore entityStore;
+    @Inject
+    protected GlossaryService glossaryService;
 
     private boolean baseLoaded = false;
+    private EntityMutationResponse hiveEntities;
+
+    private AtlasEntity timeDim;
+    private AtlasEntity productDim;
+    private AtlasEntity loggingFactMonthly;
 
     protected void setupTestData() {
         loadBaseModels();
         loadHiveDataset();
         loadEmployeeDataset();
+        assignGlossary();
     }
 
     private void loadBaseModels() {
         try {
             loadModelFromJson("0000-Area0/0010-base_model.json", typeDefStore, typeRegistry);
+            loadModelFromJson("0000-Area0/0011-glossary_model.json", typeDefStore, typeRegistry);
             baseLoaded = true;
         } catch (IOException | AtlasBaseException e) {
             fail("Base model setup is required for test to run!");
@@ -97,7 +118,7 @@ public abstract class BasicTestSetup {
         AtlasEntity.AtlasEntitiesWithExtInfo hiveTestEntities = hiveTestEntities();
 
         try {
-            entityStore.createOrUpdate(new AtlasEntityStream(hiveTestEntities), false);
+             hiveEntities = entityStore.createOrUpdate(new AtlasEntityStream(hiveTestEntities), false);
         } catch (AtlasBaseException e) {
             fail("Hive entities need to be created for test to run!");
         }
@@ -132,7 +153,7 @@ public abstract class BasicTestSetup {
         
         createClassificationTypes();
 
-        AtlasEntity salesDB = database("Sales", "Sales Database", "John ETL", "hdfs://host:8000/apps/warehouse/sales");
+        AtlasEntity salesDB = database("Sales", "/apps/warehouse/Sales Database", "John ETL", "hdfs://host:8000/apps/warehouse/sales");
         entities.add(salesDB);
 
         AtlasEntity sd =
@@ -167,12 +188,12 @@ public abstract class BasicTestSetup {
         sd = storageDescriptor("hdfs://host:8000/apps/warehouse/sales", "TextInputFormat", "TextOutputFormat", true, ImmutableList.of(column("time_id", "int", "time id")));
         entities.add(sd);
 
-        AtlasEntity timeDim = table("time_dim", "time dimension table", salesDB, sd, "John Doe", "External", timeDimColumns,
+        timeDim = table("time_dim", "time dimension table", salesDB, sd, "John Doe", "External", timeDimColumns,
                                     DIMENSION_CLASSIFICATION);
         entities.add(timeDim);
 
         AtlasEntity reportingDB =
-                database("Reporting", "reporting database", "Jane BI", "hdfs://host:8000/apps/warehouse/reporting");
+                database("Reporting", "/apps/warehouse/reporting database", "Jane BI", "hdfs://host:8000/apps/warehouse/reporting");
         entities.add(reportingDB);
 
         sd = storageDescriptor("hdfs://host:8000/apps/warehouse/sales", "TextInputFormat", "TextOutputFormat", true, ImmutableList.of(column("time_id", "int", "time id")));
@@ -208,7 +229,7 @@ public abstract class BasicTestSetup {
                                          ImmutableList.of(salesFactDaily), "create table as select ", "plan", "id", "graph", ETL_CLASSIFICATION);
         entities.add(loadSalesDaily);
 
-        AtlasEntity logDB = database("Logging", "logging database", "Tim ETL", "hdfs://host:8000/apps/warehouse/logging");
+        AtlasEntity logDB = database("Logging", "/apps/warehouse/logging", "Tim ETL", "hdfs://host:8000/apps/warehouse/logging");
         entities.add(logDB);
 
         sd = storageDescriptor("hdfs://host:8000/apps/warehouse/sales", "TextInputFormat", "TextOutputFormat", true, ImmutableList.of(column("time_id", "int", "time id")));
@@ -228,7 +249,7 @@ public abstract class BasicTestSetup {
         sd = storageDescriptor("hdfs://host:8000/apps/warehouse/sales", "TextInputFormat", "TextOutputFormat", true, ImmutableList.of(column("time_id", "int", "time id")));
         entities.add(sd);
 
-        AtlasEntity productDim =
+        productDim =
                 table("product_dim", "product dimension table", salesDB, sd, "John Doe 2", "Managed", productDimColumns,
                       DIMENSION_CLASSIFICATION);
         entities.add(productDim);
@@ -268,7 +289,7 @@ public abstract class BasicTestSetup {
         sd = storageDescriptor("hdfs://host:8000/apps/warehouse/sales", "TextInputFormat", "TextOutputFormat", true, ImmutableList.of(column("time_id", "int", "time id")));
         entities.add(sd);
 
-        AtlasEntity loggingFactMonthly =
+        loggingFactMonthly =
                 table("logging_fact_monthly_mv", "logging fact monthly materialized view", logDB, sd, "Tim ETL 2",
                       "Managed", logFactColumns, LOGDATA_CLASSIFICATION);
         entities.add(loggingFactMonthly);
@@ -284,7 +305,8 @@ public abstract class BasicTestSetup {
     }
 
     protected void createClassificationTypes() {
-        List<AtlasClassificationDef> cds =  Arrays.asList(new AtlasClassificationDef(DIMENSION_CLASSIFICATION, "Dimension Classification", "1.0"),
+        List<AtlasClassificationDef> cds =  Arrays.asList(new AtlasClassificationDef(DIMENSION_CLASSIFICATION, "Dimension Classification", "1.0",
+                        Arrays.asList(new AtlasStructDef.AtlasAttributeDef("timeAttr","string"), new AtlasStructDef.AtlasAttributeDef("productAttr","string"))),
                 new AtlasClassificationDef(FACT_CLASSIFICATION, "Fact Classification", "1.0"),
                 new AtlasClassificationDef(PII_CLASSIFICATION, "PII Classification", "1.0"),
                 new AtlasClassificationDef(METRIC_CLASSIFICATION, "Metric Classification", "1.0"),
@@ -315,8 +337,9 @@ public abstract class BasicTestSetup {
         database.setAttribute("name", name);
         database.setAttribute(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, "qualified:" + name);
         database.setAttribute("description", description);
+        database.setAttribute("userDescription", description);
         database.setAttribute("owner", owner);
-        database.setAttribute("locationUri", locationUri);
+        database.setAttribute("location", locationUri);
         database.setAttribute("createTime", System.currentTimeMillis());
         database.setAttribute("clusterName", "cl1");
         database.setClassifications(Stream.of(traitNames).map(AtlasClassification::new).collect(Collectors.toList()));
@@ -367,7 +390,15 @@ public abstract class BasicTestSetup {
         table.setAttribute("sd", getAtlasObjectId(sd));
 
         table.setAttribute("columns", getAtlasObjectIds(columns));
-        table.setClassifications(Stream.of(traitNames).map(AtlasClassification::new).collect(Collectors.toList()));
+        if (name.equals("time_dim")) {
+            AtlasClassification classification = new AtlasClassification(traitNames[0], new HashMap<String, Object>() {{ put("timeAttr", "timeValue"); }});
+            table.setClassifications(Collections.singletonList(classification));
+        } else if (name.equals("product_dim")) {
+            AtlasClassification classification = new AtlasClassification(traitNames[0], new HashMap<String, Object>() {{ put("productAttr", "productValue"); }});
+            table.setClassifications(Collections.singletonList(classification));
+        } else {
+            table.setClassifications(Stream.of(traitNames).map(AtlasClassification::new).collect(Collectors.toList()));
+        }
 
         sd.setAttribute(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, dbName + "." + name + "@" + clusterName + "_storage");
 
@@ -450,12 +481,13 @@ public abstract class BasicTestSetup {
         return datasetSubType;
     }
 
-    public void createDummyEntity(String name, String type, String... traitNames) throws AtlasBaseException {
+    public EntityMutationResponse createDummyEntity(String name, String type, String... traitNames) throws AtlasBaseException {
         AtlasEntity entity = new AtlasEntity(type);
         entity.setAttribute("name", name);
         entity.setAttribute(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, name);
         entity.setClassifications(Stream.of(traitNames).map(AtlasClassification::new).collect(Collectors.toList()));
-        entityStore.createOrUpdate(new AtlasEntityStream(new AtlasEntity.AtlasEntitiesWithExtInfo(entity)), false);
+        EntityMutationResponse resp = entityStore.createOrUpdate(new AtlasEntityStream(new AtlasEntity.AtlasEntitiesWithExtInfo(entity)), false);
+        return resp;
     }
 
     public SearchParameters.FilterCriteria getSingleFilterCondition(String attName, SearchParameters.Operator op, String attrValue) {
@@ -472,4 +504,67 @@ public abstract class BasicTestSetup {
         return filterCriteria;
     }
 
+    public void assignGlossary() {
+        try {
+            AtlasGlossary glossary = new AtlasGlossary();
+            glossary.setName(SALES_GLOSSARY);
+            glossary = glossaryService.createGlossary(glossary);
+
+            AtlasGlossaryTerm term = new AtlasGlossaryTerm();
+            term.setAnchor(new AtlasGlossaryHeader(glossary.getGuid()));
+            term.setName(SALES_TERM);
+            term = glossaryService.createTerm(term);
+
+            AtlasGlossaryTerm modernTrade = new AtlasGlossaryTerm();
+            modernTrade.setAnchor(new AtlasGlossaryHeader(glossary.getGuid()));
+            modernTrade.setName(MODERNTRADE_TERM);
+            modernTrade = glossaryService.createTerm(modernTrade);
+
+            AtlasGlossaryTerm ecomm = new AtlasGlossaryTerm();
+            ecomm.setAnchor(new AtlasGlossaryHeader(glossary.getGuid()));
+            ecomm.setName(ECOMMERCE_TERM);
+            ecomm = glossaryService.createTerm(ecomm);
+
+            List<AtlasRelatedObjectId> guids = new ArrayList<>();
+            List<AtlasRelatedObjectId> mordernTradeGuids = new ArrayList<>();
+            List<AtlasRelatedObjectId> ecomGuid = new ArrayList<>();
+
+            for (AtlasEntityHeader p : hiveEntities.getCreatedEntities()) {
+                if (StringUtils.equals(p.getTypeName(), HIVE_TABLE_TYPE)) {
+                    AtlasRelatedObjectId obj = new AtlasRelatedObjectId();
+                    obj.setGuid(p.getGuid());
+                    obj.setTypeName(p.getTypeName());
+                    guids.add(obj);
+
+                    if (p.getAttribute("name").equals(timeDim.getAttribute("name"))) {
+                        timeDim.setGuid(p.getGuid());
+                        AtlasRelatedObjectId obj1 = new AtlasRelatedObjectId();
+                        obj1.setGuid(p.getGuid());
+                        obj1.setTypeName(p.getTypeName());
+                        mordernTradeGuids.add(obj1);
+                    } else if (p.getAttribute("name").equals(productDim.getAttribute("name"))) {
+                        productDim.setGuid(p.getGuid());
+                        AtlasRelatedObjectId obj2 = new AtlasRelatedObjectId();
+                        obj2.setGuid(p.getGuid());
+                        obj2.setTypeName(p.getTypeName());
+                        ecomGuid.add(obj2);
+                    } else if (p.getAttribute("name").equals(loggingFactMonthly.getAttribute("name"))) {
+                        loggingFactMonthly.setGuid(p.getGuid());
+                        AtlasRelatedObjectId obj3 = new AtlasRelatedObjectId();
+                        obj3.setGuid(p.getGuid());
+                        obj3.setTypeName(p.getTypeName());
+                        mordernTradeGuids.add(obj3);
+                    }
+                }
+            }
+
+            glossaryService.assignTermToEntities(term.getGuid(), guids);
+            glossaryService.assignTermToEntities(modernTrade.getGuid(), mordernTradeGuids);
+            glossaryService.assignTermToEntities(ecomm.getGuid(), ecomGuid);
+        } catch (AtlasBaseException e) {
+            fail("Failed to assign glossary term");
+        }
+
+    }
+
 }
