@@ -22,6 +22,7 @@ import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.TypeCategory;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.query.antlr4.AtlasDSLParser;
+import org.apache.atlas.type.AtlasClassificationType;
 import org.apache.atlas.type.AtlasEntityType;
 import org.apache.atlas.type.AtlasStructType;
 import org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection;
@@ -31,6 +32,7 @@ import org.apache.commons.lang.StringUtils;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.IN;
 import static org.mockito.Matchers.anyString;
 import static org.mockito.Matchers.eq;
 import static org.mockito.Mockito.mock;
@@ -39,6 +41,14 @@ import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.fail;
 import static org.apache.atlas.type.AtlasStructType.AtlasAttribute.AtlasRelationshipEdgeDirection.OUT;
 public class GremlinQueryComposerTest {
+    @Test
+    public void withinClause() {
+        String expected1 = "g.V().has('__typeName', within('Asset','Table')).has('Asset.__s_name', within('t1','t2','t3')).dedup().limit(25).toList()";
+        String expected2 = "g.V().has('__typeName', within('Asset','Table')).has('Asset.__s_name', within(\"t1\",\"t2\",\"t3\")).dedup().limit(25).toList()";
+        verify("Asset where name=['t1', 't2', 't3']", expected1);
+        verify("Asset where name=[\"t1\", \"t2\", \"t3\"]", expected2);
+    }
+
     @Test
     public void classification() {
         String expected = "g.V().outE('classifiedAs').has('__name', within('PII')).outV().dedup().limit(25).toList()";
@@ -53,6 +63,19 @@ public class GremlinQueryComposerTest {
         verify("Table where Table is Dimension", expected);
     }
 
+    @Test()
+    public void classificationAttributes() {
+        String expected1 = "g.V().outE('classifiedAs').has('__name', within('Dimension')).outV().and(__.out('classifiedAs').has('Dimension.timeAttr', eq('value')).dedup().in('classifiedAs')).dedup().limit(25).toList()";
+        String expected2 = "g.V().outE('classifiedAs').has('__name', within('Dimension')).outV().as('d').and(__.out('classifiedAs').has('Dimension.timeAttr', eq('value')).dedup().in('classifiedAs')).dedup().limit(25).toList()";
+        String expected3 = "g.V().has('__typeName', 'Table').and(__.has('Table.name', eq('time_dim')),__.out('classifiedAs').has('Dimension.timeAttr', eq('value')).dedup().in('classifiedAs')).dedup().limit(25).toList()";
+        String expected4 = "g.V().has('__typeName', 'Table').and(__.out('classifiedAs').has('Dimension.timeAttr', eq('value')).dedup().in('classifiedAs')).dedup().limit(25).toList()";
+
+        verify("Dimension where Dimension.timeAttr = 'value'", expected1);
+        verify("Dimension as d where d.timeAttr = 'value'",expected2);
+        verify("Table where (name = 'time_dim' and Dimension.timeAttr = 'value')",expected3);
+        verify("Table where Dimension.timeAttr = 'value'", expected4);
+    }
+
     @Test
     public void fromDB() {
         String expected10 = "g.V().has('__typeName', 'DB').dedup().limit(10).toList()";
@@ -75,8 +98,10 @@ public class GremlinQueryComposerTest {
 
     @Test
     public void DBasDSelect() {
-        String expected = "def f(r){ t=[['d.name','d.owner']];  r.each({t.add([it.value('DB.name'),it.value('DB.owner')])}); t.unique(); }; " +
-                                  "f(g.V().has('__typeName', 'DB').as('d').has('DB.name').has('DB.owner')";
+        String expected = "def f(r){ t=[['d.name','d.owner']];  r.each({t.add([" +
+                "it.property('DB.name').isPresent() ? it.value('DB.name') : \"\"," +
+                "it.property('DB.owner').isPresent() ? it.value('DB.owner') : \"\"])}); t.unique(); }; " +
+                "f(g.V().has('__typeName', 'DB').as('d')";
         verify("DB as d select d.name, d.owner", expected + ".dedup().limit(25).toList())");
         verify("DB as d select d.name, d.owner limit 10", expected + ".dedup().limit(10).toList())");
         verify("DB as d select d","def f(r){ r }; f(g.V().has('__typeName', 'DB').as('d').dedup().limit(25).toList())");
@@ -86,7 +111,8 @@ public class GremlinQueryComposerTest {
     public void tableSelectColumns() {
         String exMain = "g.V().has('__typeName', 'Table').out('__Table.columns').dedup().limit(10).toList()";
         String exSel = "def f(r){ r }";
-        String exSel1 = "def f(r){ t=[['db.name']];  r.each({t.add([it.value('DB.name')])}); t.unique(); }";
+        String exSel1 = "def f(r){ t=[['db.name']];  r.each({t.add([" +
+                "it.property('DB.name').isPresent() ? it.value('DB.name') : \"\"])}); t.unique(); }";
         verify("Table select columns limit 10", getExpected(exSel, exMain));
 
         String exMain2 = "g.V().has('__typeName', 'Table').out('__Table.db').dedup().limit(25).toList()";
@@ -114,9 +140,12 @@ public class GremlinQueryComposerTest {
     public void groupByOrderBy() {
         verify("Table groupby(owner) select name, owner, clusterName orderby name",
                 "def f(l){ h=[['name','owner','clusterName']]; t=[]; " +
-                        "l.get(0).each({k,r -> L:{  r.each({t.add([it.value('Table.name'),it.value('Table.owner'),it.value('Table.clusterName')])}) } }); " +
+                        "l.get(0).each({k,r -> L:{  r.each({t.add([" +
+                        "it.property('Table.name').isPresent() ? it.value('Table.name') : \"\"," +
+                        "it.property('Table.owner').isPresent() ? it.value('Table.owner') : \"\"," +
+                        "it.property('Table.clusterName').isPresent() ? it.value('Table.clusterName') : \"\"])}) } }); " +
                         "h.plus(t.unique().sort{a,b -> a[0] <=> b[0]}); }; " +
-                        "f(g.V().has('__typeName', 'Table').has('Table.name').has('Table.owner').has('Table.clusterName').group().by('Table.owner').dedup().limit(25).toList())");
+                        "f(g.V().has('__typeName', 'Table').group().by('Table.owner').dedup().limit(25).toList())");
     }
 
     @Test
@@ -133,12 +162,16 @@ public class GremlinQueryComposerTest {
         verify("from DB as d orderby d.owner limit 3", "g.V().has('__typeName', 'DB').as('d').order().by('DB.owner').dedup().limit(3).toList()");
         verify("DB as d orderby d.owner limit 3", "g.V().has('__typeName', 'DB').as('d').order().by('DB.owner').dedup().limit(3).toList()");
 
-        String exSel = "def f(r){ t=[['d.name','d.owner']];  r.each({t.add([it.value('DB.name'),it.value('DB.owner')])}); t.unique(); }";
-        String exMain = "g.V().has('__typeName', 'DB').as('d').has('DB.name').has('DB.owner').order().by('DB.owner').dedup().limit(25).toList()";
+        String exSel = "def f(r){ t=[['d.name','d.owner']];  r.each({t.add([" +
+                "it.property('DB.name').isPresent() ? it.value('DB.name') : \"\"," +
+                "it.property('DB.owner').isPresent() ? it.value('DB.owner') : \"\"])}); t.unique(); }";
+        String exMain = "g.V().has('__typeName', 'DB').as('d').order().by('DB.owner').dedup().limit(25).toList()";
         verify("DB as d select d.name, d.owner orderby (d.owner) limit 25", getExpected(exSel, exMain));
 
         String exMain2 = "g.V().has('__typeName', 'Table').and(__.has('Table.name', eq(\"sales_fact\")),__.has('Table.createTime', gt('1418265300000'))).order().by('Table.createTime').dedup().limit(25).toList()";
-        String exSel2 = "def f(r){ t=[['_col_0','_col_1']];  r.each({t.add([it.value('Table.name'),it.value('Table.createTime')])}); t.unique(); }";
+        String exSel2 = "def f(r){ t=[['_col_0','_col_1']];  r.each({t.add([" +
+                "it.property('Table.name').isPresent() ? it.value('Table.name') : \"\"," +
+                "it.property('Table.createTime').isPresent() ? it.value('Table.createTime') : \"\"])}); t.unique(); }";
         verify("Table where (name = \"sales_fact\" and createTime > \"2014-12-11T02:35:0.0Z\" ) select name as _col_0, createTime as _col_1 orderby _col_1",
                 getExpected(exSel2, exMain2));
     }
@@ -150,8 +183,10 @@ public class GremlinQueryComposerTest {
 
     @Test
     public void fromDBSelect() {
-        String expected = "def f(r){ t=[['DB.name','DB.owner']];  r.each({t.add([it.value('DB.name'),it.value('DB.owner')])}); t.unique(); }; " +
-                                  "f(g.V().has('__typeName', 'DB').has('DB.name').has('DB.owner').dedup().limit(25).toList())";
+        String expected = "def f(r){ t=[['DB.name','DB.owner']];  r.each({t.add([" +
+                "it.property('DB.name').isPresent() ? it.value('DB.name') : \"\"," +
+                "it.property('DB.owner').isPresent() ? it.value('DB.owner') : \"\"])}); t.unique(); }; " +
+                "f(g.V().has('__typeName', 'DB').dedup().limit(25).toList())";
         verify("from DB select DB.name, DB.owner", expected);
     }
 
@@ -162,8 +197,10 @@ public class GremlinQueryComposerTest {
 
     @Test
     public void whereClauseTextContains() {
-        String exMain = "g.V().has('__typeName', 'DB').has('DB.name', eq(\"Reporting\")).has('DB.owner').dedup().limit(25).toList()";
-        String exSel = "def f(r){ t=[['name','owner']];  r.each({t.add([it.value('DB.name'),it.value('DB.owner')])}); t.unique(); }";
+        String exMain = "g.V().has('__typeName', 'DB').has('DB.name', eq(\"Reporting\")).dedup().limit(25).toList()";
+        String exSel = "def f(r){ t=[['name','owner']];  r.each({t.add([" +
+                "it.property('DB.name').isPresent() ? it.value('DB.name') : \"\"," +
+                "it.property('DB.owner').isPresent() ? it.value('DB.owner') : \"\"])}); t.unique(); }";
         verify("from DB where name = \"Reporting\" select name, owner", getExpected(exSel, exMain));
         verify("from DB where (name = \"Reporting\") select name, owner", getExpected(exSel, exMain));
         verify("Table where Asset.name like \"Tab*\"",
@@ -174,27 +211,37 @@ public class GremlinQueryComposerTest {
                 "g.V().has('__typeName', 'Table').has('Table.owner', org.janusgraph.core.attribute.Text.textRegex(\".*Tab_.*\")).dedup().limit(25).toList()");
         verify("from Table where (db.name = \"Reporting\")",
                 "g.V().has('__typeName', 'Table').out('__Table.db').has('DB.name', eq(\"Reporting\")).dedup().in('__Table.db').dedup().limit(25).toList()");
+        verify( "Table where owner like \"Jane/*\"", "g.V().has('__typeName', 'Table').has('Table.owner', org.janusgraph.core.attribute.Text.textRegex(\"Jane\\/.*\")).dedup().limit(25).toList()");
+        verify( "Table where Asset.name like \"/sales_*\"", "g.V().has('__typeName', 'Table').has('Asset.__s_name', org.janusgraph.core.attribute.Text.textRegex(\"\\/sales_.*\")).dedup().limit(25).toList()");
+        verify( "Table where Asset.name like \"sales:*\"", "g.V().has('__typeName', 'Table').has('Asset.__s_name', org.janusgraph.core.attribute.Text.textRegex(\"sales:.*\")).dedup().limit(25).toList()");
+        verify( "Table where Asset.name like \"table[0-9]\"", "g.V().has('__typeName', 'Table').has('Asset.__s_name', org.janusgraph.core.attribute.Text.textRegex(\"table[0-9]\")).dedup().limit(25).toList()");
     }
 
     @Test
     public void whereClauseWithAsTextContains() {
-        String exSel = "def f(r){ t=[['t.name','t.owner']];  r.each({t.add([it.value('Table.name'),it.value('Table.owner')])}); t.unique(); }";
-        String exMain = "g.V().has('__typeName', 'Table').as('t').has('Table.name', eq(\"testtable_1\")).has('Table.owner').dedup().limit(25).toList()";
+        String exSel = "def f(r){ t=[['t.name','t.owner']];  r.each({t.add([" +
+                "it.property('Table.name').isPresent() ? it.value('Table.name') : \"\"," +
+                "it.property('Table.owner').isPresent() ? it.value('Table.owner') : \"\"])}); t.unique(); }";
+        String exMain = "g.V().has('__typeName', 'Table').as('t').has('Table.name', eq(\"testtable_1\")).dedup().limit(25).toList()";
         verify("Table as t where t.name = \"testtable_1\" select t.name, t.owner", getExpected(exSel, exMain));
     }
 
     @Test
     public void whereClauseWithDateCompare() {
-        String exSel = "def f(r){ t=[['t.name','t.owner']];  r.each({t.add([it.value('Table.name'),it.value('Table.owner')])}); t.unique(); }";
-        String exMain = "g.V().has('__typeName', 'Table').as('t').has('Table.createTime', eq('%s')).has('Table.name').has('Table.owner').dedup().limit(25).toList()";
+        String exSel = "def f(r){ t=[['t.name','t.owner']];  r.each({t.add([" +
+                "it.property('Table.name').isPresent() ? it.value('Table.name') : \"\"," +
+                "it.property('Table.owner').isPresent() ? it.value('Table.owner') : \"\"])}); t.unique(); }";
+        String exMain = "g.V().has('__typeName', 'Table').as('t').has('Table.createTime', eq('%s')).dedup().limit(25).toList()";
         verify("Table as t where t.createTime = \"2017-12-12T02:35:58.440Z\" select t.name, t.owner", getExpected(exSel, String.format(exMain, "1513046158440")));
         verify("Table as t where t.createTime = \"2017-12-12\" select t.name, t.owner", getExpected(exSel, String.format(exMain, "1513036800000")));
     }
 
     @Test
     public void subType() {
-        String exMain = "g.V().has('__typeName', within('Asset','Table')).has('Asset.__s_name').has('Asset.__s_owner').dedup().limit(25).toList()";
-        String exSel = "def f(r){ t=[['name','owner']];  r.each({t.add([it.value('Asset.__s_name'),it.value('Asset.__s_owner')])}); t.unique(); }";
+        String exMain = "g.V().has('__typeName', within('Asset','Table')).dedup().limit(25).toList()";
+        String exSel = "def f(r){ t=[['name','owner']];  r.each({t.add([" +
+                "it.property('Asset.__s_name').isPresent() ? it.value('Asset.__s_name') : \"\"," +
+                "it.property('Asset.__s_owner').isPresent() ? it.value('Asset.__s_owner') : \"\"])}); t.unique(); }";
 
         verify("Asset select name, owner", getExpected(exSel, exMain));
     }
@@ -292,6 +339,8 @@ public class GremlinQueryComposerTest {
                         "g.V().has('__typeName', 'hive_db').and(__.has('hive_db.name', eq('Reporting')),__.has('hive_db.createTime', lt('1513046158440'))).dedup().limit(25).toList()"},
                 {"Table where db.name='Sales' and db.clusterName='cl1'",
                         "g.V().has('__typeName', 'Table').and(__.out('__Table.db').has('DB.name', eq('Sales')).dedup().in('__Table.db'),__.out('__Table.db').has('DB.clusterName', eq('cl1')).dedup().in('__Table.db')).dedup().limit(25).toList()"},
+                {"hive_db where (hive_db.name='Reporting' or ((hive_db.name='Reporting' and hive_db.createTime > '2017-12-12T02:35:58.440Z') and (hive_db.name='Reporting' and hive_db.createTime < '2017-12-12T02:35:58.440Z')))",
+                        "g.V().has('__typeName', 'hive_db').or(__.has('hive_db.name', eq('Reporting')),__.and(__.and(__.has('hive_db.name', eq('Reporting')),__.has('hive_db.createTime', gt('1513046158440'))),__.and(__.has('hive_db.name', eq('Reporting')),__.has('hive_db.createTime', lt('1513046158440'))))).dedup().limit(25).toList()"}
         };
     }
 
@@ -301,6 +350,23 @@ public class GremlinQueryComposerTest {
         verify(query.replace("where", " "), expectedGremlin);
     }
 
+    @Test
+    public void glossaryTermQueries() {
+        verify("Table hasTerm sales", "g.V().has('__typeName', 'Table')" +
+                ".where(in('r:AtlasGlossarySemanticAssignment').has('AtlasGlossaryTerm.name', 'sales'))" +
+                ".dedup().limit(25).toList()");
+        verify("Table hasTerm \"sales@glossary\"", "g.V().has('__typeName', 'Table')." +
+                "where(in('r:AtlasGlossarySemanticAssignment').has('AtlasGlossaryTerm.qualifiedName', 'sales@glossary'))" +
+                ".dedup().limit(25).toList()");
+        verify("Table hasTerm \"sales@glossary\" and owner = \"fetl\"",
+                "g.V().has('__typeName', 'Table')" +
+                        ".and(" +
+                            "__.where(in('r:AtlasGlossarySemanticAssignment').has('AtlasGlossaryTerm.qualifiedName', 'sales@glossary'))" +
+                            ",__.has('Table.owner', eq(\"fetl\"))" +
+                        ").dedup().limit(25).toList()");
+    }
+
+
     @Test
     public void keywordsInWhereClause() {
         verify("Table as t where t has name and t isa Dimension",
@@ -325,18 +391,27 @@ public class GremlinQueryComposerTest {
     @Test
     public void systemAttributes() {
         verify("Table has __state", "g.V().has('__typeName', 'Table').has('__state').dedup().limit(25).toList()");
-        verify("Table select __guid", "def f(r){ t=[['__guid']];  r.each({t.add([it.value('__guid')])}); t.unique(); }; f(g.V().has('__typeName', 'Table').has('__guid').dedup().limit(25).toList())");
+        verify("Table select __guid", "def f(r){ t=[['__guid']];  r.each({t.add([" +
+                "it.property('__guid').isPresent() ? it.value('__guid') : \"\"])}); t.unique(); }; " +
+                "f(g.V().has('__typeName', 'Table').dedup().limit(25).toList())");
         verify("Table as t where t.__state = 'ACTIVE'", "g.V().has('__typeName', 'Table').as('t').has('__state', eq('ACTIVE')).dedup().limit(25).toList()");
     }
 
     @Test
     public void whereComplexAndSelect() {
-        String exSel = "def f(r){ t=[['name']];  r.each({t.add([it.value('Table.name')])}); t.unique(); }";
+        String exSel = "def f(r){ t=[['name']];  r.each({t.add([" +
+                "it.property('Table.name').isPresent() ? it.value('Table.name') : \"\"])}); t.unique(); }";
         String exMain = "g.V().has('__typeName', 'Table').and(__.out('__Table.db').has('DB.name', eq(\"Reporting\")).dedup().in('__Table.db'),__.has('Table.name', eq(\"sales_fact\"))).dedup().limit(25).toList()";
         verify("Table where db.name = \"Reporting\" and name =\"sales_fact\" select name", getExpected(exSel, exMain));
         verify("Table where db.name = \"Reporting\" and name =\"sales_fact\"", exMain);
     }
 
+    @Test
+    public void whereClauseWithNEQCondition() {
+        verify("Table where owner != 'random'",
+                "g.V().has('__typeName', 'Table').or(__.has('Table.owner', neq('random')), __.hasNot('Table.owner')).dedup().limit(25).toList()");
+    }
+
     @Test
     public void invalidQueries() {
         verify("hdfs_path like h1", "");
@@ -387,7 +462,7 @@ public class GremlinQueryComposerTest {
     }
 
     private String getGremlinQuery(String dsl, AtlasDSLParser.QueryContext queryContext, int expectedNumberOfErrors) {
-        AtlasTypeRegistry             registry = mock(AtlasTypeRegistry.class);
+        AtlasTypeRegistry registry = mock(AtlasTypeRegistry.class);
         org.apache.atlas.query.Lookup lookup   = new TestLookup(registry);
         GremlinQueryComposer.Context  context  = new GremlinQueryComposer.Context(lookup);
         AtlasDSL.QueryMetadata queryMetadata   = new AtlasDSL.QueryMetadata(queryContext);
@@ -413,7 +488,7 @@ public class GremlinQueryComposerTest {
         public AtlasType getType(String typeName) throws AtlasBaseException {
             AtlasType type;
             if(typeName.equals("PII") || typeName.equals("Dimension")) {
-                type = mock(AtlasType.class);
+                type = mock(AtlasClassificationType.class);
                 when(type.getTypeCategory()).thenReturn(TypeCategory.CLASSIFICATION);
             } else {
                 type = mock(AtlasEntityType.class);
@@ -478,17 +553,30 @@ public class GremlinQueryComposerTest {
                 return "__Table.columns";
             if (attributeName.equalsIgnoreCase("db"))
                 return "__Table.db";
+            if (attributeName.equalsIgnoreCase("meanings"))
+                return "r:AtlasGlossarySemanticAssignment";
             else
                 return "__DB.Table";
         }
 
         @Override
         public AtlasRelationshipEdgeDirection getRelationshipEdgeDirection(GremlinQueryComposer.Context context, String attributeName) {
+            if (attributeName.equalsIgnoreCase("meanings")){
+                return IN;
+            }
             return OUT;
         }
 
         @Override
         public boolean hasAttribute(GremlinQueryComposer.Context context, String attributeName) {
+            if (context.getActiveType() instanceof AtlasClassificationType) {
+                return attributeName.equals("timeAttr");
+            }
+
+            if (context.getActiveEntityType() == null) {
+                return false;
+            }
+
             return (context.getActiveTypeName().equals("Table") && attributeName.equals("db")) ||
                     (context.getActiveTypeName().equals("Table") && attributeName.equals("columns")) ||
                     (context.getActiveTypeName().equals("Table") && attributeName.equals("createTime")) ||
@@ -499,6 +587,7 @@ public class GremlinQueryComposerTest {
                     (context.getActiveTypeName().equals("Table") && attributeName.equals("__guid")) ||
                     (context.getActiveTypeName().equals("Table") && attributeName.equals("__state")) ||
                     (context.getActiveTypeName().equals("Table") && attributeName.equals("partitionSize")) ||
+                    (context.getActiveTypeName().equals("Table") && attributeName.equals("meanings")) ||
                     (context.getActiveTypeName().equals("hive_db") && attributeName.equals("name")) ||
                     (context.getActiveTypeName().equals("hive_db") && attributeName.equals("owner")) ||
                     (context.getActiveTypeName().equals("hive_db") && attributeName.equals("createTime")) ||
@@ -506,7 +595,9 @@ public class GremlinQueryComposerTest {
                     (context.getActiveTypeName().equals("DB") && attributeName.equals("owner")) ||
                     (context.getActiveTypeName().equals("DB") && attributeName.equals("clusterName")) ||
                     (context.getActiveTypeName().equals("Asset") && attributeName.equals("name")) ||
-                    (context.getActiveTypeName().equals("Asset") && attributeName.equals("owner"));
+                    (context.getActiveTypeName().equals("Asset") && attributeName.equals("owner")) ||
+                    (context.getActiveTypeName().equals("AtlasGlossaryTerm") && attributeName.equals("name")) ||
+                    (context.getActiveTypeName().equals("AtlasGlossaryTerm") && attributeName.equals("qualifiedName"));
         }
 
         @Override
@@ -543,6 +634,10 @@ public class GremlinQueryComposerTest {
                 return "Column";
             }
 
+            if(context.getActiveTypeName().equals("Table") && item.equals("meanings")) {
+                return "AtlasGlossaryTerm";
+            }
+
             if(context.getActiveTypeName().equals(item)) {
                 return null;
             }
