@@ -18,7 +18,9 @@
 package org.apache.atlas.repository.store.graph.v2;
 
 import com.google.common.collect.ImmutableSet;
+import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasErrorCode;
+import org.apache.atlas.GraphTransactionInterceptor;
 import org.apache.atlas.RequestContext;
 import org.apache.atlas.TestModules;
 import org.apache.atlas.TestUtilsV2;
@@ -60,6 +62,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import static org.apache.atlas.AtlasConfiguration.STORE_DIFFERENTIAL_AUDITS;
 import static org.apache.atlas.AtlasErrorCode.INVALID_CUSTOM_ATTRIBUTE_KEY_CHARACTERS;
 import static org.apache.atlas.AtlasErrorCode.INVALID_CUSTOM_ATTRIBUTE_KEY_LENGTH;
 import static org.apache.atlas.AtlasErrorCode.INVALID_CUSTOM_ATTRIBUTE_VALUE;
@@ -852,6 +855,45 @@ public class AtlasEntityStoreV2Test extends AtlasEntityTestBase {
         assertEquals(col3.getAttribute("description"), updatedCol3Entity.getAttribute("description"));
     }
 
+    @Test
+    public void testDifferentialEntitiesOnUpdate () throws Exception {
+
+        ApplicationProperties.get().setProperty(STORE_DIFFERENTIAL_AUDITS.getPropertyName(), true);
+        init();
+        // test for entity attributes
+
+        AtlasEntity dbEntity = new AtlasEntity(TestUtilsV2.DATABASE_TYPE);
+        dbEntity.setAttribute("name", TestUtilsV2.randomString(10));
+        dbEntity.setAttribute("description", "us db");
+        dbEntity.setAttribute("namespace", "db namespace");
+        dbEntity.setAttribute("cluster", "Fenton_Cluster");
+
+        EntityStream           dbStream        = new AtlasEntityStream(new AtlasEntitiesWithExtInfo(dbEntity));
+        EntityMutationResponse response        = entityStore.createOrUpdate(dbStream, false);
+        AtlasEntityHeader      dbHeader        = response.getFirstEntityCreated();
+        AtlasEntity            createdDbEntity = getEntityFromStore(dbHeader);
+
+        assertEquals(RequestContext.get().getDifferentialEntities().size(), 0);
+
+        // update the db entity
+        dbEntity = new AtlasEntity(TestUtilsV2.DATABASE_TYPE);
+        dbEntity.setGuid(createdDbEntity.getGuid());
+        dbEntity.setAttribute("description", "new description");
+
+        dbStream = new AtlasEntityStream(new AtlasEntitiesWithExtInfo(dbEntity));
+
+        response = entityStore.createOrUpdate(dbStream, true);
+        dbHeader = response.getFirstEntityPartialUpdated();
+        AtlasEntity updatedDbEntity = getEntityFromStore(dbHeader);
+
+        assertEquals(RequestContext.get().getDifferentialEntities().size(), 1);
+        AtlasEntity diffEntity = RequestContext.get().getDifferentialEntity(updatedDbEntity.getGuid());
+        assertNotNull(diffEntity.getAttribute("description"));
+        Assert.assertNull(diffEntity.getAttribute("namespace"));
+        Assert.assertNull(diffEntity.getAttribute("name"));
+        ApplicationProperties.get().setProperty(STORE_DIFFERENTIAL_AUDITS.getPropertyName(), false);
+    }
+
     @Test
     public void testSetObjectIdAttrToNull() throws Exception {
         final AtlasEntity dbEntity  = TestUtilsV2.createDBEntity();
@@ -859,6 +901,7 @@ public class AtlasEntityStoreV2Test extends AtlasEntityTestBase {
 
         entityStore.createOrUpdate(new AtlasEntityStream(dbEntity), false);
         entityStore.createOrUpdate(new AtlasEntityStream(db2Entity), false);
+        GraphTransactionInterceptor.clearCache();
 
         final AtlasEntity tableEntity = TestUtilsV2.createTableEntity(dbEntity);
 
@@ -873,6 +916,8 @@ public class AtlasEntityStoreV2Test extends AtlasEntityTestBase {
         createdTblEntity.setAttribute("databaseComposite", null);
 
         final EntityMutationResponse tblUpdateResponse = entityStore.createOrUpdate(new AtlasEntityStream(createdTblEntity), true);
+        GraphTransactionInterceptor.clearCache();
+
         final AtlasEntityHeader      updatedTblHeader  = tblUpdateResponse.getFirstEntityPartialUpdated();
         final AtlasEntity            updatedTblEntity  = getEntityFromStore(updatedTblHeader);
         final AtlasEntity            deletedDb2Entity  = getEntityFromStore(db2Entity.getGuid());
@@ -1000,7 +1045,10 @@ public class AtlasEntityStoreV2Test extends AtlasEntityTestBase {
     }
 
     @Test (dependsOnMethods = "testCreate")
-    public void addCustomAttributesToEntity() throws AtlasBaseException {
+    public void addCustomAttributesToEntity() throws Exception {
+
+        ApplicationProperties.get().setProperty(STORE_DIFFERENTIAL_AUDITS.getPropertyName(), true);
+        init();
         AtlasEntity tblEntity = getEntityFromStore(tblEntityGuid);
 
         Map<String, String> customAttributes = new HashMap<>();
@@ -1013,10 +1061,13 @@ public class AtlasEntityStoreV2Test extends AtlasEntityTestBase {
         tblEntity.setCustomAttributes(customAttributes);
 
         entityStore.createOrUpdate(new AtlasEntityStream(tblEntity), false);
+        assertEquals(RequestContext.get().getDifferentialEntities().size(), 1);
 
         tblEntity = getEntityFromStore(tblEntityGuid);
 
         assertEquals(customAttributes, tblEntity.getCustomAttributes());
+
+        ApplicationProperties.get().setProperty(STORE_DIFFERENTIAL_AUDITS.getPropertyName(), false);
     }
 
     @Test (dependsOnMethods = "addCustomAttributesToEntity")
