@@ -40,6 +40,7 @@ import org.apache.atlas.notification.NotificationInterface.NotificationType;
 import org.apache.atlas.notification.preprocessor.EntityPreprocessor;
 import org.apache.atlas.notification.preprocessor.PreprocessorContext;
 import org.apache.atlas.notification.preprocessor.PreprocessorContext.PreprocessAction;
+import org.apache.atlas.repository.store.graph.EntityCorrelationStore;
 import org.apache.atlas.util.AtlasMetricsCounter;
 import org.apache.atlas.utils.AtlasJson;
 import org.apache.atlas.utils.LruCache;
@@ -67,6 +68,7 @@ import org.apache.commons.collections.MapUtils;
 import org.apache.commons.collections4.map.PassiveExpiringMap;
 import org.apache.commons.configuration.Configuration;
 import org.apache.commons.lang.StringUtils;
+import org.apache.kafka.common.TopicPartition;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.context.annotation.DependsOn;
@@ -84,6 +86,7 @@ import java.time.Instant;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
@@ -117,6 +120,8 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
     private static final String TYPE_HIVE_COLUMN_LINEAGE = "hive_column_lineage";
     private static final String ATTRIBUTE_INPUTS         = "inputs";
     private static final String ATTRIBUTE_QUALIFIED_NAME = "qualifiedName";
+    private static final String EXCEPTION_CLASS_NAME_JANUSGRAPH_EXCEPTION = "JanusGraphException";
+    private static final String EXCEPTION_CLASS_NAME_PERMANENTLOCKING_EXCEPTION = "PermanentLockingException";
 
     // from org.apache.hadoop.hive.ql.parse.SemanticAnalyzer
     public static final String DUMMY_DATABASE               = "_dummy_database";
@@ -186,6 +191,8 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
     private final Configuration                 applicationProperties;
     private       ExecutorService               executors;
     private       Instant                       nextStatsLogTime = AtlasMetricsCounter.getNextHourStartTime(Instant.now());
+    private final Map<TopicPartition, Long>     lastCommittedPartitionOffset;
+    private final EntityCorrelationManager      entityCorrelationManager;
 
     @VisibleForTesting
     final int consumerRetryInterval;
@@ -196,14 +203,16 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
     @Inject
     public NotificationHookConsumer(NotificationInterface notificationInterface, AtlasEntityStore atlasEntityStore,
                                     ServiceState serviceState, AtlasInstanceConverter instanceConverter,
-                                    AtlasTypeRegistry typeRegistry, AtlasMetricsUtil metricsUtil) throws AtlasException {
+                                    AtlasTypeRegistry typeRegistry, AtlasMetricsUtil metricsUtil,
+                                    EntityCorrelationStore entityCorrelationStore) throws AtlasException {
         this.notificationInterface = notificationInterface;
         this.atlasEntityStore      = atlasEntityStore;
         this.serviceState          = serviceState;
         this.instanceConverter     = instanceConverter;
         this.typeRegistry          = typeRegistry;
         this.applicationProperties = ApplicationProperties.get();
-        this.metricsUtil           = metricsUtil;
+        this.metricsUtil                    = metricsUtil;
+        this.lastCommittedPartitionOffset   = new HashMap<>();
 
         maxRetries            = applicationProperties.getInt(CONSUMER_RETRIES_PROPERTY, 3);
         failedMsgCacheSize    = applicationProperties.getInt(CONSUMER_FAILEDCACHESIZE_PROPERTY, 1);
@@ -302,7 +311,7 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
         hiveTypesRemoveOwnedRefAttrs  = applicationProperties.getBoolean(CONSUMER_PREPROCESS_HIVE_TYPES_REMOVE_OWNEDREF_ATTRS, true);
         rdbmsTypesRemoveOwnedRefAttrs = applicationProperties.getBoolean(CONSUMER_PREPROCESS_RDBMS_TYPES_REMOVE_OWNEDREF_ATTRS, true);
         preprocessEnabled             = skipHiveColumnLineageHive20633 || updateHiveProcessNameWithQualifiedName || hiveTypesRemoveOwnedRefAttrs || rdbmsTypesRemoveOwnedRefAttrs || !hiveTablesToIgnore.isEmpty() || !hiveTablesToPrune.isEmpty() || !hiveDummyDatabasesToIgnore.isEmpty() || !hiveDummyTablesToIgnore.isEmpty() || !hiveTablePrefixesToIgnore.isEmpty();
-
+        entityCorrelationManager      = new EntityCorrelationManager(entityCorrelationStore);
         LOG.info("{}={}", CONSUMER_SKIP_HIVE_COLUMN_LINEAGE_HIVE_20633, skipHiveColumnLineageHive20633);
         LOG.info("{}={}", CONSUMER_SKIP_HIVE_COLUMN_LINEAGE_HIVE_20633_INPUTS_THRESHOLD, skipHiveColumnLineageHive20633InputsThreshold);
         LOG.info("{}={}", CONSUMER_PREPROCESS_HIVE_TYPES_REMOVE_OWNEDREF_ATTRS, hiveTypesRemoveOwnedRefAttrs);
@@ -515,14 +524,10 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
         private final List<String>                           failedMessages = new ArrayList<>();
         private final AdaptiveWaiter                         adaptiveWaiter = new AdaptiveWaiter(minWaitDuration, maxWaitDuration, minWaitDuration);
 
-        @VisibleForTesting
-        final FailedCommitOffsetRecorder failedCommitOffsetRecorder;
-
         public HookConsumer(NotificationConsumer<HookNotification> consumer) {
             super("atlas-hook-consumer-thread", false);
 
             this.consumer = consumer;
-            failedCommitOffsetRecorder = new FailedCommitOffsetRecorder();
         }
 
         @Override
@@ -538,7 +543,7 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
             try {
                 while (shouldRun.get()) {
                     try {
-                        List<AtlasKafkaMessage<HookNotification>> messages = consumer.receive();
+                        List<AtlasKafkaMessage<HookNotification>> messages = consumer.receiveWithCheckedCommit(lastCommittedPartitionOffset);
 
                         for (AtlasKafkaMessage<HookNotification> msg : messages) {
                             handleMessage(msg);
@@ -584,11 +589,6 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
             }
 
             try {
-                if(failedCommitOffsetRecorder.isMessageReplayed(kafkaMsg.getOffset())) {
-                    commit(kafkaMsg);
-                    return;
-                }
-
                 // covert V1 messages to V2 to enable preProcess
                 try {
                     switch (message.getType()) {
@@ -624,6 +624,7 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
                 }
 
                 // Used for intermediate conversions during create and update
+                String exceptionClassName = StringUtils.EMPTY;
                 for (int numRetries = 0; numRetries < maxRetries; numRetries++) {
                     if (LOG.isDebugEnabled()) {
                         LOG.debug("handleMessage({}): attempt {}", message.getType().name(), numRetries);
@@ -690,6 +691,7 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
                                     EntityMutationResponse response = atlasEntityStore.deleteByUniqueAttributes(type, Collections.singletonMap(deleteRequest.getAttribute(), (Object) deleteRequest.getAttributeValue()));
 
                                     stats.updateStats(response);
+                                    entityCorrelationManager.add(kafkaMsg.getSpooled(), kafkaMsg.getMsgCreated(), response.getDeletedEntities());
                                 } catch (ClassCastException cle) {
                                     LOG.error("Failed to delete entity {}", deleteRequest);
                                 }
@@ -772,6 +774,7 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
                                         EntityMutationResponse response = atlasEntityStore.deleteByUniqueAttributes(type, entity.getUniqueAttributes());
 
                                         stats.updateStats(response);
+                                        entityCorrelationManager.add(kafkaMsg.getSpooled(), kafkaMsg.getMsgCreated(), response.getDeletedEntities());
                                     }
                                 } catch (ClassCastException cle) {
                                     LOG.error("Failed to do delete entities {}", entities);
@@ -783,9 +786,15 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
                                 throw new IllegalStateException("Unknown notification type: " + message.getType().name());
                         }
 
+                        if (StringUtils.isNotEmpty(exceptionClassName)) {
+                            LOG.warn("{}: Pausing & retry: Try: {}: Pause: {} ms. Handled!",
+                                                exceptionClassName, numRetries, adaptiveWaiter.waitDuration);
+                            exceptionClassName = StringUtils.EMPTY;
+                        }
                         break;
                     } catch (Throwable e) {
                         RequestContext.get().resetEntityGuidUpdates();
+                        exceptionClassName = e.getClass().getSimpleName();
 
                         if (numRetries == (maxRetries - 1)) {
                             String strMessage = AbstractNotification.getMessageJson(message);
@@ -800,6 +809,14 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
                                 recordFailedMessages();
                             }
                             return;
+                        } else if (e instanceof org.apache.atlas.repository.graphdb.AtlasSchemaViolationException) {
+                            LOG.warn("{}: Continuing: {}", exceptionClassName, e.getMessage());
+                        } else if (exceptionClassName.equals(EXCEPTION_CLASS_NAME_JANUSGRAPH_EXCEPTION)
+                                || exceptionClassName.equals(EXCEPTION_CLASS_NAME_PERMANENTLOCKING_EXCEPTION)) {
+                            LOG.warn("{}: Pausing & retry: Try: {}: Pause: {} ms. {}",
+                                    exceptionClassName, numRetries, adaptiveWaiter.waitDuration, e.getMessage());
+
+                            adaptiveWaiter.pause((Exception) e);
                         } else {
                             LOG.warn("Error handling message", e);
 
@@ -877,6 +894,8 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
 
                     RequestContext.get().resetEntityGuidUpdates();
 
+                    entityCorrelationManager.add(context.isSpooledMessage(), context.getMsgCreated(), response.getDeletedEntities());
+
                     RequestContext.get().clearCache();
                 }
             }
@@ -902,16 +921,11 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
         }
 
         private void commit(AtlasKafkaMessage<HookNotification> kafkaMessage) {
-            boolean commitSucceessStatus = false;
-            try {
-                recordFailedMessages();
+            recordFailedMessages();
 
-                consumer.commit(kafkaMessage.getTopicPartition(), kafkaMessage.getOffset() + 1);
-
-                commitSucceessStatus = true;
-            } finally {
-                failedCommitOffsetRecorder.recordIfFailed(commitSucceessStatus, kafkaMessage.getOffset());
-            }
+            long commitOffset = kafkaMessage.getOffset() + 1;
+            lastCommittedPartitionOffset.put(kafkaMessage.getTopicPartition(), commitOffset);
+            consumer.commit(kafkaMessage.getTopicPartition(), commitOffset);
         }
 
         boolean serverAvailable(Timer timer) {
@@ -966,7 +980,9 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
         PreprocessorContext context = null;
 
         if (preprocessEnabled) {
-            context = new PreprocessorContext(kafkaMsg, typeRegistry, hiveTablesToIgnore, hiveTablesToPrune, hiveTablesCache, hiveDummyDatabasesToIgnore, hiveDummyTablesToIgnore, hiveTablePrefixesToIgnore, hiveTypesRemoveOwnedRefAttrs, rdbmsTypesRemoveOwnedRefAttrs, updateHiveProcessNameWithQualifiedName);
+            context = new PreprocessorContext(kafkaMsg, typeRegistry, hiveTablesToIgnore, hiveTablesToPrune, hiveTablesCache,
+                    hiveDummyDatabasesToIgnore, hiveDummyTablesToIgnore, hiveTablePrefixesToIgnore, hiveTypesRemoveOwnedRefAttrs,
+                    rdbmsTypesRemoveOwnedRefAttrs, updateHiveProcessNameWithQualifiedName, entityCorrelationManager);
 
             if (context.isHivePreprocessEnabled()) {
                 preprocessHiveTypes(context);
@@ -1313,24 +1329,4 @@ public class NotificationHookConsumer implements Service, ActiveStateChangeHandl
 
         return ret;
     }
-
-    static class FailedCommitOffsetRecorder {
-        private Long currentOffset;
-
-        public void recordIfFailed(boolean commitStatus, long offset) {
-            if(commitStatus) {
-                currentOffset = null;
-            } else {
-                currentOffset = offset;
-            }
-        }
-
-        public boolean isMessageReplayed(long offset) {
-            return currentOffset != null && currentOffset == offset;
-        }
-
-        public Long getCurrentOffset() {
-            return currentOffset;
-        }
-    }
 }
