@@ -333,10 +333,10 @@ public class AtlasAuthenticationFilter extends AuthenticationFilter {
             HttpServletResponse         httpResponse    = (HttpServletResponse) response;
             AtlasResponseRequestWrapper responseWrapper = new AtlasResponseRequestWrapper(httpResponse);
 
-            responseWrapper.setHeader("X-Frame-Options", "DENY");
-            responseWrapper.setHeader("X-Content-Type-Options", "nosniff");
-            responseWrapper.setHeader("X-XSS-Protection", "1; mode=block");
-            responseWrapper.setHeader("Strict-Transport-Security", "max-age=31536000; includeSubDomains");
+            HeadersUtil.setHeaderMapAttributes(responseWrapper, HeadersUtil.X_FRAME_OPTIONS_KEY);
+            HeadersUtil.setHeaderMapAttributes(responseWrapper, HeadersUtil.X_CONTENT_TYPE_OPTIONS_KEY);
+            HeadersUtil.setHeaderMapAttributes(responseWrapper, HeadersUtil.X_XSS_PROTECTION_KEY);
+            HeadersUtil.setHeaderMapAttributes(responseWrapper, HeadersUtil.STRICT_TRANSPORT_SEC_KEY);
 
             if (headerProperties != null) {
                 for (String headerKey : headerProperties.stringPropertyNames()) {
@@ -389,6 +389,7 @@ public class AtlasAuthenticationFilter extends AuthenticationFilter {
         HttpServletResponse        httpResponse         = (HttpServletResponse) response;
         boolean                    isHttps              = "https".equals(httpRequest.getScheme());
         AuthenticationHandler      authHandler          = getAuthenticationHandler();
+        String doAsUser = supportTrustedProxy ? Servlets.getDoAsUser(httpRequest) : null;
 
         try {
             boolean             newToken = false;
@@ -445,7 +446,6 @@ public class AtlasAuthenticationFilter extends AuthenticationFilter {
                     };
 
                     // Create the proxy user if doAsUser exists
-                    String doAsUser = supportTrustedProxy ? Servlets.getDoAsUser(httpRequest) : null;
 
                     if (supportTrustedProxy && doAsUser != null && !doAsUser.equals(httpRequest.getRemoteUser())) {
                         LOG.debug("doAsUser is {}", doAsUser);
@@ -507,8 +507,10 @@ public class AtlasAuthenticationFilter extends AuthenticationFilter {
                     errCode = HttpServletResponse.SC_FORBIDDEN;
                 }
 
+                boolean isKerberosOnBrowser = supportKeyTabBrowserLogin || doAsUser != null;
+
                 if (authenticationEx == null) { // added this code for atlas error handling and fallback
-                    if (!supportKeyTabBrowserLogin && isBrowser(httpRequest.getHeader("User-Agent"))) {
+                    if (!isKerberosOnBrowser && isBrowser(httpRequest.getHeader("User-Agent"))) {
                         filterChain.doFilter(request, response);
                     } else {
                         boolean            chk         = true;
