@@ -21,8 +21,8 @@ package org.apache.atlas.web.resources;
 import com.sun.jersey.multipart.FormDataParam;
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasClient;
+import org.apache.atlas.AtlasConfiguration;
 import org.apache.atlas.AtlasErrorCode;
-import org.apache.atlas.RequestContext;
 import org.apache.atlas.authorize.AtlasAdminAccessRequest;
 import org.apache.atlas.authorize.AtlasAuthorizationUtils;
 import org.apache.atlas.authorize.AtlasEntityAccessRequest;
@@ -44,9 +44,11 @@ import org.apache.atlas.model.impexp.MigrationStatus;
 import org.apache.atlas.model.instance.AtlasCheckStateRequest;
 import org.apache.atlas.model.instance.AtlasCheckStateResult;
 import org.apache.atlas.model.instance.AtlasEntityHeader;
+import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.EntityMutationResponse;
 import org.apache.atlas.model.metrics.AtlasMetrics;
 import org.apache.atlas.model.patches.AtlasPatch.AtlasPatches;
+import org.apache.atlas.model.tasks.AtlasTask;
 import org.apache.atlas.repository.audit.AtlasAuditService;
 import org.apache.atlas.repository.audit.EntityAuditRepository;
 import org.apache.atlas.repository.impexp.AtlasServerService;
@@ -58,18 +60,21 @@ import org.apache.atlas.repository.impexp.ZipSink;
 import org.apache.atlas.repository.patches.AtlasPatchManager;
 import org.apache.atlas.repository.store.graph.AtlasEntityStore;
 import org.apache.atlas.services.MetricsService;
+import org.apache.atlas.tasks.TaskManagement;
 import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.util.SearchTracker;
 import org.apache.atlas.utils.AtlasJson;
 import org.apache.atlas.utils.AtlasPerfTracer;
 import org.apache.atlas.web.filters.AtlasCSRFPreventionFilter;
+import org.apache.atlas.web.service.AtlasDebugMetricsSink;
 import org.apache.atlas.web.service.ServiceState;
 import org.apache.atlas.web.util.Servlets;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.configuration.Configuration;
 import org.apache.commons.configuration.ConfigurationException;
 import org.apache.commons.configuration.PropertiesConfiguration;
+import org.apache.commons.lang.RandomStringUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -98,10 +103,10 @@ import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 import java.io.IOException;
 import java.io.InputStream;
+import java.security.SecureRandom;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -109,6 +114,9 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TimeZone;
 import java.util.concurrent.locks.ReentrantLock;
+import java.util.stream.Collectors;
+
+import static org.apache.atlas.web.filters.AtlasCSRFPreventionFilter.CSRF_TOKEN;
 
 
 /**
@@ -131,7 +139,11 @@ public class AdminResource {
     private static final String DEFAULT_EDITABLE_ENTITY_TYPES  = "hdfs_path";
     private static final String DEFAULT_UI_VERSION             = "atlas.ui.default.version";
     private static final String UI_VERSION_V2                  = "v2";
-    private static final List TIMEZONE_LIST  = Arrays.asList(TimeZone.getAvailableIDs());
+    private static final String UI_DATE_TIMEZONE_FORMAT_ENABLED = "atlas.ui.date.timezone.format.enabled";
+    private static final String UI_DATE_FORMAT                 = "atlas.ui.date.format";
+    private static final String UI_DATE_DEFAULT_FORMAT         = "MM/DD/YYYY hh:mm:ss A";
+    private static final String OPERATION_STATUS               = "operationStatus";
+    private static final List TIMEZONE_LIST                    = Arrays.asList(TimeZone.getAvailableIDs());
 
     @Context
     private HttpServletRequest httpServletRequest;
@@ -151,12 +163,18 @@ public class AdminResource {
     private final  MigrationProgressService migrationProgressService;
     private final  ReentrantLock            importExportOperationLock;
     private final  ExportImportAuditService exportImportAuditService;
+    private final  TaskManagement           taskManagement;
     private final  AtlasServerService       atlasServerService;
     private final  AtlasEntityStore         entityStore;
     private final  AtlasPatchManager        patchManager;
     private final  AtlasAuditService        auditService;
     private final  String                   defaultUIVersion;
     private final  EntityAuditRepository    auditRepository;
+    private final  boolean                  isTimezoneFormatEnabled;
+    private final  String                   uiDateFormat;
+    private final  AtlasDebugMetricsSink    debugMetricsRESTSink;
+    private final  boolean                  isDebugMetricsEnabled;
+    private final  boolean                  isTasksEnabled;
 
     static {
         try {
@@ -172,7 +190,8 @@ public class AdminResource {
                          MigrationProgressService migrationProgressService,
                          AtlasServerService serverService,
                          ExportImportAuditService exportImportAuditService, AtlasEntityStore entityStore,
-                         AtlasPatchManager patchManager, AtlasAuditService auditService, EntityAuditRepository auditRepository) {
+                         AtlasPatchManager patchManager, AtlasAuditService auditService, EntityAuditRepository auditRepository,
+                         TaskManagement taskManagement, AtlasDebugMetricsSink debugMetricsRESTSink) {
         this.serviceState              = serviceState;
         this.metricsService            = metricsService;
         this.exportService             = exportService;
@@ -187,11 +206,21 @@ public class AdminResource {
         this.patchManager              = patchManager;
         this.auditService              = auditService;
         this.auditRepository           = auditRepository;
+        this.taskManagement            = taskManagement;
+        this.debugMetricsRESTSink      = debugMetricsRESTSink;
 
         if (atlasProperties != null) {
-            defaultUIVersion = atlasProperties.getString(DEFAULT_UI_VERSION, UI_VERSION_V2);
+            this.defaultUIVersion = atlasProperties.getString(DEFAULT_UI_VERSION, UI_VERSION_V2);
+            this.isTimezoneFormatEnabled = atlasProperties.getBoolean(UI_DATE_TIMEZONE_FORMAT_ENABLED, true);
+            this.uiDateFormat = atlasProperties.getString(UI_DATE_FORMAT, UI_DATE_DEFAULT_FORMAT);
+            this.isDebugMetricsEnabled = AtlasConfiguration.DEBUG_METRICS_ENABLED.getBoolean();
+            this.isTasksEnabled = AtlasConfiguration.TASKS_USE_ENABLED.getBoolean();
         } else {
-            defaultUIVersion = UI_VERSION_V2;
+            this.defaultUIVersion = UI_VERSION_V2;
+            this.isTimezoneFormatEnabled = true;
+            this.uiDateFormat = UI_DATE_DEFAULT_FORMAT;
+            this.isDebugMetricsEnabled = false;
+            this.isTasksEnabled = false;
         }
     }
 
@@ -301,7 +330,7 @@ public class AdminResource {
     @GET
     @Path("session")
     @Produces(Servlets.JSON_MEDIA_TYPE)
-    public Response getUserProfile() {
+    public Response getUserProfile(@Context HttpServletRequest request) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> AdminResource.getUserProfile()");
         }
@@ -337,6 +366,18 @@ public class AdminResource {
         responseData.put("userName", userName);
         responseData.put("groups", groups);
         responseData.put("timezones", TIMEZONE_LIST);
+        responseData.put(UI_DATE_TIMEZONE_FORMAT_ENABLED, isTimezoneFormatEnabled);
+        responseData.put(UI_DATE_FORMAT, uiDateFormat);
+        responseData.put(AtlasConfiguration.DEBUG_METRICS_ENABLED.getPropertyName(), isDebugMetricsEnabled);
+        responseData.put(AtlasConfiguration.TASKS_USE_ENABLED.getPropertyName(), isTasksEnabled);
+
+        String salt = (String) request.getSession().getAttribute(CSRF_TOKEN);
+        if (StringUtils.isEmpty(salt)) {
+            salt = RandomStringUtils.random(20, 0, 0, true, true, null, new SecureRandom());
+            request.getSession().setAttribute(CSRF_TOKEN, salt);
+        }
+
+        responseData.put(CSRF_TOKEN, salt);
 
         response = Response.ok(AtlasJson.toV1Json(responseData)).build();
 
@@ -378,12 +419,19 @@ public class AdminResource {
 
         AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_EXPORT), "export");
 
-        acquireExportImportLock("export");
+        boolean preventMultipleRequests = request != null && request.getOptions() != null
+                && !(request.getOptions().containsKey(AtlasExportRequest.OPTION_SKIP_LINEAGE)
+                     || request.getOptions().containsKey(AtlasExportRequest.OPTION_KEY_REPLICATED_TO));
+        if (preventMultipleRequests) {
+            acquireExportImportLock("export");
+        }
 
         ZipSink exportSink = null;
+        boolean isSuccessful = false;
+        AtlasExportResult result = null;
         try {
             exportSink = new ZipSink(httpServletResponse.getOutputStream());
-            AtlasExportResult result = exportService.run(exportSink, request, AtlasAuthorizationUtils.getCurrentUserName(),
+            result = exportService.run(exportSink, request, AtlasAuthorizationUtils.getCurrentUserName(),
                                                          Servlets.getHostName(httpServletRequest),
                                                          AtlasAuthorizationUtils.getRequestIpAddress(httpServletRequest));
 
@@ -396,18 +444,23 @@ public class AdminResource {
             httpServletResponse.setHeader("Transfer-Encoding", "chunked");
 
             httpServletResponse.getOutputStream().flush();
+            isSuccessful = true;
             return Response.ok().build();
         } catch (IOException excp) {
             LOG.error("export() failed", excp);
 
             throw new AtlasBaseException(excp);
         } finally {
-            releaseExportImportLock();
+            if (preventMultipleRequests) {
+                releaseExportImportLock();
+            }
 
             if (exportSink != null) {
                 exportSink.close();
             }
 
+            addToExportOperationAudits(isSuccessful, result);
+
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AdminResource.export()");
             }
@@ -426,11 +479,16 @@ public class AdminResource {
 
         AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_IMPORT), "importData");
 
-        acquireExportImportLock("import");
         AtlasImportResult result = null;
+        boolean preventMultipleRequests = true;
 
         try {
             AtlasImportRequest request = AtlasType.fromJson(jsonData, AtlasImportRequest.class);
+            preventMultipleRequests = request != null && request.getOptions() != null
+                    && !request.getOptions().containsKey(AtlasImportRequest.OPTION_KEY_REPLICATED_FROM);
+            if (preventMultipleRequests) {
+                acquireExportImportLock("import");
+            }
 
             result = importService.run(inputStream, request, Servlets.getUserName(httpServletRequest),
                     Servlets.getHostName(httpServletRequest),
@@ -449,13 +507,17 @@ public class AdminResource {
 
             throw new AtlasBaseException(excp);
         } finally {
-            releaseExportImportLock();
+            if (preventMultipleRequests) {
+                releaseExportImportLock();
+            }
 
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AdminResource.importData(binary)");
             }
         }
 
+        addToImportOperationAudits(result);
+
         return result;
     }
 
@@ -481,13 +543,8 @@ public class AdminResource {
 
             final List<AtlasEntityHeader> purgedEntities = resp.getPurgedEntities();
             if(purgedEntities != null && purgedEntities.size() > 0) {
-                final String clientId = RequestContext.get().getClientIPAddress();
-                final Date startTime = new Date(RequestContext.get().getRequestTime());
-                final Date endTime = new Date();
-
-                auditService.add(AtlasAuthorizationUtils.getCurrentUserName(), AuditOperation.PURGE,
-                        clientId != null ? clientId : "", startTime, endTime, guids.toString(),
-                        resp.getPurgedEntitiesIds(), resp.getPurgedEntities().size());
+                auditService.add(AuditOperation.PURGE, guids.toString(), resp.getPurgedEntitiesIds(),
+                        resp.getPurgedEntities().size());
             }
 
             return resp;
@@ -505,13 +562,18 @@ public class AdminResource {
         }
 
         AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_IMPORT), "importFile");
-
-        acquireExportImportLock("importFile");
-
+        boolean preventMultipleRequests = true;
         AtlasImportResult result;
 
         try {
             AtlasImportRequest request = AtlasType.fromJson(jsonData, AtlasImportRequest.class);
+            preventMultipleRequests = request != null && request.getOptions() != null
+                    && request.getOptions().containsKey(AtlasImportRequest.OPTION_KEY_REPLICATED_FROM);
+
+            if (preventMultipleRequests) {
+                acquireExportImportLock("importFile");
+            }
+
             result = importService.run(request, AtlasAuthorizationUtils.getCurrentUserName(),
                                        Servlets.getHostName(httpServletRequest),
                                        AtlasAuthorizationUtils.getRequestIpAddress(httpServletRequest));
@@ -528,7 +590,9 @@ public class AdminResource {
 
             throw new AtlasBaseException(excp);
         } finally {
-            releaseExportImportLock();
+            if (preventMultipleRequests) {
+                releaseExportImportLock();
+            }
 
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AdminResource.importFile()");
@@ -599,6 +663,8 @@ public class AdminResource {
                 perf = AtlasPerfTracer.getPerfTracer(PERF_LOG, "AdminResource.getAtlasAudits(" + auditSearchParameters + ")");
             }
 
+            AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_AUDITS), "Admin Audits");
+
             return auditService.get(auditSearchParameters);
         } finally {
             AtlasPerfTracer.log(perf);
@@ -700,6 +766,29 @@ public class AdminResource {
         return ret;
     }
 
+    @GET
+    @Path("/tasks")
+    @Produces(Servlets.JSON_MEDIA_TYPE)
+    public List<AtlasTask> getTaskStatus(@QueryParam("guids") List<String> guids) throws AtlasBaseException {
+        return CollectionUtils.isNotEmpty(guids) ? taskManagement.getByGuids(guids) : taskManagement.getAll();
+    }
+
+    @DELETE
+    @Path("/tasks")
+    @Produces(Servlets.JSON_MEDIA_TYPE)
+    public void deleteTask(@QueryParam("guids") List<String> guids) throws AtlasBaseException {
+        if (CollectionUtils.isNotEmpty(guids)) {
+            taskManagement.deleteByGuids(guids);
+        }
+    }
+
+    @GET
+    @Path("/debug/metrics")
+    @Produces(MediaType.APPLICATION_JSON)
+    public Map getDebugMetrics() {
+        return debugMetricsRESTSink.getMetrics();
+    }
+
     private String getEditableEntityTypes(Configuration config) {
         String ret = DEFAULT_EDITABLE_ENTITY_TYPES;
 
@@ -736,4 +825,38 @@ public class AdminResource {
 
         importExportOperationLock.lock();
     }
+
+    private void addToImportOperationAudits(AtlasImportResult result) throws AtlasBaseException {
+        List<AtlasObjectId> objectIds = result.getExportResult().getRequest().getItemsToExport();
+
+        Map<String, Object> optionMap = new HashMap<>();
+        optionMap.put(OPERATION_STATUS, result.getOperationStatus().name());
+        String params = AtlasJson.toJson(optionMap);
+
+        auditImportExportOperations(objectIds, AuditOperation.IMPORT, params);
+    }
+
+    private void addToExportOperationAudits(boolean isSuccessful, AtlasExportResult result) throws AtlasBaseException {
+        if (!isSuccessful
+                || CollectionUtils.isEmpty(result.getRequest().getItemsToExport())
+                || result.getRequest().getOptions() == null) {
+            return;
+        }
+
+        Map<String, Object> optionMap = result.getRequest().getOptions();
+        optionMap.put(OPERATION_STATUS, result.getOperationStatus().name());
+        String params = AtlasJson.toJson(optionMap);
+
+        List<AtlasObjectId> objectIds = result.getRequest().getItemsToExport();
+
+        auditImportExportOperations(objectIds, AuditOperation.EXPORT, params);
+    }
+
+    private void auditImportExportOperations(List<AtlasObjectId> objectIds, AuditOperation auditOperation, String params) throws AtlasBaseException {
+
+        Map<String, Long> entityCountByType = objectIds.stream().collect(Collectors.groupingBy(AtlasObjectId::getTypeName, Collectors.counting()));
+        int resultCount = objectIds.size();
+
+        auditService.add(auditOperation, params, AtlasJson.toJson(entityCountByType), resultCount);
+    }
 }
