@@ -20,6 +20,7 @@ package org.apache.atlas.web.rest;
 import org.apache.atlas.AtlasClient;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.SortOrder;
+import org.apache.atlas.annotation.Timed;
 import org.apache.atlas.authorize.AtlasAuthorizationUtils;
 import org.apache.atlas.discovery.AtlasDiscoveryService;
 import org.apache.atlas.discovery.EntityDiscoveryService;
@@ -100,6 +101,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("/dsl")
+    @Timed
     public AtlasSearchResult searchUsingDSL(@QueryParam("query")          String query,
                                             @QueryParam("typeName")       String typeName,
                                             @QueryParam("classification") String classification,
@@ -147,6 +149,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("/fulltext")
+    @Timed
     public AtlasSearchResult searchUsingFullText(@QueryParam("query")                  String  query,
                                                  @QueryParam("excludeDeletedEntities") boolean excludeDeletedEntities,
                                                  @QueryParam("limit")                  int     limit,
@@ -186,6 +189,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("/basic")
+    @Timed
     public AtlasSearchResult searchUsingBasic(@QueryParam("query")                  String  query,
                                               @QueryParam("typeName")               String  typeName,
                                               @QueryParam("classification")         String  classification,
@@ -193,7 +197,8 @@ public class DiscoveryREST {
                                               @QueryParam("sortOrder")              SortOrder sortOrder,
                                               @QueryParam("excludeDeletedEntities") boolean excludeDeletedEntities,
                                               @QueryParam("limit")                  int     limit,
-                                              @QueryParam("offset")                 int     offset) throws AtlasBaseException {
+                                              @QueryParam("offset")                 int     offset,
+                                              @QueryParam("marker")                 String  marker) throws AtlasBaseException {
         Servlets.validateQueryParamLength("typeName", typeName);
         Servlets.validateQueryParamLength("classification", classification);
         Servlets.validateQueryParamLength("sortBy", sortByAttribute);
@@ -216,6 +221,7 @@ public class DiscoveryREST {
             searchParameters.setExcludeDeletedEntities(excludeDeletedEntities);
             searchParameters.setLimit(limit);
             searchParameters.setOffset(offset);
+            searchParameters.setMarker(marker);
             searchParameters.setSortBy(sortByAttribute);
             searchParameters.setSortOrder(sortOrder);
 
@@ -241,6 +247,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("/attribute")
+    @Timed
     public AtlasSearchResult searchUsingAttribute(@QueryParam("attrName")        String attrName,
                                                   @QueryParam("attrValuePrefix") String attrValuePrefix,
                                                   @QueryParam("typeName")        String typeName,
@@ -314,6 +321,7 @@ public class DiscoveryREST {
      */
     @Path("basic")
     @POST
+    @Timed
     public AtlasSearchResult searchWithParameters(SearchParameters parameters) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
@@ -326,9 +334,16 @@ public class DiscoveryREST {
                 throw new AtlasBaseException(AtlasErrorCode.BAD_REQUEST, "Limit/offset should be non-negative");
             }
 
+            if (StringUtils.isEmpty(parameters.getTypeName()) && !isEmpty(parameters.getEntityFilters())) {
+                throw new AtlasBaseException(AtlasErrorCode.BAD_REQUEST, "EntityFilters specified without Type name");
+            }
+
+            if (StringUtils.isEmpty(parameters.getClassification()) && !isEmpty(parameters.getTagFilters())) {
+                throw new AtlasBaseException(AtlasErrorCode.BAD_REQUEST, "TagFilters specified without tag name");
+            }
+
             if (StringUtils.isEmpty(parameters.getTypeName()) && StringUtils.isEmpty(parameters.getClassification()) &&
-                StringUtils.isEmpty(parameters.getQuery()) && StringUtils.isEmpty(parameters.getTermName()) &&
-                isEmpty(parameters.getEntityFilters()) && isEmpty(parameters.getTagFilters())) {
+                StringUtils.isEmpty(parameters.getQuery()) && StringUtils.isEmpty(parameters.getTermName())) {
                 throw new AtlasBaseException(AtlasErrorCode.INVALID_SEARCH_PARAMS);
             }
 
@@ -357,14 +372,17 @@ public class DiscoveryREST {
      */
     @GET
     @Path("relationship")
-    public AtlasSearchResult searchRelatedEntities(@QueryParam("guid")                   String      guid,
-                                                   @QueryParam("relation")               String      relation,
-                                                   @QueryParam("attributes")             Set<String> attributes,
-                                                   @QueryParam("sortBy")                 String      sortByAttribute,
-                                                   @QueryParam("sortOrder")              SortOrder   sortOrder,
-                                                   @QueryParam("excludeDeletedEntities") boolean     excludeDeletedEntities,
-                                                   @QueryParam("limit")                  int         limit,
-                                                   @QueryParam("offset")                 int         offset) throws AtlasBaseException {
+    @Timed
+    public AtlasSearchResult searchRelatedEntities(@QueryParam("guid")                            String      guid,
+                                                   @QueryParam("relation")                        String      relation,
+                                                   @QueryParam("attributes")                      Set<String> attributes,
+                                                   @QueryParam("sortBy")                          String      sortByAttribute,
+                                                   @QueryParam("sortOrder")                       SortOrder   sortOrder,
+                                                   @QueryParam("excludeDeletedEntities")          boolean     excludeDeletedEntities,
+                                                   @QueryParam("includeClassificationAttributes") boolean     includeClassificationAttributes,
+                                                   @QueryParam("getApproximateCount")             boolean     getApproximateCount,
+                                                   @QueryParam("limit")                           int         limit,
+                                                   @QueryParam("offset")                          int         offset) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
         Servlets.validateQueryParamLength("relation", relation);
         Servlets.validateQueryParamLength("sortBy", sortByAttribute);
@@ -373,11 +391,20 @@ public class DiscoveryREST {
 
         try {
             if (AtlasPerfTracer.isPerfTraceEnabled(PERF_LOG)) {
-                perf = AtlasPerfTracer.getPerfTracer(PERF_LOG, "DiscoveryREST.relatedEntitiesSearchUsingGremlin(" + guid +
-                        ", " + relation + ", " + sortByAttribute + ", " + sortOrder + ", " + excludeDeletedEntities + ", " + ", " + limit + ", " + offset + ")");
+                perf = AtlasPerfTracer.getPerfTracer(PERF_LOG, "DiscoveryREST.relatedEntitiesSearch(" + guid +
+                        ", " + relation + ", " + sortByAttribute + ", " + sortOrder + ", " + excludeDeletedEntities + ", " + getApproximateCount + ", " + limit + ", " + offset + ")");
             }
 
-            return discoveryService.searchRelatedEntities(guid, relation, attributes, sortByAttribute, sortOrder, excludeDeletedEntities, limit, offset);
+            SearchParameters parameters = new SearchParameters();
+            parameters.setAttributes(attributes);
+            parameters.setSortBy(sortByAttribute);
+            parameters.setSortOrder(sortOrder);
+            parameters.setExcludeDeletedEntities(excludeDeletedEntities);
+            parameters.setLimit(limit);
+            parameters.setOffset(offset);
+            parameters.setIncludeClassificationAttributes(includeClassificationAttributes);
+            return discoveryService.searchRelatedEntities(guid, relation, getApproximateCount, parameters);
+
         } finally {
             AtlasPerfTracer.log(perf);
         }
@@ -391,6 +418,7 @@ public class DiscoveryREST {
      */
     @POST
     @Path("saved")
+    @Timed
     public AtlasUserSavedSearch addSavedSearch(AtlasUserSavedSearch savedSearch) throws AtlasBaseException, IOException {
         validateUserSavedSearch(savedSearch);
 
@@ -415,6 +443,7 @@ public class DiscoveryREST {
      */
     @PUT
     @Path("saved")
+    @Timed
     public AtlasUserSavedSearch updateSavedSearch(AtlasUserSavedSearch savedSearch) throws AtlasBaseException {
         validateUserSavedSearch(savedSearch);
 
@@ -440,6 +469,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("saved/{name}")
+    @Timed
     public AtlasUserSavedSearch getSavedSearch(@PathParam("name") String searchName,
                                                @QueryParam("user") String userName) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", searchName);
@@ -466,6 +496,7 @@ public class DiscoveryREST {
      */
     @GET
     @Path("saved")
+    @Timed
     public List<AtlasUserSavedSearch> getSavedSearches(@QueryParam("user") String userName) throws AtlasBaseException {
         Servlets.validateQueryParamLength("user", userName);
 
@@ -487,6 +518,7 @@ public class DiscoveryREST {
      */
     @DELETE
     @Path("saved/{guid}")
+    @Timed
     public void deleteSavedSearch(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -514,6 +546,7 @@ public class DiscoveryREST {
      */
     @Path("saved/execute/{name}")
     @GET
+    @Timed
     public AtlasSearchResult executeSavedSearchByName(@PathParam("name") String searchName,
                                                       @QueryParam("user") String userName) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", searchName);
@@ -544,6 +577,7 @@ public class DiscoveryREST {
      */
     @Path("saved/execute/guid/{guid}")
     @GET
+    @Timed
     public AtlasSearchResult executeSavedSearchByGuid(@PathParam("guid") String searchGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", searchGuid);
 
@@ -571,6 +605,7 @@ public class DiscoveryREST {
      */
     @Path("/quick")
     @GET
+    @Timed
     public AtlasQuickSearchResult quickSearch(@QueryParam("query")                  String  query,
                                               @QueryParam("typeName")               String  typeName,
                                               @QueryParam("excludeDeletedEntities") boolean excludeDeletedEntities,
@@ -615,6 +650,7 @@ public class DiscoveryREST {
      */
     @Path("/quick")
     @POST
+    @Timed
     public AtlasQuickSearchResult quickSearch(QuickSearchParameters quickSearchParameters) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
@@ -647,6 +683,7 @@ public class DiscoveryREST {
 
     @Path("suggestions")
     @GET
+    @Timed
     public AtlasSuggestionsResult getSuggestions(@QueryParam("prefixString") String prefixString, @QueryParam("fieldName") String fieldName) {
         AtlasPerfTracer perf = null;
 
