@@ -17,10 +17,15 @@
  */
 package org.apache.atlas.web.rest;
 
+import com.sun.jersey.core.header.FormDataContentDisposition;
+import com.sun.jersey.multipart.FormDataParam;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.SortOrder;
+import org.apache.atlas.annotation.Timed;
+import org.apache.atlas.bulkimport.BulkImportResponse;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.glossary.GlossaryService;
+import org.apache.atlas.glossary.GlossaryTermUtils;
 import org.apache.atlas.model.glossary.AtlasGlossary;
 import org.apache.atlas.model.glossary.AtlasGlossaryCategory;
 import org.apache.atlas.model.glossary.AtlasGlossaryTerm;
@@ -35,8 +40,22 @@ import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Service;
 
 import javax.inject.Inject;
-import javax.ws.rs.*;
+import javax.ws.rs.Consumes;
+import javax.ws.rs.DELETE;
+import javax.ws.rs.DefaultValue;
+import javax.ws.rs.GET;
+import javax.ws.rs.POST;
+import javax.ws.rs.PUT;
+import javax.ws.rs.Path;
+import javax.ws.rs.PathParam;
+import javax.ws.rs.Produces;
+import javax.ws.rs.QueryParam;
+import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.MediaType;
+import javax.ws.rs.core.StreamingOutput;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
@@ -67,6 +86,7 @@ public class GlossaryREST {
      * @throws AtlasBaseException
      */
     @GET
+    @Timed
     public List<AtlasGlossary> getGlossaries(@DefaultValue("-1") @QueryParam("limit") final String limit,
                                              @DefaultValue("0") @QueryParam("offset") final String offset,
                                              @DefaultValue("ASC") @QueryParam("sort") final String sort) throws AtlasBaseException {
@@ -94,6 +114,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}")
+    @Timed
     public AtlasGlossary getGlossary(@PathParam("glossaryGuid") String glossaryGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("glossaryGuid", glossaryGuid);
 
@@ -125,6 +146,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}/detailed")
+    @Timed
     public AtlasGlossary.AtlasGlossaryExtInfo getDetailedGlossary(@PathParam("glossaryGuid") String glossaryGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("glossaryGuid", glossaryGuid);
 
@@ -156,6 +178,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/term/{termGuid}")
+    @Timed
     public AtlasGlossaryTerm getGlossaryTerm(@PathParam("termGuid") String termGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("termGuid", termGuid);
 
@@ -186,6 +209,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/category/{categoryGuid}")
+    @Timed
     public AtlasGlossaryCategory getGlossaryCategory(@PathParam("categoryGuid") String categoryGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("categoryGuid", categoryGuid);
         AtlasPerfTracer perf = null;
@@ -216,6 +240,7 @@ public class GlossaryREST {
      * @HTTP 409 If Glossary definition already exists (duplicate qualifiedName)
      */
     @POST
+    @Timed
     public AtlasGlossary createGlossary(AtlasGlossary atlasGlossary) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
         try {
@@ -240,6 +265,7 @@ public class GlossaryREST {
      */
     @POST
     @Path("/term")
+    @Timed
     public AtlasGlossaryTerm createGlossaryTerm(AtlasGlossaryTerm glossaryTerm) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
         try {
@@ -265,6 +291,7 @@ public class GlossaryREST {
      */
     @POST
     @Path("/terms")
+    @Timed
     public List<AtlasGlossaryTerm> createGlossaryTerms(List<AtlasGlossaryTerm> glossaryTerm) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
         try {
@@ -294,6 +321,7 @@ public class GlossaryREST {
      */
     @POST
     @Path("/category")
+    @Timed
     public AtlasGlossaryCategory createGlossaryCategory(AtlasGlossaryCategory glossaryCategory) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
         try {
@@ -319,6 +347,7 @@ public class GlossaryREST {
      */
     @POST
     @Path("/categories")
+    @Timed
     public List<AtlasGlossaryCategory> createGlossaryCategories(List<AtlasGlossaryCategory> glossaryCategory) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
         try {
@@ -349,6 +378,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/{glossaryGuid}")
+    @Timed
     public AtlasGlossary updateGlossary(@PathParam("glossaryGuid") String glossaryGuid, AtlasGlossary updatedGlossary) throws AtlasBaseException {
         Servlets.validateQueryParamLength("glossaryGuid", glossaryGuid);
         AtlasPerfTracer perf = null;
@@ -375,6 +405,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/{glossaryGuid}/partial")
+    @Timed
     public AtlasGlossary partialUpdateGlossary(@PathParam("glossaryGuid") String glossaryGuid, Map<String, String> partialUpdates) throws AtlasBaseException {
         Servlets.validateQueryParamLength("glossaryGuid", glossaryGuid);
 
@@ -414,6 +445,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/term/{termGuid}")
+    @Timed
     public AtlasGlossaryTerm updateGlossaryTerm(@PathParam("termGuid") String termGuid, AtlasGlossaryTerm glossaryTerm) throws AtlasBaseException {
         Servlets.validateQueryParamLength("termGuid", termGuid);
         AtlasPerfTracer perf = null;
@@ -440,6 +472,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/term/{termGuid}/partial")
+    @Timed
     public AtlasGlossaryTerm partialUpdateGlossaryTerm(@PathParam("termGuid") String termGuid, Map<String, String> partialUpdates) throws AtlasBaseException {
         Servlets.validateQueryParamLength("termGuid", termGuid);
 
@@ -479,6 +512,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/category/{categoryGuid}")
+    @Timed
     public AtlasGlossaryCategory updateGlossaryCategory(@PathParam("categoryGuid") String categoryGuid, AtlasGlossaryCategory glossaryCategory) throws AtlasBaseException {
         Servlets.validateQueryParamLength("categoryGuid", categoryGuid);
         AtlasPerfTracer perf = null;
@@ -505,6 +539,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/category/{categoryGuid}/partial")
+    @Timed
     public AtlasGlossaryCategory partialUpdateGlossaryCategory(@PathParam("categoryGuid") String categoryGuid, Map<String, String> partialUpdates) throws AtlasBaseException {
         Servlets.validateQueryParamLength("categoryGuid", categoryGuid);
 
@@ -541,6 +576,7 @@ public class GlossaryREST {
      */
     @DELETE
     @Path("/{glossaryGuid}")
+    @Timed
     public void deleteGlossary(@PathParam("glossaryGuid") String glossaryGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("glossaryGuid", glossaryGuid);
         AtlasPerfTracer perf = null;
@@ -563,6 +599,7 @@ public class GlossaryREST {
      */
     @DELETE
     @Path("/term/{termGuid}")
+    @Timed
     public void deleteGlossaryTerm(@PathParam("termGuid") String termGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("termGuid", termGuid);
         AtlasPerfTracer perf = null;
@@ -585,6 +622,7 @@ public class GlossaryREST {
      */
     @DELETE
     @Path("/category/{categoryGuid}")
+    @Timed
     public void deleteGlossaryCategory(@PathParam("categoryGuid") String categoryGuid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("categoryGuid", categoryGuid);
         AtlasPerfTracer perf = null;
@@ -611,6 +649,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}/terms")
+    @Timed
     public List<AtlasGlossaryTerm> getGlossaryTerms(@PathParam("glossaryGuid") String glossaryGuid,
                                                          @DefaultValue("-1") @QueryParam("limit") String limit,
                                                          @DefaultValue("0") @QueryParam("offset") String offset,
@@ -642,6 +681,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}/terms/headers")
+    @Timed
     public List<AtlasRelatedTermHeader> getGlossaryTermHeaders(@PathParam("glossaryGuid") String glossaryGuid,
                                                          @DefaultValue("-1") @QueryParam("limit") String limit,
                                                          @DefaultValue("0") @QueryParam("offset") String offset,
@@ -673,6 +713,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}/categories")
+    @Timed
     public List<AtlasGlossaryCategory> getGlossaryCategories(@PathParam("glossaryGuid") String glossaryGuid,
                                                                   @DefaultValue("-1") @QueryParam("limit") String limit,
                                                                   @DefaultValue("0") @QueryParam("offset") String offset,
@@ -704,6 +745,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/{glossaryGuid}/categories/headers")
+    @Timed
     public List<AtlasRelatedCategoryHeader> getGlossaryCategoriesHeaders(@PathParam("glossaryGuid") String glossaryGuid,
                                                                   @DefaultValue("-1") @QueryParam("limit") String limit,
                                                                   @DefaultValue("0") @QueryParam("offset") String offset,
@@ -735,6 +777,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/category/{categoryGuid}/terms")
+    @Timed
     public List<AtlasRelatedTermHeader> getCategoryTerms(@PathParam("categoryGuid") String categoryGuid,
                                                          @DefaultValue("-1") @QueryParam("limit") String limit,
                                                          @DefaultValue("0") @QueryParam("offset") String offset,
@@ -767,6 +810,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/terms/{termGuid}/related")
+    @Timed
     public Map<AtlasGlossaryTerm.Relation, Set<AtlasRelatedTermHeader>> getRelatedTerms(@PathParam("termGuid") String termGuid,
                                                                                         @DefaultValue("-1") @QueryParam("limit") String limit,
                                                                                         @DefaultValue("0") @QueryParam("offset") String offset,
@@ -799,6 +843,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/terms/{termGuid}/assignedEntities")
+    @Timed
     public List<AtlasRelatedObjectId> getEntitiesAssignedWithTerm(@PathParam("termGuid") String termGuid,
                                                                   @DefaultValue("-1") @QueryParam("limit") String limit,
                                                                   @DefaultValue("0") @QueryParam("offset") String offset,
@@ -830,6 +875,7 @@ public class GlossaryREST {
      */
     @POST
     @Path("/terms/{termGuid}/assignedEntities")
+    @Timed
     public void assignTermToEntities(@PathParam("termGuid") String termGuid, List<AtlasRelatedObjectId> relatedObjectIds) throws AtlasBaseException {
         Servlets.validateQueryParamLength("termGuid", termGuid);
 
@@ -856,6 +902,7 @@ public class GlossaryREST {
      */
     @DELETE
     @Path("/terms/{termGuid}/assignedEntities")
+    @Timed
     public void removeTermAssignmentFromEntities(@PathParam("termGuid") String termGuid, List<AtlasRelatedObjectId> relatedObjectIds) throws AtlasBaseException {
         removeTermFromGlossary(termGuid, relatedObjectIds);
     }
@@ -872,6 +919,7 @@ public class GlossaryREST {
      */
     @PUT
     @Path("/terms/{termGuid}/assignedEntities")
+    @Timed
     public void disassociateTermAssignmentFromEntities(@PathParam("termGuid") String termGuid, List<AtlasRelatedObjectId> relatedObjectIds) throws AtlasBaseException {
         removeTermFromGlossary(termGuid, relatedObjectIds);
     }
@@ -906,6 +954,7 @@ public class GlossaryREST {
      */
     @GET
     @Path("/category/{categoryGuid}/related")
+    @Timed
     public Map<String, List<AtlasRelatedCategoryHeader>> getRelatedCategories(@PathParam("categoryGuid") String categoryGuid,
                                                                          @DefaultValue("-1") @QueryParam("limit") String limit,
                                                                          @DefaultValue("0") @QueryParam("offset") String offset,
@@ -934,4 +983,41 @@ public class GlossaryREST {
         return ret;
     }
 
-}
+    /**
+     * Get sample template for uploading/creating bulk AtlasGlossaryTerm
+     *
+     * @return Template File
+     * @HTTP 400 If the provided fileType is not supported
+     */
+    @GET
+    @Path("/import/template")
+    @Produces(MediaType.APPLICATION_OCTET_STREAM)
+    public StreamingOutput produceTemplate() {
+        return new StreamingOutput() {
+            @Override
+            public void write(OutputStream outputStream) throws IOException, WebApplicationException {
+                outputStream.write(GlossaryTermUtils.getGlossaryTermHeaders().getBytes());
+            }
+        };
+    }
+
+    /**
+     * Upload glossary file for creating AtlasGlossaryTerms in bulk
+     *
+     * @param inputStream InputStream of file
+     * @param fileDetail  FormDataContentDisposition metadata of file
+     * @return
+     * @throws AtlasBaseException
+     * @HTTP 200 If glossary term creation was successful
+     * @HTTP 400 If Glossary term definition has invalid or missing information
+     * @HTTP 409 If Glossary term already exists (duplicate qualifiedName)
+     */
+    @POST
+    @Path("/import")
+    @Consumes(MediaType.MULTIPART_FORM_DATA)
+    @Timed
+    public BulkImportResponse importGlossaryData(@FormDataParam("file") InputStream inputStream,
+                                                 @FormDataParam("file") FormDataContentDisposition fileDetail) throws AtlasBaseException {
+        return glossaryService.importGlossaryData(inputStream, fileDetail.getFileName());
+    }
+}
\ No newline at end of file
