@@ -17,6 +17,7 @@
  */
 package org.apache.atlas.web.rest;
 
+import org.apache.atlas.annotation.Timed;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.model.SearchFilter;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
@@ -81,6 +82,7 @@ public class TypesREST {
      */
     @GET
     @Path("/typedef/name/{name}")
+    @Timed
     public AtlasBaseTypeDef getTypeDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -98,6 +100,7 @@ public class TypesREST {
      */
     @GET
     @Path("/typedef/guid/{guid}")
+    @Timed
     public AtlasBaseTypeDef getTypeDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -115,6 +118,7 @@ public class TypesREST {
      */
     @GET
     @Path("/typedefs/headers")
+    @Timed
     public List<AtlasTypeDefHeader> getTypeDefHeaders(@Context HttpServletRequest httpServletRequest) throws AtlasBaseException {
         SearchFilter searchFilter = getSearchFilter(httpServletRequest);
 
@@ -131,6 +135,7 @@ public class TypesREST {
      */
     @GET
     @Path("/typedefs")
+    @Timed
     public AtlasTypesDef getAllTypeDefs(@Context HttpServletRequest httpServletRequest) throws AtlasBaseException {
         SearchFilter searchFilter = getSearchFilter(httpServletRequest);
 
@@ -149,6 +154,7 @@ public class TypesREST {
      */
     @GET
     @Path("/enumdef/name/{name}")
+    @Timed
     public AtlasEnumDef getEnumDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -167,6 +173,7 @@ public class TypesREST {
      */
     @GET
     @Path("/enumdef/guid/{guid}")
+    @Timed
     public AtlasEnumDef getEnumDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -186,6 +193,7 @@ public class TypesREST {
      */
     @GET
     @Path("/structdef/name/{name}")
+    @Timed
     public AtlasStructDef getStructDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -204,6 +212,7 @@ public class TypesREST {
      */
     @GET
     @Path("/structdef/guid/{guid}")
+    @Timed
     public AtlasStructDef getStructDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -222,6 +231,7 @@ public class TypesREST {
      */
     @GET
     @Path("/classificationdef/name/{name}")
+    @Timed
     public AtlasClassificationDef getClassificationDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -240,6 +250,7 @@ public class TypesREST {
      */
     @GET
     @Path("/classificationdef/guid/{guid}")
+    @Timed
     public AtlasClassificationDef getClassificationDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -258,6 +269,7 @@ public class TypesREST {
      */
     @GET
     @Path("/entitydef/name/{name}")
+    @Timed
     public AtlasEntityDef getEntityDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -276,6 +288,7 @@ public class TypesREST {
      */
     @GET
     @Path("/entitydef/guid/{guid}")
+    @Timed
     public AtlasEntityDef getEntityDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -293,6 +306,7 @@ public class TypesREST {
      */
     @GET
     @Path("/relationshipdef/name/{name}")
+    @Timed
     public AtlasRelationshipDef getRelationshipDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -311,6 +325,7 @@ public class TypesREST {
      */
     @GET
     @Path("/relationshipdef/guid/{guid}")
+    @Timed
     public AtlasRelationshipDef getRelationshipDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -329,6 +344,7 @@ public class TypesREST {
      */
     @GET
     @Path("/businessmetadatadef/guid/{guid}")
+    @Timed
     public AtlasBusinessMetadataDef getBusinessMetadataDefByGuid(@PathParam("guid") String guid) throws AtlasBaseException {
         Servlets.validateQueryParamLength("guid", guid);
 
@@ -347,6 +363,7 @@ public class TypesREST {
      */
     @GET
     @Path("/businessmetadatadef/name/{name}")
+    @Timed
     public AtlasBusinessMetadataDef getBusinessMetadataDefByName(@PathParam("name") String name) throws AtlasBaseException {
         Servlets.validateQueryParamLength("name", name);
 
@@ -369,6 +386,7 @@ public class TypesREST {
      */
     @POST
     @Path("/typedefs")
+    @Timed
     public AtlasTypesDef createAtlasTypeDefs(final AtlasTypesDef typesDef) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
@@ -395,6 +413,7 @@ public class TypesREST {
     @PUT
     @Path("/typedefs")
     @Experimental
+    @Timed
     public AtlasTypesDef updateAtlasTypeDefs(final AtlasTypesDef typesDef) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
@@ -420,6 +439,7 @@ public class TypesREST {
     @DELETE
     @Path("/typedefs")
     @Experimental
+    @Timed
     public void deleteAtlasTypeDefs(final AtlasTypesDef typesDef) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
@@ -446,6 +466,7 @@ public class TypesREST {
      */
     @DELETE
     @Path("/typedef/name/{typeName}")
+    @Timed
     public void deleteAtlasTypeByName(@PathParam("typeName") final String typeName) throws AtlasBaseException {
         AtlasPerfTracer perf = null;
 
