@@ -18,9 +18,9 @@
 
 package org.apache.atlas.examples;
 
-import org.apache.atlas.AtlasClient;
 import org.apache.atlas.AtlasServiceException;
 import org.apache.atlas.model.instance.AtlasClassification;
+import org.apache.atlas.model.instance.AtlasClassification.AtlasClassifications;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntityHeader;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntitiesWithExtInfo;
@@ -32,7 +32,6 @@ import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
@@ -50,27 +49,30 @@ public class QuickStartV2IT extends BaseResourceIT {
     @BeforeClass
     public void runQuickStart() throws Exception {
         super.setUp();
+
         QuickStartV2.runQuickstart(new String[]{}, new String[]{"admin", "admin"});
     }
 
     @Test
     public void testDBIsAdded() throws Exception {
-        AtlasEntity db = getDB(QuickStartV2.SALES_DB);
+        AtlasEntity         db           = getDB(SALES_DB);
         Map<String, Object> dbAttributes = db.getAttributes();
-        assertEquals(QuickStartV2.SALES_DB, dbAttributes.get("name"));
+
+        assertEquals(SALES_DB, dbAttributes.get("name"));
         assertEquals("sales database", dbAttributes.get("description"));
     }
 
     private AtlasEntity getDB(String dbName) throws AtlasServiceException {
-        Map<String, String> attributes = new HashMap<>();
-        attributes.put("name", dbName);
-        AtlasEntity dbEntity = atlasClientV2.getEntityByAttribute(QuickStartV2.DATABASE_TYPE, attributes).getEntity();
+        Map<String, String> attributes = Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, dbName + CLUSTER_SUFFIX);
+        AtlasEntity         dbEntity   = atlasClientV2.getEntityByAttribute(QuickStartV2.DATABASE_TYPE, attributes).getEntity();
+
         return dbEntity;
     }
 
     @Test
     public void testTablesAreAdded() throws AtlasServiceException {
-        AtlasEntity table = getTable(QuickStart.SALES_FACT_TABLE);
+        AtlasEntity table = getTable(SALES_FACT_TABLE);
+
         verifySimpleTableAttributes(table);
 
         verifyDBIsLinkedToTable(table);
@@ -94,9 +96,9 @@ public class QuickStartV2IT extends BaseResourceIT {
     }
 
     private AtlasEntity getTable(String tableName) throws AtlasServiceException {
-        Map<String, String> attributes = new HashMap<>();
-        attributes.put(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, tableName);
-        AtlasEntity tableEntity = atlasClientV2.getEntityByAttribute(QuickStartV2.TABLE_TYPE, attributes).getEntity();
+        Map<String, String> attributes  = Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, tableName + CLUSTER_SUFFIX);
+        AtlasEntity         tableEntity = atlasClientV2.getEntityByAttribute(QuickStartV2.TABLE_TYPE, attributes).getEntity();
+
         return tableEntity;
     }
 
@@ -104,67 +106,72 @@ public class QuickStartV2IT extends BaseResourceIT {
         List<Map<String, String>> attributesList = new ArrayList<>();
 
         for (String tableName : tableNames) {
-            attributesList.add(Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, tableName));
+            attributesList.add(Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, tableName + CLUSTER_SUFFIX));
         }
 
         return atlasClientV2.getEntitiesByAttribute(QuickStartV2.TABLE_TYPE, attributesList);
     }
 
     private AtlasEntity getProcess(String processName) throws AtlasServiceException {
-        Map<String, String> attributes = new HashMap<>();
-        attributes.put(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, processName);
-        AtlasEntity processEntity = atlasClientV2.getEntityByAttribute(QuickStartV2.LOAD_PROCESS_TYPE, attributes).getEntity();
+        Map<String, String> attributes    = Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, processName + CLUSTER_SUFFIX);
+        AtlasEntity         processEntity = atlasClientV2.getEntityByAttribute(LOAD_PROCESS_TYPE, attributes).getEntity();
+
         return processEntity;
     }
 
 
     private void verifyTrait(AtlasEntity table) throws AtlasServiceException {
-        AtlasClassification.AtlasClassifications classfications = atlasClientV2.getClassifications(table.getGuid());
-        List<AtlasClassification> traits = classfications.getList();
+        AtlasClassifications      classfications = atlasClientV2.getClassifications(table.getGuid());
+        List<AtlasClassification> traits         = classfications.getList();
+
         assertNotNull(traits.get(0).getTypeName());
     }
 
     private void verifyColumnsAreAddedToTable(AtlasEntity table) {
-        Map<String, Object> tableAttributes = table.getAttributes();
-        List<Map> columns = (List<Map>) tableAttributes.get("columns");
+        Map<String, Object> tableAttributes = table.getRelationshipAttributes();
+        List<Map>           columns         = (List<Map>) tableAttributes.get("columns");
+
         assertEquals(4, columns.size());
 
         for (Map colMap : columns) {
             String colGuid = (String) colMap.get("guid");
+
             assertNotNull(UUID.fromString(colGuid));
         }
     }
 
     private void verifyDBIsLinkedToTable(AtlasEntity table) throws AtlasServiceException {
-        AtlasEntity db = getDB(QuickStartV2.SALES_DB);
-        Map<String, Object> tableAttributes = table.getAttributes();
-        Map dbFromTable = (Map) tableAttributes.get("db");
+        AtlasEntity         db              = getDB(SALES_DB);
+        Map<String, Object> tableAttributes = table.getRelationshipAttributes();
+        Map                 dbFromTable     = (Map) tableAttributes.get("db");
+
         assertEquals(db.getGuid(), dbFromTable.get("guid"));
     }
 
     private void verifySimpleTableAttributes(AtlasEntity table) {
         Map<String, Object> tableAttributes = table.getAttributes();
-        assertEquals(QuickStartV2.SALES_FACT_TABLE, tableAttributes.get("name"));
+
+        assertEquals(SALES_FACT_TABLE, tableAttributes.get("name"));
         assertEquals("sales fact table", tableAttributes.get("description"));
     }
 
     @Test
     public void testProcessIsAdded() throws AtlasServiceException {
-        Map<String, String> attributes = new HashMap<>();
-        attributes.put(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, QuickStartV2.LOAD_SALES_DAILY_PROCESS);
-        AtlasEntity loadProcess = atlasClientV2.getEntityByAttribute(QuickStartV2.LOAD_PROCESS_TYPE, attributes).getEntity();
+        Map<String, String> attributes         = Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, LOAD_SALES_DAILY_PROCESS + CLUSTER_SUFFIX);
+        AtlasEntity         loadProcess        = atlasClientV2.getEntityByAttribute(LOAD_PROCESS_TYPE, attributes).getEntity();
+        Map                 loadProcessAttribs = loadProcess.getAttributes();
 
-        Map loadProcessAttribs = loadProcess.getAttributes();
-        assertEquals(QuickStartV2.LOAD_SALES_DAILY_PROCESS, loadProcessAttribs.get(AtlasClient.NAME));
+        assertEquals(LOAD_SALES_DAILY_PROCESS, loadProcessAttribs.get(NAME));
         assertEquals("hive query for daily summary", loadProcessAttribs.get("description"));
 
         List inputs = (List) loadProcessAttribs.get("inputs");
         List outputs = (List) loadProcessAttribs.get("outputs");
+
         assertEquals(2, inputs.size());
 
-        String salesFactTableId = getTableId(QuickStartV2.SALES_FACT_TABLE);
-        String timeDimTableId = getTableId(QuickStartV2.TIME_DIM_TABLE);
-        String salesFactDailyMVId = getTableId(QuickStartV2.SALES_FACT_DAILY_MV_TABLE);
+        String salesFactTableId   = getTableId(SALES_FACT_TABLE);
+        String timeDimTableId     = getTableId(TIME_DIM_TABLE);
+        String salesFactDailyMVId = getTableId(SALES_FACT_DAILY_MV_TABLE);
 
         assertEquals(salesFactTableId, ((Map) inputs.get(0)).get("guid"));
         assertEquals(timeDimTableId, ((Map) inputs.get(1)).get("guid"));
@@ -181,16 +188,16 @@ public class QuickStartV2IT extends BaseResourceIT {
 
     @Test
     public void testLineageIsMaintained() throws AtlasServiceException {
-        String salesFactTableId      = getTableId(QuickStartV2.SALES_FACT_TABLE);
-        String timeDimTableId        = getTableId(QuickStartV2.TIME_DIM_TABLE);
-        String salesFactDailyMVId    = getTableId(QuickStartV2.SALES_FACT_DAILY_MV_TABLE);
-        String salesFactMonthlyMvId  = getTableId(QuickStartV2.SALES_FACT_MONTHLY_MV_TABLE);
-        String salesDailyProcessId   = getProcessId(QuickStartV2.LOAD_SALES_DAILY_PROCESS);
-        String salesMonthlyProcessId = getProcessId(QuickStartV2.LOAD_SALES_MONTHLY_PROCESS);
+        String salesFactTableId      = getTableId(SALES_FACT_TABLE);
+        String timeDimTableId        = getTableId(TIME_DIM_TABLE);
+        String salesFactDailyMVId    = getTableId(SALES_FACT_DAILY_MV_TABLE);
+        String salesFactMonthlyMvId  = getTableId(SALES_FACT_MONTHLY_MV_TABLE);
+        String salesDailyProcessId   = getProcessId(LOAD_SALES_DAILY_PROCESS);
+        String salesMonthlyProcessId = getProcessId(LOAD_SALES_MONTHLY_PROCESS);
 
-        AtlasLineageInfo inputLineage = atlasClientV2.getLineageInfo(salesFactDailyMVId, LineageDirection.BOTH, 0);
-        List<LineageRelation> relations = new ArrayList<>(inputLineage.getRelations());
-        Map<String, AtlasEntityHeader> entityMap = inputLineage.getGuidEntityMap();
+        AtlasLineageInfo               inputLineage = atlasClientV2.getLineageInfo(salesFactDailyMVId, LineageDirection.BOTH, 0);
+        List<LineageRelation>          relations    = new ArrayList<>(inputLineage.getRelations());
+        Map<String, AtlasEntityHeader> entityMap    = inputLineage.getGuidEntityMap();
 
         assertEquals(relations.size(), 5);
         assertEquals(entityMap.size(), 6);
@@ -205,15 +212,17 @@ public class QuickStartV2IT extends BaseResourceIT {
 
     @Test
     public void testViewIsAdded() throws AtlasServiceException {
-        Map<String, String> attributes = new HashMap<>();
-        attributes.put(AtlasClient.REFERENCEABLE_ATTRIBUTE_NAME, QuickStartV2.PRODUCT_DIM_VIEW);
-        AtlasEntity view = atlasClientV2.getEntityByAttribute(QuickStartV2.VIEW_TYPE, attributes).getEntity();
-        Map<String, Object> viewAttributes = view.getAttributes();
-        assertEquals(QuickStartV2.PRODUCT_DIM_VIEW, viewAttributes.get(AtlasClient.NAME));
+        Map<String, String> attributes                 = Collections.singletonMap(REFERENCEABLE_ATTRIBUTE_NAME, PRODUCT_DIM_VIEW + CLUSTER_SUFFIX);
+        AtlasEntity         view                       = atlasClientV2.getEntityByAttribute(VIEW_TYPE, attributes).getEntity();
+        Map<String, Object> viewAttributes             = view.getAttributes();
+        Map<String, Object> viewRelationshipAttributes = view.getRelationshipAttributes();
+
+        assertEquals(PRODUCT_DIM_VIEW, viewAttributes.get(NAME));
+
+        String productDimId   = getTable(PRODUCT_DIM_TABLE).getGuid();
+        List   inputTables    = (List) viewRelationshipAttributes.get("inputTables");
+        Map    inputTablesMap = (Map) inputTables.get(0);
 
-        String productDimId = getTable(QuickStartV2.PRODUCT_DIM_TABLE).getGuid();
-        List inputTables = (List) viewAttributes.get("inputTables");
-        Map inputTablesMap = (Map) inputTables.get(0);
         assertEquals(productDimId, inputTablesMap.get("guid"));
     }
 }
